@@ -26,7 +26,7 @@
  * (c) 2001 James Courtier-Dutton <James@superbug.demon.co.uk>
  *
  * 
- * $Id: audio_alsa_out.c,v 1.45 2001/12/24 13:06:10 f1rmb Exp $
+ * $Id: audio_alsa_out.c,v 1.52 2002/04/27 22:11:12 matt2000 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,7 +41,15 @@
 #include <fcntl.h>
 #include <math.h>
 #include <alloca.h>
+
+#ifdef HAVE_ALSA_ASOUNDLIB_H
+#include <alsa/asoundlib.h>
+#elif HAVE_SYS_ASOUNDLIB_H
 #include <sys/asoundlib.h>
+#else
+#error "required asoundlib.h neither in sys/ nor alsa/ - unable to compile."
+#endif
+
 #include <sys/ioctl.h>
 #include <inttypes.h>
 #include <pthread.h>
@@ -60,7 +68,7 @@
 # endif
 #endif
 
-#define AO_OUT_ALSA_IFACE_VERSION 3
+#define AO_OUT_ALSA_IFACE_VERSION 4
 
 #define GAP_TOLERANCE             5000
 
@@ -77,6 +85,7 @@ typedef struct alsa_driver_s {
   snd_pcm_t    *audio_fd;
   int           capabilities;
   int           open_mode;
+  int		has_pause_resume;
 
   int32_t       output_sample_rate, input_sample_rate;
   double        sample_rate_factor;
@@ -87,6 +96,7 @@ typedef struct alsa_driver_s {
 
   struct {
     pthread_t          thread;
+    pthread_mutex_t    mutex;
     char              *name;
     snd_mixer_t       *handle;
     snd_mixer_elem_t  *elem;
@@ -109,7 +119,9 @@ static void *ao_alsa_handle_event_thread(void *data) {
 
   do {
     snd_mixer_wait(this->mixer.handle, -1);
+    pthread_mutex_lock(&this->mixer.mutex);
     snd_mixer_handle_events(this->mixer.handle);
+    pthread_mutex_unlock(&this->mixer.mutex);
   } while(1);
   
   pthread_exit(NULL);
@@ -365,6 +377,8 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
     printf ("audio_alsa_out: buffer time and period time match, could not use\n");
     goto __close;
   }
+  this->has_pause_resume = ( snd_pcm_hw_params_can_pause (params)
+			    && snd_pcm_hw_params_can_resume (params) );
   if ((err = snd_pcm_hw_params(this->audio_fd, params)) < 0) {
     printf ("audio_alsa_out: pcm hw_params failed: %s\n", snd_strerror(err));
     goto __close;
@@ -478,6 +492,7 @@ static void ao_alsa_close(ao_driver_t *this_gen)
   alsa_driver_t *this = (alsa_driver_t *) this_gen;
   if(this->audio_fd) snd_pcm_close(this->audio_fd);
   this->audio_fd = NULL;
+  this->has_pause_resume = 0; /* This is set at open time */
 }
 
 static uint32_t ao_alsa_get_capabilities (ao_driver_t *this_gen) {
@@ -499,6 +514,7 @@ static void ao_alsa_exit(ao_driver_t *this_gen)
 						      this->mixer.min, this->mixer.max))) /2));
   config->save(config);
 
+  pthread_mutex_destroy(&this->mixer.mutex);
   /*
    * Destroy the mixer thread and cleanup the mixer, so that
    * any child processes (such as xscreensaver) cannot inherit
@@ -525,6 +541,9 @@ static int ao_alsa_get_property (ao_driver_t *this_gen, int property) {
   case AO_PROP_MIXER_VOL:
   case AO_PROP_PCM_VOL:
     if(this->mixer.elem) {
+
+      pthread_mutex_lock(&this->mixer.mutex);
+
       if((err = snd_mixer_selem_get_playback_volume(this->mixer.elem, SND_MIXER_SCHN_FRONT_LEFT,
 						    &this->mixer.left_vol)) < 0) {
 	printf("audio_alsa_out: snd_mixer_selem_get_playback_volume(): %s\n",  snd_strerror(err));
@@ -538,6 +557,8 @@ static int ao_alsa_get_property (ao_driver_t *this_gen, int property) {
       }
       
     __done:
+      pthread_mutex_unlock(&this->mixer.mutex);
+
       return (((ao_alsa_get_percent_from_volume(this->mixer.left_vol, 
 						this->mixer.min, this->mixer.max)) +
 	       (ao_alsa_get_percent_from_volume(this->mixer.right_vol, 
@@ -565,20 +586,24 @@ static int ao_alsa_set_property (ao_driver_t *this_gen, int property, int value)
   case AO_PROP_PCM_VOL:
     if(this->mixer.elem) {
 
+      pthread_mutex_lock(&this->mixer.mutex);
+
       this->mixer.left_vol = this->mixer.right_vol = ao_alsa_get_volume_from_percent(value, this->mixer.min, this->mixer.max);
       
       if((err = snd_mixer_selem_set_playback_volume(this->mixer.elem, SND_MIXER_SCHN_FRONT_LEFT,
 						    this->mixer.left_vol)) < 0) {
 	printf("audio_alsa_out: snd_mixer_selem_get_playback_volume(): %s\n",  snd_strerror(err));
+	pthread_mutex_unlock(&this->mixer.mutex);
 	return ~value;
       }
-
+      
       if((err = snd_mixer_selem_set_playback_volume(this->mixer.elem, SND_MIXER_SCHN_FRONT_RIGHT,
 						    this->mixer.right_vol)) < 0) {
 	printf("audio_alsa_out: snd_mixer_selem_get_playback_volume(): %s\n",  snd_strerror(err));
+	pthread_mutex_unlock(&this->mixer.mutex);
 	return ~value;
       }
-
+      pthread_mutex_unlock(&this->mixer.mutex);
       return value;
     }
     break;
@@ -588,6 +613,8 @@ static int ao_alsa_set_property (ao_driver_t *this_gen, int property, int value)
       int sw;
       int old_mute = this->mixer.mute;
       
+      pthread_mutex_lock(&this->mixer.mutex);
+
       this->mixer.mute = (value) ? MIXER_MASK_STEREO : 0;
       
       if ((this->mixer.mute != old_mute) 
@@ -607,10 +634,10 @@ static int ao_alsa_set_property (ao_driver_t *this_gen, int property, int value)
 	  }
 	}
       }
-
-    return value;
+      
+      pthread_mutex_unlock(&this->mixer.mutex);
+      return value;
     }
-
     return ~value;
     break;
   }
@@ -618,6 +645,45 @@ static int ao_alsa_set_property (ao_driver_t *this_gen, int property, int value)
   return ~value;
 }
 
+
+static int ao_alsa_ctrl(ao_driver_t *this_gen, int cmd, ...) {
+  alsa_driver_t *this = (alsa_driver_t *) this_gen;
+  int result;
+#if 0
+  /* Alsa 0.9.x pause and result is not stable enough at the moment.
+   * FIXME: Change these to snd_pcm_drop and restart.
+   */
+  switch (cmd) {
+
+  case AO_CTRL_PLAY_PAUSE:
+    if ((this->has_pause_resume) && (this->audio_fd > 0)) {
+      if ((result=snd_pcm_pause(this->audio_fd, 1)) < 0) {
+        printf("Pause call failed err=%d\n",result);
+      }
+    }
+    break;
+
+  case AO_CTRL_PLAY_RESUME:
+    if ((this->has_pause_resume) && (this->audio_fd > 0) ) {
+      if ((result=snd_pcm_pause(this->audio_fd, 0)) < 0) {
+        printf("Resume call failed err=%d\n",result);
+      }
+    }
+    break;
+
+  case AO_CTRL_FLUSH_BUFFERS:
+    if (this->audio_fd > 0) {
+      snd_pcm_drop(this->audio_fd);
+      snd_pcm_prepare(this->audio_fd);
+    }
+    break;
+  }
+#endif
+
+  return 0;
+}
+
+
 static void ao_alsa_mixer_init(ao_driver_t *this_gen) {
   alsa_driver_t        *this = (alsa_driver_t *) this_gen;
   config_values_t      *config = this->config;
@@ -946,6 +1012,8 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
     printf ("(5.1-channel not enabled in xine config) " );
   }
  
+  this->has_pause_resume = 0; /* This is checked at open time instead */
+
   snd_pcm_close (this->audio_fd);
   this->audio_fd=NULL;
   this->output_sample_rate = 0;
@@ -975,6 +1043,8 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
                                              NULL,
                                              NULL,
                                              NULL);
+
+  pthread_mutex_init(&this->mixer.mutex, NULL);
   ao_alsa_mixer_init(&this->ao_driver);
    
   this->ao_driver.get_capabilities    = ao_alsa_get_capabilities;
@@ -988,6 +1058,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   this->ao_driver.close               = ao_alsa_close;
   this->ao_driver.exit                = ao_alsa_exit;
   this->ao_driver.get_gap_tolerance   = ao_alsa_get_gap_tolerance;
+  this->ao_driver.control	      = ao_alsa_ctrl;
 
   return &this->ao_driver;
 }
