@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2000, 2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_oss_out.c,v 1.55 2002/01/15 12:23:05 siggi Exp $
+ * $Id: audio_oss_out.c,v 1.62 2002/04/28 16:11:12 guenter Exp $
  *
  * 20-8-2001 First implementation of Audio sync and Audio driver separation.
  * Copyright (C) 2001 James Courtier-Dutton James@superbug.demon.co.uk
@@ -79,7 +79,7 @@
 #       define AFMT_AC3         0x00000400  
 #endif
 
-#define AO_OUT_OSS_IFACE_VERSION 3
+#define AO_OUT_OSS_IFACE_VERSION 4
 
 #define AUDIO_NUM_FRAGMENTS     15
 #define AUDIO_FRAGMENT_SIZE   8192
@@ -94,6 +94,7 @@
 #define OSS_SYNC_GETODELAY    1
 #define OSS_SYNC_GETOPTR      2
 #define OSS_SYNC_SOFTSYNC     3
+#define OSS_SYNC_PROBEBUFFER  4
 
 #ifdef CONFIG_DEVFS_FS
 #define DSP_TEMPLATE "/dev/sound/dsp%d"
@@ -121,6 +122,7 @@ typedef struct oss_driver_s {
   int              audio_started;
   int              sync_method;
   int              latency;
+  int              buffer_size;
 
   struct {
     char          *name;
@@ -182,7 +184,7 @@ static int ao_oss_open(ao_driver_t *this_gen,
    * configure audio device
    * In A52 mode, skip all other SNDCTL commands
    */
-  if(!(mode & AO_CAP_MODE_A52)) {
+  if(!(mode & (AO_CAP_MODE_A52 | AO_CAP_MODE_AC5))) {
     tmp = (mode & AO_CAP_MODE_STEREO) ? 1 : 0;
     ioctl(this->audio_fd,SNDCTL_DSP_STEREO,&tmp);
 
@@ -248,9 +250,9 @@ static int ao_oss_open(ao_driver_t *this_gen,
   printf ("audio_oss_out : %d channels output\n",this->num_channels);
   this->bytes_per_frame=(this->bits_per_sample*this->num_channels)/8;
   
- /*
-  * set format
-  */
+  /*
+   * set format
+   */
 
   switch (mode) {
   case AO_CAP_MODE_MONO:
@@ -258,23 +260,35 @@ static int ao_oss_open(ao_driver_t *this_gen,
   case AO_CAP_MODE_4CHANNEL:
   case AO_CAP_MODE_5CHANNEL:
   case AO_CAP_MODE_5_1CHANNEL:
-    tmp = AFMT_S16_NE;
+    if (bits==8)
+      tmp = AFMT_U8;
+    else
+      tmp = AFMT_S16_NE;
     if (ioctl(this->audio_fd, SNDCTL_DSP_SETFMT, &tmp) < 0
-	|| tmp != AFMT_S16_NE) {
-        printf("audio_oss_out: SNDCTL_DSP_SETFMT failed for AFMT_S16_NE.\n");
-       if (tmp != AFMT_S16_NE)
-         printf("audio_oss_out: ioctl succeeded but set format to %x.\n", tmp);
-       else
-         printf("audio_oss_out: The AFMT_S16_NE ioctl failed.\n");
-       return 0;
+	|| (tmp!=AFMT_S16_NE && tmp!=AFMT_U8)) {
+      if (bits==8) {
+	printf("audio_oss_out: SNDCTL_DSP_SETFMT failed for AFMT_U8.\n");
+        if (tmp != AFMT_U8)
+          printf("audio_oss_out: ioctl succeeded but set format to 0x%x.\n",tmp);
+        else
+          printf("audio_oss_out: The AFMT_U8 ioctl failed.\n");
+        return 0;
+      } else {
+	printf("audio_oss_out: SNDCTL_DSP_SETFMT failed for AFMT_S16_NE.\n");
+        if (tmp != AFMT_S16_NE)
+          printf("audio_oss_out: ioctl succeeded but set format to 0x%x.\n",tmp);
+        else
+          printf("audio_oss_out: The AFMT_S16_NE ioctl failed.\n");
+        return 0;
+      }          
     }
     break;
   case AO_CAP_MODE_A52:
   case AO_CAP_MODE_AC5:
     tmp = AFMT_AC3;
     if (ioctl(this->audio_fd, SNDCTL_DSP_SETFMT, &tmp) < 0 || tmp != AFMT_AC3) {
-        printf("audio_oss_out: AC3 SNDCTL_DSP_SETFMT failed. %d\n",tmp);
-       return 0;
+      printf("audio_oss_out: AC3 SNDCTL_DSP_SETFMT failed. %d\n",tmp);
+      return 0;
     }
     break;
   }
@@ -286,19 +300,19 @@ static int ao_oss_open(ao_driver_t *this_gen,
    */
 
   /* WARNING: let's hope for good defaults here...
-  tmp=0 ;
-  fsize = AUDIO_FRAGMENT_SIZE;
-  while (fsize>0) {
-    fsize /=2;
-    tmp++;
-  }
-  tmp--;
+     tmp=0 ;
+     fsize = AUDIO_FRAGMENT_SIZE;
+     while (fsize>0) {
+     fsize /=2;
+     tmp++;
+     }
+     tmp--;
 
-  tmp = (AUDIO_NUM_FRAGMENTS << 16) | tmp ;
+     tmp = (AUDIO_NUM_FRAGMENTS << 16) | tmp ;
 
-  printf ("audio_oss_out: audio buffer fragment info : %x\n",tmp);
+     printf ("audio_oss_out: audio buffer fragment info : %x\n",tmp);
 
-  ioctl(this->audio_fd,SNDCTL_DSP_SETFRAGMENT,&tmp); 
+     ioctl(this->audio_fd,SNDCTL_DSP_SETFRAGMENT,&tmp); 
   */
 
   return this->output_sample_rate;
@@ -333,6 +347,12 @@ static int ao_oss_delay(ao_driver_t *this_gen) {
   struct        timeval tv;
 
   switch (this->sync_method) {
+  case OSS_SYNC_PROBEBUFFER:
+    if( this->bytes_in_buffer < this->buffer_size )
+      bytes_left = this->bytes_in_buffer;
+    else
+      bytes_left = this->buffer_size;
+    break;
 
   case OSS_SYNC_SOFTSYNC:
     /* use system real-time clock to get pseudo audio frame position */
@@ -442,29 +462,30 @@ static int ao_oss_get_property (ao_driver_t *this_gen, int property) {
   switch(property) {
   case AO_PROP_PCM_VOL:
   case AO_PROP_MIXER_VOL:
-    mixer_fd = open(this->mixer.name, O_RDONLY);
-    if(mixer_fd != -1) {
-      int cmd = 0;
-      int v;
-      
-      ioctl(mixer_fd, SOUND_MIXER_READ_DEVMASK, &audio_devs);
-      
-      if(audio_devs & SOUND_MASK_PCM)
-	cmd = SOUND_MIXER_READ_PCM;
-      else if(audio_devs & SOUND_MASK_VOLUME)
-	cmd = SOUND_MIXER_READ_VOLUME;
-      else {
+    if(!this->mixer.mute) {
+      mixer_fd = open(this->mixer.name, O_RDONLY);
+      if(mixer_fd != -1) {
+	int cmd = 0;
+	int v;
+	
+	ioctl(mixer_fd, SOUND_MIXER_READ_DEVMASK, &audio_devs);
+	
+	if(audio_devs & SOUND_MASK_PCM)
+	  cmd = SOUND_MIXER_READ_PCM;
+	else if(audio_devs & SOUND_MASK_VOLUME)
+	  cmd = SOUND_MIXER_READ_VOLUME;
+	else {
+	  close(mixer_fd);
+	  return 0;
+	}
+	ioctl(mixer_fd, cmd, &v);
+	this->mixer.volume = (((v & 0xFF00) >> 8) + (v & 0x00FF)) / 2;
 	close(mixer_fd);
-	return 0;
       }
-      ioctl(mixer_fd, cmd, &v);
-      this->mixer.volume = (((v & 0xFF00) >> 8) + (v & 0x00FF)) / 2;
-      close(mixer_fd);
+      else
+	printf("%s(): open() %s failed: %s\n", 
+	       __XINE_FUNCTION__, this->mixer.name, strerror(errno));
     }
-    else
-      printf("%s(): open() %s failed: %s\n", 
-	     __XINE_FUNCTION__, this->mixer.name, strerror(errno));
-    
     return this->mixer.volume;
     break;
 
@@ -486,7 +507,6 @@ static int ao_oss_set_property (ao_driver_t *this_gen, int property, int value)
   case AO_PROP_PCM_VOL:
   case AO_PROP_MIXER_VOL:
     if(!this->mixer.mute) {
-
       mixer_fd = open(this->mixer.name, O_RDONLY);
 
       if(mixer_fd != -1) {
@@ -561,6 +581,24 @@ static int ao_oss_set_property (ao_driver_t *this_gen, int property, int value)
   return ~value;
 }
 
+static int ao_oss_ctrl(ao_driver_t *this_gen, int cmd, ...) {
+  /*oss_driver_t *this = (oss_driver_t *) this_gen;*/
+
+  switch (cmd) {
+
+  case AO_CTRL_PLAY_PAUSE:
+    break;
+
+  case AO_CTRL_PLAY_RESUME:
+    break;
+
+  case AO_CTRL_FLUSH_BUFFERS:
+    break;
+  }
+
+  return 0;
+}
+
 ao_driver_t *init_audio_out_plugin (config_values_t *config) {
 
   oss_driver_t *this;
@@ -575,7 +613,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   int              devnum;
   int              audio_fd;
   int              num_channels, status, arg;
-  static char     *sync_methods[] = {"auto", "getodelay", "getoptr", "softsync", NULL};
+  static char     *sync_methods[] = {"auto", "getodelay", "getoptr", "softsync", "probebuffer", NULL};
   
   this = (oss_driver_t *) malloc (sizeof (oss_driver_t));
 
@@ -692,6 +730,41 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
 
     gettimeofday(&this->start_time, NULL);
   }
+  
+  if (this->sync_method == OSS_SYNC_PROBEBUFFER) {
+    char *buf;
+    int c;
+  
+    printf ("audio_oss_out: Audio driver realtime sync disabled...\n");
+    printf ("audio_oss_out: ...probing output buffer size: ");
+    this->buffer_size = 0;
+    
+    if( (buf=malloc(1024)) != NULL ) {
+      memset(buf,0,1024);
+     
+      do {
+        c = write(audio_fd,buf,1024);
+        if( c != -1 )
+          this->buffer_size += c;
+      } while( c == 1024 );
+      
+      free(buf);
+    }
+    close(audio_fd);
+    printf ("%d bytes\n", this->buffer_size );
+    printf ("audio_oss_out: ...there may be audio/video synchronization issues\n");
+  
+    audio_fd=open(this->audio_dev, O_WRONLY|O_NONBLOCK);
+
+    if(audio_fd < 0) 
+    {
+      printf("audio_oss_out: opening audio device %s failed:\n%s\n",
+	   this->audio_dev, strerror(errno));
+
+      free (this);
+      return NULL;
+    }
+  }
 
   this->latency = config->register_range (config, "audio.oss_latency", 0,
 					  -3000, 3000, 
@@ -806,11 +879,18 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
     
     this->mixer.mute = 0;
     this->mixer.volume = ao_oss_get_property (&this->ao_driver, this->mixer.prop);
-    this->mixer.volume = config->register_range (config, "audio.oss_mixer_volume", 50,
-						 0, 100, "Audio volume", NULL, NULL, NULL);
-    
-    (void) ao_oss_set_property(&this->ao_driver, this->mixer.prop, this->mixer.volume);
-    
+    if (config->register_bool (config, "audio.remember_volume", 0,
+			       "restore volume level at startup", 
+			       "if this not set, xine will not touch any mixer settings at startup",
+			       NULL, NULL)) {
+      this->mixer.volume = config->register_range (config,
+						   "audio.oss_mixer_volume",
+						   50, 0, 100, "Audio volume",
+						   NULL, NULL, NULL);
+      (void) ao_oss_set_property(&this->ao_driver,
+				 this->mixer.prop, this->mixer.volume);
+      
+    }
   }
   close (audio_fd);
 
@@ -830,6 +910,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   this->ao_driver.close               = ao_oss_close;
   this->ao_driver.exit                = ao_oss_exit;
   this->ao_driver.get_gap_tolerance   = ao_oss_get_gap_tolerance;
+  this->ao_driver.control	      = ao_oss_ctrl;
 
   return &this->ao_driver;
 }
