@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2000, 2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_asf.c,v 1.20 2002/01/14 00:34:22 guenter Exp $
+ * $Id: demux_asf.c,v 1.35 2002/04/19 22:57:07 miguelfreitas Exp $
  *
  * demultiplexer for asf streams
  *
@@ -47,6 +47,10 @@
 #include "libw32dll/wine/vfw.h"
 #include "libw32dll/wine/mmreg.h"
 
+/*
+#define LOG
+*/
+
 #define PACKET_SIZE        3200
 #define PACKET_HEADER_SIZE   12
 #define FRAME_HEADER_SIZE    17
@@ -58,26 +62,6 @@
 
 #define VALID_ENDS    "asf,wmv"
 
-#ifdef __GNUC__
-#define LOG_MSG_STDERR(xine, message, args...) {                     \
-    xine_log(xine, XINE_LOG_DEMUX, message, ##args);                 \
-    fprintf(stderr, message, ##args);                                \
-  }
-#define LOG_MSG(xine, message, args...) {                            \
-    xine_log(xine, XINE_LOG_DEMUX, message, ##args);                 \
-    printf(message, ##args);                                         \
-  }
-#else
-#define LOG_MSG_STDERR(xine, ...) {                                  \
-    xine_log(xine, XINE_LOG_DEMUX, __VA_ARGS__);                     \
-    fprintf(stderr, __VA_ARGS__);                                    \
-  }
-#define LOG_MSG(xine, ...) {                                         \
-    xine_log(xine, XINE_LOG_DEMUX, __VA_ARGS__);                     \
-    printf(__VA_ARGS__);                                             \
-  }
-#endif
-
 typedef struct {
   int               num;
   int               seq;
@@ -132,6 +116,11 @@ typedef struct demux_asf_s {
 
   /* packet filling */
   int               packet_size_left;
+
+  /* frame rate calculations */
+
+  int64_t           last_video_pts;
+  int32_t           frame_duration;
   
   /* only for reading */
   int               packet_padsize;
@@ -140,16 +129,20 @@ typedef struct demux_asf_s {
   int               frame;
   
   pthread_t         thread;
+  pthread_mutex_t   mutex;
 
   int               status;
 
   int               send_end_buffers;
+
+  int               send_discontinuity;
   
   /* byte reordering from audio streams */
   int               reorder_h;
   int               reorder_w;
   int               reorder_b;
 
+  off_t             header_size;
 } demux_asf_t ;
 
 
@@ -235,7 +228,9 @@ static uint8_t get_byte (demux_asf_t *this) {
   /* printf ("%02x ", buf); */
   
   if (i != 1) {
-    LOG_MSG(this->xine, _("demux_asf: end of data\n"));
+#ifdef LOG
+    printf ("demux_asf: end of data\n");
+#endif
     this->status = DEMUX_FINISHED;
   }
 
@@ -252,7 +247,7 @@ static uint16_t get_le16 (demux_asf_t *this) {
   /* printf (" [%02x %02x] ", buf[0], buf[1]); */
 
   if (i != 2) {
-    LOG_MSG(this->xine, _("demux_asf: end of data\n"));
+    printf ("demux_asf: end of data\n");
     this->status = DEMUX_FINISHED;
   }
 
@@ -269,7 +264,7 @@ static uint32_t get_le32 (demux_asf_t *this) {
   /* printf ("%02x %02x %02x %02x ", buf[0], buf[1], buf[2], buf[3]); */
 
   if (i != 4) {
-    LOG_MSG(this->xine, _("demux_asf: end of data\n"));
+    printf ("demux_asf: end of data\n");
     this->status = DEMUX_FINISHED;
   }
 
@@ -284,7 +279,7 @@ static uint64_t get_le64 (demux_asf_t *this) {
   i = this->input->read (this->input, buf, 8);
 
   if (i != 8) {
-    LOG_MSG(this->xine, _("demux_asf: end of data\n"));
+    printf ("demux_asf: end of data\n");
     this->status = DEMUX_FINISHED;
   }
 
@@ -337,13 +332,13 @@ static void asf_send_audio_header (demux_asf_t *this, int stream_id) {
     formattag_to_buf_audio ( wavex->wFormatTag );
     
   if ( !this->streams[this->num_streams].buf_type ) {
-    LOG_MSG(this->xine, _("demux_asf: unknown audio type 0x%x\n"), wavex->wFormatTag);
+    printf ("demux_asf: unknown audio type 0x%x\n", wavex->wFormatTag);
     this->streams[this->num_streams].buf_type     = BUF_CONTROL_NOP;
-  }
-  else
-    LOG_MSG(this->xine, _("demux_asf: audio format : %s (wFormatTag 0x%x)\n"), 
-	    buf_audio_name(this->streams[this->num_streams].buf_type),
-	    wavex->wFormatTag);
+  } else
+    xine_log (this->xine, XINE_LOG_FORMAT,
+	      _("demux_asf: audio format : %s (wFormatTag 0x%x)\n"), 
+	      buf_audio_name(this->streams[this->num_streams].buf_type),
+	      wavex->wFormatTag);
 
   this->streams[this->num_streams].buf_type   |= this->num_audio_streams;
   this->streams[this->num_streams].fifo        = this->audio_fifo;
@@ -361,11 +356,13 @@ static void asf_send_audio_header (demux_asf_t *this, int stream_id) {
   buf->content = buf->mem;
   memcpy (buf->content, this->wavex, this->wavex_size);
 
-  LOG_MSG(this->xine, _("demux_asf: wavex header is %d bytes long\n"), this->wavex_size);
+#ifdef LOG
+  printf ("demux_asf: wavex header is %d bytes long\n", this->wavex_size);
+#endif
 
   buf->size = this->wavex_size;
   buf->type = this->streams[this->num_streams].buf_type;
-  buf->decoder_info[0] = 0; /* first package, containing wavex */
+  buf->decoder_flags   = BUF_FLAG_HEADER;
   buf->decoder_info[1] = wavex->nSamplesPerSec;
   buf->decoder_info[2] = wavex->wBitsPerSample;
   buf->decoder_info[3] = wavex->nChannels;
@@ -388,7 +385,7 @@ static void asf_send_video_header (demux_asf_t *this, int stream_id) {
     fourcc_to_buf_video((void*)&bih->biCompression);
     
   if( !this->streams[this->num_streams].buf_type ) {
-    LOG_MSG(this->xine, _("demux_asf: unknown video format %.4s\n"),
+    printf ("demux_asf: unknown video format %.4s\n",
 	    (char*)&bih->biCompression);
     
     this->status = DEMUX_FINISHED;
@@ -401,15 +398,13 @@ static void asf_send_video_header (demux_asf_t *this, int stream_id) {
   this->streams[this->num_streams].frag_offset  = 0;
   this->streams[this->num_streams].defrag = 0;
   
-  /* 
-  printf ("demux_asf: video format : %.4s\n", (char*)&bih->biCompression);
-  */
-  LOG_MSG(this->xine, _("demux_asf: video format : %s\n"), 
-	  buf_video_name(this->streams[this->num_streams].buf_type));
+  xine_log (this->xine, XINE_LOG_FORMAT,
+	    _("demux_asf: video format : %s\n"), 
+	    buf_video_name(this->streams[this->num_streams].buf_type));
 
   buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
   buf->content = buf->mem;
-  buf->decoder_info[0] = 0; /* first package, containing bih */
+  buf->decoder_flags   = BUF_FLAG_HEADER;
   buf->decoder_info[1] = 3000; /* FIXME ? */
   memcpy (buf->content, &this->bih, this->bih_size);
   buf->size = this->bih_size;
@@ -428,7 +423,7 @@ static int asf_read_header (demux_asf_t *this) {
 
   get_guid(this, &g);
   if (memcmp(&g, &asf_header, sizeof(GUID))) {
-    LOG_MSG(this->xine, _("demux_asf: file doesn't start with an asf header\n"));
+    printf ("demux_asf: file doesn't start with an asf header\n");
     return 0;
   }
   get_le64(this);
@@ -456,14 +451,14 @@ static int asf_read_header (demux_asf_t *this) {
       end_time =  get_le64 (this); 
       
       this->length = get_le64(this) / 10000000; 
-      if ( (this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE)
-	   && (this->length) )
+      if (this->length)
 	this->rate = this->input->get_length (this->input) / this->length;
       else
 	this->rate = 0;
 
-      LOG_MSG(this->xine, _("demux_asf: stream length is %d sec, rate is %d bytes/sec\n"),
-	      this->length, this->rate);
+      xine_log (this->xine, XINE_LOG_FORMAT,
+		_("demux_asf: stream length is %d sec, rate is %d bytes/sec\n"),
+		this->length, this->rate);
 
       start_time = get_le32(this); /* start timestamp in 1/1000 s*/
 
@@ -513,16 +508,18 @@ static int asf_read_header (demux_asf_t *this) {
           this->reorder_w=(buffer[2]<<8)|buffer[1];
           this->reorder_b=(buffer[4]<<8)|buffer[3];
   	  this->reorder_w/=this->reorder_b;
-          LOG_MSG(this->xine, _("demux_asf: audio conceal interleave detected (%d x %d x %d)\n"),
+          printf ("demux_asf: audio conceal interleave detected (%d x %d x %d)\n",
 		  this->reorder_w, this->reorder_h, this->reorder_b );
 	} else {
 	  this->reorder_b=this->reorder_h=this->reorder_w=1;        
         }
 
 	this->wavex_size = total_size; /* 18 + this->wavex[8]; */
-
+     
 	asf_send_audio_header (this, stream_id);
-
+#ifdef LOG
+        printf ("found a_stream id=%d \n", stream_id);
+#endif
       } else {
 
 	get_le32(this); /* width */
@@ -533,6 +530,9 @@ static int asf_read_header (demux_asf_t *this) {
 	this->input->read (this->input, (uint8_t *) this->bih, this->bih_size);
 
 	asf_send_video_header (this, stream_id);
+#ifdef LOG
+	printf ("found v_stream id=%d \n", stream_id);
+#endif
       }
       pos2 = this->input->get_current_pos (this->input);
       this->input->seek (this->input, gsize - (pos2 - pos1 + 24), SEEK_CUR);
@@ -594,9 +594,10 @@ static void asf_reorder(demux_asf_t *this, uint8_t *src, int len){
 
 static int asf_get_packet(demux_asf_t *this) {
 
-  int timestamp, hdr_size;
-
+  int      timestamp, hdr_size;
   uint32_t sig = 0;
+  int      duration;
+  int      packet_size;
 
   hdr_size = 11;
 
@@ -607,35 +608,52 @@ static int asf_get_packet(demux_asf_t *this) {
   this->packet_flags = get_byte(this);
   this->segtype = get_byte(this);
   this->packet_padsize = 0;
-  
-  if (this->packet_flags & 0x40) {
-    get_le16(this);
-    LOG_MSG(this->xine, _("demux_asf: absolute size ignored\n"));
+
+  if (this->packet_flags & 0x40) { 
+
+    /* packet size given */
+
+    packet_size = get_le16(this);
+#ifdef LOG
+    printf ("demux_asf: absolute packet size is %d\n", packet_size);
+#endif
     hdr_size += 2;
-  }
+
+    if (this->packet_flags & 0x10) {
+      /* FIXME: ignore ? this->packet_padsize =*/ get_le16(this);
+      hdr_size += 2;
+    } else if (this->packet_flags & 0x08) {
+      /* FIXME: ignore ? this->packet_padsize =*/ get_byte(this);
+      hdr_size++;
+    }
+  } else {
+    packet_size = this->packet_size;
       
-  if (this->packet_flags & 0x10) {
-    this->packet_padsize = get_le16(this);
-    hdr_size += 2;
-  } else if (this->packet_flags & 0x08) {
-    this->packet_padsize = get_byte(this);
-    hdr_size++;
+    if (this->packet_flags & 0x10) {
+      this->packet_padsize = get_le16(this);
+      hdr_size += 2;
+    } else if (this->packet_flags & 0x08) {
+      this->packet_padsize = get_byte(this);
+      hdr_size++;
+    }
   }
+
   timestamp = get_le32(this);
-  get_le16(this); /* duration */
+  duration  = get_le16(this) ; /* duration */
   if (this->packet_flags & 0x01) {
     this->nb_frames = get_byte(this); /* nb_frames */
     hdr_size++;
   }
   else
     this->nb_frames = 1;
+
   this->frame = 0;
     
-  this->packet_size_left = this->packet_size - hdr_size;
+  this->packet_size_left = packet_size - hdr_size;
 
   /*
-  printf ("demux_asf: new packet, size = %d, flags = 0x%02x, padsize = %d\n",
-	  this->packet_size_left, this->packet_flags, this->packet_padsize);
+    printf ("demux_asf: new packet, size = %d, flags = 0x%02x, padsize = %d\n",
+    this->packet_size_left, this->packet_flags, this->packet_padsize);
   */
 
   return 1;
@@ -645,14 +663,33 @@ static void hexdump (unsigned char *data, int len, xine_t *xine) {
   int i;
 
   for (i=0; i<len; i++)
-    LOG_MSG(xine, "%02x ", data[i]);
-  LOG_MSG(xine, "\n");
+    printf ( "%02x ", data[i]);
+  printf ("\n");
 
 }
 
+static void asf_send_discontinuity (demux_asf_t *this, int64_t pts) {
+
+  buf_element_t  *buf;
+
+  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
+  buf->type          = BUF_CONTROL_NEWPTS;
+  buf->disc_off      = pts;
+  this->video_fifo->put (this->video_fifo, buf);
+
+  if(this->audio_fifo) {
+    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+    buf->type          = BUF_CONTROL_NEWPTS;
+    buf->disc_off      = pts;
+    this->audio_fifo->put (this->audio_fifo, buf);
+  }
+
+}
+
+
 static void asf_send_buffer_nodefrag (demux_asf_t *this, asf_stream_t *stream, 
-			     int frag_offset, int seq, int timestamp, 
-			     int frag_len, int payload_size) {
+				      int frag_offset, int seq, int timestamp, 
+				      int frag_len, int payload_size) {
 
   buf_element_t *buf;
   int bufsize;
@@ -699,8 +736,23 @@ static void asf_send_buffer_nodefrag (demux_asf_t *this, asf_stream_t *stream,
       buf->input_pos  = 0 ;
       buf->input_time = 0 ;
     }
-    buf->PTS        = timestamp * 90;
-    buf->SCR        = timestamp * 90;
+
+#ifdef LOG
+    printf ("demux_asf: input pos is %lld, input time is %d\n",
+	    buf->input_pos,
+	    buf->input_time);
+#endif
+
+    buf->pts        = timestamp * 90;
+
+    if (buf->pts && this->send_discontinuity && this->keyframe_found) {
+      this->send_discontinuity--;
+      if( !this->send_discontinuity )
+        asf_send_discontinuity (this, buf->pts);
+      else
+        buf->pts = 0;
+    }
+
     buf->type       = stream->buf_type;
     buf->size       = bufsize;
     timestamp       = 0;
@@ -710,24 +762,42 @@ static void asf_send_buffer_nodefrag (demux_asf_t *this, asf_stream_t *stream,
 
     /* test if whole packet read */
     if (stream->frag_offset == payload_size) {
-      buf->decoder_info[0] = 2;
-      stream->frag_offset = 0;
-    } else 
-      buf->decoder_info[0] = 1;
 
+      if ( (buf->type & BUF_MAJOR_MASK) == BUF_VIDEO_BASE) {
+	if (buf->pts && this->last_video_pts) 
+	  this->frame_duration = (3*this->frame_duration + (buf->pts - this->last_video_pts)) / 4;
+
+	/*
+	printf ("demux_asf: frame_duration is %d\n", this->frame_duration);
+	*/
+
+	this->last_video_pts = buf->pts;
+
+	buf->decoder_flags   = BUF_FLAG_FRAME_END | BUF_FLAG_FRAMERATE;
+	buf->decoder_info[0] = this->frame_duration;
+      } else
+	buf->decoder_flags   = BUF_FLAG_FRAME_END;
+
+      stream->frag_offset = 0;
+
+    } else 
+      buf->decoder_flags   = 0;
+
+    if( !this->keyframe_found )
+      buf->decoder_flags   |= BUF_FLAG_PREVIEW;
+    
     stream->fifo->put (stream->fifo, buf);
   }
 }
 
-
 static void asf_send_buffer_defrag (demux_asf_t *this, asf_stream_t *stream, 
-			     int frag_offset, int seq, int timestamp, 
-			     int frag_len, int payload_size) {
-
+				    int frag_offset, int seq, int timestamp, 
+				    int frag_len, int payload_size) {
+  
   buf_element_t *buf;
 
   /*
-  printf("asf_send_buffer seq=%d frag_offset=%d frag_len=%d\n",
+    printf("asf_send_buffer seq=%d frag_offset=%d frag_len=%d\n",
     seq, frag_offset, frag_len );
   */
   
@@ -741,55 +811,62 @@ static void asf_send_buffer_defrag (demux_asf_t *this, asf_stream_t *stream,
     } else {
       /* cannot continue current packet: free it */
       if( stream->frag_offset )
-      {
-        int bufsize;
-        uint8_t *p;
+	{
+	  int bufsize;
+	  uint8_t *p;
                
-        if (stream->fifo == this->audio_fifo && 
-            this->reorder_h > 1 && this->reorder_w > 1 ) {
-          asf_reorder(this,stream->buffer,stream->frag_offset);
-        }
+	  if (stream->fifo == this->audio_fifo && 
+	      this->reorder_h > 1 && this->reorder_w > 1 ) {
+	    asf_reorder(this,stream->buffer,stream->frag_offset);
+	  }
         
-        p = stream->buffer;
-        while( stream->frag_offset ) {
-          if ( stream->frag_offset < stream->fifo->buffer_pool_buf_size )
-            bufsize = stream->frag_offset;
-          else
-            bufsize = stream->fifo->buffer_pool_buf_size;
-
-          buf = stream->fifo->buffer_pool_alloc (stream->fifo);
-          buf->content = buf->mem;
-          xine_fast_memcpy (buf->content, p, bufsize);
-
-          if (stream->fifo == this->video_fifo) {
-            buf->input_pos  = this->input->get_current_pos (this->input);
-	    if (this->rate)
-	      buf->input_time = buf->input_pos / this->rate;
+	  p = stream->buffer;
+	  while( stream->frag_offset ) {
+	    if ( stream->frag_offset < stream->fifo->buffer_pool_buf_size )
+	      bufsize = stream->frag_offset;
 	    else
-	      buf->input_time = 0;
-          } else {
-            buf->input_pos  = 0 ;
-            buf->input_time = 0 ;
-          }
-          
-          buf->PTS        = stream->timestamp * 90 + stream->ts_per_kbyte * 
-                            (p-stream->buffer) / 1024; 
-          buf->SCR        = buf->PTS;
-          buf->type       = stream->buf_type;
-          buf->size       = bufsize;
-          
-          stream->frag_offset -= bufsize;
-          p+=bufsize;
-          
-          /* test if whole packet read */
-          if ( !stream->frag_offset )
-            buf->decoder_info[0] = 2;
-          else 
-            buf->decoder_info[0] = 1;
+	      bufsize = stream->fifo->buffer_pool_buf_size;
 
-          stream->fifo->put (stream->fifo, buf);
-        }
-      }
+	    buf = stream->fifo->buffer_pool_alloc (stream->fifo);
+	    buf->content = buf->mem;
+	    xine_fast_memcpy (buf->content, p, bufsize);
+
+	    if (stream->fifo == this->video_fifo) {
+	      buf->input_pos  = this->input->get_current_pos (this->input);
+	      if (this->rate)
+		buf->input_time = buf->input_pos / this->rate;
+	      else
+		buf->input_time = 0;
+	    } else {
+	      buf->input_pos  = 0 ;
+	      buf->input_time = 0 ;
+	    }
+          
+	    buf->pts        = stream->timestamp * 90 + stream->ts_per_kbyte * 
+	      (p-stream->buffer) / 1024; 
+
+	    if (buf->pts && this->send_discontinuity) {
+	      buf->pts = 0;
+	    }
+
+	    buf->type       = stream->buf_type;
+	    buf->size       = bufsize;
+          
+	    stream->frag_offset -= bufsize;
+	    p+=bufsize;
+          
+	    /* test if whole packet read */
+	    if ( !stream->frag_offset )
+	      buf->decoder_flags   = BUF_FLAG_FRAME_END;
+	    else 
+	      buf->decoder_flags = 0;
+	  
+	    if( !this->keyframe_found )
+	      buf->decoder_flags   |= BUF_FLAG_PREVIEW;
+	      
+	    stream->fifo->put (stream->fifo, buf);
+	  }
+	}
       
       stream->frag_offset = 0;
       if (frag_offset != 0) {
@@ -813,17 +890,12 @@ static void asf_send_buffer_defrag (demux_asf_t *this, asf_stream_t *stream,
   }
   
   if( stream->frag_offset + frag_len > DEFRAG_BUFSIZE ) {
-    LOG_MSG(this->xine, _("demux_asf: buffer overflow on defrag!\n"));
+    printf ("demux_asf: buffer overflow on defrag!\n");
   }
   else {  
     this->input->read (this->input, &stream->buffer[stream->frag_offset], frag_len);
     stream->frag_offset += frag_len;
   }
-  
-  /*
-  printf ("demux_asf: read %d bytes :", frag_len);
-  hexdump (buf->content, frag_len, this->xine);
-  */
 }
 
 
@@ -832,19 +904,46 @@ static void asf_read_packet(demux_asf_t *this) {
   int            raw_id, stream_id, seq, frag_offset, payload_size, frag_len;
   int            timestamp, flags, i;
   asf_stream_t  *stream;
+  static int nb_all_p=0;
+  static int nb_p_c=0;
+  static int num_stream_c=0;
+  int id;
+
+ if ( !num_stream_c ){
+   if(this->num_streams > 0){
+     num_stream_c= this->num_streams ;
+     id=this->streams[num_stream_c-1].stream_id;
+    }
+   else
+     return ;
+  
+  }
+  else
+   id  = this->streams[num_stream_c-1].stream_id;;
+   
+ if (this->num_streams > 0 && nb_p_c < nb_all_p /this->num_streams -10){
+   if ( num_stream_c > 2 ){ /* still have at least one video stream except this*/
+       printf ("changing a videoo stream \n"); 
+       num_stream_c--;
+       id= this->streams[num_stream_c-1].stream_id;
+   }
+   nb_p_c = nb_all_p =0;
+ }
 
   if ((this->packet_size_left < FRAME_HEADER_SIZE) ||
       (this->packet_size_left <= this->packet_padsize) ||
       (++this->frame == (this->nb_frames & 0x3f)) ) {
     /* fail safe */
 
-    /* printf ("demux_asf: reading new packet, packet size left %d\n", this->packet_size_left);
-    */
+#ifdef LOG
+    printf ("demux_asf: reading new packet, packet size left %d\n", this->packet_size_left);
+#endif
+
     if (this->packet_size_left)
       this->input->seek (this->input, this->packet_size_left, SEEK_CUR);
     
     if (!asf_get_packet(this)) {
-      LOG_MSG(this->xine, _("demux_asf: get_packet failed\n"));
+      printf ("demux_asf: get_packet failed\n");
       this->status = DEMUX_FINISHED;
       return ;
     }
@@ -856,12 +955,23 @@ static void asf_read_packet(demux_asf_t *this) {
   stream_id  = raw_id & 0x7f;
 
   stream    = NULL;
-  if ( (raw_id & 0x80) || this->keyframe_found ) {
-    for (i=0; i<this->num_streams; i++)
-      if (this->streams[i].stream_id == stream_id)
+  nb_all_p++;
+  if ( (raw_id & 0x80) || this->keyframe_found || (this->num_video_streams==0)) {
+#if LOG
+    printf ("asf_demuxer: got stream =%d \n", stream_id);
+#endif
+    for (i=0; i<this->num_streams; i++){
+      if (this->streams[i].stream_id == stream_id && (stream_id==1
+       ||  stream_id==id)) {
 	stream = &this->streams[i];
-    this->keyframe_found = 1;
+	nb_p_c++;
+	
+	if( raw_id & 0x80 )
+	  this->keyframe_found = 1;
+      }
+    }
   }
+   
 
   seq           = get_byte(this);
   switch (this->segtype){
@@ -878,17 +988,16 @@ static void asf_read_packet(demux_asf_t *this) {
     this->packet_size_left -= 4;
     break;
   default:
-    LOG_MSG(this->xine, _("demux_asf: unknow segtype %x\n"), this->segtype);
-    frag_offset = get_le32(this);
-    this->packet_size_left -= 4;
+    printf ("demux_asf: unknown segtype %x\n", this->segtype);
+    frag_offset = 0;
     break;
   }
   flags         = get_byte(this); 
 
-  /*
-  printf ("\ndemux_asf: segment header, stream id %02x, frag_offset %d, flags : %02x\n", 
-	  stream_id, frag_offset, flags);
-  */
+#ifdef LOG
+    printf ("demux_asf: segment header, stream id %02x, frag_offset %d, flags : %02x\n", 
+	    stream_id, frag_offset, flags);
+#endif
 
   if (flags == 1) {
     int data_length, data_sent=0;
@@ -908,8 +1017,8 @@ static void asf_read_packet(demux_asf_t *this) {
       this->packet_size_left -= data_length + 4;
       
       /*
-      printf ("demux_asf: reading grouping part segment, size = %d\n",
-	      data_length);
+	printf ("demux_asf: reading grouping part segment, size = %d\n",
+	data_length);
       */
 
     } else {
@@ -918,7 +1027,7 @@ static void asf_read_packet(demux_asf_t *this) {
       this->packet_size_left = this->packet_padsize;
 
       /*
-      printf ("demux_asf: reading grouping single segment, size = %d\n", data_length); 
+	printf ("demux_asf: reading grouping single segment, size = %d\n", data_length); 
       */
     }
 
@@ -926,21 +1035,25 @@ static void asf_read_packet(demux_asf_t *this) {
       int object_length = get_byte(this);
 
       /*
-      printf ("demux_asf: sending grouped object, len = %d\n", object_length);
+	printf ("demux_asf: sending grouped object, len = %d\n", object_length);
       */
 
       if (stream) {
+#ifdef LOG
+      printf ("demux_asf: sending buffer of type %08x\n", stream->buf_type);
+#endif
+
 	if (stream->defrag)
           asf_send_buffer_defrag (this, stream, 0, seq, timestamp, 
-			 object_length, object_length);
+				  object_length, object_length);
         else
           asf_send_buffer_nodefrag (this, stream, 0, seq, timestamp, 
-			 object_length, object_length);
+				    object_length, object_length);
       }
       else {
-        /*
+#ifdef LOG
 	printf ("demux_asf: unhandled stream type, id %d\n", stream_id);
-        */
+#endif
 	this->input->seek (this->input, object_length, SEEK_CUR);
       }
 
@@ -964,9 +1077,9 @@ static void asf_read_packet(demux_asf_t *this) {
       }        
       this->packet_size_left -= FRAME_HEADER_SIZE + frag_len - 6;
       
-/*
-      printf ("demux_asf: reading part segment, size = %d\n",
-	      frag_len);
+      /*
+	printf ("demux_asf: reading part segment, size = %d\n",
+	frag_len);
       */
 
     } else {
@@ -974,13 +1087,18 @@ static void asf_read_packet(demux_asf_t *this) {
       this->packet_size_left = this->packet_padsize;
       
       /*
-      printf ("demux_asf: reading single segment, size = %d\n", frag_len); 
+	printf ("demux_asf: reading single segment, size = %d\n", frag_len); 
       */
 
     }
 
     
     if (stream) {
+
+#ifdef LOG
+      printf ("demux_asf: sending buffer of type %08x\n", stream->buf_type);
+#endif
+
       if (stream->defrag)
         asf_send_buffer_defrag (this, stream, frag_offset, seq, timestamp, 
                                 frag_len, payload_size);
@@ -988,9 +1106,10 @@ static void asf_read_packet(demux_asf_t *this) {
         asf_send_buffer_nodefrag (this, stream, frag_offset, seq, timestamp,
                                   frag_len, payload_size);
     } else {
-      /*
+
+#ifdef LOG
       printf ("demux_asf: unhandled stream type, id %d\n", stream_id);
-      */
+#endif
       this->input->seek (this->input, frag_len, SEEK_CUR);
     }  
   }
@@ -1009,10 +1128,17 @@ static void *demux_asf_loop (void *this_gen) {
 
   this->send_end_buffers = 1;
 
-  while (this->status == DEMUX_OK) {
-
+  while(1) {
+    
+    pthread_mutex_lock( &this->mutex );
+    
+    if( this->status != DEMUX_OK)
+      break;
+    
     asf_read_packet (this);
-
+    
+    pthread_mutex_unlock( &this->mutex );
+  
   }
 
   /*
@@ -1025,17 +1151,19 @@ static void *demux_asf_loop (void *this_gen) {
   if (this->send_end_buffers) {
     buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
     buf->type            = BUF_CONTROL_END;
-    buf->decoder_info[0] = 0; /* stream finished */
+    buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
     this->video_fifo->put (this->video_fifo, buf);
     
     if(this->audio_fifo) {
       buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
       buf->type            = BUF_CONTROL_END;
-      buf->decoder_info[0] = 0; /* stream finished */
+      buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
       this->audio_fifo->put (this->audio_fifo, buf);
     }
 
   }
+  
+  pthread_mutex_unlock( &this->mutex );
 
   pthread_exit(NULL);
 
@@ -1055,32 +1183,32 @@ static void demux_asf_stop (demux_plugin_t *this_gen) {
   buf_element_t *buf;
   int i;
   void *p;
+  
+  pthread_mutex_lock( &this->mutex );
 
   if (this->status != DEMUX_OK) {
-    LOG_MSG(this->xine, _("demux_asf: stop...ignored\n"));
+    printf ("demux_asf: stop...ignored\n");
+    pthread_mutex_unlock( &this->mutex );
     return;
   }
 
   this->send_end_buffers = 0;
   this->status = DEMUX_FINISHED;
 
-  pthread_cancel (this->thread);
+  pthread_mutex_unlock( &this->mutex );
   pthread_join (this->thread, &p);
 
-  this->video_fifo->clear(this->video_fifo);
-  if (this->audio_fifo)
-    this->audio_fifo->clear(this->audio_fifo);
+  xine_flush_engine(this->xine);
 
   buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
   buf->type            = BUF_CONTROL_END;
-  buf->decoder_info[0] = 1; /* forced */
-
+  buf->decoder_flags   = BUF_FLAG_END_USER; 
   this->video_fifo->put (this->video_fifo, buf);
 
   if(this->audio_fifo) {
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
     buf->type            = BUF_CONTROL_END;
-    buf->decoder_info[0] = 1; /* forced */
+    buf->decoder_flags   = BUF_FLAG_END_USER;
     this->audio_fifo->put (this->audio_fifo, buf);
   }
   
@@ -1106,90 +1234,139 @@ static void demux_asf_start (demux_plugin_t *this_gen,
   demux_asf_t *this = (demux_asf_t *) this_gen;
   buf_element_t *buf;
   int err;
-
-  this->video_fifo  = video_fifo;
-  this->audio_fifo  = audio_fifo;
-
-  /* 
-   * send start buffer
-   */
-
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->type    = BUF_CONTROL_START;
-  this->video_fifo->put (this->video_fifo, buf);
-
-  if(this->audio_fifo) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type    = BUF_CONTROL_START;
-    this->audio_fifo->put (this->audio_fifo, buf);
-  }
-
-  /*
-   * initialize asf engine
-   */
-
-  this->num_streams              = 0;
-  this->num_audio_streams        = 0;
-  this->num_video_streams        = 0;
-  this->packet_size              = 0;
-  this->seqno                    = 0;
+  int starting;
   
-  if (this->input->get_capabilities (this->input) & INPUT_CAP_SEEKABLE)
-    this->input->seek (this->input, 0, SEEK_SET);
+  pthread_mutex_lock( &this->mutex );
 
-  if (!asf_read_header (this)) {
-    
-    this->status = DEMUX_FINISHED;
-    return;
-  } 
-
-  LOG_MSG(this->xine, _("demux_asf: title        : %s\n"), this->title);
-  LOG_MSG(this->xine, _("demux_asf: author       : %s\n"), this->author);
-  LOG_MSG(this->xine, _("demux_asf: copyright    : %s\n"), this->copyright);
-  LOG_MSG(this->xine, _("demux_asf: comment      : %s\n"), this->comment);
-
-  /*
-   * seek to start position
-   */
-
-  if (this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) {
-
-    off_t cur_pos = this->input->get_current_pos (this->input);
-    
-    if ( (!start_pos) && (start_time))
-      start_pos = start_time * this->rate;
-
-    if (start_pos<cur_pos)
-      start_pos = cur_pos;
-
-    this->input->seek (this->input, start_pos, SEEK_SET);
-  }
-
-  /*
-   * now start demuxing
-   */
-
-  this->keyframe_found = 0;
+  starting = (this->status != DEMUX_OK);
   this->status = DEMUX_OK;
+  
+  if( starting ) {
+    this->audio_fifo   = audio_fifo;
+    this->video_fifo   = video_fifo;
+    
+    /* 
+     * send start buffer
+     */
 
-  if ((err = pthread_create (&this->thread,
-			     NULL, demux_asf_loop, this)) != 0) {
-    LOG_MSG_STDERR(this->xine, _("demux_asf: can't create new thread (%s)\n"),
-	     strerror(err));
-    exit (1);
+    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
+    buf->type    = BUF_CONTROL_START;
+    this->video_fifo->put (this->video_fifo, buf);
+
+    if(this->audio_fifo) {
+      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+      buf->type    = BUF_CONTROL_START;
+      this->audio_fifo->put (this->audio_fifo, buf);
+    }
+  
+    /*
+     * initialize asf engine
+     */
+
+    this->num_streams              = 0;
+    this->num_audio_streams        = 0;
+    this->num_video_streams        = 0;
+    this->packet_size              = 0;
+    this->seqno                    = 0;
+    this->frame_duration           = 3000;
+  
+    if (this->input->get_capabilities (this->input) & INPUT_CAP_SEEKABLE)
+      this->input->seek (this->input, 0, SEEK_SET);
+                                       
+    if (!asf_read_header (this)) {
+    
+      this->status = DEMUX_FINISHED;
+    } else { 
+      this->header_size = this->input->get_current_pos (this->input);
+
+      xine_log (this->xine, XINE_LOG_FORMAT,
+	        _("demux_asf: title        : %s\n"), this->title);
+      xine_log (this->xine, XINE_LOG_FORMAT,
+	        _("demux_asf: author       : %s\n"), this->author);
+      xine_log (this->xine, XINE_LOG_FORMAT,
+	        _("demux_asf: copyright    : %s\n"), this->copyright);
+      xine_log (this->xine, XINE_LOG_FORMAT,
+	        _("demux_asf: comment      : %s\n"), this->comment);
+    }
+  }
+  else {
+    xine_flush_engine(this->xine);
+  }
+  
+  if( this->status == DEMUX_OK ) {
+    /*
+     * seek to start position
+     */
+    this->send_discontinuity       = 2;
+    this->last_video_pts           = 0;
+    this->keyframe_found           = 0;
+
+    if (this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) {
+
+      if ( (!start_pos) && (start_time))
+        start_pos = start_time * this->rate;
+
+      if (start_pos < this->header_size)
+        start_pos = this->header_size;
+
+      this->input->seek (this->input, start_pos, SEEK_SET);
+    }
+
+    /*
+     * now start demuxing
+     */
+
+    if( starting ) {
+      if ((err = pthread_create (&this->thread,
+			       NULL, demux_asf_loop, this)) != 0) {
+        printf ("demux_asf: can't create new thread (%s)\n",
+	      strerror(err));
+        exit (1);
+      }
+    }
+  }
+  
+  pthread_mutex_unlock( &this->mutex );
+  
+  if( !starting && this->status != DEMUX_OK ) {
+    void *p;
+    pthread_join (this->thread, &p);
   }
 }
 
+static void demux_asf_seek (demux_plugin_t *this_gen,
+			     off_t start_pos, int start_time) {
+  demux_asf_t *this = (demux_asf_t *) this_gen;
+
+	demux_asf_start (this_gen, this->video_fifo, this->audio_fifo, 
+			 start_pos, start_time);
+}
+
+
 static int demux_asf_open(demux_plugin_t *this_gen,
 			  input_plugin_t *input, int stage) {
 
   demux_asf_t *this = (demux_asf_t *) this_gen;
+  uint8_t      buf[8192];
+  int          len;
 
   switch(stage) {
-
   case STAGE_BY_CONTENT:
+
+    /* 
+     * try to get a preview of the data
+     */
+    len = input->get_optional_data (input, buf, INPUT_OPTIONAL_DATA_PREVIEW);
+    if (len == INPUT_OPTIONAL_UNSUPPORTED)
+      return DEMUX_CANNOT_HANDLE;
+
+    if (!memcmp(buf, &asf_header, sizeof(GUID))) {
+      printf ("demux_asf: file starts with an asf header\n");
+      this->input = input;
+      return DEMUX_CAN_HANDLE;
+    }
+
     return DEMUX_CANNOT_HANDLE;
-    break;
 
   case STAGE_BY_EXTENSION: {
     char *ending;
@@ -1247,11 +1424,10 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
 
   demux_asf_t     *this;
 
-  if (iface != 6) {
-    LOG_MSG(xine, 
-	    _("demux_asf: plugin doesn't support plugin API version %d.\n"
-	      "           this means there's a version mismatch between xine and this "
-	      "           demuxer plugin.\nInstalling current demux plugins should help.\n"),
+  if (iface != 7) {
+    printf ("demux_asf: plugin doesn't support plugin API version %d.\n"
+	    "           this means there's a version mismatch between xine and this "
+	    "           demuxer plugin.\nInstalling current demux plugins should help.\n",
 	    iface);
     return NULL;
   }
@@ -1268,6 +1444,7 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
   this->demux_plugin.interface_version = DEMUXER_PLUGIN_IFACE_VERSION;
   this->demux_plugin.open              = demux_asf_open;
   this->demux_plugin.start             = demux_asf_start;
+  this->demux_plugin.seek              = demux_asf_seek;
   this->demux_plugin.stop              = demux_asf_stop;
   this->demux_plugin.close             = demux_asf_close;
   this->demux_plugin.get_status        = demux_asf_get_status;
@@ -1275,5 +1452,8 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
   this->demux_plugin.get_stream_length = demux_asf_get_stream_length;
   this->demux_plugin.get_mimetypes     = demux_asf_get_mimetypes;
   
+  this->status = DEMUX_FINISHED;
+  pthread_mutex_init( &this->mutex, NULL );
+  
   return (demux_plugin_t *) this;
 }
