@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2000, 2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_mpeg.c,v 1.48 2002/01/02 18:16:07 jkeil Exp $
+ * $Id: demux_mpeg.c,v 1.58 2002/04/28 01:50:54 miguelfreitas Exp $
  *
  * demultiplexer for mpeg 1/2 program streams
  * reads streams of variable blocksizes
@@ -45,28 +45,13 @@
 #define VALID_MRLS          "stdin,fifo"
 #define VALID_ENDS          "mpg,mpeg,mpe"
 
-#ifdef __GNUC__
-#define LOG_MSG_STDERR(xine, message, args...) {                     \
-    xine_log(xine, XINE_LOG_DEMUX, message, ##args);                 \
-    fprintf(stderr, message, ##args);                                \
-  }
-#define LOG_MSG(xine, message, args...) {                            \
-    xine_log(xine, XINE_LOG_DEMUX, message, ##args);                 \
-    printf(message, ##args);                                         \
-  }
-#else
-#define LOG_MSG_STDERR(xine, ...) {                                  \
-    xine_log(xine, XINE_LOG_DEMUX, __VA_ARGS__);                     \
-    fprintf(stderr, __VA_ARGS__);                                    \
-  }
-#define LOG_MSG(xine, ...) {                                         \
-    xine_log(xine, XINE_LOG_DEMUX, __VA_ARGS__);                     \
-    printf(__VA_ARGS__);                                             \
-  }
-#endif
-
 #define NUM_PREVIEW_BUFFERS 150
 
+#define WRAP_THRESHOLD       120000 
+
+#define PTS_AUDIO 0
+#define PTS_VIDEO 1
+
 typedef struct demux_mpeg_s {
 
   demux_plugin_t       demux_plugin;
@@ -81,6 +66,7 @@ typedef struct demux_mpeg_s {
   input_plugin_t      *input;
 
   pthread_t            thread;
+  pthread_mutex_t      mutex;
 
   unsigned char        dummy_space[100000];
 
@@ -91,9 +77,10 @@ typedef struct demux_mpeg_s {
 
   int                  send_end_buffers;
 
-  uint32_t             last_scr;
-
-} demux_mpeg_t ;
+  int64_t              last_pts[2];
+  int                  send_newpts;
+ 
+} demux_mpeg_t;
 
 static uint32_t read_bytes (demux_mpeg_t *this, int n) {
   
@@ -102,15 +89,13 @@ static uint32_t read_bytes (demux_mpeg_t *this, int n) {
   unsigned char buf[6];
 
   buf[4]=0;
-
-
+  
   i = this->input->read (this->input, buf, n);
 
   if (i != n) {
     
     this->status = DEMUX_FINISHED;
   }
-  
 
   switch (n)  {
   case 1:
@@ -126,18 +111,52 @@ static uint32_t read_bytes (demux_mpeg_t *this, int n) {
     res = (buf[2]<<8) | buf[3] | (buf[1]<<16) | (buf[0] << 24);
     break;
   default:
-    LOG_MSG_STDERR(this->xine,
-		   _("How how - something wrong in wonderland demux:read_bytes (%d)\n"), n);
+    printf ("demux_mpeg: how how - something wrong in wonderland demux:read_bytes (%d)\n", n);
     exit (1);
   }
 
   return res;
 }
 
-static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, uint32_t scr) {
+/* redefine abs as macro to handle 64-bit diffs.
+   i guess llabs may not be available everywhere */
+#define abs(x) ( (x<0) ? (-x) : (x) )
+
+static void check_newpts( demux_mpeg_t *this, int64_t pts, int video )
+{
+  int64_t diff;
+  
+  diff = pts - this->last_pts[video];
+  
+  if( !this->preview_mode && pts && 
+      (this->send_newpts || (this->last_pts[video] && abs(diff)>WRAP_THRESHOLD) ) ) {
+    
+    buf_element_t *buf;
+  
+    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
+    buf->type = BUF_CONTROL_NEWPTS;
+    buf->disc_off = pts;
+    this->video_fifo->put (this->video_fifo, buf);
+
+    if (this->audio_fifo) {
+      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+      buf->type = BUF_CONTROL_NEWPTS;
+      buf->disc_off = pts;
+      this->audio_fifo->put (this->audio_fifo, buf);
+    }
+    this->send_newpts = 0;
+    this->last_pts[1-video] = 0;
+  }
+  
+  if( !this->preview_mode && pts )
+    this->last_pts[video] = pts;
+}
+
+static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, int64_t scr) {
 
   int            len, i;
-  uint32_t       w, flags, header_len, pts;
+  uint32_t       w, flags, header_len;
+  int64_t        pts;
   buf_element_t *buf = NULL;
 
   len = read_bytes(this, 2);
@@ -163,7 +182,7 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, uint32_t scr)
       w = read_bytes(this, 2);
       pts |= (w & 0xFFFE) >> 1;
 
-      header_len -= 5 ;
+      header_len -= 5 ; 
     }
 
     /* read rest of header */
@@ -184,13 +203,15 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, uint32_t scr)
       this->status = DEMUX_FINISHED;
       return ;
     }
+
     buf->type      = BUF_AUDIO_A52 + track;
-    buf->PTS       = pts;
-    buf->SCR       = scr;
+    buf->pts       = pts;
+    check_newpts( this, pts, PTS_AUDIO );
+    
     if (this->preview_mode)
-      buf->decoder_info[0] = 0;
+      buf->decoder_flags = BUF_FLAG_PREVIEW;
     else
-      buf->decoder_info[0] = 1;
+      buf->decoder_flags = 0;
 
     buf->input_pos = this->input->get_current_pos (this->input);
 
@@ -234,13 +255,15 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, uint32_t scr)
       this->status = DEMUX_FINISHED;
       return ;
     }
+   
     buf->type      = BUF_AUDIO_MPEG + track;
-    buf->PTS       = pts;
-    buf->SCR       = scr;
+    buf->pts       = pts;
+    check_newpts( this, pts, PTS_AUDIO );
+    
     if (this->preview_mode)
-      buf->decoder_info[0] = 0;
+      buf->decoder_flags = BUF_FLAG_PREVIEW;
     else
-      buf->decoder_info[0] = 1;
+      buf->decoder_flags = 0;
     buf->input_pos = this->input->get_current_pos(this->input);
 
     if(this->audio_fifo)
@@ -280,12 +303,13 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, uint32_t scr)
       return ;
     }
     buf->type = BUF_VIDEO_MPEG;
-    buf->PTS  = pts;
-    buf->SCR  = scr;
+    buf->pts  = pts;
+    check_newpts( this, pts, PTS_VIDEO );
+
     if (this->preview_mode)
-      buf->decoder_info[0] = 0;
+      buf->decoder_flags = BUF_FLAG_PREVIEW;
     else
-      buf->decoder_info[0] = 1;
+      buf->decoder_flags = 0;
     buf->input_pos = this->input->get_current_pos(this->input);
 
     this->video_fifo->put (this->video_fifo, buf);
@@ -298,12 +322,12 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, uint32_t scr)
 
 }
 
-static void parse_mpeg1_packet (demux_mpeg_t *this, int stream_id, uint32_t scr) {
+static void parse_mpeg1_packet (demux_mpeg_t *this, int stream_id, int64_t scr) {
 
   int             len;
   uint32_t        w;
   int             i;
-  int             pts;
+  int64_t         pts;
   buf_element_t  *buf = NULL;
 
   len = read_bytes(this, 2);
@@ -362,7 +386,7 @@ static void parse_mpeg1_packet (demux_mpeg_t *this, int stream_id, uint32_t scr)
 
       pts |= (w & 0xFFFE) >> 1;
 
-/*       printf ("pts2=%d\n",pts); */
+/*       printf ("pts2=%lld\n",pts); */
 
       /* Decoding Time Stamp */
       w = read_bytes(this, 3); len -= 3;
@@ -392,12 +416,13 @@ static void parse_mpeg1_packet (demux_mpeg_t *this, int stream_id, uint32_t scr)
       return ;
     }
     buf->type      = BUF_AUDIO_MPEG + track ;
-    buf->PTS       = pts;
-    buf->SCR       = scr;
+    buf->pts       = pts;
+    check_newpts( this, pts, PTS_AUDIO );
+    
     if (this->preview_mode)
-      buf->decoder_info[0] = 0;
+      buf->decoder_flags = BUF_FLAG_PREVIEW;
     else
-      buf->decoder_info[0] = 1;
+      buf->decoder_flags = 0;
     buf->input_pos = this->input->get_current_pos(this->input);
     if (this->rate)
       buf->input_time = buf->input_pos / (this->rate * 50);
@@ -407,7 +432,6 @@ static void parse_mpeg1_packet (demux_mpeg_t *this, int stream_id, uint32_t scr)
 
   } else if ((stream_id & 0xf0) == 0xe0) {
 
-
     buf = this->input->read_block (this->input, this->video_fifo, len);
 
     if (buf == NULL) {
@@ -415,12 +439,13 @@ static void parse_mpeg1_packet (demux_mpeg_t *this, int stream_id, uint32_t scr)
       return ;
     }
     buf->type = BUF_VIDEO_MPEG;
-    buf->PTS  = pts;
-    buf->SCR  = scr;
+    buf->pts  = pts;
+    check_newpts( this, pts, PTS_VIDEO );
+
     if (this->preview_mode)
-      buf->decoder_info[0] = 0;
+      buf->decoder_flags = BUF_FLAG_PREVIEW;
     else
-      buf->decoder_info[0] = 1;
+      buf->decoder_flags = 0;
     buf->input_pos = this->input->get_current_pos(this->input);
     if (this->rate)
       buf->input_time = buf->input_pos / (this->rate * 50);
@@ -441,7 +466,7 @@ static uint32_t parse_pack(demux_mpeg_t *this) {
 
   uint32_t  buf ;
   int       mpeg_version;
-  uint32_t  scr;
+  int64_t   scr;
 
 
   buf = read_bytes (this, 1);
@@ -509,27 +534,31 @@ static uint32_t parse_pack(demux_mpeg_t *this) {
   }
 
   /* discontinuity ? */
+#if 0
+  /* scr-wrap detection disabled due bad streams */
+  if( scr && !this->preview_mode )
   {  
-    int32_t scr_diff = scr - this->last_scr;
-    if (abs(scr_diff) > 60000) {
-      
+    int64_t scr_diff = scr - this->last_scr;
+    
+    if (abs(scr_diff) > 60000 && !this->send_newpts) {
+     
       buf_element_t *buf;
 
       buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-      buf->type = BUF_CONTROL_AVSYNC_RESET;
-      buf->SCR  = scr;
+      buf->type = BUF_CONTROL_DISCONTINUITY;
+      buf->disc_off = scr_diff;
       this->video_fifo->put (this->video_fifo, buf);
 
       if (this->audio_fifo) {
 	buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-	buf->type = BUF_CONTROL_AVSYNC_RESET;
-	buf->SCR  = scr;
+	buf->type = BUF_CONTROL_DISCONTINUITY;
+	buf->disc_off = scr_diff;
 	this->audio_fifo->put (this->audio_fifo, buf);
       }
     }
     this->last_scr = scr;
   }
-
+#endif
 
   /* system header */
 
@@ -640,33 +669,43 @@ static void *demux_mpeg_loop (void *this_gen) {
 
   demux_mpeg_t *this = (demux_mpeg_t *) this_gen;
   buf_element_t *buf;
-  uint32_t w;
+  uint32_t w=0;
 
-  do {
+  while(1) {
+    
+    pthread_mutex_lock( &this->mutex );
+    
+    if( this->status != DEMUX_OK)
+      break;
+    
     w = parse_pack (this);
 
     if (w != 0x000001ba)
       demux_mpeg_resync (this, w);
     
-  } while (this->status == DEMUX_OK) ;
+    pthread_mutex_unlock( &this->mutex );
+  
+  }
   
   if (this->send_end_buffers) {
     buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
     buf->type            = BUF_CONTROL_END;
-    buf->decoder_info[0] = 0; /* stream finished */
+    buf->decoder_flags   = BUF_FLAG_END_STREAM;
     this->video_fifo->put (this->video_fifo, buf);
 
     if(this->audio_fifo) {
       buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
       buf->type            = BUF_CONTROL_END;
-      buf->decoder_info[0] = 0; /* stream finished */
+      buf->decoder_flags   = BUF_FLAG_END_STREAM;
       this->audio_fifo->put (this->audio_fifo, buf);
     }
   }
 
-  LOG_MSG(this->xine, _("demux loop finished (status: %d, buf:%x)\n"),
+  printf ("demux_mpeg: demux loop finished (status: %d, buf:%x)\n",
 	  this->status, w);
 
+  pthread_mutex_unlock( &this->mutex );
+  
   pthread_exit(NULL);
 
   return NULL;
@@ -678,36 +717,31 @@ static void demux_mpeg_stop (demux_plugin_t *this_gen) {
   buf_element_t *buf;
   void *p;
 
-  LOG_MSG(this->xine, _("demux_mpeg: stop...\n"));
-
+  pthread_mutex_lock( &this->mutex );
+  
   if (this->status != DEMUX_OK) {
-
-    this->video_fifo->clear(this->video_fifo);
-    if(this->audio_fifo) 
-      this->audio_fifo->clear(this->audio_fifo);
-
+    printf ("demux_mpeg: stop...ignored\n");
+    pthread_mutex_unlock( &this->mutex );
     return;
   }
-
+  
   this->send_end_buffers = 0;
   this->status = DEMUX_FINISHED;
-
-  pthread_cancel (this->thread);
+  
+  pthread_mutex_unlock( &this->mutex );
   pthread_join (this->thread, &p);
 
-  this->video_fifo->clear(this->video_fifo);
-  if(this->audio_fifo) 
-    this->audio_fifo->clear(this->audio_fifo);
+  xine_flush_engine(this->xine);
 
   buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
   buf->type            = BUF_CONTROL_END;
-  buf->decoder_info[0] = 1; /* forced */
+  buf->decoder_flags   = BUF_FLAG_END_USER; /* user finished */
   this->video_fifo->put (this->video_fifo, buf);
 
   if(this->audio_fifo) {
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
     buf->type            = BUF_CONTROL_END;
-    buf->decoder_info[0] = 1; /* forced */
+    buf->decoder_flags   = BUF_FLAG_END_USER; /* user finished */
     this->audio_fifo->put (this->audio_fifo, buf);
   }
 
@@ -727,64 +761,94 @@ static void demux_mpeg_start (demux_plugin_t *this_gen,
   buf_element_t *buf;
   int err;
 
-  this->video_fifo    = video_fifo;
-  this->audio_fifo    = audio_fifo;
+  pthread_mutex_lock( &this->mutex );
 
-  this->rate          = 0; /* fixme */
-  this->last_scr      = 0;
-
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->type    = BUF_CONTROL_START;
-  this->video_fifo->put (this->video_fifo, buf);
-
-  if(this->audio_fifo) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+  if( this->status != DEMUX_OK ) {
+  
+    this->video_fifo  = video_fifo;
+    this->audio_fifo  = audio_fifo;
+  
+    this->rate          = 0; /* fixme */
+    this->last_pts[0]   = 0;
+    this->last_pts[1]   = 0;
+    
+    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
     buf->type    = BUF_CONTROL_START;
-    this->audio_fifo->put (this->audio_fifo, buf);
-  }
+    this->video_fifo->put (this->video_fifo, buf);
 
+    if(this->audio_fifo) {
+      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+      buf->type    = BUF_CONTROL_START;
+      this->audio_fifo->put (this->audio_fifo, buf);
+    }
+
+    if ((this->input->get_capabilities (this->input) & INPUT_CAP_PREVIEW) != 0 ) {
+
+      uint32_t w;
+      int num_buffers = NUM_PREVIEW_BUFFERS;
+
+      this->preview_mode = 1;
+
+      this->input->seek (this->input, 4, SEEK_SET);
+
+      this->status = DEMUX_OK ;
+
+      do {
+
+        w = parse_pack_preview (this, &num_buffers);
+      
+        if (w != 0x000001ba)
+	  demux_mpeg_resync (this, w);
+      
+        num_buffers --;
+
+      } while ( (this->status == DEMUX_OK) && (num_buffers>0)) ;
+
+      /* printf ("demux_mpeg: rate %d\n", this->rate); */
+      this->status = DEMUX_FINISHED;
+    }
+  }  
+  
   if ((this->input->get_capabilities (this->input) & INPUT_CAP_SEEKABLE) != 0 ) {
-
-    uint32_t w;
-    int num_buffers = NUM_PREVIEW_BUFFERS;
-
-    this->preview_mode = 1;
-
-    this->input->seek (this->input, 4, SEEK_SET);
-
-    this->status = DEMUX_OK ;
-
-    do {
-
-      w = parse_pack_preview (this, &num_buffers);
-      
-      if (w != 0x000001ba)
-	demux_mpeg_resync (this, w);
-      
-      num_buffers --;
-
-    } while ( (this->status == DEMUX_OK) && (num_buffers>0)) ;
-
-    /* printf ("demux_mpeg: rate %d\n", this->rate); */
-
+    
     if ( (!start_pos) && (start_time))
       start_pos = start_time * this->rate * 50;
 
     this->input->seek (this->input, start_pos+4, SEEK_SET);
 
+    if( start_pos )
+      demux_mpeg_resync (this, read_bytes(this, 4) );
+  
   } else
     read_bytes(this, 4);
+  
+  this->send_newpts = 1;
+    
+  if( this->status != DEMUX_OK ) {
+    this->preview_mode = 0;
+    this->send_end_buffers = 1;
+    this->status = DEMUX_OK ;
 
-  this->preview_mode = 0;
-  this->send_end_buffers = 1;
-  this->status = DEMUX_OK ;
-
-  if ((err = pthread_create (&this->thread,
+    if ((err = pthread_create (&this->thread,
 			     NULL, demux_mpeg_loop, this)) != 0) {
-    LOG_MSG_STDERR(this->xine, _("demux_mpeg: can't create new thread (%s)\n"),
-		   strerror(err));
-    exit (1);
+      printf ("demux_mpeg: can't create new thread (%s)\n",
+	      strerror(err));
+      exit (1);
+    }
   }
+  else {
+    xine_flush_engine(this->xine);
+  }
+  pthread_mutex_unlock( &this->mutex );
+  
+}
+
+static void demux_mpeg_seek (demux_plugin_t *this_gen,
+			     off_t start_pos, int start_time) {
+  demux_mpeg_t *this = (demux_mpeg_t *) this_gen;
+
+	demux_mpeg_start (this_gen, this->video_fifo, this->audio_fifo,
+			 start_pos, start_time);
 }
 
 static int demux_mpeg_open(demux_plugin_t *this_gen, 
@@ -870,7 +934,7 @@ static int demux_mpeg_open(demux_plugin_t *this_gen,
 	    return DEMUX_CANNOT_HANDLE;
 	  }
 	  
-	  LOG_MSG_STDERR(this->xine, _("You should specify mpeg(mpeg1/mpeg2) stream type.\n"));
+	  xine_log (this->xine, XINE_LOG_MSG, _("demux_mpeg: please specify mpeg(mpeg1/mpeg2) stream type.\n"));
 	  return DEMUX_CANNOT_HANDLE;
 	}
       }
@@ -915,7 +979,7 @@ static char *demux_mpeg_get_mimetypes(void) {
 }
 
 static void demux_mpeg_close (demux_plugin_t *this) {
-  /* nothing */
+  free (this);
 }
 
 static int demux_mpeg_get_stream_length (demux_plugin_t *this_gen) {
@@ -933,11 +997,10 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
 
   demux_mpeg_t    *this;
 
-  if (iface != 6) {
-    LOG_MSG(xine,
-	    _("demux_mpeg: plugin doesn't support plugin API version %d.\n"
-	      "            this means there's a version mismatch between xine and this "
-	      "            demuxer plugin.\nInstalling current demux plugins should help.\n"),
+  if (iface != 7) {
+    printf ("demux_mpeg: plugin doesn't support plugin API version %d.\n"
+	    "            this means there's a version mismatch between xine and this "
+	    "            demuxer plugin.\nInstalling current demux plugins should help.\n",
 	    iface);
     return NULL;
   }
@@ -958,6 +1021,7 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
   this->demux_plugin.interface_version = DEMUXER_PLUGIN_IFACE_VERSION;
   this->demux_plugin.open              = demux_mpeg_open;
   this->demux_plugin.start             = demux_mpeg_start;
+  this->demux_plugin.seek              = demux_mpeg_seek;
   this->demux_plugin.stop              = demux_mpeg_stop;
   this->demux_plugin.close             = demux_mpeg_close;
   this->demux_plugin.get_status        = demux_mpeg_get_status;
@@ -965,6 +1029,9 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
   this->demux_plugin.get_stream_length = demux_mpeg_get_stream_length;
   this->demux_plugin.get_mimetypes     = demux_mpeg_get_mimetypes;
   
+  this->status = DEMUX_FINISHED;
+  pthread_mutex_init( &this->mutex, NULL );
+  
   return (demux_plugin_t *) this;
 }
 
