@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2000, 2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_mpeg_block.c,v 1.73 2002/01/15 13:51:10 guenter Exp $
+ * $Id: demux_mpeg_block.c,v 1.95 2002/04/27 16:33:24 miguelfreitas Exp $
  *
  * demultiplexer for mpeg 1/2 program streams
  *
@@ -45,29 +45,13 @@
 #define VALID_MRLS          "dvd,stdin,fifo"
 #define VALID_ENDS          "vob"
 
-#ifdef __GNUC__
-#define LOG_MSG_STDERR(xine, message, args...) {                     \
-    xine_log(xine, XINE_LOG_DEMUX, message, ##args);                 \
-    fprintf(stderr, message, ##args);                                \
-  }
-#define LOG_MSG(xine, message, args...) {                            \
-    xine_log(xine, XINE_LOG_DEMUX, message, ##args);                 \
-    printf(message, ##args);                                         \
-  }
-#else
-#define LOG_MSG_STDERR(xine, ...) {                                  \
-    xine_log(xine, XINE_LOG_DEMUX, __VA_ARGS__);                     \
-    fprintf(stderr, __VA_ARGS__);                                    \
-  }
-#define LOG_MSG(xine, ...) {                                         \
-    xine_log(xine, XINE_LOG_DEMUX, __VA_ARGS__);                     \
-    printf(__VA_ARGS__);                                             \
-  }
-#endif
-
 #define NUM_PREVIEW_BUFFERS   250
 #define DISC_TRESHOLD       90000
 
+/* redefine abs as macro to handle 64-bit diffs.
+   i guess llabs may not be available everywhere */
+#define abs(x) ( (x<0) ? (-x) : (x) )
+
 typedef struct demux_mpeg_block_s {
   demux_plugin_t        demux_plugin;
 
@@ -81,6 +65,7 @@ typedef struct demux_mpeg_block_s {
   input_plugin_t       *input;
 
   pthread_t             thread;
+  pthread_mutex_t       mutex;
 
   int                   status;
   
@@ -92,9 +77,11 @@ typedef struct demux_mpeg_block_s {
 
   char                  cur_mrl[256];
 
-  uint8_t              *scratch;
+  uint8_t              *scratch, *scratch_base;
 
-  uint32_t              last_scr;
+  int64_t               last_scr;
+  int                   ignore_scr_discont;
+  int64_t               nav_last_end_pts;
 } demux_mpeg_block_t ;
 
 
@@ -104,17 +91,23 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
   uint8_t       *p;
   int            bMpeg1=0;
   uint32_t       header_len;
-  uint32_t       PTS;
+  int64_t        pts;
   uint32_t       packet_len;
   uint32_t       stream_id;
-  uint32_t       scr = this->last_scr;
+  int64_t        scr = this->last_scr;
+
+#ifdef LOG
+  printf ("demux_mpeg_block: read_block\n");
+#endif
 
   buf = this->input->read_block (this->input, this->video_fifo, this->blocksize);
 
   if (buf==NULL) {
     xine_next_mrl_event_t event;
 
-    LOG_MSG(this->xine, _("demux_mpeg_block: read_block failed\n"));
+#ifdef LOG
+    printf ("demux_mpeg_block: read_block failed\n");
+#endif
 
     /*
      * check if seamless branching is possible
@@ -127,13 +120,17 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
     if (event.handled) {
 
       char *next_mrl = event.mrl;
-      
-      LOG_MSG(this->xine, _("demux_mpeg_block: checking if we can branch to %s\n"), next_mrl);
+
+#ifdef LOG      
+      printf ("demux_mpeg_block: checking if we can branch to %s\n", next_mrl);
+#endif
 
       if (next_mrl && this->input->is_branch_possible 
 	  && this->input->is_branch_possible (this->input, next_mrl)) {
 
-        LOG_MSG(this->xine, _("demux_mpeg_block: branching\n"));
+#ifdef LOG      
+        printf ("demux_mpeg_block: branching\n");
+#endif
 
 	this->input->close (this->input);
         this->input->open (this->input, next_mrl);
@@ -159,7 +156,7 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
 
   /* If this is not a block for the demuxer, pass it
    * straight through. */
-  if(buf->type != BUF_DEMUX_BLOCK) {
+  if (buf->type != BUF_DEMUX_BLOCK) {
     buf_element_t *cbuf;
 
     this->video_fifo->put (this->video_fifo, buf);
@@ -168,21 +165,35 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
 
     if (this->audio_fifo) {
       cbuf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+
       cbuf->type = buf->type;
+      cbuf->decoder_flags = buf->decoder_flags;
       cbuf->decoder_info[0] = buf->decoder_info[0];
+      cbuf->decoder_info[1] = buf->decoder_info[1];
+      cbuf->decoder_info[2] = buf->decoder_info[2];
+      cbuf->decoder_info[3] = buf->decoder_info[3];
+
       this->audio_fifo->put (this->audio_fifo, cbuf);
     }
 
+#ifdef LOG
+    printf ("demux_mpeg_block: type %08x != BUF_DEMUX_BLOCK\n", buf->type);
+#endif
+    /*
+    if (buf->type == BUF_CONTROL_NOP) 
+      this->send_disc = 1;
+    */
     return;
   }
 
   p = buf->content; /* len = this->mnBlocksize; */
   if (preview_mode)
-    buf->decoder_info[0] = 0;
+    buf->decoder_flags = BUF_FLAG_PREVIEW;
   else
-    buf->decoder_info[0] = 1;
+    buf->decoder_flags = 0;
 
   buf->input_pos = this->input->get_current_pos (this->input);
+  buf->input_length = this->input->get_length (this->input);
 
   if (this->rate)
     buf->input_time = buf->input_pos / (this->rate * 50);
@@ -202,7 +213,7 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
       scr |= (p[6] & 0xFF) <<  7;
       scr |= (p[7] & 0xFE) >>  1;
 
-      buf->SCR = scr;
+      /* buf->scr = scr; */
 
       /* mux_rate */
 
@@ -230,11 +241,15 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
       scr |= p[7] << 5;
       scr |= (p[8] & 0xF8) >> 3;
       /*  optional - decode extension:
-      scr *=300;
-      scr += ( (p[8] & 0x03 << 7) | (p[9] & 0xFE >> 1) );
+	  scr *=300;
+	  scr += ( (p[8] & 0x03 << 7) | (p[9] & 0xFE >> 1) );
       */
 
-      buf->SCR = scr;
+#ifdef LOG
+      printf ("demux_mpeg_block: SCR=%lld\n", scr);
+#endif
+
+      /* buf->scr = scr; */
 
       /* mux_rate */
 
@@ -248,8 +263,7 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
 
       p += 14 + num_stuffing_bytes;
     }
-  }
-
+  } 
 
   if (p[3] == 0xbb) { /* program stream system header */
 
@@ -263,63 +277,116 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
   /* we should now have a PES packet here */
 
   if (p[0] || p[1] || (p[2] != 1)) {
-    LOG_MSG(this->xine, _("demux_mpeg_block: error! %02x %02x %02x (should be 0x000001) \n"),
+    printf ("demux_mpeg_block: error! %02x %02x %02x (should be 0x000001)\n",
 	    p[0], p[1], p[2]);
     buf->free_buffer (buf);
 
     this->warned++;
     if (this->warned > 5) {
-      LOG_MSG(this->xine, _("demux_mpeg_block: too many errors, stopping playback. Maybe this stream is scrambled?\n"));
+      xine_log (this->xine, XINE_LOG_MSG,
+		_("demux_mpeg_block: too many errors, stopping playback. Maybe this stream is scrambled?\n"));
       this->status = DEMUX_FINISHED;
     }
 
     return;
   }
 
+  packet_len = p[4] << 8 | p[5];
+  stream_id  = p[3];
+
+  if (stream_id == 0xbf) {  /* NAV Packet */
+
+    int64_t start_pts, end_pts;
+
+    start_pts  = (p[7+12] << 24);
+    start_pts |= (p[7+13] << 16);
+    start_pts |= (p[7+14] << 8);
+    start_pts |= p[7+15];
+
+    end_pts  = (p[7+16] << 24);
+    end_pts |= (p[7+17] << 16);
+    end_pts |= (p[7+18] << 8);
+    end_pts |= p[7+19];
+
+    buf->content   = p;
+    buf->size      = packet_len;
+    buf->type      = BUF_SPU_NAV;
+    buf->pts       = 0;   /* NAV packets do not have PES values */
+    buf->input_pos = this->input->get_current_pos(this->input);
+    buf->input_length = this->input->get_length (this->input);
+    this->video_fifo->put (this->video_fifo, buf);
+
+#ifdef LOG
+    printf ("demux_mpeg_block: NAV packet, start pts = %lld, end_pts = %lld\n",
+	    start_pts, end_pts);
+#endif
+
+    if (this->nav_last_end_pts != start_pts && !preview_mode) {
+
+#ifdef LOG
+      printf ("demux_mpeg_block: informing metronom about new start pts\n");
+#endif
+
+      buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
+      buf->type = BUF_CONTROL_NEWPTS;
+      buf->disc_off = start_pts;
+      this->video_fifo->put (this->video_fifo, buf);
+
+      if (this->audio_fifo) {
+	buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+	buf->type = BUF_CONTROL_NEWPTS;
+	buf->disc_off = start_pts;
+	this->audio_fifo->put (this->audio_fifo, buf);
+      }
+
+    }
+    
+    this->nav_last_end_pts = end_pts;
+    this->ignore_scr_discont = 1;
+
+    /*
+    for (i=0; i<120; i++)
+      printf ("%02x ", p[i]);
+    printf ("\n");
+    */
+
+    return ;
+  }
+
   /* discontinuity ? */
-  {  
-    int32_t scr_diff = scr - this->last_scr;
+  if (scr && !preview_mode)  {  
+    int64_t scr_diff = scr - this->last_scr;
 
 #ifdef LOG
     printf ("demux_mpeg_block: scr %d last_scr %d diff %d\n",
 	    scr, this->last_scr, scr_diff);
 #endif
 
-    if (abs(scr_diff) > DISC_TRESHOLD) {
+    if ( (abs(scr_diff) > DISC_TRESHOLD) &&
+        !this->ignore_scr_discont) {
       
       buf_element_t *buf;
 
 #ifdef LOG
-      printf ("demux_mpeg_block: DISCONTINUITY!\n");
+      printf ("demux_mpeg_block: DISCONTINUITY/NEWPTS!\n");
 #endif
 
       buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-      buf->type = BUF_CONTROL_AVSYNC_RESET;
-      buf->SCR  = scr;
+      buf->type = BUF_CONTROL_DISCONTINUITY;
+      buf->disc_off = scr_diff;
       this->video_fifo->put (this->video_fifo, buf);
 
       if (this->audio_fifo) {
 	buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-	buf->type = BUF_CONTROL_AVSYNC_RESET;
-	buf->SCR  = scr;
+	buf->type = BUF_CONTROL_DISCONTINUITY;
+	buf->disc_off = scr_diff;
 	this->audio_fifo->put (this->audio_fifo, buf);
       }
     }
+    this->ignore_scr_discont = 0;
     this->last_scr = scr;
   }
 
-  packet_len = p[4] << 8 | p[5];
-  stream_id  = p[3];
-
-  if (stream_id == 0xbf) {  /* NAV Packet */
-    buf->content   = p;
-    buf->size      = packet_len;
-    buf->type      = BUF_SPU_NAV;
-    buf->PTS       = 0;   /* NAV packets do not have PES values */
-    buf->input_pos = this->input->get_current_pos(this->input);
-    this->video_fifo->put (this->video_fifo, buf);
-    return ;
-  }
 
   if (bMpeg1) {
 
@@ -342,27 +409,27 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
       packet_len -=2;
     }
 
-    PTS = 0; 
+    pts = 0; 
     if ((p[0] & 0xf0) == 0x20) {
-      PTS  = (p[ 0] & 0x0E) << 29 ;
-      PTS |=  p[ 1]         << 22 ;
-      PTS |= (p[ 2] & 0xFE) << 14 ;
-      PTS |=  p[ 3]         <<  7 ;
-      PTS |= (p[ 4] & 0xFE) >>  1 ;
+      pts  = (p[ 0] & 0x0E) << 29 ;
+      pts |=  p[ 1]         << 22 ;
+      pts |= (p[ 2] & 0xFE) << 14 ;
+      pts |=  p[ 3]         <<  7 ;
+      pts |= (p[ 4] & 0xFE) >>  1 ;
       p   += 5;
       packet_len -=5;
     } else if ((p[0] & 0xf0) == 0x30) {
-      PTS  = (p[ 0] & 0x0E) << 29 ;
-      PTS |=  p[ 1]         << 22 ;
-      PTS |= (p[ 2] & 0xFE) << 14 ;
-      PTS |=  p[ 3]         <<  7 ;
-      PTS |= (p[ 4] & 0xFE) >>  1 ;
+      pts  = (p[ 0] & 0x0E) << 29 ;
+      pts |=  p[ 1]         << 22 ;
+      pts |= (p[ 2] & 0xFE) << 14 ;
+      pts |=  p[ 3]         <<  7 ;
+      pts |= (p[ 4] & 0xFE) >>  1 ;
       /* DTS decoding code is working, but not used in xine
-      DTS  = (p[ 5] & 0x0E) << 29 ;
-      DTS |=  p[ 6]         << 22 ;
-      DTS |= (p[ 7] & 0xFE) << 14 ;
-      DTS |=  p[ 8]         <<  7 ;
-      DTS |= (p[ 9] & 0xFE) >>  1 ;
+	 DTS  = (p[ 5] & 0x0E) << 29 ;
+	 DTS |=  p[ 6]         << 22 ;
+	 DTS |= (p[ 7] & 0xFE) << 14 ;
+	 DTS |=  p[ 8]         <<  7 ;
+	 DTS |= (p[ 9] & 0xFE) >>  1 ;
       */
       p   += 10;
       packet_len -= 10;
@@ -374,33 +441,39 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
   } else { /* mpeg 2 */
     /* check PES scrambling_control */
     if (((p[6] & 0x30) != 0) && !this->warned) {
-      LOG_MSG(this->xine, _("demux_mpeg_block: warning: pes header indicates that this stream may be encrypted (encryption mode %d)\n"), (p[6] & 0x30) >> 4);
+
+      xine_log (this->xine, XINE_LOG_MSG,
+		_("demux_mpeg_block: warning: pes header indicates that this stream may be encrypted (encryption mode %d)\n"), (p[6] & 0x30) >> 4);
 
       this->warned = 1;
     }
 
-    if (p[7] & 0x80) { /* PTS avail */
+    if (p[7] & 0x80) { /* pts avail */
 
-      PTS  = (p[ 9] & 0x0E) << 29 ;
-      PTS |=  p[10]         << 22 ;
-      PTS |= (p[11] & 0xFE) << 14 ;
-      PTS |=  p[12]         <<  7 ;
-      PTS |= (p[13] & 0xFE) >>  1 ;
+      pts  = (p[ 9] & 0x0E) << 29 ;
+      pts |=  p[10]         << 22 ;
+      pts |= (p[11] & 0xFE) << 14 ;
+      pts |=  p[12]         <<  7 ;
+      pts |= (p[13] & 0xFE) >>  1 ;
+
+#ifdef LOG
+      printf ("demux_mpeg_block: pts = %lld\n", pts);
+#endif
 
     } else
-      PTS = 0;
+      pts = 0;
 
     /* code is working but not used in xine
-    if (p[7] & 0x40) {  
+       if (p[7] & 0x40) {  
       
-      DTS  = (p[14] & 0x0E) << 29 ;
-      DTS |=  p[15]         << 22 ;
-      DTS |= (p[16] & 0xFE) << 14 ;
-      DTS |=  p[17]         <<  7 ;
-      DTS |= (p[18] & 0xFE) >>  1 ;
+       DTS  = (p[14] & 0x0E) << 29 ;
+       DTS |=  p[15]         << 22 ;
+       DTS |= (p[16] & 0xFE) << 14 ;
+       DTS |=  p[17]         <<  7 ;
+       DTS |= (p[18] & 0xFE) >>  1 ;
       
-    } else
-      DTS = 0;
+       } else
+       DTS = 0;
     */
 
 
@@ -410,8 +483,6 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
     packet_len -= header_len + 3;
   }
 
-  PTS &= 0x7FFFFFFF ; /* 31 bit only (for signed calculations) */
-
   if (stream_id == 0xbd) {
 
     int track, spu_id;
@@ -424,8 +495,9 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
       buf->content   = p+1;
       buf->size      = packet_len-1;
       buf->type      = BUF_SPU_PACKAGE + spu_id;
-      buf->PTS       = PTS;
+      buf->pts       = pts;
       buf->input_pos = this->input->get_current_pos(this->input);
+      buf->input_length = this->input->get_length (this->input);
       
       this->video_fifo->put (this->video_fifo, buf);    
       
@@ -433,7 +505,7 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
     }
 
     if ((p[0]&0xF0) == 0x80) {
-
+    
       /*  printf ( "ac3 PES packet, track %02x\n",track);  */
       buf->decoder_info[1] = p[1]; /* Number of frame headers */
       buf->decoder_info[2] = p[2] << 8 | p[3]; /* First access unit pointer */
@@ -445,9 +517,10 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
       } else {
         buf->type      = BUF_AUDIO_A52 + track;
       }
-      buf->PTS       = PTS;
+      buf->pts       = pts;
 
       buf->input_pos = this->input->get_current_pos(this->input);
+      buf->input_length = this->input->get_length (this->input);
 
       if(this->audio_fifo)
 	this->audio_fifo->put (this->audio_fifo, buf);
@@ -488,7 +561,7 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
       switch ((p[5]>>6) & 3) {
       case 3: /* illegal, use 16-bits? */
       default:
-	LOG_MSG(this->xine, _("illegal lpcm sample format (%d), assume 16-bit samples\n"),
+	printf ("illegal lpcm sample format (%d), assume 16-bit samples\n",
 		(p[5]>>6) & 3 );
       case 0: bits_per_sample = 16; break;
       case 1: bits_per_sample = 20; break;
@@ -505,9 +578,10 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
       buf->content   = p+pcm_offset;
       buf->size      = packet_len-pcm_offset;
       buf->type      = BUF_AUDIO_LPCM_BE + track;
-      buf->PTS       = PTS;
+      buf->pts       = pts;
 
       buf->input_pos = this->input->get_current_pos(this->input);
+      buf->input_length = this->input->get_length (this->input);
 
       if(this->audio_fifo)
 	this->audio_fifo->put (this->audio_fifo, buf);
@@ -522,9 +596,10 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
     buf->content   = p;
     buf->size      = packet_len;
     buf->type      = BUF_VIDEO_MPEG;
-    buf->PTS       = PTS;
+    buf->pts       = pts;
 
     buf->input_pos = this->input->get_current_pos(this->input);
+    buf->input_length = this->input->get_length (this->input);
 
     this->video_fifo->put (this->video_fifo, buf);
 
@@ -538,9 +613,10 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
     buf->content   = p;
     buf->size      = packet_len;
     buf->type      = BUF_AUDIO_MPEG + track;
-    buf->PTS       = PTS;
+    buf->pts       = pts;
 
     buf->input_pos = this->input->get_current_pos(this->input);
+    buf->input_length = this->input->get_length (this->input);
 
     if(this->audio_fifo)
       this->audio_fifo->put (this->audio_fifo, buf);
@@ -550,7 +626,6 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
     return ;
 
   } 
-
   buf->free_buffer (buf);
 
   return ;
@@ -566,11 +641,18 @@ static void *demux_mpeg_block_loop (void *this_gen) {
 
   this->send_end_buffers = 1;
 
-  do {
-
+  while(1) {
+    
+    pthread_mutex_lock( &this->mutex );
+    
+    if( this->status != DEMUX_OK)
+      break;
+    
     demux_mpeg_block_parse_pack(this, 0);
-
-  } while (this->status == DEMUX_OK) ;
+    
+    pthread_mutex_unlock( &this->mutex );
+  
+  }
 
   /*
   printf ("demux_mpeg_block: demux loop finished (status: %d)\n",
@@ -582,18 +664,20 @@ static void *demux_mpeg_block_loop (void *this_gen) {
   if (this->send_end_buffers) {
     buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
     buf->type            = BUF_CONTROL_END;
-    buf->decoder_info[0] = 0; /* stream finished */
+    buf->decoder_flags   = BUF_FLAG_END_STREAM;
     this->video_fifo->put (this->video_fifo, buf);
     
     if(this->audio_fifo) {
       buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
       buf->type            = BUF_CONTROL_END;
-      buf->decoder_info[0] = 0; /* stream finished */
+      buf->decoder_flags   = BUF_FLAG_END_STREAM;
       this->audio_fifo->put (this->audio_fifo, buf);
     }
 
   }
 
+  pthread_mutex_unlock( &this->mutex );
+  
   pthread_exit(NULL);
 
   return NULL;
@@ -608,7 +692,7 @@ static int demux_mpeg_block_estimate_rate (demux_mpeg_block_t *this) {
   int            is_mpeg1=0;
   off_t          pos, last_pos;
   off_t          step;
-  uint32_t       PTS, last_PTS;
+  int64_t        pts, last_pts;
   int            rate;
   int            count;
   int            stream_id;
@@ -618,7 +702,7 @@ static int demux_mpeg_block_estimate_rate (demux_mpeg_block_t *this) {
     return 0;
 
   last_pos = 0;
-  last_PTS = 0;
+  last_pts = 0;
   rate     = 0;
   step     = this->input->get_length (this->input) / 10;
   step     = (step / this->blocksize) * this->blocksize;
@@ -648,14 +732,14 @@ static int demux_mpeg_block_estimate_rate (demux_mpeg_block_t *this) {
     /* we should now have a PES packet here */
 
     if (p[0] || p[1] || (p[2] != 1)) {
-      LOG_MSG(this->xine, _("demux_mpeg_block: error %02x %02x %02x (should be 0x000001) \n"),
+      printf ("demux_mpeg_block: error %02x %02x %02x (should be 0x000001) \n",
 	      p[0], p[1], p[2]);
       buf->free_buffer (buf);
       return rate;
     }
 
     stream_id  = p[3];
-    PTS = 0; 
+    pts = 0; 
 
     if ((stream_id < 0xbc) || ((stream_id & 0xf0) != 0xe0)) {
       pos += (off_t) this->blocksize;
@@ -679,47 +763,47 @@ static int demux_mpeg_block_estimate_rate (demux_mpeg_block_t *this) {
 	}
 
 	if ( ((p[0] & 0xf0) == 0x20) || ((p[0] & 0xf0) == 0x30) ) {
-	  PTS  = (p[ 0] & 0x0E) << 29 ;
-	  PTS |=  p[ 1]         << 22 ;
-	  PTS |= (p[ 2] & 0xFE) << 14 ;
-	  PTS |=  p[ 3]         <<  7 ;
-	  PTS |= (p[ 4] & 0xFE) >>  1 ;
+	  pts  = (p[ 0] & 0x0E) << 29 ;
+	  pts |=  p[ 1]         << 22 ;
+	  pts |= (p[ 2] & 0xFE) << 14 ;
+	  pts |=  p[ 3]         <<  7 ;
+	  pts |= (p[ 4] & 0xFE) >>  1 ;
 	} 
       }
     } else { /* mpeg 2 */
       
-      if (p[7] & 0x80) { /* PTS avail */
+      if (p[7] & 0x80) { /* pts avail */
 	
-	PTS  = (p[ 9] & 0x0E) << 29 ;
-	PTS |=  p[10]         << 22 ;
-	PTS |= (p[11] & 0xFE) << 14 ;
-	PTS |=  p[12]         <<  7 ;
-	PTS |= (p[13] & 0xFE) >>  1 ;
+	pts  = (p[ 9] & 0x0E) << 29 ;
+	pts |=  p[10]         << 22 ;
+	pts |= (p[11] & 0xFE) << 14 ;
+	pts |=  p[12]         <<  7 ;
+	pts |= (p[13] & 0xFE) >>  1 ;
 	
       } else
-	PTS = 0;
+	pts = 0;
     }
 
-    if (PTS) {
+    if (pts) {
 
 
-      if ( (pos>last_pos) && (PTS>last_PTS) ) {
+      if ( (pos>last_pos) && (pts>last_pts) ) {
 	int cur_rate;
       
-	cur_rate = ((pos - last_pos)*90000) / ((PTS - last_PTS) * 50);
+	cur_rate = ((pos - last_pos)*90000) / ((pts - last_pts) * 50);
 	
 	rate = (count * rate + cur_rate) / (count+1);
 
 	count ++;
 	
 	/*
-	printf ("demux_mpeg_block: stream_id %02x, pos: %lld, PTS: %d, cur_rate = %d, overall rate : %d\n", 
-		stream_id, pos, PTS, cur_rate, rate); 
+	printf ("demux_mpeg_block: stream_id %02x, pos: %lld, pts: %d, cur_rate = %d, overall rate : %d\n", 
+		stream_id, pos, pts, cur_rate, rate); 
 	*/
       }
 
       last_pos = pos;
-      last_PTS = PTS;
+      last_pts = pts;
       pos += step;
     } else
       pos += (off_t) this->blocksize;
@@ -738,6 +822,7 @@ static int demux_mpeg_block_estimate_rate (demux_mpeg_block_t *this) {
 static void demux_mpeg_block_close (demux_plugin_t *this_gen) {
 
   demux_mpeg_block_t *this = (demux_mpeg_block_t *) this_gen;
+  free (this->scratch_base);
   free (this);
   
 }
@@ -748,31 +833,32 @@ static void demux_mpeg_block_stop (demux_plugin_t *this_gen) {
   buf_element_t *buf;
   void *p;
 
+  pthread_mutex_lock( &this->mutex );
+  
   if (this->status != DEMUX_OK) {
-    LOG_MSG(this->xine, _("demux_mpeg_block: stop...ignored\n"));
+    printf ("demux_mpeg_block: stop...ignored\n");
+    pthread_mutex_unlock( &this->mutex );
     return;
   }
 
   this->send_end_buffers = 0;
   this->status = DEMUX_FINISHED;
-
-  pthread_cancel (this->thread);
+  
+  pthread_mutex_unlock( &this->mutex );
   pthread_join (this->thread, &p);
 
-  this->video_fifo->clear(this->video_fifo);
-  if (this->audio_fifo)
-    this->audio_fifo->clear(this->audio_fifo);
-
+  xine_flush_engine(this->xine);
+  
   buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
   buf->type            = BUF_CONTROL_END;
-  buf->decoder_info[0] = 1; /* forced */
+  buf->decoder_flags   = BUF_FLAG_END_USER;
 
   this->video_fifo->put (this->video_fifo, buf);
 
   if(this->audio_fifo) {
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
     buf->type            = BUF_CONTROL_END;
-    buf->decoder_info[0] = 1; /* forced */
+    buf->decoder_flags   = BUF_FLAG_END_USER; 
     this->audio_fifo->put (this->audio_fifo, buf);
   }
 }
@@ -792,42 +878,50 @@ static void demux_mpeg_block_start (demux_plugin_t *this_gen,
   buf_element_t *buf;
   int err;
 
+  pthread_mutex_lock( &this->mutex );
+
   this->video_fifo  = video_fifo;
   this->audio_fifo  = audio_fifo;
 
-  /* 
-   * send start buffer
-   */
+  if( this->status != DEMUX_OK ) {
+    /* 
+     * send start buffer
+     */
 
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->type    = BUF_CONTROL_START;
-  this->video_fifo->put (this->video_fifo, buf);
-
-  if(this->audio_fifo) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
     buf->type    = BUF_CONTROL_START;
-    this->audio_fifo->put (this->audio_fifo, buf);
-  }
+    this->video_fifo->put (this->video_fifo, buf);
 
-  if (!this->rate)
-    this->rate = demux_mpeg_block_estimate_rate (this);
-
-
-  this->last_scr = 0;
-
-  if((this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) != 0) {
-
-    int num_buffers = NUM_PREVIEW_BUFFERS;
-
-    this->input->seek (this->input, 0, SEEK_SET);
-
-    this->status = DEMUX_OK ;
-    while ( (num_buffers>0) && (this->status == DEMUX_OK) ) {
-
-      demux_mpeg_block_parse_pack(this, 1);
-      num_buffers --;
+    if(this->audio_fifo) {
+      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+      buf->type    = BUF_CONTROL_START;
+      this->audio_fifo->put (this->audio_fifo, buf);
     }
 
+    if (!this->rate)
+      this->rate = demux_mpeg_block_estimate_rate (this);
+
+    this->last_scr         = 0;
+    this->nav_last_end_pts = 0;
+    this->ignore_scr_discont = 0;
+
+    if((this->input->get_capabilities(this->input) & INPUT_CAP_PREVIEW) != 0) {
+
+      int num_buffers = NUM_PREVIEW_BUFFERS;
+
+      this->input->seek (this->input, 0, SEEK_SET);
+
+      this->status = DEMUX_OK ;
+      while ( (num_buffers>0) && (this->status == DEMUX_OK) ) {
+
+        demux_mpeg_block_parse_pack(this, 1);
+        num_buffers --;
+      }
+    }
+    this->status = DEMUX_FINISHED;
+  }
+  
+  if((this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) != 0) {
     if (start_pos) {
       start_pos /= (off_t) this->blocksize;
       start_pos *= (off_t) this->blocksize;
@@ -849,11 +943,12 @@ static void demux_mpeg_block_start (demux_plugin_t *this_gen,
 
   buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
 
-  if(this->input->get_optional_data(this->input, buf->mem,
-     INPUT_OPTIONAL_DATA_CLUT) == INPUT_OPTIONAL_SUCCESS) {
+  if ((this->input->get_capabilities(this->input) & INPUT_CAP_CLUT) &&
+      ((this->input->get_optional_data(this->input, buf->mem, INPUT_OPTIONAL_DATA_CLUT)
+	== INPUT_OPTIONAL_SUCCESS))) {
     buf->type = BUF_SPU_CLUT;
     buf->content = buf->mem;
-
+    
     this->video_fifo->put(this->video_fifo, buf);
   } else {
     buf->free_buffer(buf);
@@ -863,17 +958,35 @@ static void demux_mpeg_block_start (demux_plugin_t *this_gen,
    * now start demuxing
    */
 
-  this->status   = DEMUX_OK ;
-  this->last_scr = 0;
+  if( this->status != DEMUX_OK )
+  {
+    this->status   = DEMUX_OK ;
+    this->last_scr = 0;
+    this->nav_last_end_pts = 0;
+    this->ignore_scr_discont = 0;
 
-  if ((err = pthread_create (&this->thread,
+    if ((err = pthread_create (&this->thread,
 			     NULL, demux_mpeg_block_loop, this)) != 0) {
-    LOG_MSG_STDERR(this->xine, _("demux_mpeg_block: can't create new thread (%s)\n"),
-		   strerror(err));
-    exit (1);
+      printf ("demux_mpeg_block: can't create new thread (%s)\n",
+	      strerror(err));
+      exit (1);
+    }
   }
+  else {
+    xine_flush_engine(this->xine);
+  }
+  pthread_mutex_unlock( &this->mutex );
 }
 
+static void demux_mpeg_block_seek (demux_plugin_t *this_gen,
+			     off_t start_pos, int start_time) {
+  demux_mpeg_block_t *this = (demux_mpeg_block_t *) this_gen;
+
+	demux_mpeg_block_start (this_gen, this->video_fifo, this->audio_fifo,
+			 start_pos, start_time);
+}
+
+
 static void demux_mpeg_block_accept_input (demux_mpeg_block_t *this,
 					   input_plugin_t *input) {
 
@@ -885,11 +998,11 @@ static void demux_mpeg_block_accept_input (demux_mpeg_block_t *this,
 
     strncpy (this->cur_mrl, input->get_mrl(input), 256);
 
-    LOG_MSG(this->xine, _("demux_mpeg_block: mrl %s is new, will estimated bitrate\n"),
+    printf ("demux_mpeg_block: mrl %s is new, will estimated bitrate\n",
 	    this->cur_mrl);
 
   } else
-    LOG_MSG(this->xine, _("demux_mpeg_block: mrl %s is known, estimated bitrate: %d\n"),
+    printf ("demux_mpeg_block: mrl %s is known, estimated bitrate: %d\n",
 	    this->cur_mrl, this->rate * 50 * 8);
 }
 
@@ -969,8 +1082,9 @@ static int demux_mpeg_block_open(demux_plugin_t *this_gen,
 	
 	while(*m == ' ' || *m == '\t') m++;
 	
-	if((!strncmp(MRL, m, strlen(m))) 
-	   || ((!strncmp(MRL, m, strlen(m))) && (!strncmp((media + 3), "mpeg2", 5)))) {
+	if(((!strncmp(MRL, m, strlen(m))) && ((strncmp((media + 3), "mpeg2", 5)) && (strncmp((media + 3), "mpeg1", 5))))
+	   || (((!strncmp(MRL, m, strlen(m))) && (!strncmp((media + 3), "mpeg2", 5))) &&
+	       ((!strncmp(MRL, m, strlen(m))) && (strncmp((media + 3), "mpeg1", 5))))) {
 	  
 	  this->blocksize = 2048;
 	  demux_mpeg_block_accept_input(this, input);
@@ -1031,6 +1145,9 @@ static char *demux_mpeg_block_get_mimetypes(void) {
 static int demux_mpeg_block_get_stream_length (demux_plugin_t *this_gen) {
 
   demux_mpeg_block_t *this = (demux_mpeg_block_t *) this_gen;
+                        /*
+   * find input plugin
+   */
 
   if (this->rate) 
     return this->input->get_length (this->input) / (this->rate * 50);
@@ -1043,13 +1160,12 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
 
   demux_mpeg_block_t *this;
 
-  if (iface != 6) {
-    LOG_MSG(xine,
-	    _("demux_mpeg_block: plugin doesn't support plugin API version %d.\n"
-	      "                  this means there's a version mismatch between xine and this "
-	      "                  demuxer plugin.\nInstalling current demux plugins should help.\n"),
+  if (iface != 7) {
+    printf ("demux_mpeg_block: plugin doesn't support plugin API version %d.\n"
+	    "                  this means there's a version mismatch between xine and this "
+	    "                  demuxer plugin.\nInstalling current demux plugins should help.\n",
 	    iface);
-    return NULL;
+  return NULL;
   }
 
   this         = xine_xmalloc (sizeof (demux_mpeg_block_t));
@@ -1068,6 +1184,7 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
   this->demux_plugin.interface_version = DEMUXER_PLUGIN_IFACE_VERSION;
   this->demux_plugin.open              = demux_mpeg_block_open;
   this->demux_plugin.start             = demux_mpeg_block_start;
+  this->demux_plugin.seek              = demux_mpeg_block_seek;
   this->demux_plugin.stop              = demux_mpeg_block_stop;
   this->demux_plugin.close             = demux_mpeg_block_close;
   this->demux_plugin.get_status        = demux_mpeg_block_get_status;
@@ -1075,7 +1192,9 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
   this->demux_plugin.get_stream_length = demux_mpeg_block_get_stream_length;
   this->demux_plugin.get_mimetypes     = demux_mpeg_block_get_mimetypes;
   
-  this->scratch = xine_xmalloc_aligned (512, 4096);
+  this->scratch = xine_xmalloc_aligned (512, 4096, (void**) &this->scratch_base);
+  this->status = DEMUX_FINISHED;
+  pthread_mutex_init( &this->mutex, NULL );
     
   return (demux_plugin_t *) this;
 }
