@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: dxr3_decoder.c,v 1.58 2002/01/10 21:30:10 jcdutton Exp $
+ * $Id: dxr3_decoder.c,v 1.74 2002/04/24 20:26:06 jcdutton Exp $
  *
  * dxr3 video and spu decoder plugin. Accepts the video and spu data
  * from XINE and sends it directly to the corresponding dxr3 devices.
@@ -78,23 +78,26 @@
 #include "xine_internal.h"
 #include "buffer.h"
 #include "xine-engine/bswap.h"
+#include "nav_types.h"
+#include "nav_read.h"
 
 /* for DXR3_VO_UPDATE_FLAG */
 #include "dxr3_video_out.h"
 
 #define LOOKUP_DEV "dxr3.devicename"
 #define DEFAULT_DEV "/dev/em8300"
-static char *devname;
+static char devname[128];
+static char devnum[3];
 
 /* lots of poohaa about pts things */
-#define LOG_PTS 0 
+#define LOG_PTS 0
 #define LOG_SPU 0 
 
 #define MV_COMMAND 0
-#define MV_STATUS  1
-#ifndef MVCOMMAND_SCAN
- #define MVCOMMAND_SCAN 4
-#endif
+
+/* the number of frames to pass after an out-of-sync situation
+   before locking the stream again */
+#define RESYNC_WINDOW_SIZE 50
 
 
 typedef struct dxr3_decoder_s {
@@ -104,7 +107,7 @@ typedef struct dxr3_decoder_s {
 	
 	int fd_control;
 	int fd_video;
-	int last_pts;
+	int64_t last_pts;
 	scr_plugin_t *scr;
 	int scr_prio;
 	int width;
@@ -112,57 +115,34 @@ typedef struct dxr3_decoder_s {
 	int aspect;
 	int frame_rate_code;
 	int repeat_first_field;
-	/* try to sync PTS every frame. will be disabled if non-progessive
+	/* try to sync pts every frame. will be disabled if non-progessive
 	   video is detected via repeat first field */
 	int sync_every_frame;
 	/* if disabled by repeat first field, retry after 500 frames */
 	int sync_retry; 
 	int enhanced_mode;
+	int resync_window;
 	int have_header_info;
 	int in_buffer_fill;
 	pthread_t decoder_thread; /* reference to self */
 } dxr3_decoder_t;
 
-/* Function to check whether the dxr3 video out plugin is active.
- * Without it, we can't work and must give it to libmpeg2.
- * We (ab)use a config value for this (set by dxr3 video out init/exit)
- */
-static int dxr3_check_vo(config_values_t* cfg) 
-{
-	cfg_entry_t* entry;
-	entry = cfg->lookup_entry(cfg, "dxr3.active");
-	return (entry && entry->num_value);
-}
-
 static int dxr3_tested = 0;
 static int dxr3_ok;
 
 static void dxr3_presence_test( xine_t* xine)
 {
-	int fd, val;
-        vo_info_t *info;
-
-	if (dxr3_tested)
-		return;
-
+        int info;
+/* FIXME: Get rid of global vars */
 	dxr3_tested = 1;
 	dxr3_ok = 0;
         if (xine && xine->video_driver ) {
-          info = xine->video_driver->get_info();
-          if ((strncmp (info->id, "dxr3", 4)) != 0) {
-              return;
+          info = xine->video_driver->get_property( xine->video_driver, VO_PROP_VO_TYPE);
+          printf("dxr3_presence_test:info=%d\n",info);
+          if (info != VO_TYPE_DXR3) {
+            return;
           }
         }
-	if ((fd = open(devname, O_WRONLY))<0) {
-		printf("dxr3: not detected (%s: %s)\n",
-			devname, strerror(errno));
-		return;
-	}
-	if (ioctl(fd, EM8300_IOCTL_GET_AUDIOMODE, &val)<0) {
-		printf("dxr3: ioctl failed (%s)\n", strerror(errno));
-		return;
-	}
-	close(fd);
 	dxr3_ok = 1;
 }
 
@@ -175,7 +155,7 @@ static void dxr3_presence_test( xine_t* xine)
    4 	- Play intra frames only (for FFWD/FBackward)
    6 	- Alternate playmode - not much is known about this mode
      	  other than it buffers frames, possibly re-organising them
-     	  on-the-fly to match SCR vs PTS values
+     	  on-the-fly to match scr vs pts values
    0x11 - Flush the onboard buffer???????
    0x10 - as above??? 
 */
@@ -192,7 +172,9 @@ typedef struct dxr3scr_s {
 	scr_plugin_t scr;
 	int fd_control;
 	int priority;
-	int offset; /* a little offset < 7200 */
+	int64_t offset;     /* difference between real scr and internal dxr3 clock */
+	uint32_t last_pts;  /* last known value of internal dxr3 clock to detect wrap around */
+	pthread_mutex_t mutex;
 } dxr3scr_t;
 
 static int dxr3scr_get_priority (scr_plugin_t *scr) {
@@ -223,32 +205,33 @@ static int dxr3scr_set_speed (scr_plugin_t *scr, int speed) {
 		break;
 	case SPEED_SLOW_2:
 		em_speed = 0x900/2;
-		playmode=MVCOMMAND_START;		
+		playmode=MVCOMMAND_START;
 		break;
 	case SPEED_NORMAL:
 		em_speed = 0x900;
-		playmode=MVCOMMAND_START;
+		playmode=MVCOMMAND_SYNC;
 		break;
 	case SPEED_FAST_2:
 		em_speed = 0x900*2;
-		playmode=MVCOMMAND_SCAN;
+		playmode=MVCOMMAND_START;
 		break;
 	case SPEED_FAST_4:
 		em_speed = 0x900*4;
-		playmode=MVCOMMAND_SCAN;
+		playmode=MVCOMMAND_START;
 		break;
 	default:
 		em_speed = 0;
 		playmode = MVCOMMAND_PAUSE;
 	}
+	
+	if (dxr3_mvcommand(self->fd_control, playmode))
+		printf("dxr3scr: failed to playmode (%s)\n", strerror(errno));
+		
 	if(em_speed>0x900)
 		scanning_mode=1;
 	else
 		scanning_mode=0;
 
-	if(dxr3_mvcommand(self->fd_control,playmode))
-		printf("dxr3scr: failed to playmode (%s)\n", strerror(errno));
-		
 	if (ioctl(self->fd_control, EM8300_IOCTL_SCR_SETSPEED, &em_speed))
 		printf("dxr3scr: failed to set speed (%s)\n", strerror(errno));
 
@@ -257,26 +240,32 @@ static int dxr3scr_set_speed (scr_plugin_t *scr, int speed) {
 
 
 /* *** dxr3scr_adjust ***
-   Adjusts the SCR value of the card to match that given.
-   This function is only called if the dxr3 SCR plugin is
+   Adjusts the scr value of the card to match that given.
+   This function is only called if the dxr3 scr plugin is
    _NOT_ master...
    Harm: wish that were so. It's called by audio_out
    (those adjusting master clock x->y messages)
 */
-static void dxr3scr_adjust (scr_plugin_t *scr, uint32_t vpts) {
+static void dxr3scr_adjust (scr_plugin_t *scr, int64_t vpts) {
 	dxr3scr_t *self = (dxr3scr_t*) scr;
-	uint32_t cpts;
-	if (ioctl(self->fd_control, EM8300_IOCTL_SCR_GET, &cpts))
+	uint32_t cpts32;
+	int32_t offset32;
+	
+	pthread_mutex_lock(&self->mutex);
+	if (ioctl(self->fd_control, EM8300_IOCTL_SCR_GET, &cpts32))
 		printf("dxr3scr: adjust get failed (%s)\n", strerror(errno));
-	cpts <<= 1;
-	self->offset = vpts - cpts;
-	/* kernel driver ignores diffs < 7200 */
-	if (self->offset < -7200 || self->offset > 7200) {
-		vpts >>= 1;
-		if (ioctl(self->fd_control, EM8300_IOCTL_SCR_SET, &vpts))
+	self->last_pts = cpts32;
+	self->offset = vpts - ((int64_t)cpts32 << 1);
+	offset32 = self->offset / 4;
+	/* kernel driver ignores diffs < 7200, so abs(offset32) must be > 7200 / 4 */
+	if (offset32 < -7200/4 || offset32 > 7200/4) {
+		uint32_t vpts32 = vpts >> 1;
+		if (ioctl(self->fd_control, EM8300_IOCTL_SCR_SET, &vpts32))
 			printf("dxr3scr: adjust set failed (%s)\n", strerror(errno));
-		self->offset = 0;
+		self->last_pts = vpts32;
+		self->offset = vpts - ((int64_t)vpts32 << 1);
 	}
+	pthread_mutex_unlock(&self->mutex);
 }
 
 /* *** dxr3scr_start ***
@@ -284,16 +273,19 @@ static void dxr3scr_adjust (scr_plugin_t *scr, uint32_t vpts) {
    it in start_vpts.  also sets the speed of the clock to 0x900 - which
    is normal speed.
 */
-static void dxr3scr_start (scr_plugin_t *scr, uint32_t start_vpts) {
+static void dxr3scr_start (scr_plugin_t *scr, int64_t start_vpts) {
 	dxr3scr_t *self = (dxr3scr_t*) scr;
-	start_vpts >>= 1;
+	uint32_t vpts32 = start_vpts >> 1;
 
-	self->offset = 0;
-	if (ioctl(self->fd_control, EM8300_IOCTL_SCR_SET, &start_vpts))
+	pthread_mutex_lock(&self->mutex);
+	self->last_pts = vpts32;
+	self->offset = start_vpts - ((int64_t)vpts32 << 1);
+	if (ioctl(self->fd_control, EM8300_IOCTL_SCR_SET, &vpts32))
 		printf("dxr3scr: start failed (%s)\n", strerror(errno));
-	/* mis-use start_vpts */
-	start_vpts = 0x900;
-	ioctl(self->fd_control, EM8300_IOCTL_SCR_SETSPEED, &start_vpts);
+	/* mis-use vpts32 */
+	vpts32 = 0x900;
+	ioctl(self->fd_control, EM8300_IOCTL_SCR_SETSPEED, &vpts32);
+	pthread_mutex_unlock(&self->mutex);
 }
 
 
@@ -302,14 +294,33 @@ static void dxr3scr_start (scr_plugin_t *scr, uint32_t start_vpts) {
    on the dxr3 - apparently only called when the dxr3_scr plugin is
    master..
 */
-static uint32_t dxr3scr_get_current (scr_plugin_t *scr) {
+static int64_t dxr3scr_get_current (scr_plugin_t *scr) {
 	dxr3scr_t *self = (dxr3scr_t*) scr;
 	uint32_t pts;
+	int64_t current;
 
-	if (ioctl(self->fd_control, EM8300_IOCTL_SCR_GET, &pts))
+	pthread_mutex_lock(&self->mutex);
+        if (ioctl(self->fd_control, EM8300_IOCTL_SCR_GET, &pts))
 		printf("dxr3scr: get current failed (%s)\n", strerror(errno));
+	/* FIXME: The pts value read from the card sometimes drops to zero for 
+	   unknown reasons. We catch this here, but we should better find out, why. */
+	if (pts == 0) pts = self->last_pts;
+	if (pts < self->last_pts) /* wrap around detected, compensate with offset */
+		self->offset += (int64_t)1 << 33;
+	self->last_pts = pts;
+	current = ((int64_t)pts << 1) + self->offset;
+        pthread_mutex_unlock(&self->mutex);
 
-	return (pts << 1) + self->offset;
+	return current;
+}
+ /* *** dxr3scr_exit ***
+    clean up for exit
+ */
+static void dxr3scr_exit (scr_plugin_t *scr) {
+       dxr3scr_t *self = (dxr3scr_t*) scr;
+
+       pthread_mutex_destroy(&self->mutex);
+       free (self);
 }
 
 
@@ -318,6 +329,7 @@ static uint32_t dxr3scr_get_current (scr_plugin_t *scr) {
 */
 static scr_plugin_t* dxr3scr_init (dxr3_decoder_t *dxr3) {
 	dxr3scr_t *self;
+	char tmpstr[128];
 
 	self = malloc(sizeof(*self));
 	memset(self, 0, sizeof(*self));
@@ -328,64 +340,61 @@ static scr_plugin_t* dxr3scr_init (dxr3_decoder_t *dxr3) {
 	self->scr.adjust            = dxr3scr_adjust;
 	self->scr.start             = dxr3scr_start;
 	self->scr.get_current       = dxr3scr_get_current;
+	self->scr.exit              = dxr3scr_exit;
 
 	self->offset = 0;
+	self->last_pts = 0;
+	pthread_mutex_init(&self->mutex, NULL);
 
-	if ((self->fd_control = open (devname, O_WRONLY)) < 0) {
+	snprintf (tmpstr, sizeof(tmpstr), "%s%s", devname, devnum);
+	if ((self->fd_control = open (tmpstr, O_WRONLY)) < 0) {
 		printf("dxr3scr: Failed to open control device %s (%s)\n",
-		 devname, strerror(errno));
+		 tmpstr, strerror(errno));
 		return NULL;
 	}
 
 	self->priority = dxr3->scr_prio;
 
-	printf("dxr3scr: init complete\n");
+	printf("dxr3scr_init: init complete\n");
 	return &self->scr;
 }
 
 static int dxr3_can_handle (video_decoder_t *this_gen, int buf_type)
 {
-	if (! dxr3_check_vo(((dxr3_decoder_t*)this_gen)->config)) {
-		/* dxr3 video out is not active. Play dead. */
-		return 0;
-	}
 	buf_type &= 0xFFFF0000;
-	return (buf_type == BUF_VIDEO_MPEG) || (buf_type == BUF_VIDEO_FILL);
+	return (buf_type == BUF_VIDEO_MPEG);
 }
 
 static void dxr3_init (video_decoder_t *this_gen, vo_instance_t *video_out)
 {
 	dxr3_decoder_t *this = (dxr3_decoder_t *) this_gen;
+	metronom_t *metronom = this->video_decoder.metronom;
+	char tmpstr[128];
+	int64_t cur_offset;
 
-	printf("dxr3: Entering video init, devname=%s.\n",devname);
+	snprintf (tmpstr, sizeof(tmpstr), "%s%s", devname, devnum);
+	printf("dxr3: Entering video init, devname=%s.\n",tmpstr);
    
 	this->fd_video = -1; /* open later */
 
-	if ((this->fd_control = open (devname, O_WRONLY)) < 0) {
+	if ((this->fd_control = open (tmpstr, O_WRONLY)) < 0) {
 		printf("dxr3: Failed to open control device %s (%s)\n",
-		 devname, strerror(errno));
+		 tmpstr, strerror(errno));
 		return;
 	}
 
 	video_out->open(video_out);
 	this->video_out = video_out;
 
-	this->last_pts = 0;
+	this->last_pts = metronom->get_current_time(metronom);
 	this->decoder_thread = pthread_self();
-
-	this->scr = dxr3scr_init(this);
-	this->video_decoder.metronom->register_scr(
-	 this->video_decoder.metronom, this->scr);
-
-	if (this->video_decoder.metronom->scr_master == this->scr) {
-		printf("dxr3: dxr3scr plugin is master\n");
-	}
-	else {
-		printf("dxr3: dxr3scr plugin is NOT master\n");
-	}
-	/* dxr3_init is called while the master scr already runs.
-	   therefore the scr must be started here */
-	this->scr->start(this->scr, 0);
+	
+	this->resync_window = 0;
+	
+	cur_offset = metronom->get_option(metronom, METRONOM_AV_OFFSET);
+	metronom->set_option(metronom, METRONOM_AV_OFFSET, cur_offset - 21600);
+	/* a note on the 21600: this seems to be a necessary offset,
+	   maybe the dxr3 needs some internal time, however it improves sync a lot */
 }
 
 
@@ -426,7 +435,7 @@ static int get_duration(int framecode, int repeat_first_field)
 		/*duration=3600;*/
 		break;
 	case 4: /* 29.970 */
-		duration=repeat_first_field ? 3754 : 3003;
+		duration=repeat_first_field ? 4505 : 3003;
 		/*duration=3003;*/
 		break;
 	case 5: /* 30.000 */
@@ -449,6 +458,11 @@ static int get_duration(int framecode, int repeat_first_field)
 	return duration;
 }
 
+static void dxr3_reset(video_decoder_t *this_gen)
+{
+/*	dxr3_decoder_t *this = (dxr3_decoder_t *) this_gen; */
+}
+
 /* *** dxr3_flush ***
    flush the dxr3's onboard buffers - but I'm not sure that this is 
    doing that - more testing is required.
@@ -459,8 +473,8 @@ static void dxr3_flush (video_decoder_t *this_gen)
 	/* Don't flush, causes still images to disappear. We don't seem
 	 * to need it anyway... */
 	printf("dxr3_decoder: flush requested, disabled for the moment.\n");
-	/* dxr3_mvcommand(this->fd_control, 0x11); */
-	this->have_header_info = 0;
+	/* dxr3_mvcommand(this->fd_control, 0x11);
+	this->have_header_info = 0;*/
 	if (this->fd_video >= 0)
 		fsync(this->fd_video);
 }
@@ -470,43 +484,16 @@ static void dxr3_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 {
 	dxr3_decoder_t *this = (dxr3_decoder_t *) this_gen;
 	ssize_t written;
-	int vpts, i, duration, skip;
+	int64_t vpts;
+	int i, duration, skip;
         vo_frame_t *img;
 	uint8_t *buffer, byte;
 	uint32_t shift;
 
+	if (buf->decoder_flags & BUF_FLAG_PREVIEW) return;
+
 	vpts = 0;
 
-#if 0
-	/* if we're just coming from a BUF_VIDEO_FILL situation,
-	 * do a flush for good riddance. (doesn't help much though) */
-	if (this->in_buffer_fill && buf->type != BUF_VIDEO_FILL)
-		dxr3_flush(this_gen);
-	this->in_buffer_fill = (buf->type == BUF_VIDEO_FILL);
-
-	/* FIXME What are we supposed to do with this? */
-	if (buf->type == BUF_VIDEO_FILL && this->have_header_info) {
-		duration = get_duration(this->frame_rate_code, 0); 
-		/* printf("dxr3enc: BUF_VIDEO_FILL\n"); */
-		/* require have_header_info, otherwise width and height
-	 	 * settings may be random */
-    		img = this->video_out->get_frame (this->video_out,
-                             this->width,
-                             this->height,
-                             this->aspect,
-                             IMGFMT_YV12,
-			     duration,
-                             DXR3_VO_UPDATE_FLAG);
-		img->PTS=0;
-		img->bad_frame = 0;
-	        img->draw(img);
-		vpts = img->PTS;
-		img->free(img);
-		this->last_pts += duration; /* predict vpts */
-		return;
-	}
-#endif
-
 	/* count the number of frame codes in this block of data 
 	 * this code borrowed from libmpeg2. 
 	 * Note: this uses the 'naive' approach of constant durations,
@@ -561,26 +548,45 @@ static void dxr3_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 		duration = get_duration(this->frame_rate_code, 
 					this->repeat_first_field);
 		/* pretend like we have decoded a frame */
-    		img = this->video_out->get_frame (this->video_out,
+		img = this->video_out->get_frame (this->video_out,
                              this->width,
                              this->height,
                              this->aspect,
-                             IMGFMT_YV12,
-                             duration,
-                             DXR3_VO_UPDATE_FLAG);
-		img->PTS=buf->PTS;
+                             IMGFMT_MPEG,
+                             VO_BOTH_FIELDS);
+		img->pts=buf->pts;
 		img->bad_frame = 0;
+		img->duration = duration;
 		/* draw calls metronom->got_video_frame with img pts and scr
-		   and stores the return value back in img->PTS
-		   Calling draw with buf->PTS==0 is okay; metronome will
+		   and stores the return value in img->vpts
+		   Calling draw with buf->pts==0 is okay; metronome will
 		   extrapolate a value. */
 		skip = img->draw(img);
 	        if (skip <= 0) { /* don't skip */
-			vpts = img->PTS; /* copy so we can free img */
+			vpts = img->vpts; /* copy so we can free img */
+			
+			if (this->resync_window == 0 && this->scr && this->enhanced_mode && !scanning_mode) {
+				/* we are in sync, so we can lock the stream now */
+				printf("dxr3: in sync, stream locked\n");
+				dxr3_mvcommand(this->fd_control, MVCOMMAND_SYNC);
+				this->resync_window = -RESYNC_WINDOW_SIZE;
+			}
+			if (this->resync_window != 0 && this->resync_window > -RESYNC_WINDOW_SIZE)
+				this->resync_window--;
 		}
-		else { /* metronom says skip, so don't set PTS */
+		else { /* metronom says skip, so don't set vpts */
 			printf("dxr3: skip = %d\n", skip);
 			vpts = 0;
+			
+			if (scanning_mode) this->resync_window = 0;
+			if (this->resync_window == 0 && this->scr && this->enhanced_mode && !scanning_mode) {
+				/* switch off sync mode in the card to allow resyncing */
+				printf("dxr3: out of sync, allowing stream resync\n");
+				dxr3_mvcommand(this->fd_control, MVCOMMAND_START);
+				this->resync_window = RESYNC_WINDOW_SIZE;
+			}
+			if (this->resync_window != 0 && this->resync_window < RESYNC_WINDOW_SIZE)
+				this->resync_window++;
 		}
 		img->free(img);
 		this->last_pts += duration; /* predict vpts */
@@ -604,7 +610,7 @@ static void dxr3_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 	 * wants to open it) */
 	if (this->fd_video < 0) {	
 		char tmpstr[128];
-		snprintf (tmpstr, sizeof(tmpstr), "%s_mv", devname);
+		snprintf (tmpstr, sizeof(tmpstr), "%s_mv%s", devname, devnum);
 		if ((this->fd_video = open (tmpstr, O_WRONLY | O_NONBLOCK)) < 0) {
 			printf("dxr3: Failed to open video device %s (%s)\n",
 				tmpstr, strerror(errno)); 
@@ -612,30 +618,56 @@ static void dxr3_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 		}
 	}
 
+	/* We may want to issue a SETPTS, so make sure the scr plugin
+	   is running and registered. Unfortuantely wa cannot do this
+	   earlier, because the dxr3's internal scr gets confused
+	   when started with a closed video device. Maybe this is a
+	   driver bug and gets fixed somewhen. FIXME: We might then
+	   want to move this code to dxr3_init. */
+	if (!this->scr) {
+		int64_t time;
+		
+		time = this->video_decoder.metronom->get_current_time(
+			this->video_decoder.metronom);
+		
+		this->scr = dxr3scr_init(this);
+		this->scr->start(this->scr, time);
+		this->video_decoder.metronom->register_scr(
+			this->video_decoder.metronom, this->scr);
+		if (this->video_decoder.metronom->scr_master == this->scr) {
+			printf("dxr3: dxr3scr plugin is master\n");
+		} else {
+			printf("dxr3: dxr3scr plugin is NOT master\n");
+		}
+	}
+
 	/* From time to time, update the pts value 
 	 * FIXME: the exact conditions here are a bit uncertain... */
 	if (vpts)
 	{
-		int delay;
+		int64_t delay;
+		
 		delay = vpts - this->video_decoder.metronom->get_current_time(
-				this->video_decoder.metronom);
+			this->video_decoder.metronom);
 #if LOG_PTS
-		printf("dxr3: SETPTS got %d expected = %d (delta %d) delay = %d\n", 
+		printf("dxr3: SETPTS got %lld expected = %lld (delta %lld) delay = %lld\n", 
 			vpts, this->last_pts, vpts-this->last_pts, delay);
 #endif
 		this->last_pts = vpts;
+		
 		/* SETPTS only if less then one second in the future and
-		 * either buffer has PTS or sync_every_frame is set */
+		 * either buffer has pts or sync_every_frame is set */
 		if ((delay > 0) && (delay < 90000) &&
-		    (this->sync_every_frame || buf->PTS)) {
-			/* update the dxr3's current pts value */	
-			if (ioctl(this->fd_video, EM8300_IOCTL_VIDEO_SETPTS, &vpts))
+		    (this->sync_every_frame || buf->pts)) {
+			uint32_t vpts32 = vpts;
+			/* update the dxr3's current pts value */
+			if (ioctl(this->fd_video, EM8300_IOCTL_VIDEO_SETPTS, &vpts32))
 				printf("dxr3: set video pts failed (%s)\n",
 					 strerror(errno));
 		}
 		if (delay >= 90000) {
 			/* frame more than 1 sec ahead */
-			printf("dxr3: WARNING: vpts %d is %.02f seconds ahead of time!\n",
+			printf("dxr3: WARNING: vpts %lld is %.02f seconds ahead of time!\n",
 				vpts, delay/90000.0); 
 		}
 		if (delay < 0) {
@@ -643,14 +675,11 @@ static void dxr3_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 		}
 	}
 #if LOG_PTS
-	else if (buf->PTS) {
-		printf("dxr3: skip buf->PTS = %d (no vpts) last_vpts = %d\n", 
-			buf->PTS, this->last_pts);
+	else if (buf->pts) {
+		printf("dxr3: skip buf->pts = %lld (no vpts) last_vpts = %lld\n", 
+			buf->pts, this->last_pts);
 	}
 #endif
-	/* if the dxr3_alt_play option is used, change the dxr3 playmode */
-	if(this->enhanced_mode && !scanning_mode)
-		dxr3_mvcommand(this->fd_control, 6);
 
 	/* now write the content to the dxr3 mpeg device and, in a dramatic
 	   break with open source tradition, check the return value */
@@ -675,12 +704,19 @@ static void dxr3_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 static void dxr3_close (video_decoder_t *this_gen)
 {
 	dxr3_decoder_t *this = (dxr3_decoder_t *) this_gen;
+	metronom_t *metronom = this->video_decoder.metronom;
+	int64_t cur_offset;
 
 	if (this->scr) {
-		this->video_decoder.metronom->unregister_scr(
-			this->video_decoder.metronom, this->scr);
+		metronom->unregister_scr(metronom, this->scr);
+		this->scr->exit(this->scr);
+		this->scr = 0;
 	}
-
+	
+	/* revert av_offset change done in dxr3_init */
+	cur_offset = metronom->get_option(metronom, METRONOM_AV_OFFSET);
+	metronom->set_option(metronom, METRONOM_AV_OFFSET, cur_offset + 21600);
+	
 	if (this->fd_video >= 0)
 		close(this->fd_video);
 	this->fd_video = -1;
@@ -723,13 +759,19 @@ static void dxr3_flush_decoder(void *this_gen, cfg_entry_t *entry)
 	entry->num_value = 0;	
 }
 
+static void dxr3_dispose (video_decoder_t *this_gen) {
+  free (this_gen);
+}
+
 video_decoder_t *init_video_decoder_plugin (int iface_version,
  xine_t *xine)
 {
 	dxr3_decoder_t *this ;
 	config_values_t *cfg;
+	char *tmpstr;
+	int dashpos;
 
-	if (iface_version != 5) {
+	if (iface_version != 7) {
 		printf( "dxr3: plugin doesn't support plugin API version %d.\n"
 		 "dxr3: this means there's a version mismatch between xine and this\n"
 		 "dxr3: decoder plugin. Installing current plugins should help.\n",
@@ -738,7 +780,19 @@ video_decoder_t *init_video_decoder_plugin (int iface_version,
 	}
  
 	cfg = xine->config;
-	devname = cfg->register_string (cfg, LOOKUP_DEV, DEFAULT_DEV, "Dxr3: Device Name",NULL,NULL,NULL);
+	tmpstr = cfg->register_string (cfg, LOOKUP_DEV, DEFAULT_DEV, "Dxr3: Device Name",NULL,NULL,NULL);
+	strncpy(devname, tmpstr, 128);
+	devname[127] = '\0';
+	dashpos = strlen(devname) - 2; /* the dash in the new device naming scheme would be here */
+	if (devname[dashpos] == '-') {
+		/* use new device naming scheme with trailing number */
+		strncpy(devnum, &devname[dashpos], 3);
+		devname[dashpos] = '\0';
+	} else {
+		/* use old device naming scheme without trailing number */
+		/* FIXME: remove this when everyone uses em8300 >=0.12.0 */
+		devnum[0] = '\0';
+	}
 
 	dxr3_presence_test ( xine );
 	if (!dxr3_ok) return NULL;
@@ -752,13 +806,18 @@ video_decoder_t *init_video_decoder_plugin (int iface_version,
 	this->video_decoder.close               = dxr3_close;
 	this->video_decoder.get_identifier      = dxr3_get_id;
 	this->video_decoder.flush		= dxr3_flush;
-	this->video_decoder.priority            = 10;
+	this->video_decoder.reset		= dxr3_reset;
+	this->video_decoder.dispose		= dxr3_dispose;
+	this->video_decoder.priority            = cfg->register_num(cfg,
+		"dxr3.decoder_priority", 10, "Dxr3: video decoder priority", NULL, NULL, NULL);
+	
 	this->config			        = cfg;
 
 	this->frame_rate_code = 0;
 	this->repeat_first_field = 0;
 	this->sync_every_frame = 1;
 
+	this->scr = NULL;
 	this->scr_prio = cfg->register_num(cfg, "dxr3.scr_priority", 10, "Dxr3: SCR plugin priority",NULL,NULL,NULL); 
         
 	this->sync_every_frame = cfg->register_bool(cfg,
@@ -772,7 +831,7 @@ video_decoder_t *init_video_decoder_plugin (int iface_version,
 
 	this->enhanced_mode = cfg->register_bool(cfg,
 		"dxr3.alt_play_mode", 
-		0, 
+		1, 
 		"Use alternate Play mode",
 		"Enabling this option will utilise a slightly different play mode",
 		dxr3_update_enhanced_mode, this);
@@ -805,16 +864,15 @@ typedef struct spudec_decoder_s {
 	vo_instance_t   	*vo_out;
 	int 			fd_spu;
 	int			menu; /* are we in a menu? */
+	pci_t			pci;
+	uint32_t		buttonN;  /* currently highlighted button */
+	int64_t			button_vpts; /* time to show the menu enter button */
 	xine_t			*xine;
 } spudec_decoder_t;
 
 static int spudec_can_handle (spu_decoder_t *this_gen, int buf_type)
 {
 	int type = buf_type & 0xFFFF0000;
-	if (! dxr3_check_vo(((spudec_decoder_t*)this_gen)->xine->config) ) {
-		/* dxr3 video out is not active. Play dead. */
-		return 0;
-	}
 	return (type == BUF_SPU_PACKAGE || type == BUF_SPU_CLUT || 
 		type == BUF_SPU_NAV || type == BUF_SPU_SUBP_CONTROL);
 }
@@ -828,7 +886,7 @@ static void spudec_init (spu_decoder_t *this_gen, vo_instance_t *vo_out)
 	this->vo_out = vo_out;
 
 	/* open spu device */
-	snprintf (tmpstr, sizeof(tmpstr), "%s_sp", devname);
+	snprintf (tmpstr, sizeof(tmpstr), "%s_sp%s", devname, devnum);
 	if ((this->fd_spu = open (tmpstr, O_WRONLY)) < 0) {
 		printf("dxr3: Failed to open spu device %s (%s)\n",
 		 tmpstr, strerror(errno));
@@ -855,6 +913,39 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf)
 	ssize_t written;
 	uint32_t stream_id = buf->type & 0x1f ;
 	
+	if (this->button_vpts &&
+		this->spu_decoder.metronom->get_current_time(this->spu_decoder.metronom) > this->button_vpts) {
+		/* we have a scheduled menu enter button and it's time to show it now */
+		em8300_button_t button;
+		int buttonNr;
+		btni_t *button_ptr;
+		
+		buttonNr = this->buttonN;
+		
+		if (this->pci.hli.hl_gi.fosl_btnn > 0)
+			buttonNr = this->pci.hli.hl_gi.fosl_btnn ;
+		
+		if ((buttonNr <= 0) || (buttonNr > this->pci.hli.hl_gi.btn_ns)) {
+			printf("dxr3_spu: Unable to select button number %i as it doesn't exist. Forcing button 1\n", buttonNr);
+			buttonNr = 1;
+		}
+		
+		button_ptr = &this->pci.hli.btnit[buttonNr-1];
+		button.left = button_ptr->x_start;
+		button.top  = button_ptr->y_start;
+		button.right = button_ptr->x_end;
+		button.bottom = button_ptr->y_end;
+		button.color = this->pci.hli.btn_colit.btn_coli[button_ptr->btn_coln-1][0] >> 16;
+		button.contrast = this->pci.hli.btn_colit.btn_coli[button_ptr->btn_coln-1][0] & 0xffff;
+		
+#if LOG_SPU
+		printf("dxr3_spu: now showing menu enter button %i.\n", buttonNr);
+#endif
+		ioctl(this->fd_spu, EM8300_IOCTL_SPU_BUTTON, &button);
+		this->button_vpts = 0;
+		this->pci.hli.hl_gi.hli_s_ptm = 0;
+	}
+	
 	if (buf->type == BUF_SPU_CLUT) {
 #if LOG_SPU
         printf ("dxr3_spu: BUF_SPU_CLUT\n" );
@@ -880,12 +971,30 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf)
 	}
         if(buf->type == BUF_SPU_NAV){
 #if LOG_SPU
-          printf ("dxr3_spu: Got NAV packet\n");
+		printf("dxr3_spu: Got NAV packet\n");
 #endif
-     	  return;
+		uint8_t *p = buf->content;
+		
+		/* just watch out for menus */
+		if (p[3] == 0xbf && p[6] == 0x00) { /* Private stream 2 */
+			pci_t pci;
+			
+			nav_read_pci(&pci, p + 7);
+			
+			if (pci.hli.hl_gi.hli_ss == 1)
+				/* menu ahead, remember pci for later evaluation */
+				xine_fast_memcpy(&this->pci, &pci, sizeof(pci_t));
+			
+			if ((pci.hli.hl_gi.hli_ss == 0) && (this->pci.hli.hl_gi.hli_ss == 1)) {
+				/* leaving menu */
+				xine_fast_memcpy(&this->pci, &pci, sizeof(pci_t));
+				ioctl(this->fd_spu, EM8300_IOCTL_SPU_BUTTON, NULL);
+			}
+		}
+		return;
 	}
 	/* Is this also needed for subpicture? */
-	if (buf->decoder_info[0] == 0) {
+	if (buf->decoder_flags & BUF_FLAG_PREVIEW) {
 #if LOG_SPU
         printf ("dxr3_spu: Dropping SPU channel %d. Preview data\n", stream_id);
 #endif
@@ -906,19 +1015,32 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf)
 #endif
           return;
         }
+	if ( (this->menu == 0) && (this->xine->spu_channel & 0x80) ) { 
+#if LOG_SPU
+	  printf ("dxr3_spu: Dropping SPU channel %d. Only allow forced display SPUs\n", stream_id);
+#endif
+	  return;
+	}
 //        if (this->xine->spu_channel != stream_id && this->menu!=1 ) return; 
         /* Hide any previous button highlights */
 	ioctl(this->fd_spu, EM8300_IOCTL_SPU_BUTTON, NULL);
-	if (buf->PTS) {
-		int vpts;
+	if (buf->pts) {
+		int64_t vpts;
+		uint32_t vpts32;
+		
 		vpts = this->spu_decoder.metronom->got_spu_packet
-		 (this->spu_decoder.metronom, buf->PTS, 0, buf->SCR);
+		 (this->spu_decoder.metronom, buf->pts);
 #if LOG_SPU
-                printf ("dxr3_spu: PTS=%u VPTS=%u\n", buf->PTS, vpts);
+                printf ("dxr3_spu: pts=%lld vpts=%lld\n", buf->pts, vpts);
 #endif
-
-		if (ioctl(this->fd_spu, EM8300_IOCTL_SPU_SETPTS, &vpts))
+		vpts32 = vpts;
+		if (ioctl(this->fd_spu, EM8300_IOCTL_SPU_SETPTS, &vpts32))
 			printf("dxr3: spu setpts failed (%s)\n", strerror(errno));
+			
+		if (buf->pts == this->pci.hli.hl_gi.hli_s_ptm)
+			/* schedule the menu enter button for current packet's vpts, so
+			that the card has decoded this SPU when we try to show the button */
+			this->button_vpts = vpts;
 	}
 
 
@@ -936,6 +1058,10 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf)
 		 written, buf->size);
 }
 
+static void spudec_reset (spu_decoder_t *this_gen)
+{
+}
+
 static void spudec_close (spu_decoder_t *this_gen)
 {
 	spudec_decoder_t *this = (spudec_decoder_t *) this_gen;
@@ -970,6 +1096,9 @@ static void spudec_event_listener (void *this_gen, xine_event_t *event_gen) {
 //	ioctl(this->fd_spu, EM8300_IOCTL_SPU_BUTTON, NULL);
 	break;
       }
+      
+      this->buttonN = but->buttonN;
+      
       btn.color = btn.contrast = 0;
 #if LOG_SPU
         printf ("dxr3_spu: buttonN = %u\n",but->buttonN);
@@ -993,7 +1122,7 @@ static void spudec_event_listener (void *this_gen, xine_event_t *event_gen) {
 
   case XINE_EVENT_SPU_CLUT:
     {
-      spu_cltbl_t *clut = event->data;
+      spudec_clut_table_t *clut = event->data;
 #if LOG_SPU
         printf ("dxr3_spu: SPU_CLUT\n");
 #endif
@@ -1037,12 +1166,22 @@ static char *spudec_get_id(void)
 	return "dxr3-spudec";
 }
 
+static void spudec_dispose (spu_decoder_t *this_gen) {
+  spudec_decoder_t *this  = (spudec_decoder_t *) this_gen;
+
+  xine_remove_event_listener (this->xine, spudec_event_listener);
+
+  free (this);
+}
+
 spu_decoder_t *init_spu_decoder_plugin (int iface_version, xine_t *xine)
 {
   spudec_decoder_t *this;
   config_values_t  *cfg;
+  char *tmpstr;
+  int dashpos;
 
-  if (iface_version != 4) {
+  if (iface_version != 6) {
     printf( "dxr3: plugin doesn't support plugin API version %d.\n"
 	    "dxr3: this means there's a version mismatch between xine and this "
 	    "dxr3: decoder plugin.\nInstalling current plugins should help.\n",
@@ -1051,7 +1190,19 @@ spu_decoder_t *init_spu_decoder_plugin (int iface_version, xine_t *xine)
   }
 
   cfg = xine->config;
-  devname = cfg->register_string (cfg, LOOKUP_DEV, DEFAULT_DEV, NULL,NULL,NULL,NULL);
+  tmpstr = cfg->register_string (cfg, LOOKUP_DEV, DEFAULT_DEV, NULL,NULL,NULL,NULL);
+  strncpy(devname, tmpstr, 128);
+  devname[127] = '\0';
+  dashpos = strlen(devname) - 2; /* the dash in the new device naming scheme would be here */
+  if (devname[dashpos] == '-') {
+	/* use new device naming scheme with trailing number */
+	strncpy(devnum, &devname[dashpos], 3);
+	devname[dashpos] = '\0';
+  } else {
+	/* use old device naming scheme without trailing number */
+	/* FIXME: remove this when everyone uses em8300 >=0.12.0 */
+	devnum[0] = '\0';
+  }
 
   dxr3_presence_test ( xine );
   if (!dxr3_ok) {
@@ -1060,19 +1211,24 @@ spu_decoder_t *init_spu_decoder_plugin (int iface_version, xine_t *xine)
 
   this = (spudec_decoder_t *) malloc (sizeof (spudec_decoder_t));
 
-  this->spu_decoder.interface_version   = 4;
+  this->spu_decoder.interface_version   = iface_version;
   this->spu_decoder.can_handle          = spudec_can_handle;
   this->spu_decoder.init                = spudec_init;
   this->spu_decoder.decode_data         = spudec_decode_data;
+  this->spu_decoder.reset               = spudec_reset;
   this->spu_decoder.close               = spudec_close;
   this->spu_decoder.get_identifier      = spudec_get_id;
+  this->spu_decoder.dispose             = spudec_dispose;
   this->spu_decoder.priority            = 10;
   this->xine				= xine;
   this->menu				= 0;
   this->fd_spu				= 0;
+  this->pci.hli.hl_gi.hli_ss		= 0;
+  this->pci.hli.hl_gi.hli_s_ptm		= 0;
+  this->buttonN				= 1;
+  this->button_vpts			= 0;
   
   xine_register_event_listener(xine, spudec_event_listener, this);
   
   return (spu_decoder_t *) this;
 }
-
