@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: dxr3_video_out.c,v 1.5 2002/01/09 22:33:04 jcdutton Exp $
+ * $Id: dxr3_video_out.c,v 1.15 2002/04/04 00:08:36 miguelfreitas Exp $
  *
  * mpeg1 encoding video out plugin for the dxr3.  
  *
@@ -160,32 +160,6 @@ int dxr3_rte_init(dxr3_driver_t *);
 int dxr3_fame_init(dxr3_driver_t *);
 #endif
 
-#define MV_COMMAND 0
-
-vo_info_t *get_video_out_plugin_info();
-
-/* some helper stuff so that the decoder plugin can test for the
- * presence of the dxr3 vo driver */
-/* to be called by dxr3 video out init and exit handlers */
-static void dxr3_set_vo(dxr3_driver_t* this, int active)
-{
-	cfg_entry_t *entry;
-	config_values_t *config = this->config;
-
-	entry = config->lookup_entry(config, "dxr3.active");
-	if (! entry) {
-		/* register first */
-		config->register_num(config, "dxr3.active", active, 
-			"state of dxr3 video out", 
-			"(internal variable; do not edit)",
-			NULL, NULL);
-	}
-	else {
-		entry->num_value = active;
-	}
-	printf("dxr3: %s dxr3 video out.\n", (active ? "enabled" : "disabled"));
-}
-
 static uint32_t dxr3_get_capabilities (vo_driver_t *this_gen)
 {
 	return VO_CAP_YV12 | VO_CAP_YUY2 |
@@ -221,6 +195,7 @@ static vo_frame_t *dxr3_alloc_frame (vo_driver_t *this_gen)
     frame->vo_frame.copy = 0;
   frame->vo_frame.field   = dummy_frame_field; 
   frame->vo_frame.dispose = dxr3_frame_dispose;
+  frame->vo_frame.driver  = this_gen;
 
   return (vo_frame_t*) frame;
 }
@@ -235,41 +210,33 @@ static void dxr3_update_frame_format (vo_driver_t *this_gen,
   dxr3_frame_t  *frame = (dxr3_frame_t *) frame_gen; 
   int image_size, oheight; 
 
+  /* update the overlay window co-ords if required */
+  dxr3_overlay_update(this);
+
   /* reset the copy calls counter (number of calls to dxr3_frame_copy) */	
   frame->copy_calls = 0;
-  frame->vo_instance = this;
+  frame->vo_frame.driver = this_gen;
 
   aspect = this->aspectratio;
   oheight = this->oheight;
 
-  if (flags == DXR3_VO_UPDATE_FLAG) { /* talking to dxr3 decoder */
-	this->mpeg_source = 1;
+  if (format == IMGFMT_MPEG) { /* talking to dxr3 decoder */
+	int aspect;
 	/* a bit of a hack. we must release the em8300_mv fd for
 	 * the dxr3 decoder plugin */
 	if (this->fd_video >= 0) {
 		close(this->fd_video);
-		this->fd_video = -1;
+		this->fd_video = CLOSED_FOR_DECODER;
 	}
-  }
-  else {
-	/* FIXME: Disable reset of mpeg_source 
-	 * video_out.c can call us without the DXR3_VO_UPDATE_FLAG in
-	 * the still frames code. Needs a better fix... */
-	/* this->mpeg_source = 0; */
-  }
-
-  /* for mpeg source, we don't have to do much. */
-  if (this->mpeg_source) {
-	int aspect;
+	/* for mpeg source, we don't have to do much. */
 	this->video_width  = width;
 	this->video_height = height;
 	this->video_aspect = ratio_code;
 	/* remember, there are no buffers malloc'ed for this frame!
- 	 * the dxr3 decoder plugin is cool about this */
+	 * the dxr3 decoder plugin is cool about this */
 	frame->width  = width;
 	frame->height = height;
 	frame->oheight = oheight;
-	frame->format = format;
 	if (frame->mem) {
 		free (frame->mem);
 		frame->mem = NULL;
@@ -288,6 +255,11 @@ static void dxr3_update_frame_format (vo_driver_t *this_gen,
   }
 
   /* the following is for the mpeg encoding part only */
+  
+  if (this->fd_video == CLOSED_FOR_DECODER) { /* decoder should have released it */
+	this->fd_video = CLOSED_FOR_ENCODER; /* allow encoder to reopen it */
+	this->need_redraw = 1;
+  }
 
   if (this->add_bars == 0) {
 	/* don't add black bars; assume source is in 4:3 */
@@ -331,6 +303,7 @@ static void dxr3_update_frame_format (vo_driver_t *this_gen,
     this->video_aspect = ratio_code;
     this->fps = 90000.0/frame->vo_frame.duration;
     this->format = format;
+    this->need_redraw = 1;
 
     if (! this->enc) {
       /* no encoder plugin! Let's bug the user! */
@@ -347,7 +320,7 @@ static void dxr3_update_frame_format (vo_driver_t *this_gen,
 	);
     }
 
-    if (this->mpeg_source == 0 && this->enc && this->enc->on_update_format)
+    if (this->enc && this->enc->on_update_format)
       this->enc->on_update_format(this);
   }
 
@@ -421,18 +394,31 @@ static void dxr3_update_frame_format (vo_driver_t *this_gen,
   frame->width  = width;
   frame->height = height;
   frame->oheight = oheight;
-  frame->format = format;
   frame->swap_fields = this->swap_fields;
   if(this->aspectratio!=aspect)
     dxr3_set_property (this_gen,VO_PROP_ASPECT_RATIO, aspect);
 
 }
 
+int dxr3_redraw_needed(vo_driver_t *this_gen)
+{
+	dxr3_driver_t *this = (dxr3_driver_t *)this_gen;
+	
+	dxr3_overlay_update(this);
+
+	if (this->need_redraw) {
+		this->need_redraw = 0;
+  		return 1;
+	}
+
+	return 0;
+}
+
 static void dxr3_frame_copy(vo_frame_t *frame_gen, uint8_t **src)
 {
 	dxr3_frame_t *frame = (dxr3_frame_t *) frame_gen;
-	dxr3_driver_t *this = frame->vo_instance;
-	if (this->mpeg_source == 0 && this->enc && this->enc->on_frame_copy)
+	dxr3_driver_t *this = (dxr3_driver_t *) frame_gen->driver;
+	if (frame_gen->format != IMGFMT_MPEG && this->enc && this->enc->on_frame_copy)
 		this->enc->on_frame_copy(this, frame, src);
 }
 
@@ -441,10 +427,11 @@ static void dxr3_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen)
   dxr3_driver_t *this = (dxr3_driver_t*)this_gen;
   dxr3_frame_t *frame = (dxr3_frame_t*)frame_gen;
 
-  if (this->mpeg_source == 0 && this->enc && this->enc->on_display_frame)
-	this->enc->on_display_frame(this, frame);
-  if (this->mpeg_source)
-	frame_gen->displayed(frame_gen);
+  if (frame_gen->format != IMGFMT_MPEG && this->enc && this->enc->on_display_frame) {
+        this->enc->on_display_frame(this, frame);
+  } else {
+        frame_gen->displayed(frame_gen);
+  }
   /* for non-mpeg, the encoder plugin is responsible for calling 
    * frame_gen->displayed(frame_gen) ! */
 }
@@ -452,16 +439,18 @@ static void dxr3_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen)
 static void dxr3_overlay_blend (vo_driver_t *this_gen, vo_frame_t *frame_gen,
  vo_overlay_t *overlay)
 {
-	dxr3_driver_t *this = (dxr3_driver_t*)this_gen;
-	if ( this->mpeg_source == 0 )
+	/* FIXME: We only blend non-mpeg frames here.
+	   Is there any way to provide overlays for mpeg content? Subpictures? */
+	if ( frame_gen->format != IMGFMT_MPEG )
 	{
-		/* we have regular YUV frames, so in principle we can blend
-		 * it just like the Xv driver does. Problem is that the
-		 * alphablend.c code file is not nearby */
-	}
-	else {
-		/* we're using hardware mpeg decoding and have no YUV frames
-		 * we need something else to blend... */
+		dxr3_frame_t *frame = (dxr3_frame_t*)frame_gen;
+		
+		if (overlay->rle) {
+			if( frame_gen->format == IMGFMT_YV12 )
+				blend_yuv( frame->vo_frame.base, overlay, frame->width, frame->height);
+			else
+				blend_yuy2( frame->vo_frame.base[0], overlay, frame->width, frame->height);
+		}
 	}
 }
 
@@ -472,11 +461,9 @@ void dxr3_exit (vo_driver_t *this_gen)
 	if (this->enc && this->enc->on_close)
 		this->enc->on_close(this);
 	printf("dxr3: vo exit called\n");
-	this->mpeg_source = 0;
 
 	if(this->overlay_enabled)
 		dxr3_overlay_set_mode(&this->overlay, EM8300_OVERLAY_MODE_OFF);
-	dxr3_set_vo(this, 0);
 }
 
 void dxr3_update_add_bars(void *data, cfg_entry_t* entry)
@@ -508,6 +495,8 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen)
 	char tmpstr[100];
 	const char *encoder; 
 	char *available_encoders,*default_encoder;
+	char *confstr;
+	int dashpos;
 
 	/*
 	* allocate plugin struct
@@ -532,10 +521,9 @@ printf("dxr3_video_out:init_plugin\n");
 	this->vo_driver.set_property         = dxr3_set_property;
 	this->vo_driver.get_property_min_max = dxr3_get_property_min_max;
 	this->vo_driver.gui_data_exchange    = dxr3_gui_data_exchange;
+	this->vo_driver.redraw_needed        = dxr3_redraw_needed;
 	this->vo_driver.exit                 = dxr3_exit;
-	this->vo_driver.get_info             = get_video_out_plugin_info;
 	this->config=config;
-	this->mpeg_source = 0; /* set by update_frame, by checking the flag */
 
 	this->swap_fields = config->register_bool(config, "dxr3.enc_swap_fields", 0, "swap odd and even lines", NULL, dxr3_update_swap_fields, this);
 
@@ -543,12 +531,25 @@ printf("dxr3_video_out:init_plugin\n");
 	
 	this->enhanced_mode = config->register_bool(config,"dxr3.enc_alt_play_mode", 1, "dxr3: use alternate play mode for mpeg encoder playback","Enabling this option will utilise a slightly different play mode",dxr3_update_enhanced_mode,this);
 	/* open control device */
-	this->devname = config->register_string (config, LOOKUP_DEV, DEFAULT_DEV,NULL,NULL,NULL,NULL);
+	confstr = config->register_string (config, LOOKUP_DEV, DEFAULT_DEV,NULL,NULL,NULL,NULL);
+	strncpy(this->devname, confstr, 128);
+	this->devname[127] = '\0';
+	dashpos = strlen(this->devname) - 2; /* the dash in the new device naming scheme would be here */
+	if (this->devname[dashpos] == '-') {
+		/* use new device naming scheme with trailing number */
+		strncpy(this->devnum, &this->devname[dashpos], 3);
+		this->devname[dashpos] = '\0';
+	} else {
+		/* use old device naming scheme without trailing number */
+		/* FIXME: remove this when everyone uses em8300 >=0.12.0 */
+		this->devnum[0] = '\0';
+	}
 
-	printf("dxr3: Entering video init, devname=%s.\n",this->devname);
-	if ((this->fd_control = open(this->devname, O_WRONLY)) < 0) {
+	snprintf (tmpstr, sizeof(tmpstr), "%s%s", this->devname, this->devnum);
+	printf("dxr3: Entering video init, devname=%s.\n",tmpstr);
+	if ((this->fd_control = open(tmpstr, O_WRONLY)) < 0) {
 		printf("dxr3: Failed to open control device %s (%s)\n",
-			this->devname, strerror(errno));
+			tmpstr, strerror(errno));
 		return 0;
 	}
         /* output mpeg to file instead of dxr3? */
@@ -563,7 +564,7 @@ printf("dxr3_video_out:init_plugin\n");
 	}
 	else {        
 	 	/* open video device */
-		snprintf (tmpstr, sizeof(tmpstr), "%s_mv", this->devname);
+		snprintf (tmpstr, sizeof(tmpstr), "%s_mv%s", this->devname, this->devnum);
 		if ((this->fd_video = open (tmpstr, O_WRONLY | O_SYNC )) < 0) {
 			printf("dxr3: failed to open video device %s (%s)\n",
 				tmpstr, strerror(errno));
@@ -571,7 +572,7 @@ printf("dxr3_video_out:init_plugin\n");
 		}
 	        /* close now and and let the encoders reopen if they want */
 	        close(this->fd_video);
-        	this->fd_video = -1;
+        	this->fd_video = CLOSED_FOR_DECODER;
 	}
 
 	/* which encoder to use? Whadda we got? */
@@ -606,7 +607,6 @@ printf("dxr3_video_out:init_plugin\n");
 			printf(
 "dxr3: mpeg encoder \"%s\" not compiled in or not supported.\n"
 "dxr3: valid options are %s\n", encoder, available_encoders);
-			return 0;
 		}
 	}
 	else {
@@ -617,26 +617,23 @@ printf("dxr3_video_out:init_plugin\n");
 "dxr3: driver. See the README.dxr3 for details on configuring an encoder.\n" 
 		);
 	}
-
-	gather_screen_vars(this, visual_gen);
 	
 	/* default values */
 	this->overlay_enabled = 0;
 	this->aspectratio = ASPECT_FULL;
 
-	dxr3_read_config(this);
+	dxr3_read_config(this, visual_gen);
 	
 	if (this->overlay_enabled) {
 		dxr3_get_keycolor(this);
 		dxr3_overlay_buggy_preinit(&this->overlay, this->fd_control);
 	}
 
-	dxr3_set_vo(this, 1);
 	return &this->vo_driver;
 }
 
 static vo_info_t vo_info_dxr3 = {
-  3, /* api version */
+  5, /* api version */
   "dxr3",
   "xine video output plugin for dxr3 cards",
   VISUAL_TYPE_X11,
