@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: dxr3_vo_core.c,v 1.14 2002/01/10 21:30:10 jcdutton Exp $
+ * $Id: dxr3_vo_core.c,v 1.20 2002/04/03 09:40:40 mlampard Exp $
  *
  *************************************************************************
  * core functions common to both Standard and RT-Encoding vo plugins     *
@@ -58,6 +58,28 @@ void *malloc_aligned (size_t alignment, size_t size, void **mem) {
   return aligned;
 }
 
+/****** update the overlay window ******/
+void dxr3_overlay_update(dxr3_driver_t *this)
+{
+	if(this->overlay_enabled) {
+		int gui_win_x, gui_win_y, gypos,gxpos,gw,gh;
+		
+		this->frame_output_cb (this->user_data,
+                   this->video_width, this->video_height,
+                        &gxpos, &gypos, &gw, &gh,
+                             &gui_win_x, &gui_win_y );
+
+		if(this->xpos!=gxpos || this->ypos!=gypos ||
+		   this->width !=gw || this->height!=gh) {
+			this->xpos=gxpos+1;
+			this->ypos=gypos+1;
+			this->width=gw;
+			this->height=gh;
+			dxr3_overlay_adapt_area(this, this->xpos, this->ypos, this->width, this->height);
+		}
+	}
+}
+
 
 /****** detect true window position and adapt overlay to it *******/
 void dxr3_overlay_adapt_area(dxr3_driver_t *this,
@@ -99,13 +121,13 @@ void dxr3_get_keycolor(dxr3_driver_t *this)
  *  overlay setup data is read from ~/.overlay/res* in the     *
  *   		     overlay section below  		       *
  ***************************************************************/	
-void dxr3_read_config(dxr3_driver_t *this)
+void dxr3_read_config(dxr3_driver_t *this, void * visual_gen)
 {
 	char* str;
 	config_values_t *config=this->config;
 	
 	if (ioctl(this->fd_control, EM8300_IOCTL_GETBCS, &this->bcs))
-		fprintf(stderr, "dxr3_vo: cannot read bcs values (%s)\n",
+		printf("dxr3_vo: cannot read bcs values (%s)\n",
 		 strerror(errno));
 
 	this->bcs.contrast = config->register_range(config, "dxr3.contrast", this->bcs.contrast,100,900,"Dxr3: contrast control",NULL,NULL,NULL);
@@ -124,7 +146,8 @@ void dxr3_read_config(dxr3_driver_t *this)
 		this->tv_switchable=0;  /* don't allow on-the-fly switching */		
 	} else if (!strcasecmp(str, "overlay")) {
 		this->tv_mode = EM8300_VIDEOMODE_DEFAULT;
-		fprintf(stderr, "dxr3_vo: setting up overlay mode\n");
+		printf("dxr3_vo: setting up overlay mode\n");
+		gather_screen_vars(this, visual_gen);
 		if (dxr3_overlay_read_state(&this->overlay) == 0) {
 			this->overlay_enabled = 1;
 			this->tv_switchable=1;	
@@ -136,7 +159,7 @@ void dxr3_read_config(dxr3_driver_t *this)
 
 			sscanf(str, "%f", &this->overlay.color_interval);
 		} else {
-			fprintf(stderr, "dxr3_vo: please run autocal, overlay disabled\n");
+			printf("dxr3_vo: please run autocal, overlay disabled\n");
 			this->overlay_enabled=0;
 			this->tv_switchable=0;
 		}
@@ -145,13 +168,13 @@ void dxr3_read_config(dxr3_driver_t *this)
 
 	if (!strcasecmp(str, "ntsc")) {
 		this->tv_mode = EM8300_VIDEOMODE_NTSC;
-		fprintf(stderr, "dxr3_vo: setting tv_mode to NTSC\n");
+		printf("dxr3_vo: setting tv_mode to NTSC\n");
 	} else if (!strcasecmp(str, "pal")) {
 		this->tv_mode = EM8300_VIDEOMODE_PAL;
-		fprintf(stderr, "dxr3_vo: setting tv_mode to PAL 50Hz\n");
+		printf("dxr3_vo: setting tv_mode to PAL 50Hz\n");
 	} else if (!strcasecmp(str, "pal60")) {
 		this->tv_mode = EM8300_VIDEOMODE_PAL60;
-		fprintf(stderr, "dxr3_vo: setting tv_mode to PAL 60Hz\n");
+		printf("dxr3_vo: setting tv_mode to PAL 60Hz\n");
 	} else {
 		this->tv_mode = EM8300_VIDEOMODE_DEFAULT;
 	}
@@ -159,12 +182,12 @@ void dxr3_read_config(dxr3_driver_t *this)
 	
 	if (this->tv_mode != EM8300_VIDEOMODE_DEFAULT)
 		if (ioctl(this->fd_control, EM8300_IOCTL_SET_VIDEOMODE, &this->tv_mode))
-			fprintf(stderr, "dxr3_vo: setting video mode failed.");
+			printf("dxr3_vo: setting video mode failed.");
 }
 
 
 /******** is this window fullscreen? ************/
-static int is_fullscreen(dxr3_driver_t *this)
+int is_fullscreen(dxr3_driver_t *this)
 {
 	XWindowAttributes a;
 
@@ -245,14 +268,16 @@ int dxr3_get_property (vo_driver_t *this_gen, int property)
 	case VO_PROP_COLORKEY:
 		val = this->overlay.colorkey;
 		break;
-	case VO_PROP_ZOOM_X:
-	case VO_PROP_ZOOM_Y:
+	case VO_PROP_ZOOM_FACTOR:
 	case VO_PROP_TVMODE:
 		break;
 
+	case VO_PROP_VO_TYPE:
+		val = VO_TYPE_DXR3;
+		break;
 	default:
 		val = 0;
-		fprintf(stderr, "dxr3_vo: property %d not implemented!\n", property);
+		printf("dxr3_vo: property %d not implemented!\n", property);
 	}
 
 	return val;
@@ -285,48 +310,48 @@ int dxr3_set_property (vo_driver_t *this_gen,
 		if (value > ASPECT_FULL)
 			value = ASPECT_ANAMORPHIC;
 		this->aspectratio = value;
-		fullscreen = is_fullscreen(this);
+		fullscreen = this->overlay_enabled ? is_fullscreen(this) : 0;
 
 		if (value == ASPECT_ANAMORPHIC) {
-			fprintf(stderr, "dxr3_vo: setting aspect ratio to anamorphic\n");
+			printf("dxr3_vo: setting aspect ratio to anamorphic\n");
 			if (!this->overlay_enabled || fullscreen)
 				val = EM8300_ASPECTRATIO_16_9;
 			else /* The overlay window can adapt to the ratio */
 				val = EM8300_ASPECTRATIO_4_3;
 			this->desired_ratio = 16.0/9.0;
 		} else {
-			fprintf(stderr, "dxr3_vo: setting aspect ratio to full\n");
+			printf("dxr3_vo: setting aspect ratio to full\n");
 			val = EM8300_ASPECTRATIO_4_3;
 			this->desired_ratio = 4.0/3.0;
 		}
 
 		if (ioctl(this->fd_control, EM8300_IOCTL_SET_ASPECTRATIO, &val))
-			fprintf(stderr, "dxr3_vo: failed to set aspect ratio (%s)\n",
+			printf("dxr3_vo: failed to set aspect ratio (%s)\n",
 			 strerror(errno));
 		
 		if (this->overlay_enabled && !fullscreen){
 			int foo;
-			this->request_dest_size(this->user_data, this->width,
-			 this->width/this->desired_ratio, &foo, &foo, &foo, &foo);
+			this->frame_output_cb(this->user_data, this->width,
+			 this->width/this->desired_ratio, &foo, &foo, &foo, &foo, &foo, &foo);
 		}
 		break;
 	case VO_PROP_COLORKEY:
-		fprintf(stderr, "dxr3_vo: VO_PROP_COLORKEY not implemented!");
+		printf("dxr3_vo: VO_PROP_COLORKEY not implemented!");
 		this->overlay.colorkey = val;
 		break;
-	case VO_PROP_ZOOM_X: 
+	case VO_PROP_ZOOM_FACTOR:   /* FIXME: Was ZOOM_X */
 		if(!this->overlay_enabled){  /* TV-out only */
 		  if(value==1){
-			fprintf(stderr, "dxr3_vo: enabling 16:9 zoom\n");
+			printf("dxr3_vo: enabling 16:9 zoom\n");
 			val=EM8300_ASPECTRATIO_4_3;
 			if (ioctl(this->fd_control, EM8300_IOCTL_SET_ASPECTRATIO, &val))
-				fprintf(stderr, "dxr3_vo: failed to set aspect ratio (%s)\n",
+				printf("dxr3_vo: failed to set aspect ratio (%s)\n",
 				 strerror(errno));
 			dxr3_zoomTV(this);
 		  }else if (value==-1){
-			fprintf(stderr, "dxr3_vo: disabling 16:9 zoom\n");		
+			printf("dxr3_vo: disabling 16:9 zoom\n");		
 			if (ioctl(this->fd_control, EM8300_IOCTL_SET_ASPECTRATIO, &this->aspectratio))
-				fprintf(stderr, "dxr3_vo: failed to set aspect ratio (%s)\n",
+				printf("dxr3_vo: failed to set aspect ratio (%s)\n",
 				 strerror(errno));
 		  }
 		}
@@ -338,15 +363,15 @@ int dxr3_set_property (vo_driver_t *this_gen,
 		  	newmode++;
 		  	if (newmode>EM8300_VIDEOMODE_LAST)
 		  		newmode=EM8300_VIDEOMODE_PAL;
-				fprintf(stderr, "dxr3_vo: Changing TVMode to ");
+				printf("dxr3_vo: Changing TVMode to ");
 				if(newmode==EM8300_VIDEOMODE_PAL)
-					fprintf(stderr, "PAL\n");
+					printf("PAL\n");
 				if(newmode==EM8300_VIDEOMODE_PAL60)
-					fprintf(stderr, "PAL60\n");
+					printf("PAL60\n");
 				if(newmode==EM8300_VIDEOMODE_NTSC)
-					fprintf(stderr, "NTSC\n");
+					printf("NTSC\n");
 		  	if (ioctl(this->fd_control, EM8300_IOCTL_SET_VIDEOMODE, &newmode))
-			fprintf(stderr, "dxr3_vo: setting video mode failed.");
+			printf("dxr3_vo: setting video mode failed.");
 		}
 		break;
 	default:
@@ -355,7 +380,7 @@ int dxr3_set_property (vo_driver_t *this_gen,
 
 	if (bcs_changed){
 		if (ioctl(this->fd_control, EM8300_IOCTL_SETBCS, &this->bcs))
-			fprintf(stderr, "dxr3_vo: bcs set failed (%s)\n",
+			printf("dxr3_vo: bcs set failed (%s)\n",
 			 strerror(errno));
 		this->config->update_num(this->config, "dxr3.contrast", this->bcs.contrast);
 		this->config->update_num(this->config, "dxr3.saturation", this->bcs.saturation);
@@ -403,6 +428,12 @@ int dxr3_gui_data_exchange (vo_driver_t *this_gen,
 	if (!this->overlay_enabled && !this->tv_switchable) return 0;
 
 	switch (data_type) {
+/*************************************************************************
+ *  FIXME: Removed due to changes in XV by Guenter, but don't know what to replace it with
+ *	Update 2/4/02 - Mike Lampard:
+ *	  This functionality is now incorporated into dxr3_overlay_update until
+ *        a cleaner way can be found to update window changes.
+	  
 	case GUI_DATA_EX_DEST_POS_SIZE_CHANGED:{
 			x11_rectangle_t *area = (x11_rectangle_t*) data;
 			dxr3_overlay_adapt_area(this, area->x, area->y, area->w, area->h);
@@ -413,11 +444,13 @@ int dxr3_gui_data_exchange (vo_driver_t *this_gen,
 				dxr3_overlay_set_mode(&this->overlay,EM8300_OVERLAY_MODE_OVERLAY);
 		}
 		break;
+*************************************************************************/
 	case GUI_DATA_EX_EXPOSE_EVENT:{
 			XLockDisplay(this->display);
 			XFillRectangle(this->display, this->win,
 				 this->gc, 0, 0, this->width, this->height);
 			XUnlockDisplay(this->display);
+			dxr3_overlay_update(this);
 		}
 		break;
 	case GUI_DATA_EX_DRAWABLE_CHANGED:{
@@ -447,11 +480,11 @@ int dxr3_gui_data_exchange (vo_driver_t *this_gen,
 			int window_showing;
 			(int *)window_showing = (int *)data;
 			if(!window_showing){
-				fprintf(stderr, "dxr3_vo: Hiding VO window and diverting video to TV\n");
+				printf("dxr3_vo: Hiding VO window and diverting video to TV\n");
 				dxr3_overlay_set_mode(&this->overlay, EM8300_OVERLAY_MODE_OFF );
 				this->overlay_enabled=0;
 			}else{
-				fprintf(stderr, "dxr3_vo: Using VO window for overlaying video\n");
+				printf("dxr3_vo: Using VO window for overlaying video\n");
 				dxr3_overlay_set_mode(&this->overlay, EM8300_OVERLAY_MODE_OVERLAY );			
 				this->overlay_enabled=1;
 			}
@@ -498,7 +531,8 @@ void gather_screen_vars(dxr3_driver_t *this, x11_visual_t *vis)
 	}
 
 	this->overlay.screen_depth = DisplayPlanes(this->display, scrn);
-	this->request_dest_size = (void *)vis->request_dest_size;
+	this->frame_output_cb = (void *)vis->frame_output_cb;
+/* request_dest_size; */
 	printf("xres %d yres %d depth %d\n", this->overlay.screen_xres, this->overlay.screen_yres, this->overlay.screen_depth);
 }
 
