@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_cda.c,v 1.16 2002/01/15 13:20:59 jkeil Exp $
+ * $Id: input_cda.c,v 1.22 2002/04/24 08:12:13 f1rmb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -93,20 +93,20 @@ extern int errno;
 
 #ifdef __GNUC__
 #define LOG_MSG_STDERR(xine, message, args...) {                     \
-    xine_log(xine, XINE_LOG_INPUT, message, ##args);                 \
+    xine_log(xine, XINE_LOG_MSG, message, ##args);                 \
     fprintf(stderr, message, ##args);                                \
   }
 #define LOG_MSG(xine, message, args...) {                            \
-    xine_log(xine, XINE_LOG_INPUT, message, ##args);                 \
+    xine_log(xine, XINE_LOG_MSG, message, ##args);                 \
     printf(message, ##args);                                         \
   }
 #else
 #define LOG_MSG_STDERR(xine, ...) {                                  \
-    xine_log(xine, XINE_LOG_INPUT, __VA_ARGS__);                     \
+    xine_log(xine, XINE_LOG_MSG, __VA_ARGS__);                     \
     fprintf(stderr, __VA_ARGS__);                                    \
   }
 #define LOG_MSG(xine, ...) {                                         \
-    xine_log(xine, XINE_LOG_INPUT, __VA_ARGS__);                     \
+    xine_log(xine, XINE_LOG_MSG, __VA_ARGS__);                     \
     printf(__VA_ARGS__);                                             \
   }
 #endif
@@ -268,36 +268,6 @@ static void _cda_mkdir_recursive_safe(char *path) {
   }
 }
 
-/*
- * Return user name.
- */
-static char *_cda_get_username_safe(void) {
-  char          *un;
-  struct passwd *pw;
-
-  pw = getpwuid(geteuid());
-
-  un = strdup(((pw && pw->pw_name) ? strdup(pw->pw_name) : "unknown"));
-
-  return un;
-}
-
-/*
- * Return host name.
- */
-static char *_cda_get_hostname_safe(void) {
-  char *hn;
-  char  buf[2048];
-
-  memset(&buf, 0, sizeof(buf));
-  if(gethostname(&buf[0], sizeof(buf)) == 0)
-    hn = strdup(buf);
-  else
-    hn = strdup("unknown");
-  
-  return hn;
-}
-
 /*
  * ************* CDDB *********************
  */
@@ -590,7 +560,6 @@ static void _cda_cddb_socket_close(cda_input_plugin_t *this) {
  */
 static void _cda_cddb_retrieve(cda_input_plugin_t *this) {
   char  buffer[2048];
-  char *username, *hostname;
   int   err, i;
 
   if(this == NULL)
@@ -623,9 +592,6 @@ static void _cda_cddb_retrieve(cda_input_plugin_t *this) {
       return;
     }
     
-    username = _cda_get_username_safe();
-    hostname = _cda_get_hostname_safe();
-    
     memset(&buffer, 0, sizeof(buffer));
     
     /* Get welcome message */
@@ -633,7 +599,11 @@ static void _cda_cddb_retrieve(cda_input_plugin_t *this) {
       if((err = _cda_cddb_handle_code(buffer)) >= 0) {
 	/* send hello */
 	memset(&buffer, 0, sizeof(buffer));
-	sprintf(buffer, "cddb hello %s %s xine %s\n", username, hostname, VERSION);
+        /* We don't send current user/host name to prevent spam.
+         * Software that sends this is considered spyware
+         * that most people don't like.
+         */
+	sprintf(buffer, "cddb hello unknown localhost xine %s\n", VERSION);
 	if((err = _cda_cddb_send_command(this, buffer)) > 0) {
 	  /* Get answer from hello */
 	  memset(&buffer, 0, sizeof(buffer));
@@ -712,8 +682,6 @@ static void _cda_cddb_retrieve(cda_input_plugin_t *this) {
       }
     }
     _cda_cddb_socket_close(this);
-    free(username);
-    free(hostname);
   }
  
 }
@@ -1578,7 +1546,7 @@ static uint32_t cda_plugin_get_capabilities (input_plugin_t *this_gen) {
   _ENTER_FUNC();
   _LEAVE_FUNC();
   
-  return INPUT_CAP_SEEKABLE | INPUT_CAP_AUTOPLAY | INPUT_CAP_GET_DIR;
+  return INPUT_CAP_SEEKABLE | INPUT_CAP_PREVIEW | INPUT_CAP_AUTOPLAY | INPUT_CAP_GET_DIR;
 }
 
 /*
@@ -1800,6 +1768,19 @@ static int cda_plugin_get_optional_data (input_plugin_t *this_gen, void *data, i
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
+static void cda_plugin_dispose (input_plugin_t *this_gen ) {
+  cda_input_plugin_t *this = (cda_input_plugin_t *) this_gen;
+  int i;
+
+  for (i = 0; i < 100; i++)
+    free (this->filelist[i]);
+
+  free (this->cda->device_name);
+  free (this->cda);
+  free (this->mrls);
+  free (this);
+}
+
 /*
  * Initialize plugin.
  */
@@ -1810,7 +1791,7 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
 
   _ENTER_FUNC();
 
-  if (iface != 5) {
+  if (iface != 6) {
     LOG_MSG(xine,
 	    _("cda input plugin doesn't support plugin API version %d.\n"
 	      "PLUGIN DISABLED.\n"
@@ -1845,6 +1826,7 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
   this->input_plugin.get_mrl            = cda_plugin_get_mrl;
   this->input_plugin.get_autoplay_list  = cda_plugin_get_autoplay_list;
   this->input_plugin.get_optional_data  = cda_plugin_get_optional_data;
+  this->input_plugin.dispose            = cda_plugin_dispose;
   this->input_plugin.is_branch_possible = NULL;
   
   this->xine           = xine;
