@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_dvd.c,v 1.42 2002/01/02 18:16:07 jkeil Exp $
+ * $Id: input_dvd.c,v 1.47 2002/04/24 08:12:13 f1rmb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -56,20 +56,20 @@ extern int errno;
 
 #ifdef __GNUC__
 #define LOG_MSG_STDERR(xine, message, args...) {                     \
-    xine_log(xine, XINE_LOG_INPUT, message, ##args);                 \
+    xine_log(xine, XINE_LOG_MSG, message, ##args);                 \
     fprintf(stderr, message, ##args);                                \
   }
 #define LOG_MSG(xine, message, args...) {                            \
-    xine_log(xine, XINE_LOG_INPUT, message, ##args);                 \
+    xine_log(xine, XINE_LOG_MSG, message, ##args);                 \
     printf(message, ##args);                                         \
   }
 #else
 #define LOG_MSG_STDERR(xine, ...) {                                  \
-    xine_log(xine, XINE_LOG_INPUT, __VA_ARGS__);                     \
+    xine_log(xine, XINE_LOG_MSG, __VA_ARGS__);                     \
     fprintf(stderr, __VA_ARGS__);                                    \
   }
 #define LOG_MSG(xine, ...) {                                         \
-    xine_log(xine, XINE_LOG_INPUT, __VA_ARGS__);                     \
+    xine_log(xine, XINE_LOG_MSG, __VA_ARGS__);                     \
     printf(__VA_ARGS__);                                             \
   }
 #endif
@@ -370,7 +370,7 @@ static int openDVDFile (dvd_input_plugin_t *this,
  *
  */
 static uint32_t dvd_plugin_get_capabilities (input_plugin_t *this) {
-  return INPUT_CAP_SEEKABLE | INPUT_CAP_BLOCK | INPUT_CAP_AUTOPLAY | INPUT_CAP_GET_DIR;
+  return INPUT_CAP_SEEKABLE | INPUT_CAP_PREVIEW | INPUT_CAP_BLOCK | INPUT_CAP_AUTOPLAY | INPUT_CAP_GET_DIR;
 }
 
 /*
@@ -786,6 +786,21 @@ static int dvd_plugin_get_optional_data (input_plugin_t *this_gen,
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
+static void dvd_plugin_dispose (input_plugin_t *this_gen ) {
+  dvd_input_plugin_t *this = (dvd_input_plugin_t *) this_gen;
+  int i;
+
+  read_cache_free (this->read_cache);
+
+  for (i = 0; i < MAX_DIR_ENTRIES; i++) {
+    free (this->filelist[i]);
+    free (this->filelist2[i]);
+  }
+
+  free (this->mrls);
+  free (this);
+}
+
 
 input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
 
@@ -793,7 +808,7 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
   config_values_t    *config;
   int i;
 
-  if (iface != 5) {
+  if (iface != 6) {
     LOG_MSG(xine,
 	    _("dvd input plugin doesn't support plugin API version %d.\n"
 	      "PLUGIN DISABLED.\n"
@@ -830,6 +845,7 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
   this->input_plugin.get_mrl           = dvd_plugin_get_mrl;
   this->input_plugin.get_autoplay_list = dvd_plugin_get_autoplay_list;
   this->input_plugin.get_optional_data = dvd_plugin_get_optional_data;
+  this->input_plugin.dispose           = dvd_plugin_dispose;
   this->input_plugin.is_branch_possible= NULL;
   /* disable branch until we fix the problems branching from 
      menu vob to video vob
