@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2000 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_file.c,v 1.37 2002/01/02 18:16:07 jkeil Exp $
+ * $Id: input_file.c,v 1.47 2002/04/24 08:12:13 f1rmb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,20 +45,20 @@ extern int errno;
 
 #ifdef __GNUC__
 #define LOG_MSG_STDERR(xine, message, args...) {                     \
-    xine_log(xine, XINE_LOG_INPUT, message, ##args);                 \
+    xine_log(xine, XINE_LOG_MSG, message, ##args);                 \
     fprintf(stderr, message, ##args);                                \
   }
 #define LOG_MSG(xine, message, args...) {                            \
-    xine_log(xine, XINE_LOG_INPUT, message, ##args);                 \
+    xine_log(xine, XINE_LOG_MSG, message, ##args);                 \
     printf(message, ##args);                                         \
   }
 #else
 #define LOG_MSG_STDERR(xine, ...) {                                  \
-    xine_log(xine, XINE_LOG_INPUT, __VA_ARGS__);                     \
+    xine_log(xine, XINE_LOG_MSG, __VA_ARGS__);                     \
     fprintf(stderr, __VA_ARGS__);                                    \
   }
 #define LOG_MSG(xine, ...) {                                         \
-    xine_log(xine, XINE_LOG_INPUT, __VA_ARGS__);                     \
+    xine_log(xine, XINE_LOG_MSG, __VA_ARGS__);                     \
     printf(__VA_ARGS__);                                             \
   }
 #endif
@@ -266,7 +266,7 @@ static off_t get_file_size(char *filepathname, char *origin) {
  */
 static uint32_t file_plugin_get_capabilities (input_plugin_t *this_gen) {
 
-  return INPUT_CAP_SEEKABLE | INPUT_CAP_GET_DIR;
+  return INPUT_CAP_SEEKABLE | INPUT_CAP_PREVIEW | INPUT_CAP_GET_DIR | INPUT_CAP_SPULANG;
 }
 
 /*
@@ -277,14 +277,17 @@ static int file_plugin_open (input_plugin_t *this_gen, char *mrl) {
   char                *filename, *subtitle;
   file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
 
-  this->mrl = strdup(mrl); /* FIXME: small memory leak */ 
+  if (this->mrl)
+    free (this->mrl);
+
+  this->mrl = strdup(mrl);
 
   if (!strncasecmp (this->mrl, "file:",5))
     filename = &this->mrl[5];
   else
     filename = this->mrl;
 
-  subtitle = strrchr (filename, ':');
+  subtitle = strrchr (filename, '%');
   if (subtitle) {
     *subtitle = 0;
     subtitle++;
@@ -379,6 +382,9 @@ static off_t file_plugin_seek (input_plugin_t *this_gen, off_t offset, int origi
 static off_t file_plugin_get_current_pos (input_plugin_t *this_gen){
   file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
 
+  if (this->fh <0)
+    return 0;
+
   return lseek (this->fh, 0, SEEK_CUR);
 }
 
@@ -390,6 +396,9 @@ static off_t file_plugin_get_length (input_plugin_t *this_gen) {
   struct stat          buf ;
   file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
 
+  if (this->fh <0)
+    return 0;
+
   if (fstat (this->fh, &buf) == 0) {
     return buf.st_size;
   } else
@@ -432,6 +441,7 @@ static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen,
   int                   num_norm_files  = 0;
   int                   num_files       = -1;
   int                 (*func) ()        = _sortfiles_default;
+  int                   already_tried   = 0;
 
   *nFiles = 0;
   memset(current_dir, 0, sizeof(current_dir));
@@ -452,6 +462,8 @@ static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen,
   }
 
   /* Store new origin path */
+ __try_again_from_home:
+  
   this->config->update_string(this->config, "input.file_origin_path", current_dir);
 
   if(strcasecmp(current_dir, "/"))
@@ -462,8 +474,17 @@ static mrl_t **file_plugin_get_dir (input_plugin_t *this_gen,
   /*
    * Ooch!
    */
-  if((pdir = opendir(current_dir)) == NULL)
+  if((pdir = opendir(current_dir)) == NULL) {
+
+    if(!already_tried) {
+      /* Try one more time with user homedir */
+      snprintf(current_dir, XINE_PATH_MAX, "%s", xine_get_homedir());
+      already_tried++;
+      goto __try_again_from_home;
+    }
+
     return NULL;
+  }
   
   dir_files  = (mrl_t *) xine_xmalloc(sizeof(mrl_t) * MAXFILES);
   hide_files = (mrl_t *) xine_xmalloc(sizeof(mrl_t) * MAXFILES);
@@ -788,22 +809,43 @@ static int file_plugin_get_optional_data (input_plugin_t *this_gen,
 	  data_type, this->sub);
 
 
-  if ( (data_type == INPUT_OPTIONAL_DATA_TEXTSPU0)
-       && this->sub ) {
-
-    FILE **tmp;
-
-    /* dirty hacks... */
-
-    tmp = data;
-    *tmp = this->sub;
-
+  switch(data_type) {
+  case INPUT_OPTIONAL_DATA_TEXTSPU0:
+    if(this->sub) {
+      FILE **tmp;
+      
+      /* dirty hacks... */
+      tmp = data;
+      *tmp = this->sub;
+      
+      return INPUT_OPTIONAL_SUCCESS;
+    }
+    break;
+    
+  case INPUT_OPTIONAL_DATA_SPULANG:
+    sprintf(data, "%3s", (this->sub) ? "on" : "off");
     return INPUT_OPTIONAL_SUCCESS;
+    break;
+    
+  default:
+    return INPUT_OPTIONAL_UNSUPPORTED;
+    break;
+
   }
 
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
+static void file_plugin_dispose (input_plugin_t *this_gen ) {
+  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+
+  if (this->mrl)
+    free (this->mrl);
+
+  free (this->mrls);
+  free (this);
+}
+
 /*
  *
  */
@@ -812,7 +854,7 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
   file_input_plugin_t *this;
   config_values_t     *config;
 
-  if (iface != 5) {
+  if (iface != 6) {
     LOG_MSG(xine,
 	    _("file input plugin doesn't support plugin API version %d.\n"
 	      "PLUGIN DISABLED.\n"
@@ -844,6 +886,7 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
   this->input_plugin.get_identifier     = file_plugin_get_identifier;
   this->input_plugin.get_autoplay_list  = NULL;
   this->input_plugin.get_optional_data  = file_plugin_get_optional_data;
+  this->input_plugin.dispose            = file_plugin_dispose;
   this->input_plugin.is_branch_possible = NULL;
 
   this->fh                     = -1;
