@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,29 +39,11 @@
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "input_plugin.h"
+#include "net_buf_ctrl.h"
 
 extern int errno;
 
-#ifdef __GNUC__
-#define LOG_MSG_STDERR(xine, message, args...) {                     \
-    xine_log(xine, XINE_LOG_INPUT, message, ##args);                 \
-    fprintf(stderr, message, ##args);                                \
-  }
-#define LOG_MSG(xine, message, args...) {                            \
-    xine_log(xine, XINE_LOG_INPUT, message, ##args);                 \
-    printf(message, ##args);                                         \
-  }
-#else
-#define LOG_MSG_STDERR(xine, ...) {                                  \
-    xine_log(xine, XINE_LOG_INPUT, __VA_ARGS__);                     \
-    fprintf(stderr, __VA_ARGS__);                                    \
-  }
-#define LOG_MSG(xine, ...) {                                         \
-    xine_log(xine, XINE_LOG_INPUT, __VA_ARGS__);                     \
-    printf(__VA_ARGS__);                                             \
-  }
-#endif
-
+#define PREVIEW_SIZE            2200
 #define BUFSIZE 1024
 
 #define DEFAULT_HTTP_PORT 80
@@ -75,6 +57,8 @@ typedef struct {
   char            *mrl;
   config_values_t *config;
 
+  nbc_t           *nbc; 
+
   off_t            curpos;
   off_t            contentlength;
     
@@ -95,6 +79,11 @@ typedef struct {
   char            *proxypassword;
   char            *proxyhost;
   int              proxyport;
+
+  char             preview[PREVIEW_SIZE];
+  off_t            preview_size;
+  off_t            preview_pos;
+
 } http_input_plugin_t;
 
 
@@ -106,7 +95,7 @@ static int http_plugin_host_connect_attempt(struct in_addr ia, int port, xine_t
   s=socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
 
   if (s==-1) {
-    LOG_MSG(xine, _("input_http: failed to open socket\n"));
+    xine_log (xine, XINE_LOG_MSG, _("input_http: failed to open socket\n"));
     return -1;
   }
 
@@ -115,7 +104,7 @@ static int http_plugin_host_connect_attempt(struct in_addr ia, int port, xine_t
   sin.sin_port   = htons(port);
 	
   if (connect(s, (struct sockaddr *)&sin, sizeof(sin))==-1 && errno != EINPROGRESS) {
-    LOG_MSG(xine, _("input_http: cannot connect to host\n"));
+    xine_log (xine, XINE_LOG_MSG, _("input_http: cannot connect to host\n"));
     close(s);
     return -1;
   }	
@@ -130,7 +119,7 @@ static int http_plugin_host_connect(const char *host, int port, xine_t *xine) {
 	
   h=gethostbyname(host);
   if (h==NULL) {
-    LOG_MSG(xine, _("input_http: unable to resolve >%s<\n"), host);
+    xine_log (xine, XINE_LOG_MSG, _("input_http: unable to resolve >%s<\n"), host);
     return -1;
   }
 	
@@ -142,7 +131,7 @@ static int http_plugin_host_connect(const char *host, int port, xine_t *xine) {
       return s;
   }
 
-  LOG_MSG(xine, _("http: unable to connect to >%s<\n"), host);
+  xine_log (xine, XINE_LOG_MSG, _("http: unable to connect to >%s<\n"), host);
   return -1;
 }
 
@@ -293,6 +282,9 @@ static int http_plugin_basicauth (const char *user, const char *password,
   return 0;
 }
 
+static off_t http_plugin_read (input_plugin_t *this_gen, 
+			       char *buf, off_t nlen) ;
+
 static int http_plugin_open (input_plugin_t *this_gen, char *mrl) {
 
   http_input_plugin_t *this = (http_input_plugin_t *) this_gen;
@@ -346,7 +338,9 @@ static int http_plugin_open (input_plugin_t *this_gen, char *mrl) {
     
     sprintf(buf, "%s\n", buf);
 
-    LOG_MSG(this->xine, buf);
+#ifdef LOG
+    printf (buf);
+#endif
   }
   
   if (proxy != NULL)
@@ -397,8 +391,10 @@ static int http_plugin_open (input_plugin_t *this_gen, char *mrl) {
     return 0 ;
   }
 
+#ifdef LOG
   printf ("input_http: request sent: >%s<\n",
 	  this->buf);
+#endif
 
   /* read and parse reply */
   done = 0; len = 0; linenum = 0;
@@ -414,10 +410,10 @@ static int http_plugin_open (input_plugin_t *this_gen, char *mrl) {
       
       switch (errno) {
       case EAGAIN:
-	LOG_MSG(this->xine, _("input_http: EAGAIN\n"));
+	xine_log (this->xine, XINE_LOG_MSG, _("input_http: EAGAIN\n"));
 	continue;
       default:
-	LOG_MSG(this->xine, _("input_http: read error\n"));
+	xine_log (this->xine, XINE_LOG_MSG, _("input_http: read error\n"));
 	return 0;
       }
     }
@@ -434,7 +430,9 @@ static int http_plugin_open (input_plugin_t *this_gen, char *mrl) {
 
       linenum++;
       
-      LOG_MSG(this->xine, _("input_http: answer: >%s<\n"), this->buf);
+#ifdef LOG
+      printf ("input_http: answer: >%s<\n", this->buf);
+#endif
 
       if (linenum == 1)
       {
@@ -444,17 +442,17 @@ static int http_plugin_open (input_plugin_t *this_gen, char *mrl) {
 	if (sscanf(this->buf, "HTTP/%d.%d %d %[^\015\012]", &httpver, &httpsub,
 		   &httpcode, httpstatus) != 4)
 	{
-      	  LOG_MSG(this->xine, _("input_http: invalid http answer\n"));
+      	  xine_log (this->xine, XINE_LOG_MSG, _("input_http: invalid http answer\n"));
 	  return 0;
 	}
 	
 	if (httpcode >= 300 && httpcode < 400) {
-      	  LOG_MSG(this->xine, _("input_http: 3xx redirection not implemented: >%d %s<\n"),
+      	  xine_log (this->xine, XINE_LOG_MSG, _("input_http: 3xx redirection not implemented: >%d %s<\n"),
 		  httpcode, httpstatus);
 	  return 0;
 	}
 	if (httpcode < 200 || httpcode >= 300) {
-      	  LOG_MSG(this->xine, _("input_http: http status not 2xx: >%d %s<\n"),
+      	  xine_log (this->xine, XINE_LOG_MSG, _("input_http: http status not 2xx: >%d %s<\n"),
 		  httpcode, httpstatus);
 	  return 0;
 	}
@@ -463,14 +461,14 @@ static int http_plugin_open (input_plugin_t *this_gen, char *mrl) {
 	  off_t contentlength;
 	  
 	  if (sscanf(this->buf, "Content-Length: %Ld", &contentlength) == 1) {
-      	    LOG_MSG(this->xine, _("input_http: content length = %Ld bytes\n"), contentlength);
+      	    xine_log (this->xine, XINE_LOG_MSG, _("input_http: content length = %Ld bytes\n"), contentlength);
 	    this->contentlength = contentlength;
 	  }
         }
 	
 	if (!strncasecmp(this->buf, "Location: ", 10))
 	{
-      	  LOG_MSG(this->xine, _("input_http: Location redirection not implemented\n"));
+      	  xine_log (this->xine, XINE_LOG_MSG, _("input_http: Location redirection not implemented\n"));
 	  return 0;
 	}
       }
@@ -483,7 +481,19 @@ static int http_plugin_open (input_plugin_t *this_gen, char *mrl) {
       len ++;
   }
 
-  LOG_MSG(this->xine, _("input_http: end of headers\n"));
+#ifdef LOG
+  printf ("input_http: end of headers\n");
+#endif
+
+  this->nbc    = nbc_init (this->xine);
+
+  /*
+   * fill preview buffer
+   */
+
+  this->preview_size = http_plugin_read (&this->input_plugin, this->preview,
+					 PREVIEW_SIZE);
+  this->preview_pos  = 0;
 
   return 1;
 }
@@ -493,20 +503,38 @@ static off_t http_plugin_read (input_plugin_t *this_gen,
   http_input_plugin_t *this = (http_input_plugin_t *) this_gen;
   off_t n, num_bytes;
 
+  nbc_check_buffers (this->nbc);
+
   num_bytes = 0;
 
   while (num_bytes < nlen) {
 
-    n = read (this->fh, &buf[num_bytes], nlen - num_bytes);
+    if (this->preview_pos < this->preview_size) {
+
+      n = this->preview_size - this->preview_pos;
+      if (n > (nlen - num_bytes)) 
+	n = nlen - num_bytes;
+
+#ifdef LOG
+      printf ("stdin: %lld bytes from preview (which has %lld bytes)\n",
+	      n, this->preview_size);
+#endif
+
+      memcpy (&buf[num_bytes], &this->preview[this->preview_pos], n);
+
+      this->preview_pos += n;
+
+    } else
+      n = read (this->fh, &buf[num_bytes], nlen - num_bytes);
 
     if (n <= 0) {
       
       switch (errno) {
       case EAGAIN:
-	LOG_MSG(this->xine, _("input_http: EAGAIN\n"));
+	xine_log (this->xine, XINE_LOG_MSG, _("input_http: EAGAIN\n"));
 	continue;
       default:
-	LOG_MSG(this->xine, _("input_http: read error\n"));
+	xine_log (this->xine, XINE_LOG_MSG, _("input_http: read error\n"));
 	return 0;
       }
     }
@@ -529,28 +557,23 @@ static void pool_release_buffer (void *arg) {
 
 static buf_element_t *http_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, off_t todo) {
 
-  off_t                 num_bytes, total_bytes;
+  off_t                 total_bytes;
   http_input_plugin_t  *this = (http_input_plugin_t *) this_gen;
   buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
 
+  nbc_check_buffers (this->nbc);
+
   pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,NULL);
   pthread_cleanup_push( pool_release_buffer, buf );
 
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
-  total_bytes = 0;
 
-  while (total_bytes < todo) {
-    pthread_testcancel();
-    num_bytes = read (this->fh, buf->mem + total_bytes, todo-total_bytes);
-    if (num_bytes < 0) {
-      LOG_MSG(this->xine, _("input_http: read error (%s)\n"), strerror (errno));
-      buf->free_buffer (buf);
-      buf = NULL;
-      break;
-    }
-    total_bytes += num_bytes;
-    this->curpos += num_bytes;
+  total_bytes = http_plugin_read (this_gen, buf->content, todo);
+
+  if (total_bytes != todo) {
+    buf->free_buffer (buf);
+    buf = NULL;
   }
 
   if (buf != NULL)
@@ -593,6 +616,11 @@ static void http_plugin_close (input_plugin_t *this_gen) {
 
   close(this->fh);
   this->fh = -1;
+
+  if (this->nbc) {
+    nbc_close (this->nbc);
+    this->nbc = NULL;
+  }
 }
 
 static void http_plugin_stop (input_plugin_t *this_gen) {
@@ -617,16 +645,31 @@ static char* http_plugin_get_mrl (input_plugin_t *this_gen) {
 static int http_plugin_get_optional_data (input_plugin_t *this_gen, 
 					  void *data, int data_type) {
 
+  http_input_plugin_t *this = (http_input_plugin_t *) this_gen;
+
+  switch (data_type) {
+  case INPUT_OPTIONAL_DATA_PREVIEW:
+
+    memcpy (data, this->preview, this->preview_size);
+    return this->preview_size;
+
+    break;
+  }
+
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
+static void http_plugin_dispose (input_plugin_t *this_gen ) {
+  free (this_gen);
+}
+
 input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
 
   http_input_plugin_t *this;
   config_values_t    *config;
 
-  if (iface != 5) {
-    LOG_MSG(xine,
+  if (iface != 6) {
+    xine_log (xine, XINE_LOG_PLUGIN,
 	    _("http input plugin doesn't support plugin API version %d.\n"
 	      "PLUGIN DISABLED.\n"
 	      "This means there's a version mismatch between xine and this input"
@@ -657,11 +700,13 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
   this->input_plugin.get_identifier    = http_plugin_get_identifier;
   this->input_plugin.get_autoplay_list = NULL;
   this->input_plugin.get_optional_data = http_plugin_get_optional_data;
+  this->input_plugin.dispose           = http_plugin_dispose;
   this->input_plugin.is_branch_possible= NULL;
 
   this->fh      = -1;
   this->config  = config;
   this->curpos  = 0;
+  this->nbc     = NULL;
   
   return (input_plugin_t *) this;
 }
