@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_vcd.c,v 1.35 2002/01/02 18:16:07 jkeil Exp $
+ * $Id: input_vcd.c,v 1.41 2002/04/24 08:12:13 f1rmb Exp $
  *
  */
 
@@ -55,20 +55,20 @@ extern int errno;
 
 #ifdef __GNUC__
 #define LOG_MSG_STDERR(xine, message, args...) {                     \
-    xine_log(xine, XINE_LOG_INPUT, message, ##args);                 \
+    xine_log(xine, XINE_LOG_MSG, message, ##args);                 \
     fprintf(stderr, message, ##args);                                \
   }
 #define LOG_MSG(xine, message, args...) {                            \
-    xine_log(xine, XINE_LOG_INPUT, message, ##args);                 \
+    xine_log(xine, XINE_LOG_MSG, message, ##args);                 \
     printf(message, ##args);                                         \
   }
 #else
 #define LOG_MSG_STDERR(xine, ...) {                                  \
-    xine_log(xine, XINE_LOG_INPUT, __VA_ARGS__);                     \
+    xine_log(xine, XINE_LOG_MSG, __VA_ARGS__);                     \
     fprintf(stderr, __VA_ARGS__);                                    \
   }
 #define LOG_MSG(xine, ...) {                                         \
-    xine_log(xine, XINE_LOG_INPUT, __VA_ARGS__);                     \
+    xine_log(xine, XINE_LOG_MSG, __VA_ARGS__);                     \
     printf(__VA_ARGS__);                                             \
   }
 #endif
@@ -859,7 +859,7 @@ static off_t vcd_plugin_get_current_pos (input_plugin_t *this_gen){
  */
 static uint32_t vcd_plugin_get_capabilities (input_plugin_t *this_gen) {
   
-  return INPUT_CAP_SEEKABLE | INPUT_CAP_BLOCK | INPUT_CAP_AUTOPLAY | INPUT_CAP_GET_DIR;
+  return INPUT_CAP_SEEKABLE | INPUT_CAP_PREVIEW | INPUT_CAP_BLOCK | INPUT_CAP_AUTOPLAY | INPUT_CAP_GET_DIR;
 }
 
 /*
@@ -960,7 +960,7 @@ static void vcd_plugin_stop (input_plugin_t *this_gen) {
  *
  */
 static char *vcd_plugin_get_description (input_plugin_t *this_gen) {
-  return _("plain file input plugin as shipped with xine");
+  return _("vcd device input plugin as shipped with xine");
 }
 
 /*
@@ -1121,6 +1121,17 @@ static int vcd_plugin_get_optional_data (input_plugin_t *this_gen,
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
+static void vcd_plugin_dispose (input_plugin_t *this_gen ) {
+  vcd_input_plugin_t *this = (vcd_input_plugin_t *) this_gen;
+  int i;
+
+  for (i = 0; i < 100; i++)
+    free (this->filelist[i]);
+
+  free (this->mrls);
+  free (this);
+}
+
 /*
  *
  */
@@ -1130,7 +1141,7 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
   config_values_t    *config;
   int                 i;
 
-  if (iface != 5) {
+  if (iface != 6) {
     LOG_MSG(xine, 
 	    _("vcd input plugin doesn't support plugin API version %d.\n"
 	      "PLUGIN DISABLED.\n"
@@ -1166,6 +1177,7 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
   this->input_plugin.get_mrl           = vcd_plugin_get_mrl;
   this->input_plugin.get_autoplay_list = vcd_plugin_get_autoplay_list;
   this->input_plugin.get_optional_data = vcd_plugin_get_optional_data;
+  this->input_plugin.dispose           = vcd_plugin_dispose;
   this->input_plugin.is_branch_possible= NULL;
   
   this->device = config->register_string(config, "input.vcd_device", CDROM,
