@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.16 2002/01/05 21:54:17 miguelfreitas Exp $
+ * $Id: xine_decoder.c,v 1.25 2002/04/23 15:45:21 esnel Exp $
  *
  * stuff needed to turn liba52 into a xine decoder plugin
  */
@@ -50,8 +50,7 @@ int a52file;
 typedef struct a52dec_decoder_s {
   audio_decoder_t  audio_decoder;
 
-  uint32_t         pts;
-  uint32_t         last_pts;
+  int64_t          pts;
 
   uint8_t          frame_buffer[3840];
   uint8_t         *frame_ptr;
@@ -69,7 +68,7 @@ typedef struct a52dec_decoder_s {
   int              a52_flags_map[11];
   int              ao_flags_map[11];
 
-  sample_t        *samples;
+  sample_t        *samples, *samples_base;
 
   ao_instance_t	  *audio_out;
   int              audio_caps;
@@ -141,8 +140,6 @@ void a52dec_reset (audio_decoder_t *this_gen) {
   this->syncword      = 0;
   this->sync_todo     = 7;
   this->pts           = 0;
-  this->last_pts      = 0;
-
 }
 
 void a52dec_init (audio_decoder_t *this_gen, ao_instance_t *audio_out) {
@@ -156,10 +153,9 @@ void a52dec_init (audio_decoder_t *this_gen, ao_instance_t *audio_out) {
   this->sync_todo     = 7;
   this->output_open   = 0;
   this->pts           = 0;
-  this->last_pts      = 0;
 
   if( !this->samples )
-    this->samples = a52_init (xine_mm_accel());
+    this->samples = a52_init (xine_mm_accel(), &this->samples_base);
 
   /*
    * find out if this driver supports a52 output
@@ -265,7 +261,7 @@ static inline void float_to_int (float * _f, int16_t * s16, int num_channels) {
   }
 }
 
-static void a52dec_decode_frame (a52dec_decoder_t *this, uint32_t pts, uint32_t scr) {
+static void a52dec_decode_frame (a52dec_decoder_t *this, int64_t pts) {
 
   int output_mode = AO_CAP_MODE_STEREO;
 
@@ -335,11 +331,13 @@ static void a52dec_decode_frame (a52dec_decoder_t *this, uint32_t pts, uint32_t
 
     buf = this->audio_out->get_buffer (this->audio_out);
     int_samples = buf->mem;
+    buf->num_frames = 256*6;
 
     for (i = 0; i < 6; i++) {
       if (a52_block (&this->a52_state, this->samples)) {
 	printf ("liba52: a52_block error\n");
-	return; 
+	buf->num_frames = 0;
+	break; 
       }
       
       switch (output_mode) {
@@ -378,14 +376,10 @@ static void a52dec_decode_frame (a52dec_decoder_t *this, uint32_t pts, uint32_t
       
     /*  output decoded samples */
       
-    buf->num_frames = 256*6;
     buf->vpts       = pts;
-    buf->scr        = scr;
 
     this->audio_out->put_buffer (this->audio_out, buf);
 
-    pts = 0;
-    
   } else {
 
     /*
@@ -431,11 +425,9 @@ static void a52dec_decode_frame (a52dec_decoder_t *this, uint32_t pts, uint32_t
 
       buf->num_frames = 1536;
       buf->vpts       = pts;
-      buf->scr        = scr;
       
       this->audio_out->put_buffer (this->audio_out, buf);
       
-      pts = 0;
     }
   }
 }
@@ -447,24 +439,16 @@ void a52dec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
   uint8_t          *end = buf->content + buf->size;
   uint8_t           byte;
   
-  if (buf->decoder_info[0] == 0)
+  if (buf->decoder_flags & BUF_FLAG_PREVIEW)
     return;
   
-  /*
-    printf ("liba52: got buffer, pts =%d, pts - last_pts=%d\n", 
-    buf->PTS, buf->PTS - this->last_pts);
-
-    this->last_pts = buf->PTS;
-  */
-
-  if (buf->PTS) 
-    this->pts = buf->PTS; 
-
+  if (buf->pts) 
+    this->pts = buf->pts; 
   
   while (current != end) {
 
     if ( (this->sync_todo == 0) && (this->frame_todo == 0) ) {
-      a52dec_decode_frame (this, this->pts, buf->SCR);
+      a52dec_decode_frame (this, this->pts);
 #ifdef DEBUG_A52
       write (a52file, this->frame_buffer, this->frame_length);
 #endif
@@ -538,7 +522,7 @@ void a52dec_close (audio_decoder_t *this_gen) {
     this->audio_out->close (this->audio_out);
 
   if( this->samples ) {
-    xine_free_aligned( this->samples );
+    free (this->samples_base);
     this->samples = NULL;
   }
   
@@ -553,12 +537,16 @@ static char *a52dec_get_id(void) {
   return "a/52dec";
 }
 
+static void a52dec_dispose (audio_decoder_t *this_gen) {
+  free (this_gen);
+}
+
 audio_decoder_t *init_audio_decoder_plugin (int iface_version, xine_t *xine) {
 
   a52dec_decoder_t *this ;
   config_values_t *cfg;
 
-  if (iface_version != 5) {
+  if (iface_version != 7) {
     printf( "liba52: plugin doesn't support plugin API version %d.\n"
 	    "liba52: this means there's a version mismatch between xine and this "
 	    "liba52: decoder plugin.\nInstalling current plugins should help.\n",
@@ -577,6 +565,7 @@ audio_decoder_t *init_audio_decoder_plugin (int iface_version, xine_t *xine) {
   this->audio_decoder.reset               = a52dec_reset;
   this->audio_decoder.close               = a52dec_close;
   this->audio_decoder.get_identifier      = a52dec_get_id;
+  this->audio_decoder.dispose             = a52dec_dispose;
   this->audio_decoder.priority            = 2;
   
 
