@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2001 the xine project
+ * Copyright (C) 2001-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.16 2002/01/05 21:54:17 miguelfreitas Exp $
+ * $Id: xine_decoder.c,v 1.31 2002/04/28 10:12:49 esnel Exp $
  *
  * xine decoder plugin using divx4
  *
@@ -60,6 +60,8 @@
 #if CATCH_SIGSEGV
 #include <signal.h>
 
+/* #define LOG */
+
 /* to be able to restore the old handler */
 void (*old_handler)(int);
 
@@ -119,6 +121,7 @@ typedef struct divx4_decoder_s {
      OpenDivx cannot, so the user can set it in .xinerc. If 0, can_handle
      only returns MPEG4, yielding 311 to ffmpeg */
   int		    can_handle_311;
+  int               skipframes;
 } divx4_decoder_t;
 
 #define VIDEOBUFSIZE 128*1024
@@ -206,6 +209,21 @@ static void divx4_set_pp(divx4_decoder_t *this) {
          "divx4: Valid range 0-6, reduce if you get frame drop\n", 
          this->postproc); 
   setpp.postproc_level=this->postproc*10;
+  setpp.deblock_hor_luma = 0;
+  setpp.deblock_ver_luma = 0;
+  setpp.deblock_hor_chr = 0;
+  setpp.deblock_ver_chr = 0;
+  setpp.dering_luma = 0;
+  setpp.dering_chr = 0;
+  setpp.pp_semaphore = 0;
+
+  setpp.deblock_hor_luma = 0;
+  setpp.deblock_ver_luma = 0;
+  setpp.deblock_hor_chr = 0;
+  setpp.deblock_ver_chr = 0;
+  setpp.dering_luma = 0;
+  setpp.dering_chr = 0;
+  setpp.pp_semaphore = 0;
 
   ret = this->decore((unsigned long)this, DEC_OPT_SETPP, &setpp, 0);
   if (ret != DEC_OK)
@@ -233,21 +251,36 @@ static int divx4_init_decoder(divx4_decoder_t *this, buf_element_t *buf)
   switch (buf->type & 0xFFFF0000) {
   case BUF_VIDEO_MSMPEG4_V12:
   case BUF_VIDEO_MSMPEG4_V3:
-    this->use_311_compat = 1;
+    if (this->version >= 20020303) { 
+      param.codec_version=311;
+      this->use_311_compat = 0;
+    } else {
+      this->use_311_compat = 1;
+    }
     break;
-  case BUF_VIDEO_MPEG4 :
+  case BUF_VIDEO_MPEG4:
+  case BUF_VIDEO_DIVX5:
+    if (this->version >= 20020303)
+      param.codec_version=500;
+
     this->use_311_compat = 0;
+    if (this->version >= 20020303)
+      param.codec_version=500;
     break;
   default:
     printf ("divx4: unknown video format (buftype: 0x%08X)\n",
-      buf->type & 0xFFFF0000);
+	    buf->type & 0xFFFF0000);
   }
 
   /* setup decoder; inspired by avifile's plugin */
   param.x_dim=this->biWidth;
   param.y_dim=this->biHeight;
   param.time_incr = 15; /* no idea what this does */
+
+  param.build_number=0;
+
   param.output_format=DEC_USER;
+  param.build_number=0;
   memset(&param.buffers, 0, sizeof(param.buffers));
 
   ret = this->decore((unsigned long)this, DEC_OPT_INIT, &param, &this->bih);
@@ -333,7 +366,8 @@ static int divx4_can_handle (video_decoder_t *this_gen, int buf_type) {
   /* divx4 currently does not support MSMPEG4 v1/v2 */
   return ( (buf_type == BUF_VIDEO_MSMPEG4_V3 && this->can_handle_311) ||
            /* buf_type == BUF_VIDEO_MSMPEG4_V12 || */
-           buf_type == BUF_VIDEO_MPEG4);
+           (buf_type == BUF_VIDEO_MPEG4) ||
+	   (buf_type == BUF_VIDEO_DIVX5));
 }
 
 /* copied verbatim from ffmpeg plugin */
@@ -356,8 +390,20 @@ static void divx4_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
   divx4_decoder_t *this = (divx4_decoder_t *) this_gen;
 
-  if (buf->decoder_info[0] == 0) { /* need to initialize */
+#ifdef LOG
+  printf ("divx4: decoding buffer %08x, flags = %08x\n", buf, buf->decoder_flags);
+#endif 
+
+  if (buf->decoder_flags & BUF_FLAG_PREVIEW)
+    return;
+
+  if (buf->decoder_flags & BUF_FLAG_HEADER) { /* need to initialize */
     /* only proceed if version is good and initialization succeeded */
+
+#ifdef LOG
+    printf ("divx4: get_version...\n");
+#endif 
+
     divx4_get_version(this);	
     this->decoder_ok = ( divx4_check_version(this) &&
                          divx4_init_decoder(this, buf) );
@@ -369,6 +415,8 @@ static void divx4_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
     
       this->buf = malloc( VIDEOBUFSIZE );
       this->bufsize = VIDEOBUFSIZE;
+    
+      this->skipframes = 0;
     }
     return;
   }
@@ -390,13 +438,19 @@ static void divx4_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
   xine_fast_memcpy (&this->buf[this->size], buf->content, buf->size);
   this->size += buf->size;
+  
+  if (buf->decoder_flags & BUF_FLAG_FRAMERATE)
+    this->video_step = buf->decoder_info[0];
 
-  if (buf->decoder_info[0] == 2)  { /* need to decode a frame */
+  if (buf->decoder_flags & BUF_FLAG_FRAME_END)  { /* need to decode a frame */
     /* allocate image (taken from ffmpeg plugin) */
     img = this->video_out->get_frame (this->video_out, this->biWidth,
-        this->biHeight, XINE_ASPECT_RATIO_DONT_TOUCH, IMGFMT_YV12, 
-        this->video_step, VO_BOTH_FIELDS);
+				      this->biHeight, XINE_ASPECT_RATIO_DONT_TOUCH, 
+				      IMGFMT_YV12, 
+				      VO_BOTH_FIELDS);
 
+    img->pts = buf->pts;
+    img->duration = this->video_step;
     /* setup the decode frame parameters, as demonstrated by avifile.
        Old versions used DEC_YV12, but that was basically wrong and just
        happened to work with the Xv driver. Now DEC_USER is the only option.
@@ -412,18 +466,20 @@ static void divx4_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
     else
       ret = this->decore((unsigned long)this, DEC_OPT_FRAME, &frame, 0);
       
-    if (ret != DEC_OK) {
-      printf("divx4: decore DEC_OPT_FRAME command returned %s.\n", 
-        decore_retval(ret));
+    if (ret != DEC_OK || this->skipframes) {
+      if( !this->skipframes )
+        printf("divx4: decore DEC_OPT_FRAME command returned %s.\n", 
+          decore_retval(ret));
       img->bad_frame = 1; /* better skip this one */
     }
     else {
       divx4_copy_frame(this, img, pict);
+      img->bad_frame = 0;
     }
 
-    /* this again from ffmpeg plugin */
-    img->PTS = buf->PTS;
-    img->draw(img);
+    this->skipframes = img->draw(img);
+    if( this->skipframes < 0 )
+      this->skipframes = 0;
     img->free(img);
       
     this->size = 0;
@@ -464,6 +520,17 @@ static char *divx4_get_id(void) {
   return "divx4 video decoder";
 }
 
+static void divx4_flush(video_decoder_t *this_gen) {
+}
+
+static void divx4_reset(video_decoder_t *this_gen) {
+  /* seems to handle seeking quite nicelly without any code here */
+}
+
+static void divx4_dispose(video_decoder_t *this_gen) {
+  free (this_gen);
+}
+
 /* This is pretty generic. I took the liberty to increase the
    priority over that of libffmpeg :-) */
 video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
@@ -474,7 +541,7 @@ video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
   decoreFunc libdecore_func = 0;
   config_values_t *cfg;
 
-  if (iface_version != 5) {
+  if (iface_version != 7) {
     printf( "divx4: plugin doesn't support plugin API version %d.\n"
 	    "divx4: this means there's a version mismatch between xine and this "
 	    "divx4: decoder plugin.\nInstalling current plugins should help.\n",
@@ -507,6 +574,9 @@ video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
   this->video_decoder.decode_data         = divx4_decode_data;
   this->video_decoder.close               = divx4_close;
   this->video_decoder.get_identifier      = divx4_get_id;
+  this->video_decoder.flush               = divx4_flush;
+  this->video_decoder.reset               = divx4_reset;
+  this->video_decoder.dispose             = divx4_dispose;
   this->video_decoder.priority            = cfg->register_num (cfg, "codec.divx4_priority", 4,
 							       "priority of the divx4 plugin (>5 => enable)",
 							       NULL, NULL, NULL); 
@@ -519,7 +589,7 @@ video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
 								"use divx4 plugin for msmpeg4v3 streams",
 								NULL, NULL, NULL);
   this->size				  = 0;
-   /* allow override of version checking by user */
+  /* allow override of version checking by user */
   this->version 			  = cfg->register_num(cfg, "codec.divx4_forceversion", 0,
 							      "Divx version to check for (set to 0 (default) if unsure)",
 							      NULL, NULL, NULL);
