@@ -1,8 +1,10 @@
 /*
  * decode.c
- * Copyright (C) 1999-2001 Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
+ * Copyright (C) 2000-2002 Michel Lespinasse <walken@zoy.org>
+ * Copyright (C) 1999-2000 Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
  *
  * This file is part of mpeg2dec, a free MPEG-2 video stream decoder.
+ * See http://libmpeg2.sourceforge.net/ for updates.
  *
  * mpeg2dec is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,10 +24,6 @@
  *
  */
 
-/*
-#define LOG_PAN_SCAN
-*/
-
 #include "config.h"
 
 #include <stdio.h>
@@ -40,28 +38,39 @@
 #include "xine_internal.h"
 #include "events.h"
 
-#define BUFFER_SIZE (224 * 1024)
+/*
+#define LOG_PAN_SCAN
+*/
 
-mpeg2_config_t config;
+/*
+#define LOG
+*/
+
+/* #define BUFFER_SIZE (224 * 1024) */
+#define BUFFER_SIZE (1194 * 1024) /* new buffer size for mpeg2dec 0.2.1 */
 
 static void process_userdata(mpeg2dec_t *mpeg2dec, uint8_t *buffer);
 
 void mpeg2_init (mpeg2dec_t * mpeg2dec, 
 		 vo_instance_t * output)
 {
-    static int do_init = 1;
+  static int do_init = 1;
+  uint32_t mm_accel;
 
     if (do_init) {
 	do_init = 0;
-	config.flags = xine_mm_accel();
-	idct_init ();
-	motion_comp_init ();
+	mm_accel = xine_mm_accel();
+	mpeg2_cpu_state_init (mm_accel);
+	mpeg2_idct_init (mm_accel);
+	mpeg2_mc_init (mm_accel);
     }
 
     if( !mpeg2dec->chunk_buffer )
-      mpeg2dec->chunk_buffer = xine_xmalloc_aligned (16, BUFFER_SIZE + 4);
+      mpeg2dec->chunk_buffer = xine_xmalloc_aligned (16, BUFFER_SIZE + 4, 
+						     (void**)&mpeg2dec->chunk_base);
     if( !mpeg2dec->picture )
-      mpeg2dec->picture = xine_xmalloc_aligned (16, sizeof (picture_t));
+      mpeg2dec->picture = xine_xmalloc_aligned (16, sizeof (picture_t),
+						(void**)&mpeg2dec->picture_base);
 
     mpeg2dec->shift = 0xffffff00;
     mpeg2dec->is_sequence_needed = 1;
@@ -75,54 +84,71 @@ void mpeg2_init (mpeg2dec_t * mpeg2dec,
 
     memset (mpeg2dec->picture, 0, sizeof (picture_t));
 
-    /* initialize supstructures */
-    header_state_init (mpeg2dec->picture);
+    /* initialize substructures */
+    mpeg2_header_state_init (mpeg2dec->picture);
 }
 
-static int get_frame_duration(mpeg2dec_t * mpeg2dec, vo_frame_t * frame)
+static inline void get_frame_duration (mpeg2dec_t * mpeg2dec, vo_frame_t *frame)
 {
-  int duration;
-  switch (frame->frame_rate_code) {
+  switch (mpeg2dec->picture->frame_rate_code) {
   case 1: /* 23.976 fps */
-    frame->duration = 3913;
+    frame->duration      = 3913;
     break;
   case 2: /* 24 fps */
-    frame->duration = 3750;
+    frame->duration      = 3750;
     break;
   case 3: /* 25 fps */
-    frame->duration = frame->repeat_first_field ? 5400 : 3600;
+    frame->duration      = 3600;
     break;
   case 4: /* 29.97 fps */
-    if ((frame->repeat_first_field) || (mpeg2dec->last_repeat_first_field)) {
-      frame->duration = 3754;
-      frame->pts_corrector = frame->repeat_first_field ? 375 : -375;
-    } else {
-      frame->duration = 3003;
-      frame->pts_corrector = 0;
-    }
-//    duration = 3754;
-//    duration = frame->repeat_first_field ? 4504 : 3003;
+    frame->duration      = 3003;
     break;
   case 5: /* 30 fps */
-    frame->duration = 3000;
+    frame->duration      = 3000;
     break;
   case 6: /* 50 fps */
-    frame->duration = 1800;
+    frame->duration      = 1800;
     break;
   case 7: /* 59.94 fps */
-    frame->duration = 1525;
+    frame->duration      = 1525;
     break;
   case 8: /* 60 fps */
-    frame->duration = 1509;
+    frame->duration      = 1509;
     break;
   default:
        /* printf ("invalid/unknown frame rate code : %d \n",
                frame->frame_rate_code); */
-    duration = 3000;
+    frame->duration      = 3000;
   }
-  /* printf("mpeg2dec: rff=%u\n",frame->repeat_first_field); */
-  mpeg2dec->last_repeat_first_field = frame->repeat_first_field;
-  return 1;
+  
+  /* this should be used to detect any special rff pattern */
+  mpeg2dec->rff_pattern = mpeg2dec->rff_pattern << 1;
+  mpeg2dec->rff_pattern |= !!frame->repeat_first_field;
+
+  if( ((mpeg2dec->rff_pattern & 0xff) == 0xaa ||
+      (mpeg2dec->rff_pattern & 0xff) == 0x55) &&
+      !mpeg2dec->picture->progressive_sequence &&
+       mpeg2dec->picture->progressive_frame ) {
+    /* special case for ntsc 3:2 pulldown */
+    frame->duration += frame->duration/4;
+  }
+  else
+  {  
+    if( frame->repeat_first_field ) {
+      if( !mpeg2dec->picture->progressive_sequence &&
+           mpeg2dec->picture->progressive_frame ) {
+        /* decoder should output 3 fields, so adjust duration to
+           count on this extra field time */
+        frame->duration += frame->duration/2;     
+      } else if( mpeg2dec->picture->progressive_sequence ) {
+        /* for progressive sequences the output should repeat the
+           frame 1 or 2 times depending on top_field_first flag. */
+        frame->duration *= (mpeg2dec->picture->top_field_first)?3:2;
+      }
+    }
+  }
+  
+  /*printf("mpeg2dec: rff=%u\n",frame->repeat_first_field);*/
 } 
 
 static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
@@ -130,11 +156,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 {
     picture_t * picture;
     int is_frame_done;
-/*
-    if (code >= 0xb0) { 
-      printf ("libmpeg2: parse_chunk 0x%02x\n", code);   
-    }
-*/
+    
     /* wait for sequence_header_code */
     if (mpeg2dec->is_sequence_needed) {
       if (code != 0xb3) {
@@ -148,7 +170,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
       return 0;
     }
 
-    stats_header (code, buffer);
+    mpeg2_stats (code, buffer);
 
     picture = mpeg2dec->picture;
     is_frame_done = mpeg2dec->in_slice && ((!code) || (code >= 0xb0));
@@ -156,42 +178,34 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
     if (is_frame_done) {
 	mpeg2dec->in_slice = 0;
 
-	if ( picture->current_frame && ((picture->picture_structure == FRAME_PICTURE) ||
+	if (((picture->picture_structure == FRAME_PICTURE) ||
 	     (picture->second_field)) ) {
+	  
+	  if (mpeg2dec->drop_frame)
+	    picture->current_frame->bad_frame = 1;
+   
+	  if (picture->picture_coding_type == B_TYPE) {
+	    if( picture->current_frame && !picture->current_frame->drawn ) {
 
-	    picture->current_frame->bad_frame |= mpeg2dec->drop_frame;
+	      /* hack against wrong mpeg1 pts */
+	      if (picture->mpeg1) 
+	        picture->current_frame->pts = 0;
 
-#if 0 
-	    printf ("type %s: %s\n",
-		    picture->picture_coding_type == I_TYPE ? "I" :
-		    picture->picture_coding_type == P_TYPE ? "P" : "B",
-		    picture->current_frame->bad_frame ? "BAD" : "good");
-#endif 
-	    if (picture->picture_coding_type == B_TYPE) {
-	      if (picture->mpeg1) {
-		picture->current_frame->PTS = 0;
-              }
-              get_frame_duration(mpeg2dec, picture->current_frame);
+	      get_frame_duration(mpeg2dec, picture->current_frame);
 	      mpeg2dec->frames_to_drop = picture->current_frame->draw (picture->current_frame);
-	      picture->current_frame->free (picture->current_frame);
-	      picture->current_frame = NULL;
-	      picture->throwaway_frame = NULL;
-	    } else if (picture->forward_reference_frame && !picture->forward_reference_frame->drawn) {
-              get_frame_duration(mpeg2dec, picture->forward_reference_frame);
-	      mpeg2dec->frames_to_drop = picture->forward_reference_frame->draw (picture->forward_reference_frame);
-	      picture->forward_reference_frame->drawn = 1;
+	      picture->current_frame->drawn = 1;
 	    }
-
-#ifdef ARCH_X86
-	    if (config.flags & MM_ACCEL_X86_MMX)
-		emms ();
-#endif
+	  } else if (picture->forward_reference_frame && !picture->forward_reference_frame->drawn) {
+	    get_frame_duration(mpeg2dec, picture->forward_reference_frame);
+	    mpeg2dec->frames_to_drop = picture->forward_reference_frame->draw (picture->forward_reference_frame);
+	    picture->forward_reference_frame->drawn = 1;
+	  }
 	}
     }
 
     switch (code) {
     case 0x00:	/* picture_start_code */
-	if (header_process_picture_header (picture, buffer)) {
+	if (mpeg2_header_picture (picture, buffer)) {
 	    fprintf (stderr, "bad picture header\n");
 	    exit (1);
 	}
@@ -208,9 +222,26 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	  case B_TYPE:
 	    
 	    if (mpeg2dec->frames_to_drop>1) {
+#ifdef LOG
+	      printf ("libmpeg2: dropping b-frame because frames_to_drop==%d\n",
+		      mpeg2dec->frames_to_drop);
+#endif
 	      mpeg2dec->drop_frame = 1;
 	    } else if (!picture->forward_reference_frame || picture->forward_reference_frame->bad_frame 
 		       || !picture->backward_reference_frame || picture->backward_reference_frame->bad_frame) {
+#ifdef LOG
+	      printf ("libmpeg2: dropping b-frame because ref is bad (");
+	      if (picture->forward_reference_frame)
+		printf ("fw ref frame %d, bad %d;", picture->forward_reference_frame->id,
+			picture->forward_reference_frame->bad_frame);
+	      else
+		printf ("fw ref frame not there;");
+	      if (picture->backward_reference_frame)
+		printf ("bw ref frame %d, bad %d)\n", picture->backward_reference_frame->id,
+			picture->backward_reference_frame->bad_frame);
+	      else
+		printf ("fw ref frame not there)\n");
+#endif
 	      mpeg2dec->drop_frame = 1;
 	    }
 	    break;
@@ -219,13 +250,26 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	    
 	    if (mpeg2dec->frames_to_drop>2) {
 	      mpeg2dec->drop_frame = 1;
+#ifdef LOG
+	      printf ("libmpeg2: dropping p-frame because frames_to_drop==%d\n",
+		      mpeg2dec->frames_to_drop);
+#endif
 	    } else if (!picture->backward_reference_frame || picture->backward_reference_frame->bad_frame) {
 	      mpeg2dec->drop_frame = 1;
+#ifdef LOG
+	      if (picture->backward_reference_frame->bad_frame)
+		printf ("libmpeg2: dropping p-frame because ref %d is bad\n", picture->backward_reference_frame->id);
+	      else
+		printf ("libmpeg2: dropping p-frame because no ref frame\n");
+#endif
 	    }
 	    break;
 	    
 	  case I_TYPE:
-	    /*
+#ifdef LOG
+	    printf ("libmpeg2: I-Frame\n");
+#endif
+	    /* for the sake of dvd menus, never drop i-frames
 	    if (mpeg2dec->frames_to_drop>4) {
 	      mpeg2dec->drop_frame = 1;
 	    }
@@ -241,15 +285,20 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
         break;
 
     case 0xb3:	/* sequence_header_code */
-	if (header_process_sequence_header (picture, buffer)) {
-  	    printf ("libmpeg2: bad sequence header\n");
+	if (mpeg2_header_sequence (picture, buffer)) {
+	    fprintf (stderr, "bad sequence header\n");
 	    /* exit (1); */
 	} else if (mpeg2dec->is_sequence_needed 
 	    || (picture->frame_width != picture->coded_picture_width)
 	    || (picture->frame_height != picture->coded_picture_height)) {
             xine_frame_change_event_t notify_event;
+
+	    xine_log (mpeg2dec->xine, XINE_LOG_FORMAT,
+		      "libmpeg2: frame size is %d x %d, ratio is %d\n",
+		      picture->frame_width, picture->frame_height,
+		      picture->frame_rate_code);
             
-	    printf ("mpeg2dec: frame size has changed to from %d x %d to %d x %d\n",
+	    printf ("libmpeg2: frame size has changed to from %d x %d to %d x %d\n",
 		    picture->frame_width, picture->frame_height,
 		    picture->coded_picture_width, picture->coded_picture_height);
 	    
@@ -264,7 +313,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	    if (picture->backward_reference_frame) 
 	      picture->backward_reference_frame->free (picture->backward_reference_frame);
 
-	    printf ("mpeg2dec: old frames freed.\n"); 
+	    printf ("libmpeg2: old frames freed.\n"); 
 
 	    mpeg2dec->is_sequence_needed = 0;
 	    picture->forward_reference_frame = NULL;
@@ -276,7 +325,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	break;
 
     case 0xb5:	/* extension_start_code */
-	if (header_process_extension (picture, buffer)) {
+	if (mpeg2_header_extension (picture, buffer)) {
 	    fprintf (stderr, "bad extension\n");
 	    exit (1);
 	}
@@ -284,16 +333,16 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 
     case 0xb7:	/* sequence end code */
 #ifdef LOG_PAN_SCAN
-      printf ("libmpeg2:SEQUENCE END CODE NOT HANDLED!\n");
+      printf ("libmpeg2: sequence end code not handled\n");
 #endif
     case 0xb8:	/* group of pictures start code */
-	if (header_process_group_of_pictures (picture, buffer)) {
-	    fprintf (stderr, "bad group of pictures\n");
-	    exit (1);
+	if (mpeg2_header_group_of_pictures (picture, buffer)) {
+	  printf ("libmpeg2: bad group of pictures\n");
+	  exit (1);
 	}
     default:
 	if (code >= 0xb9)
-	    fprintf (stderr, "stream not demultiplexed ?\n");
+	  printf ("libmpeg2: stream not demultiplexed ?\n");
 
 	if (code >= 0xb0)
 	    break;
@@ -308,14 +357,19 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	      else
 		mpeg2dec->drop_frame = 1;		
 	    } else {
+		if ( picture->current_frame && 
+		     picture->current_frame != picture->backward_reference_frame &&
+		     picture->current_frame != picture->forward_reference_frame ) {
+			picture->current_frame->free (picture->current_frame);
+		}
+		
 		if (picture->picture_coding_type == B_TYPE)
-		    picture->throwaway_frame = picture->current_frame =
+		    picture->current_frame =
 		        mpeg2dec->output->get_frame (mpeg2dec->output,
 						     picture->coded_picture_width,
 						     picture->coded_picture_height,
 						     picture->aspect_ratio_information,
 						     IMGFMT_YV12,
-						     picture->frame_duration,
 						     picture->picture_structure);
 		else {
 		    picture->current_frame =
@@ -324,7 +378,6 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 						     picture->coded_picture_height,
 						     picture->aspect_ratio_information,
 						     IMGFMT_YV12,
-						     picture->frame_duration,
 						     (VO_PREDICTION_FLAG | picture->picture_structure));
 		    if (picture->forward_reference_frame)
 		      picture->forward_reference_frame->free (picture->forward_reference_frame);
@@ -333,36 +386,29 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 			picture->backward_reference_frame;
 		    picture->backward_reference_frame = picture->current_frame;
 		}
-		picture->current_frame->bad_frame = 0;
-		picture->current_frame->drawn = 0;
-		picture->current_frame->PTS = mpeg2dec->pts;
-		picture->current_frame->SCR = mpeg2dec->scr;
-                picture->current_frame->aspect_ratio = picture->aspect_ratio_information;
-                picture->current_frame->frame_rate_code = picture->frame_rate_code;
-                picture->current_frame->top_field_first = picture->top_field_first;
-                picture->current_frame->progressive_sequence = picture->progressive_sequence;
+		picture->current_frame->bad_frame          = 1;
+		picture->current_frame->drawn              = 0;
+		picture->current_frame->pts                = mpeg2dec->pts;
+                picture->current_frame->top_field_first    = picture->top_field_first;
                 picture->current_frame->repeat_first_field = picture->repeat_first_field;
-                picture->current_frame->progressive_frame = picture->progressive_frame;
-                picture->current_frame->picture_coding_type = picture->picture_coding_type;
-                picture->current_frame->bitrate = picture->bitrate;
-		picture->current_frame->repeat_first_field = picture->repeat_first_field;
+
+#ifdef LOG
+		printf ("libmpeg2: decoding frame %d, type %s\n",
+			picture->current_frame->id, picture->picture_coding_type == I_TYPE ? "I" :
+			picture->picture_coding_type == P_TYPE ? "P" : "B");
+#endif
 		mpeg2dec->pts = 0;
 	    }
 	}
 
 	if (!(mpeg2dec->drop_frame)) {
-	    /* printf ("slice_process\n"); */
-
-	    slice_process (picture, code, buffer);
-
-#ifdef ARCH_X86
-	    if (config.flags & MM_ACCEL_X86_MMX)
-		emms ();
-#endif
+	  mpeg2_slice (picture, code, buffer);
+	  if( picture->v_offset > picture->limit_y ) 
+	    picture->current_frame->bad_frame = 0;
 	}
     }
 
-    /* printf ("parse_chunk %d completed\n", code);  */
+    /* printf ("libmpeg2: parse_chunk %d completed\n", code);  */
     return is_frame_done;
 }
 
@@ -380,11 +426,6 @@ static inline uint8_t * copy_chunk (mpeg2dec_t * mpeg2dec,
     if (limit > end)
 	limit = end;
 
-    /* 
-    printf ("copy chunk current %08x\n", current );
-    printf ("copy chunk end     %08x\n", end); fflush(stdout);
-    */
-
     while (1) {
 
 	byte = *current++;
@@ -412,7 +453,7 @@ static inline uint8_t * copy_chunk (mpeg2dec_t * mpeg2dec,
 }
 
 int mpeg2_decode_data (mpeg2dec_t * mpeg2dec, uint8_t * current, uint8_t * end,
-		       uint32_t pts, uint32_t scr)
+		       uint64_t pts)
 {
     int ret;
     uint8_t code;
@@ -429,8 +470,6 @@ int mpeg2_decode_data (mpeg2dec_t * mpeg2dec, uint8_t * current, uint8_t * end,
     if (pts)
       mpeg2dec->pts = pts;
 
-    mpeg2dec->scr = scr;
-
     while (current != end) {
 	code = mpeg2dec->code;
 	current = copy_chunk (mpeg2dec, current, end);
@@ -442,33 +481,70 @@ int mpeg2_decode_data (mpeg2dec_t * mpeg2dec, uint8_t * current, uint8_t * end,
     return ret;
 }
 
+void mpeg2_reset (mpeg2dec_t * mpeg2dec) {
+  
+  picture_t *picture = mpeg2dec->picture;
+
+  if( !picture )
+    return;
+  
+  mpeg2dec->pts = 0;
+  
+  if( !picture->mpeg1 )
+    mpeg2dec->is_sequence_needed = 1;
+  
+  mpeg2dec->in_slice = 0;
+
+  /* to free reference frames one also needs to fix slice.c to 
+   * abort when they are NULL. unfortunately it seems to break
+   * DVD menus.
+   */
+  /*
+  if ( picture->current_frame && 
+       picture->current_frame != picture->backward_reference_frame &&
+       picture->current_frame != picture->forward_reference_frame )
+    picture->current_frame->free (picture->current_frame);
+  picture->current_frame = NULL;
+  
+  if (picture->forward_reference_frame)
+    picture->forward_reference_frame->free (picture->forward_reference_frame);
+  picture->forward_reference_frame = NULL;
+  
+  if (picture->backward_reference_frame)
+    picture->backward_reference_frame->free (picture->backward_reference_frame);
+  picture->backward_reference_frame = NULL;
+  */
+}
+
 void mpeg2_flush (mpeg2dec_t * mpeg2dec) {
 
-    picture_t *picture = mpeg2dec->picture;
+  picture_t *picture = mpeg2dec->picture;
 
-    if (picture->backward_reference_frame && !picture->backward_reference_frame->drawn) {
-      vo_frame_t        *img;
-      
-      printf ("libmpeg2: blasting out backward reference frame on flush\n");
-      picture->backward_reference_frame->PTS = 0;
-      picture->backward_reference_frame->SCR = mpeg2dec->scr;
-      get_frame_duration(mpeg2dec, picture->backward_reference_frame);
-      picture->backward_reference_frame->bad_frame = 0;
-      picture->backward_reference_frame->drawn = 1;
-      picture->backward_reference_frame->displayed (picture->backward_reference_frame);
-      
-      /* output a copy instead of the frame used by decoder */
-      img = picture->backward_reference_frame->instance->duplicate_frame(
-            picture->backward_reference_frame->instance, 
-            picture->backward_reference_frame);
-      img->PTS = 0;
-      img->SCR = mpeg2dec->scr;
-      img->bad_frame = 0;
-      img->drawn = 2;
-      img->draw(img);
-      
-      img->free(img);
-    }
+  if (!picture)
+    return;
+  
+  if (picture->current_frame && !picture->current_frame->drawn &&
+      !picture->current_frame->bad_frame) {
+
+    vo_frame_t *img;
+    
+    printf ("libmpeg2: blasting out current frame %d on flush\n",
+	    picture->current_frame->id);
+    
+    picture->current_frame->drawn = 1;
+    get_frame_duration(mpeg2dec, picture->current_frame);
+    
+    /* output a copy instead of the frame used by decoder */
+    img = picture->current_frame->instance->duplicate_frame(picture->current_frame->instance, 
+								       picture->current_frame);
+    img->pts = 0;
+    img->bad_frame = 0;
+    img->drawn = 1; 
+    
+    img->draw(img);
+    img->free(img);
+
+  }
 
 }
 
@@ -487,46 +563,47 @@ void mpeg2_close (mpeg2dec_t * mpeg2dec)
       dont remove any picture->*->free() below. doing so will cause buffer 
       leak, and we only have about 15 of them.
     */ 
-    if (picture->forward_reference_frame) {
-      /*
-      printf ("libmpeg2: blasting out forward reference frame on close\n");
-//      picture->forward_reference_frame->PTS = 0;
-      picture->forward_reference_frame->bad_frame = 0;
-      get_frame_duration(mpeg2dec, picture->forward_reference_frame);
-      picture->forward_reference_frame->draw (picture->forward_reference_frame); 
-      */
-      picture->forward_reference_frame->displayed (picture->forward_reference_frame);
-      picture->forward_reference_frame->free (picture->forward_reference_frame);
+ 
+    if ( picture->current_frame ) {
+      if( !picture->current_frame->drawn ) {
+        printf ("libmpeg2: blasting out current frame on close\n");
+        picture->current_frame->pts = 0;
+        get_frame_duration(mpeg2dec, picture->current_frame);
+        picture->current_frame->draw (picture->current_frame);
+        picture->current_frame->drawn = 1;
+      }
+         
+      if( picture->current_frame != picture->backward_reference_frame &&
+          picture->current_frame != picture->forward_reference_frame ) {
+        picture->current_frame->free (picture->current_frame);
+      }
+      picture->current_frame = NULL;
     }
     
-    if (picture->throwaway_frame) {
-      printf ("libmpeg2: blasting out throwaway frame on close\n");
-      picture->throwaway_frame->PTS = 0;
-      picture->throwaway_frame->SCR = mpeg2dec->scr;
-      picture->throwaway_frame->bad_frame = 0;
-      get_frame_duration(mpeg2dec, picture->throwaway_frame);
-      picture->throwaway_frame->draw (picture->throwaway_frame);
-      picture->throwaway_frame->free (picture->throwaway_frame);
+    if (picture->forward_reference_frame) {
+      picture->forward_reference_frame->free (picture->forward_reference_frame);
+      picture->forward_reference_frame = NULL;
     }
-
+    
     if (picture->backward_reference_frame) {
-      printf ("libmpeg2: blasting out backward reference frame on close\n");
-      picture->backward_reference_frame->PTS = 0;
-      picture->backward_reference_frame->SCR = mpeg2dec->scr;
-      get_frame_duration(mpeg2dec, picture->backward_reference_frame);
-      picture->backward_reference_frame->bad_frame = 0;
-      if( !picture->backward_reference_frame->drawn)
+      if( !picture->backward_reference_frame->drawn) {
+        printf ("libmpeg2: blasting out backward reference frame on close\n");
+        picture->backward_reference_frame->pts = 0;
+        get_frame_duration(mpeg2dec, picture->backward_reference_frame);
         picture->backward_reference_frame->draw (picture->backward_reference_frame);
+        picture->backward_reference_frame->drawn = 1;      
+      }
       picture->backward_reference_frame->free (picture->backward_reference_frame);
+      picture->backward_reference_frame = NULL;
     }
 
     if ( mpeg2dec->chunk_buffer ) {
-      xine_free_aligned (mpeg2dec->chunk_buffer);
+      free (mpeg2dec->chunk_base);
       mpeg2dec->chunk_buffer = NULL;
     }
     
     if ( mpeg2dec->picture ) {
-      xine_free_aligned (mpeg2dec->picture);
+      free (mpeg2dec->picture_base);
       mpeg2dec->picture = NULL;
     }
 }
@@ -547,11 +624,10 @@ void mpeg2_find_sequence_header (mpeg2dec_t * mpeg2dec,
 
     /* printf ("looking for sequence header... %02x\n", code);  */
 
-    stats_header (code, mpeg2dec->chunk_buffer);
+    mpeg2_stats (code, mpeg2dec->chunk_buffer);
 
     if (code == 0xb3) {	/* sequence_header_code */
-
-      if (header_process_sequence_header (picture, mpeg2dec->chunk_buffer)) {
+      if (mpeg2_header_sequence (picture, mpeg2dec->chunk_buffer)) {
 	printf ("libmpeg2: bad sequence header\n");
 	continue;
       }
@@ -563,7 +639,11 @@ void mpeg2_find_sequence_header (mpeg2dec_t * mpeg2dec,
 	picture->frame_width  = picture->coded_picture_width;
 	picture->frame_height = picture->coded_picture_height;
 
-	printf ("mpeg2dec: frame size %d x %d\n",
+	xine_log (mpeg2dec->xine, XINE_LOG_FORMAT,
+		  "libmpeg2: frame size is %d x %d\n",
+		  picture->frame_width, picture->frame_height);
+            
+	printf ("libmpeg2: frame size %d x %d\n",
 		picture->frame_width, picture->frame_height);
             
 	notify_event.event.type = XINE_EVENT_FRAME_CHANGE;
@@ -572,7 +652,7 @@ void mpeg2_find_sequence_header (mpeg2dec_t * mpeg2dec,
 	xine_send_event(mpeg2dec->xine, &notify_event.event);
       }
     } else if (code == 0xb5) {	/* extension_start_code */
-      if (header_process_extension (picture, mpeg2dec->chunk_buffer)) {
+      if (mpeg2_header_extension (picture, mpeg2dec->chunk_buffer)) {
 	printf ("libmpeg2: bad extension\n");
 	continue ;
       }
@@ -603,7 +683,6 @@ static void process_userdata(mpeg2dec_t *mpeg2dec, uint8_t *buffer)
     event.buffer = &buffer[2];
     event.buf_len = end - &buffer[2];
     event.pts = mpeg2dec->pts;
-    event.scr = mpeg2dec->scr;
     xine_send_event(mpeg2dec->xine, &event.event);
   }
 }
