@@ -4,7 +4,7 @@
  * Copyright (C) Christian Vogler 
  *               cvogler@gradient.cis.upenn.edu - December 2001
  *
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: cc_decoder.c,v 1.6 2002/01/08 20:38:05 cvogler Exp $
+ * $Id: cc_decoder.c,v 1.16 2002/04/26 20:31:47 jkeil Exp $
  *
  * stuff needed to provide closed captioning decoding and display
  *
@@ -76,134 +76,151 @@ static int text_colormap[NUM_FG_COL] = {
   OSD_TEXT1, OSD_TEXT2, OSD_TEXT3, OSD_TEXT4, OSD_TEXT5, OSD_TEXT6, OSD_TEXT7
 };
 
-/* text colors */
+
+/* -------------------- caption text colors -----------------------------*/
 /* FIXME: The colors look fine on an XShm display, but they look *terrible*
    with the Xv display on the NVidia driver on a GeForce 3. The colors bleed
-   into each other. Not sure at this moment whether the YUV values
-   may be off, whether it is a bug in the NVidia Xv driver, or whether
-   it is a bug in the blending functions.
+   into each other more than I'd expect from the downsampling into YUV
+   colorspace. 
+   At this moment, it looks like a problem in the Xv YUV blending functions.
 */
-static clut_t cc_text[NUM_FG_COL][TEXT_PALETTE_SIZE] = {
+typedef struct colorinfo_s {
+  clut_t bgcol;           /* text background color */
+  clut_t bordercol;       /* text border color */
+  clut_t textcol;         /* text color */
+} colorinfo_t;
+
+
+static colorinfo_t cc_text_trans[NUM_FG_COL] = {
   /* white, black border, translucid */
   {
-    CLUT_Y_CR_CB_INIT(0x00, 0x00, 0x00), //0
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //1
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //2
-    CLUT_Y_CR_CB_INIT(0x60, 0x80, 0x80), //3
-    CLUT_Y_CR_CB_INIT(0x40, 0x80, 0x80), //4
-    CLUT_Y_CR_CB_INIT(0x20, 0x80, 0x80), //5
-    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80), //6
-    CLUT_Y_CR_CB_INIT(0x40, 0x80, 0x80), //7
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //8
-    CLUT_Y_CR_CB_INIT(0xc0, 0x80, 0x80), //9
-    CLUT_Y_CR_CB_INIT(0xff, 0x80, 0x80), //10
+    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0xff, 0x80, 0x80)
   },
 
   /* green, black border, translucid */
   {
-    CLUT_Y_CR_CB_INIT(0x00, 0x00, 0x00), //0
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //1
-    CLUT_Y_CR_CB_INIT(0x60, 0x80, 0x80), //2
-    CLUT_Y_CR_CB_INIT(0x40, 0x80, 0x80), //3
-    CLUT_Y_CR_CB_INIT(0x20, 0x80, 0x80), //4
-    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80), //5
-    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80), //6
-    CLUT_Y_CR_CB_INIT(0x20, 0x6a, 0x65), //7
-    CLUT_Y_CR_CB_INIT(0x4a, 0x50, 0x55), //8
-    CLUT_Y_CR_CB_INIT(0x70, 0x3a, 0x45), //9
-    CLUT_Y_CR_CB_INIT(0x90, 0x22, 0x35), //10
+    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x90, 0x22, 0x35)
   },
 
   /* blue, black border, translucid */
   {
-    CLUT_Y_CR_CB_INIT(0x00, 0x00, 0x00), //0
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //1
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //2
-    CLUT_Y_CR_CB_INIT(0x60, 0x80, 0x80), //3
-    CLUT_Y_CR_CB_INIT(0x40, 0x80, 0x80), //4
-    CLUT_Y_CR_CB_INIT(0x20, 0x80, 0x80), //5
-    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80), //6
-    CLUT_Y_CR_CB_INIT(0x0c, 0x7c, 0xa0), //7
-    CLUT_Y_CR_CB_INIT(0x18, 0x78, 0xc0), //8
-    CLUT_Y_CR_CB_INIT(0x20, 0x74, 0xe0), //9
-    CLUT_Y_CR_CB_INIT(0x29, 0x6e, 0xff), //10
+    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x29, 0x6e, 0xff)
   },
 
   /* cyan, black border, translucid */
   {
-    CLUT_Y_CR_CB_INIT(0x00, 0x00, 0x00), //0
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //1
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //2
-    CLUT_Y_CR_CB_INIT(0x60, 0x80, 0x80), //3
-    CLUT_Y_CR_CB_INIT(0x40, 0x80, 0x80), //4
-    CLUT_Y_CR_CB_INIT(0x20, 0x80, 0x80), //5
-    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80), //6
-    CLUT_Y_CR_CB_INIT(0x28, 0x64, 0x8a), //7
-    CLUT_Y_CR_CB_INIT(0x50, 0x48, 0x94), //8
-    CLUT_Y_CR_CB_INIT(0x78, 0x2c, 0x9e), //9
-    CLUT_Y_CR_CB_INIT(0xaa, 0x10, 0xa6), //10
+    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0xaa, 0x10, 0xa6)
   },
 
   /* red, black border, translucid */
   {
-    CLUT_Y_CR_CB_INIT(0x00, 0x00, 0x00), //0
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //1
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //2
-    CLUT_Y_CR_CB_INIT(0x60, 0x80, 0x80), //3
-    CLUT_Y_CR_CB_INIT(0x40, 0x80, 0x80), //4
-    CLUT_Y_CR_CB_INIT(0x20, 0x80, 0x80), //5
-    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80), //6
-    CLUT_Y_CR_CB_INIT(0x14, 0x9c, 0x77), //7
-    CLUT_Y_CR_CB_INIT(0x28, 0xb8, 0x6e), //8
-    CLUT_Y_CR_CB_INIT(0x3c, 0xd4, 0x65), //9
-    CLUT_Y_CR_CB_INIT(0x52, 0xf0, 0x5a), //10
+    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x52, 0xf0, 0x5a)
   },
 
   /* yellow, black border, translucid */
   {
-    CLUT_Y_CR_CB_INIT(0x00, 0x00, 0x00), //0
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //1
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //2
-    CLUT_Y_CR_CB_INIT(0x60, 0x80, 0x80), //3
-    CLUT_Y_CR_CB_INIT(0x40, 0x80, 0x80), //4
-    CLUT_Y_CR_CB_INIT(0x20, 0x80, 0x80), //5
-    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80), //6
-    CLUT_Y_CR_CB_INIT(0x34, 0x84, 0x64), //7
-    CLUT_Y_CR_CB_INIT(0x68, 0x8a, 0x48), //8
-    CLUT_Y_CR_CB_INIT(0x9c, 0x8e, 0x2c), //9
-    CLUT_Y_CR_CB_INIT(0xd4, 0x92, 0x10), //10
+    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0xd4, 0x92, 0x10)
   },
 
   /* magenta, black border, translucid */
   {
-    CLUT_Y_CR_CB_INIT(0x00, 0x00, 0x00), //0
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //1
-    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80), //2
-    CLUT_Y_CR_CB_INIT(0x60, 0x80, 0x80), //3
-    CLUT_Y_CR_CB_INIT(0x40, 0x80, 0x80), //4
-    CLUT_Y_CR_CB_INIT(0x20, 0x80, 0x80), //5
-    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80), //6
-    CLUT_Y_CR_CB_INIT(0x1a, 0x98, 0x92), //7
-    CLUT_Y_CR_CB_INIT(0x34, 0xb0, 0xa6), //8
-    CLUT_Y_CR_CB_INIT(0x4e, 0xc8, 0xb8), //9
-    CLUT_Y_CR_CB_INIT(0x6b, 0xde, 0xca), //10
+    CLUT_Y_CR_CB_INIT(0x80, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x6b, 0xde, 0xca)
+  }
+};
+
+static colorinfo_t cc_text_solid[NUM_FG_COL] = {
+  /* white, black border, solid */
+  {
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0xff, 0x80, 0x80)
   },
 
+  /* green, black border, solid */
+  {
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x90, 0x22, 0x35)
+  },
+
+  /* blue, black border, solid */
+  {
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x29, 0x6e, 0xff)
+  },
+
+  /* cyan, black border, solid */
+  {
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0xaa, 0x10, 0xa6)
+  },
+
+  /* red, black border, solid */
+  {
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x52, 0xf0, 0x5a)
+  },
+
+  /* yellow, black border, solid */
+  {
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0xd4, 0x92, 0x10)
+  },
+
+  /* magenta, black border, solid */
+  {
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x00, 0x80, 0x80),
+    CLUT_Y_CR_CB_INIT(0x6b, 0xde, 0xca)
+  }
 };
 
-static uint8_t cc_text_trans[NUM_FG_COL][TEXT_PALETTE_SIZE] = {
-  {0, 8, 9, 10, 11, 12, 13, 14, 15, 15, 15 },
-  {0, 8, 9, 10, 11, 12, 13, 14, 15, 15, 15 },
-  {0, 8, 9, 10, 11, 12, 13, 14, 15, 15, 15 },
-  {0, 8, 9, 10, 11, 12, 13, 14, 15, 15, 15 },
-  {0, 8, 9, 10, 11, 12, 13, 14, 15, 15, 15 },
-  {0, 8, 9, 10, 11, 12, 13, 14, 15, 15, 15 },
-  {0, 8, 9, 10, 11, 12, 13, 14, 15, 15, 15 }
+
+static uint8_t cc_text_trans_alpha[TEXT_PALETTE_SIZE] = {
+  0, 8, 9, 10, 11, 12, 15, 15, 15, 15, 15
 };
 
-/* caption palette and alpha channel */
-static uint32_t cc_palette[OVL_PALETTE_SIZE];
-static uint8_t cc_trans[OVL_PALETTE_SIZE];
+static uint8_t cc_text_solid_alpha[TEXT_PALETTE_SIZE] = {
+  0, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15
+};
+
+
+static colorinfo_t *cc_text_palettes[NUM_CC_PALETTES] = {
+  cc_text_trans,
+  cc_text_solid
+};
+
+static uint8_t *cc_alpha_palettes[NUM_CC_PALETTES] = {
+  cc_text_trans_alpha,
+  cc_text_solid_alpha
+};
+
+
+char *cc_schemes[NUM_CC_PALETTES + 1] = {
+  "White/Gray/Translucent",
+  "White/Black/Solid",
+  NULL
+};
+
+/* --------------------- misc. EIA 608 definitions -------------------*/
 
 #define TRANSP_SPACE 0x19   /* code for transparent space, essentially 
 			       arbitrary */
@@ -250,7 +267,21 @@ struct cc_renderer_s {
      that as the vpts of the hide event upon an osd free.
   */
 #warning "FIXME: bug in OSD or SPU?"
-  int display_vpts;           /* vpts of currently displayed caption */
+  int64_t display_vpts;       /* vpts of currently displayed caption */
+
+  /* this variable is an even worse hack: in some rare cases, the pts
+     information on the DVD gets out of sync with the caption information.
+     If this happens, the vpts of a hide caption event can actually be
+     slightly higher than the vpts of the following show caption event.
+     For this reason, we remember the vpts of the hide event and force
+     the next show event's vpts  to be at least equal to the hide event's 
+     vpts.
+  */
+  int64_t last_hide_vpts;
+     
+  /* caption palette and alpha channel */
+  uint32_t cc_palette[OVL_PALETTE_SIZE];
+  uint8_t cc_trans[OVL_PALETTE_SIZE];
 
   metronom_t *metronom;       /* the active xine metronom */
 
@@ -315,8 +346,7 @@ struct cc_decoder_s {
   uint32_t lastcode;
 
   /* The PTS and SCR at which the captioning chunk started */
-  uint32_t pts;
-  uint32_t scr;
+  int64_t pts;
   /* holds the NTSC frame offset to last known pts/scr */
   uint32_t f_offset;
 
@@ -414,25 +444,35 @@ static void build_char_table(void)
 }
 
 
-static void build_palette(void)
+static clut_t interpolate_color(clut_t src, clut_t dest, int steps,
+				int current_step)
 {
-  int i, j;
-
-  memset(cc_palette, 0, sizeof (cc_palette));
-  memset(cc_trans, 0, sizeof (cc_trans));
-  for (i = 0; i < NUM_FG_COL; i++) {
-    for (j = 0; j < TEXT_PALETTE_SIZE; j++) {
-      cc_palette[i * TEXT_PALETTE_SIZE + j + OSD_TEXT1] = *(uint32_t *) &cc_text[i][j];
-      cc_trans[i * TEXT_PALETTE_SIZE + j + OSD_TEXT1] = cc_text_trans[i][j];
-    }
-  }
+  int diff_y = ((int) dest.y) - ((int) src.y);
+  int diff_cr = ((int) dest.cr) - ((int) src.cr);
+  int diff_cb = ((int) dest.cb) - ((int) src.cb);
+  int res_y = ((int) src.y) + (diff_y * current_step / (steps + 1));
+  int res_cr = ((int) src.cr) + (diff_cr * current_step / (steps + 1));
+  int res_cb = ((int) src.cb) + (diff_cb * current_step / (steps + 1));
+#if __SUNPRO_C
+  /*
+   * Sun's Forte compiler refuses to initialize automatic structure
+   * variable with bitfields, so we use explicit assignments for now.
+   */
+  clut_t res;
+  res.y = res_y;
+  res.cr = res_cr;
+  res.cb = res_cb;
+  res.foo = 0;
+#else
+  clut_t res = CLUT_Y_CR_CB_INIT((uint8_t) res_y, (uint8_t) res_cr,
+				 (uint8_t) res_cb);
+#endif
+  return res;
 }
 
-
 /*----------------- cc_row_t methods --------------------------------*/
 
-static void ccrow_fill_transp(cc_row_t *rowbuf)
-{
+static void ccrow_fill_transp(cc_row_t *rowbuf){
   int i;
 
 #ifdef LOG_DEBUG
@@ -757,11 +797,51 @@ static void ccmem_exit(cc_memory_t *this)
 
 /*----------------- cc_renderer_t methods -------------------------------*/
 
-static uint32_t cc_renderer_calc_vpts(cc_renderer_t *this, uint32_t pts,
-				      uint32_t scr, uint32_t ntsc_frame_offset)
+static void cc_renderer_build_palette(cc_renderer_t *this)
+{
+  int i, j;
+  colorinfo_t *cc_text = cc_text_palettes[this->cc_cfg->cc_scheme];
+  uint8_t *cc_alpha = cc_alpha_palettes[this->cc_cfg->cc_scheme];
+  
+  memset(this->cc_palette, 0, sizeof (this->cc_palette));
+  memset(this->cc_trans, 0, sizeof (this->cc_trans));
+  for (i = 0; i < NUM_FG_COL; i++) {
+    /* background color */
+    this->cc_palette[i * TEXT_PALETTE_SIZE + 1 + OSD_TEXT1] =
+      *(uint32_t *) &cc_text[i].bgcol;
+    /* background -> border */
+    for (j = 2; j <= 5; j++) {
+      clut_t col = interpolate_color(cc_text[i].bgcol,
+				     cc_text[i].bordercol, 4, j - 1);
+      this->cc_palette[i * TEXT_PALETTE_SIZE + j + OSD_TEXT1] =
+	*(uint32_t *) &col;
+    }
+    /* border color */
+    this->cc_palette[i * TEXT_PALETTE_SIZE + 6 + OSD_TEXT1] =
+      *(uint32_t *) &cc_text[i].bordercol;
+    /* border -> foreground */
+    for (j = 7; j <= 9; j++) {
+      clut_t col = interpolate_color(cc_text[i].bordercol,
+				     cc_text[i].textcol, 3, j - 6);
+      this->cc_palette[i * TEXT_PALETTE_SIZE + j + OSD_TEXT1] =
+	*(uint32_t *) &col;
+    }
+    /* foreground color */
+    this->cc_palette[i * TEXT_PALETTE_SIZE + 10 + OSD_TEXT1] =
+      *(uint32_t *) &cc_text[i].textcol;
+
+    /* alpha values */
+    for (j = 0; j <= 10; j++)
+      this->cc_trans[i * TEXT_PALETTE_SIZE + j + OSD_TEXT1] = cc_alpha[j];
+  }
+}
+
+
+static int64_t cc_renderer_calc_vpts(cc_renderer_t *this, int64_t pts,
+				      uint32_t ntsc_frame_offset)
 {
   metronom_t *metronom = this->metronom;
-  uint32_t vpts = metronom->got_spu_packet(metronom, pts, 0, scr);
+  int64_t vpts = metronom->got_spu_packet(metronom, pts);
   return vpts + ntsc_frame_offset * NTSC_FRAME_DURATION;
 }
 
@@ -773,17 +853,18 @@ static int cc_renderer_on_display(cc_renderer_t *this)
 }
 
 
-static void cc_renderer_hide_caption(cc_renderer_t *this, uint32_t vpts)
+static void cc_renderer_hide_caption(cc_renderer_t *this, int64_t vpts)
 {
   if (this->displayed) {
     this->osd_renderer->hide(this->cap_display, vpts);
     this->displayed = 0;
+    this->last_hide_vpts = vpts;
   }
 }
 
 
 static void cc_renderer_show_caption(cc_renderer_t *this, cc_buffer_t *buf,
-			      uint32_t vpts)
+				     int64_t vpts)
 {
 #ifdef LOG_DEBUG
   printf("spucc: cc_renderer: show\n");
@@ -799,6 +880,7 @@ static void cc_renderer_show_caption(cc_renderer_t *this, cc_buffer_t *buf,
   this->osd_renderer->set_position(this->cap_display, 
 				   this->x,
 				   this->y);
+  vpts = MAX(vpts, this->last_hide_vpts);
   this->osd_renderer->show(this->cap_display, vpts);
   
   this->displayed = 1;
@@ -830,7 +912,8 @@ static void cc_renderer_adjust_osd_object(cc_renderer_t *this)
   this->cap_display = this->osd_renderer->new_object(this->osd_renderer,
 						     this->width,
 						     this->height);
-  this->osd_renderer->set_palette(this->cap_display, cc_palette, cc_trans);
+  this->osd_renderer->set_palette(this->cap_display, this->cc_palette,
+				  this->cc_trans);
 }
 
 
@@ -871,6 +954,9 @@ void cc_renderer_update_cfg(cc_renderer_t *this_obj, int video_width,
   this_obj->video_width = video_width;
   this_obj->video_height = video_height;
 
+  /* fill in text palette */
+  cc_renderer_build_palette(this_obj);
+
   /* calculate preferred captioning area, as per the EIA-608 standard */
   this_obj->x =  this_obj->video_width * 10 / 100;
   this_obj->y = this_obj->video_height * 10 / 100;
@@ -960,8 +1046,11 @@ static void cc_hide_displayed(cc_decoder_t *this)
 #endif
 
   if (cc_renderer_on_display(this->cc_cfg->renderer)) {
-    uint32_t vpts = cc_renderer_calc_vpts(this->cc_cfg->renderer, this->pts,
-					  this->scr, this->f_offset); 
+    int64_t vpts = cc_renderer_calc_vpts(this->cc_cfg->renderer, this->pts,
+					  this->f_offset); 
+#ifdef LOG_DEBUG
+    printf("cc_decoder: cc_hide_displayed: hiding caption %u at vpts %u\n", this->capid, vpts);
+#endif    
     cc_renderer_hide_caption(this->cc_cfg->renderer, vpts);
   }
 }
@@ -974,8 +1063,8 @@ static void cc_show_displayed(cc_decoder_t *this)
 #endif
 
   if (cc_onscreen_displayable(this)) {
-    uint32_t vpts = cc_renderer_calc_vpts(this->cc_cfg->renderer, this->pts,
-					  this->scr, this->f_offset);
+    int64_t vpts = cc_renderer_calc_vpts(this->cc_cfg->renderer, this->pts,
+					  this->f_offset);
 #ifdef LOG_DEBUG
     printf("cc_decoder: cc_show_displayed: showing caption %u at vpts %u\n", this->capid, vpts);
 #endif    
@@ -1180,6 +1269,10 @@ static void cc_decode_EIA608(cc_decoder_t *this, uint16_t data)
   uint8_t c1 = data & 0x7f;
   uint8_t c2 = (data >> 8) & 0x7f;
 
+#if LOG_DEBUG >= 3
+  printf("decoding %x %x\n", c1, c2);
+#endif
+
   if (c1 & 0x60) {             /* normal character, 0x20 <= c1 <= 0x7f */
     cc_decode_standard_char(this, c1, c2);
   }
@@ -1231,7 +1324,7 @@ static void cc_decode_EIA608(cc_decoder_t *this, uint16_t data)
 
 
 void decode_cc(cc_decoder_t *this, uint8_t *buffer, uint32_t buf_len,
-	       uint32_t pts, uint32_t scr)
+	       int64_t pts)
 {
   /* The first number may denote a channel number. I don't have the
    * EIA-708 standard, so it is hard to say.
@@ -1243,12 +1336,23 @@ void decode_cc(cc_decoder_t *this, uint8_t *buffer, uint32_t buf_len,
    *        field #2 in line 21 of the VBI. We'll ignore it for the
    *        time being.
    *
-   *   0xff starts 2 byte EIA-608 sequence, field #1 in line 21 of the VBI
+   *   0xff starts 2 byte EIA-608 sequence, field #1 in line 21 of the VBI.
+   *        Followed by a 3-code triplet that starts either with 0xff or
+   *        0xfe. In either case, the following triplet needs to be ignored
+   *        for line 21, field 1.
    *
    *   0x00 is padding, followed by 2 more 0x00.
    *
    *   0x01 always seems to appear at the beginning, always seems to
-   *        be followed by 0xf8, 0x9e. Ignored for the time being.
+   *        be followed by 0xf8, 8-bit number. 
+   *        The lower 7 bits of this 8-bit number seem to denote the
+   *        number of code triplets that follow.
+   *        The most significant bit denotes whether the Line 21 field 1 
+   *        captioning information is at odd or even triplet offsets from this
+   *        beginning triplet. 1 denotes odd offsets, 0 denotes even offsets.
+   *      
+   *        Most captions are encoded with odd offsets, so this is what we
+   *        will assume.
    *
    * until end of packet
    */
@@ -1256,12 +1360,26 @@ void decode_cc(cc_decoder_t *this, uint8_t *buffer, uint32_t buf_len,
   uint32_t curbytes = 0;
   uint8_t data1, data2;
   uint8_t cc_code;
+  int odd_offset = 1;
 
   this->f_offset = 0;
   this->pts = pts;
-  this->scr = scr;
-  
+
+#if LOG_DEBUG >= 2
+  printf("libspucc: decode_cc: got pts %u\n", pts);
+  {
+    uint8_t *cur_d = buffer;
+    printf("libspucc: decode_cc: codes: ");
+    while (cur_d < buffer + buf_len) {
+      printf("0x%0x ", *cur_d++);
+    }
+    printf("\n");
+  }
+#endif  
+
   while (curbytes < buf_len) {
+    int skip = 2;
+
     cc_code = *current++;
     curbytes++;
     
@@ -1272,14 +1390,14 @@ void decode_cc(cc_decoder_t *this, uint8_t *buffer, uint32_t buf_len,
       break;
     }
     
-    data1 = *current++;
-    data2 = *current++;
-    curbytes += 2;
+    data1 = *current;
+    data2 = *(current + 1);
     
     switch (cc_code) {
     case 0xfe:
       /* expect 2 byte encoding (perhaps CC3, CC4?) */
       /* ignore for time being */
+      skip = 2;
       break;
       
     case 0xff:
@@ -1288,22 +1406,31 @@ void decode_cc(cc_decoder_t *this, uint8_t *buffer, uint32_t buf_len,
 	cc_decode_EIA608(this, data1 | (data2 << 8));
 	this->f_offset++;
       }
+      skip = 5;
       break;
       
     case 0x00:
       /* This seems to be just padding */
+      skip = 2;
       break;
       
     case 0x01:
-      /* unknown Header info, ignore for the time being */
+      odd_offset = data2 & 0x80;
+      if (odd_offset)
+	skip = 2;
+      else
+	skip = 5;
       break;
       
     default:
 #ifdef LOG_DEBUG
       fprintf(stderr, "Unknown CC encoding: %x\n", cc_code);
 #endif
+      skip = 2;
       break;
     }
+    current += skip;
+    curbytes += skip;
   }
 }
 
@@ -1324,7 +1451,7 @@ cc_decoder_t *cc_decoder_open(cc_config_t *cc_cfg)
   this->lastcode = 0;
   this->capid = 0;
 
-  this->pts = this->scr = this->f_offset = 0;
+  this->pts = this->f_offset = 0;
 
 #ifdef LOG_DEBUG
   printf("spucc: cc_decoder_open\n");
@@ -1352,6 +1479,5 @@ void cc_decoder_init(void)
 {
   build_parity_table();
   build_char_table();
-  build_palette();
 }
 
