@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.11 2002/01/08 16:47:56 cvogler Exp $
+ * $Id: xine_decoder.c,v 1.21 2002/04/23 15:45:25 esnel Exp $
  *
  * code based on mplayer module:
  *
@@ -36,6 +36,11 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <ctype.h>
+#ifdef __FreeBSD__
+# include <sys/iconv.h>
+#else
+# include <iconv.h>
+#endif
 
 #include "buffer.h"
 #include "events.h"
@@ -91,6 +96,8 @@ typedef struct sputext_decoder_s {
   osd_renderer_t    *renderer;
   osd_object_t      *osd;
   char              *font;
+  char              *src_encoding;
+  char              *dst_encoding;
 
 } sputext_decoder_t;
 
@@ -223,9 +230,12 @@ static subtitle_t *sub_read_line_microdvd(sputext_decoder_t *this, subtitle_t *c
   
   bzero (current, sizeof(subtitle_t));
   
+  current->end=-1;
   do {
     if (!fgets (line, 1000, this->fd)) return NULL;
-  } while (sscanf (line, "{%ld}{%ld}%[^\r\n]", &(current->start), &(current->end),line2) <3);
+  } while ((sscanf (line, "{%ld}{}%[^\r\n]", &(current->start), line2) !=2) &&
+           (sscanf (line, "{%ld}{%ld}%[^\r\n]", &(current->start), &(current->end),line2) !=3)
+	  );
   
   p=line2;
   
@@ -567,7 +577,8 @@ static int sub_autodetect (sputext_decoder_t *this) {
     if (!fgets (line, 1000, this->fd))
       return -1;
     
-    if (sscanf (line, "{%d}{%d}", &i, &i)==2) {
+    if ((sscanf (line, "{%d}{}", &i)==1) ||
+        (sscanf (line, "{%d}{%d}", &i, &i)==2)) {
       this->uses_time=0;
       printf ("sputext: microdvd subtitle format detected\n");
       return FORMAT_MICRODVD;
@@ -653,6 +664,7 @@ static subtitle_t *sub_read_file (sputext_decoder_t *this) {
     sub_read_line_aqt
     
   };
+  iconv_t iconv_descr;
 
   this->format=sub_autodetect (this);
   if (this->format==-1) {
@@ -668,6 +680,7 @@ static subtitle_t *sub_read_file (sputext_decoder_t *this) {
   first = (subtitle_t *) xine_xmalloc(n_max*sizeof(subtitle_t));
   if(!first) return NULL;
     
+  iconv_descr=iconv_open(this->dst_encoding,this->src_encoding);
   while(1){
     subtitle_t *sub;
     if(this->num>=n_max){
@@ -681,10 +694,29 @@ static subtitle_t *sub_read_file (sputext_decoder_t *this) {
     if (sub==ERR) 
       ++this->errs; 
     else {
-      ++this->num; /* Error vs. Valid */
+      int i;
 
+      for(i=0; i<first[this->num].lines; i++)
+      { char *tmp;
+	char *in_buff, *out_buff;
+	int   in_len, out_len;
+
+	in_len=strlen(first[this->num].text[i])+1;
+	tmp=malloc(in_len);
+	in_buff=first[this->num].text[i];
+	out_buff=tmp;
+	out_len=in_len;
+	if ((size_t)(-1)!=iconv(iconv_descr,&in_buff,&in_len,&out_buff,&out_len))
+	{ free(first[this->num].text[i]);
+	  first[this->num].text[i]=tmp;
+	}
+	else
+        { printf("sputext: Can't convert subtitle text\n"); }
+      }
+      ++this->num; /* Error vs. Valid */
     }
   }
+  iconv_close(iconv_descr);
 
   printf ("sputext: Read %i subtitles", this->num);
   if (this->errs) 
@@ -724,7 +756,6 @@ static int spudec_can_handle (spu_decoder_t *this_gen, int buf_type) {
 }
 
 
-
 static void spudec_init (spu_decoder_t *this_gen, vo_instance_t *vo_out) {
 
   sputext_decoder_t *this = (sputext_decoder_t *) this_gen;
@@ -744,8 +775,9 @@ static void spudec_init (spu_decoder_t *this_gen, vo_instance_t *vo_out) {
 static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
 
   sputext_decoder_t *this = (sputext_decoder_t *) this_gen;
-
-  if (buf->decoder_info[0] == 0) {
+  int64_t current_time;
+  
+  if (buf->decoder_flags & BUF_FLAG_HEADER) {
 
     int y;
 
@@ -776,9 +808,11 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
 
     this->renderer->set_position (this->osd, 0, y); 
 
-    this->renderer->render_text (this->osd, 0, 0, "sputext decoder", OSD_TEXT1);
-    this->renderer->show (this->osd, 0);
-    this->renderer->hide (this->osd, 300000);      
+    /*  this->renderer->render_text (this->osd, 0, 0, "sputext decoder", OSD_TEXT1); */
+    
+    current_time = this->xine->metronom->get_current_time (this->xine->metronom);
+    this->renderer->show (this->osd, current_time);
+    this->renderer->hide (this->osd, current_time+300000);      
 
     this->fd = (FILE *) buf->content;
   
@@ -791,17 +825,21 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
 
   } else {
 
-    uint32_t    pts, pts_end;
-    int32_t     pts_factor;
+    int64_t     pts, pts_end;
+    int64_t     pts_factor;
     int         frame_num;
     subtitle_t *subtitle;
 
-    subtitle = NULL;
+    /* don't want to see subtitle */
+    if(this->xine->spu_channel_user == -2)
+      return;
 
-    pts       = buf->PTS;
+    subtitle = NULL;
+      
+    pts       = buf->pts;
     pts_end   = pts;
     frame_num = buf->decoder_info[1];
-
+    
     /* 
      * find out which subtitle to display 
      */
@@ -820,9 +858,12 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
       if (subtitle->start > frame_num)
 	return;
 
+      /* FIXME FIXME FIXME 
       pts_factor = this->xine->metronom->get_video_rate (this->xine->metronom);
+      */
+      pts_factor = 3000;
 
-      pts += this->xine->metronom->video_wrap_offset;
+      pts += this->xine->metronom->vpts_offset;
 
       pts_end = pts + (subtitle->end - subtitle->start) * pts_factor;
 
@@ -853,7 +894,7 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
       if (subtitle->start > (start_tenth+20))
 	return;
 
-      pts += this->xine->metronom->video_wrap_offset;
+      pts += this->xine->metronom->vpts_offset;
 
       pts_end = pts + (subtitle->end - subtitle->start)*900;
     }
@@ -894,7 +935,7 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
       this->renderer->hide (this->osd, pts_end);
 
 #ifdef LOG
-      printf ("sputext: scheduling subtitle >%s< at %d until %d, current time is %d\n",
+      printf ("sputext: scheduling subtitle >%s< at %lld until %lld, current time is %lld\n",
 	      subtitle->text[0], pts, pts_end, 
 	      this->xine->metronom->get_current_time (this->xine->metronom));
 #endif
@@ -906,6 +947,12 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
 }  
 
 
+static void spudec_reset (spu_decoder_t *this_gen) {
+  sputext_decoder_t *this = (sputext_decoder_t *) this_gen;
+
+  this->cur = 0;
+}
+
 static void spudec_close (spu_decoder_t *this_gen) {
   sputext_decoder_t *this = (sputext_decoder_t *) this_gen;
 
@@ -932,11 +979,33 @@ static void update_osd_font(void *this_gen, cfg_entry_t *entry)
   printf("libsputext: spu_font = %s\n", this->font );
 }
 
+static void update_osd_src_encoding(void *this_gen, cfg_entry_t *entry)
+{
+  sputext_decoder_t *this = (sputext_decoder_t *)this_gen;
+
+  this->src_encoding = entry->str_value;
+  
+  printf("libsputext: spu_src_encoding = %s\n", this->src_encoding );
+}
+
+static void update_osd_dst_encoding(void *this_gen, cfg_entry_t *entry)
+{
+  sputext_decoder_t *this = (sputext_decoder_t *)this_gen;
+
+  this->dst_encoding = entry->str_value;
+  
+  printf("libsputext: spu_dst_encoding = %s\n", this->dst_encoding );
+}
+
+static void spudec_dispose (spu_decoder_t *this_gen) {
+  free (this_gen);
+}
+
 spu_decoder_t *init_spu_decoder_plugin (int iface_version, xine_t *xine) {
 
   sputext_decoder_t *this ;
 
-  if (iface_version != 4) {
+  if (iface_version != 6) {
     printf("libsputext: doesn't support plugin api version %d.\n"
 	   "libsputext: This means there is a version mismatch between xine and\n"
 	   "libsputext: this plugin.\n", iface_version);
@@ -949,8 +1018,10 @@ spu_decoder_t *init_spu_decoder_plugin (int iface_version, xine_t *xine) {
   this->spu_decoder.can_handle          = spudec_can_handle;
   this->spu_decoder.init                = spudec_init;
   this->spu_decoder.decode_data         = spudec_decode_data;
+  this->spu_decoder.reset               = spudec_reset;
   this->spu_decoder.close               = spudec_close;
   this->spu_decoder.get_identifier      = spudec_get_id;
+  this->spu_decoder.dispose             = spudec_dispose;
   this->spu_decoder.priority            = 1;
 
   this->xine                            = xine;
@@ -959,6 +1030,16 @@ spu_decoder_t *init_spu_decoder_plugin (int iface_version, xine_t *xine) {
 									"sans", 
 									"font for avi subtitles", 
 									NULL, update_osd_font, this);
+  this->src_encoding                    = xine->config->register_string(xine->config, 
+									"codec.spu_src_encoding", 
+									"windows-1250", 
+									"source encoging of subtitles", 
+									NULL, update_osd_dst_encoding, this);
+  this->dst_encoding                    = xine->config->register_string(xine->config, 
+									"codec.spu_dst_encoding", 
+									"iso-8859-2", 
+									"target encoging for subtitles (have to match font encoding)", 
+									NULL, update_osd_dst_encoding, this);
 
   return (spu_decoder_t *) this;
 }
