@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: w32codec.c,v 1.59 2002/01/15 17:30:51 miguelfreitas Exp $
+ * $Id: w32codec.c,v 1.76 2002/04/28 15:32:41 esnel Exp $
  *
  * routines for using w32 codecs
  * DirectShow support by Miguel Freitas (Nov/2001)
@@ -46,7 +46,11 @@
 #include "buffer.h"
 #include "xineutils.h"
 #include "xine_internal.h"
- 
+
+/*
+#define LOG
+*/
+
 static GUID CLSID_Voxware =
 {
      0x73f7a062, 0x8829, 0x11d1,
@@ -78,9 +82,12 @@ static GUID dvsd_clsid =
 };
 
 
-pthread_mutex_t win32_codec_name_mutex;
+/* some data is shared inside wine loader.
+ * this mutex seems to avoid some segfaults
+ */
+static pthread_mutex_t win32_codec_mutex;
+static pthread_once_t once_control = PTHREAD_ONCE_INIT;
 extern char*   win32_codec_name; 
-
 extern char*   win32_def_path;
 
 #define VIDEOBUFSIZE 128*1024
@@ -89,7 +96,7 @@ typedef struct w32v_decoder_s {
   video_decoder_t   video_decoder;
 
   vo_instance_t    *video_out;
-  int               video_step;
+  int64_t           video_step;
   int               decoder_ok;
 
   BITMAPINFOHEADER  bih, o_bih; 
@@ -113,7 +120,8 @@ typedef struct w32v_decoder_s {
   DS_VideoDecoder  *ds_dec;
 
   int               stream_id;
-    
+  int               skipframes;  
+  
   LDT_FS *ldt_fs;
 } w32v_decoder_t;
 
@@ -126,11 +134,10 @@ typedef struct w32a_decoder_s {
 
   unsigned char    *buf;
   int               size;   
-  uint32_t          pts;
-  uint32_t          scr;
+  int64_t           pts;
   
   /* these are used for pts estimation */
-  uint32_t lastpts, sumpts, sumsize;
+  int64_t           lastpts, sumpts, sumsize;
 
   unsigned char    *outbuf;
   int               outsize;
@@ -149,6 +156,23 @@ typedef struct w32a_decoder_s {
 } w32a_decoder_t;
 
 
+/* This is a ugly hack (as all this win32 code!):
+ * unloading one of the codecs (w32a or w32v) while the other is 
+ * still playing causes segfaults. using this define will make
+ * the last of them to close both.
+ * hopefuly some day we will fix the wine loader to remove that.
+ */   
+#define SYNC_SHUTDOWN
+
+#ifdef SYNC_SHUTDOWN
+w32v_decoder_t *w32v_instance;
+w32a_decoder_t *w32a_instance;
+
+static void w32v_close (video_decoder_t *this_gen);
+static void w32a_close (audio_decoder_t *this_gen);
+#endif
+
+
 /*
  * RGB->YUY2 conversion, we need is for xine video-codec ->
  * video-output interface
@@ -394,9 +418,9 @@ static void w32v_init_codec (w32v_decoder_t *this, int buf_type) {
   uint32_t vo_cap;
   int outfmt;
 
-  w32v_init_rgb_ycc();
-
+#ifdef LOG
   printf ("w32codec: init codec...\n");
+#endif
 
   memset(&this->o_bih, 0, sizeof(BITMAPINFOHEADER));
   this->o_bih.biSize = sizeof(BITMAPINFOHEADER);
@@ -495,7 +519,7 @@ static void w32v_init_codec (w32v_decoder_t *this, int buf_type) {
 static void w32v_init_ds_codec (w32v_decoder_t *this, int buf_type) {
   uint32_t vo_cap;
   int outfmt;
-
+  
   printf ("w32codec: init Direct Show video codec...\n");
   
   memset(&this->o_bih, 0, sizeof(BITMAPINFOHEADER));
@@ -567,33 +591,53 @@ static void w32v_init_ds_codec (w32v_decoder_t *this, int buf_type) {
 static void w32v_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
   w32v_decoder_t *this = (w32v_decoder_t *) this_gen;
 
-  /*
-  printf ("w32codec: processing packet type = %08x, buf : %d, 
-          buf->decoder_info[0]=%d\n", 
-	  buf->type, buf, buf->decoder_info[0]);
-	  */
-  if (buf->decoder_info[0] == 0) {
+  
+#ifdef LOG
+  printf ("w32codec: processing packet type = %08x, buf->decoder_flags=%08x\n", 
+	  buf->type, buf->decoder_flags);
+#endif
+  
+  if (buf->decoder_flags & BUF_FLAG_PREVIEW)
+    return;
+  
+  if (buf->decoder_flags & BUF_FLAG_HEADER) {
     if ( buf->type & 0xff )
       return;
     
+#ifdef LOG
+    printf ("w32codec: processing header ...\n"); 
+#endif
+
     /* init package containing bih */
     memcpy ( &this->bih, buf->content, sizeof (BITMAPINFOHEADER));
     this->video_step = buf->decoder_info[1];
+#ifdef LOG
+    printf ("w32codec: video_step is %lld\n", this->video_step);
+#endif
 
-    pthread_mutex_lock(&win32_codec_name_mutex);
+    pthread_mutex_lock(&win32_codec_mutex);
     win32_codec_name = get_vids_codec_name (this, buf->type);
 
     if( !this->ds_driver )
       w32v_init_codec (this, buf->type);
     else
       w32v_init_ds_codec (this, buf->type);
-    pthread_mutex_unlock(&win32_codec_name_mutex);
+      
+#ifdef SYNC_SHUTDOWN
+    if( this->decoder_ok )
+      w32v_instance = this;
+#endif
+    
+    pthread_mutex_unlock(&win32_codec_mutex);
       
     this->stream_id = -1;
+    this->skipframes = 0;
     
   } else if (this->decoder_ok) {
 
-    /* printf ("w32codec: processing packet ...\n"); */
+#ifdef LOG
+    printf ("w32codec: processing packet ...\n"); 
+#endif
     if( (int) buf->size <= 0 )
         return;
        
@@ -614,109 +658,137 @@ static void w32v_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
     this->size += buf->size;
 
-    if (buf->decoder_info[0] == 2)  {
+    if (buf->decoder_flags & BUF_FLAG_FRAMERATE)
+      this->video_step = buf->decoder_info[0];
+
+    if (buf->decoder_flags & BUF_FLAG_FRAME_END)  {
 
       HRESULT     ret;
+      int         flags;
       vo_frame_t *img;
+      uint8_t    *img_buffer = this->img_buffer;
 
       /* decoder video frame */
 
       this->bih.biSizeImage = this->size;
-
+     
       img = this->video_out->get_frame (this->video_out,
 					this->bih.biWidth, 
 					this->bih.biHeight, 
 					42, 
 					IMGFMT_YUY2,
-					this->video_step,
 					VO_BOTH_FIELDS);
 
-      /*
-      ret = ICDecompress(this->hic, ICDECOMPRESS_NOTKEYFRAME, 
-			 &this->bih, this->buf,
-			 &this->o_bih, img->base[0]);
-      */
+      img->duration = this->video_step;
+
+#ifdef LOG
+	printf ("w32codec: frame duration is %lld\n", this->video_step);
+#endif
+
+      if (this->outfmt==IMGFMT_YUY2)
+         img_buffer = img->base[0];
+         
+      flags = 0;
+      if( !(buf->decoder_flags & BUF_FLAG_KEYFRAME) )
+        flags |= ICDECOMPRESS_NOTKEYFRAME;
+      if( this->skipframes )
+        flags |= ICDECOMPRESS_HURRYUP|ICDECOMPRESS_PREROL;
+      
+      pthread_mutex_lock(&win32_codec_mutex);
       if( !this->ds_driver )
         ret = (!this->ex_functions)
-              ?ICDecompress(this->hic, ICDECOMPRESS_NOTKEYFRAME, 
-			    &this->bih, this->buf,
-			    &this->o_bih, this->img_buffer)
-              :ICDecompressEx(this->hic, ICDECOMPRESS_NOTKEYFRAME, 
-			    &this->bih, this->buf,
-			    &this->o_bih, this->img_buffer);
+              ?ICDecompress(this->hic, flags,
+			    &this->bih, this->buf, &this->o_bih, 
+			    (this->skipframes)?NULL:img_buffer)
+              :ICDecompressEx(this->hic, flags,
+			    &this->bih, this->buf, &this->o_bih,
+			    (this->skipframes)?NULL:img_buffer); 
       else {
-        ret = DS_VideoDecoder_DecodeInternal(this->ds_dec, this->buf, 
-                         this->size, 0, this->img_buffer);
+        ret = DS_VideoDecoder_DecodeInternal(this->ds_dec, this->buf, this->size,
+                            buf->decoder_flags & BUF_FLAG_KEYFRAME,
+                            (this->skipframes)?NULL:img_buffer);
       }
+      pthread_mutex_unlock(&win32_codec_mutex);
                          
-      if (this->outfmt==IMGFMT_YUY2) {
-	/* already decoded into YUY2 format by DLL */
-	xine_fast_memcpy(img->base[0], this->img_buffer, this->bih.biHeight*this->bih.biWidth*2);
-      } else {
-	/* now, convert rgb to yuv */
-	int row, col;
+      if (!this->skipframes) {
+        if (this->outfmt==IMGFMT_YUY2) {
+	  /* already decoded into YUY2 format by DLL */
+	  /*
+	  xine_fast_memcpy(img->base[0], this->img_buffer,
+	                   this->bih.biHeight*this->bih.biWidth*2);
+	  */
+        } else {
+	  /* now, convert rgb to yuv */
+	  int row, col;
 #if	HAS_SLOW_MULT
-	int32_t *ctab = rgb_ycc_tab;
+	  int32_t *ctab = rgb_ycc_tab;
 #endif
-
-	xine_profiler_start_count (this->prof_rgb2yuv);
-
-	for (row=0; row<this->bih.biHeight; row++) {
-
-	  uint16_t *pixel, *out;
-
-	  pixel = (uint16_t *) ( (uint8_t *)this->img_buffer + 2 * row * this->o_bih.biWidth );
-	  out = (uint16_t *) (img->base[0] + 2 * row * this->o_bih.biWidth );
-
-	  for (col=0; col<this->o_bih.biWidth; col++, pixel++, out++) {
-	    
-	    uint8_t   r,g,b;
-	    uint8_t   y,u,v;
-	    
-	    b = (*pixel & 0x001F) << 3;
-	    g = (*pixel & 0x03E0) >> 5 << 3;
-	    r = (*pixel & 0x7C00) >> 10 << 3;
-	    
+  
+	  xine_profiler_start_count (this->prof_rgb2yuv);
+  
+	  for (row=0; row<this->bih.biHeight; row++) {
+  
+	    uint16_t *pixel, *out;
+  
+	    pixel = (uint16_t *) ( (uint8_t *)this->img_buffer + 2 * row * this->o_bih.biWidth );
+	    out = (uint16_t *) (img->base[0] + 2 * row * this->o_bih.biWidth );
+  
+	    for (col=0; col<this->o_bih.biWidth; col++, pixel++, out++) {
+  	    
+	      uint8_t   r,g,b;
+	      uint8_t   y,u,v;
+  	    
+	      b = (*pixel & 0x001F) << 3;
+	      g = (*pixel & 0x03E0) >> 5 << 3;
+	      r = (*pixel & 0x7C00) >> 10 << 3;
+  	    
 #if	HAS_SLOW_MULT
-	    y = (ctab[r+R_Y_OFF] + ctab[g+G_Y_OFF] + ctab[b+B_Y_OFF]) >> SCALEBITS;
-	    if (!(col & 0x0001)) {
-	      /* even pixel, do u */
-	      u = (ctab[r+R_CB_OFF] + ctab[g+G_CB_OFF] + ctab[b+B_CB_OFF]) >> SCALEBITS;
-	      *out = ( (uint16_t) u << 8) | (uint16_t) y;
-	    } else {
-	      /* odd pixel, do v */
-	      v = (ctab[r+R_CR_OFF] + ctab[g+G_CR_OFF] + ctab[b+B_CR_OFF]) >> SCALEBITS;
-	      *out = ( (uint16_t) v << 8) | (uint16_t) y;
-	    }
+	      y = (ctab[r+R_Y_OFF] + ctab[g+G_Y_OFF] + ctab[b+B_Y_OFF]) >> SCALEBITS;
+	      if (!(col & 0x0001)) {
+	        /* even pixel, do u */
+	        u = (ctab[r+R_CB_OFF] + ctab[g+G_CB_OFF] + ctab[b+B_CB_OFF]) >> SCALEBITS;
+	        *out = ( (uint16_t) u << 8) | (uint16_t) y;
+	      } else {
+	        /* odd pixel, do v */
+	        v = (ctab[r+R_CR_OFF] + ctab[g+G_CR_OFF] + ctab[b+B_CR_OFF]) >> SCALEBITS;
+	        *out = ( (uint16_t) v << 8) | (uint16_t) y;
+	      }
 #else
-	    y = (FIX(0.299) * r + FIX(0.587) * g + FIX(0.114) * b + ONE_HALF) >> SCALEBITS;
-	    if (!(col & 0x0001)) {
-	      /* even pixel, do u */
-	      u = (- FIX(0.16874) * r - FIX(0.33126) * g + FIX(0.5) * b + CBCR_OFFSET + ONE_HALF-1) >> SCALEBITS;
-	      *out = ( (uint16_t) u << 8) | (uint16_t) y;
-	    } else {
-	      /* odd pixel, do v */
-	      v = (FIX(0.5) * r - FIX(0.41869) * g - FIX(0.08131) * b + CBCR_OFFSET + ONE_HALF-1) >> SCALEBITS;
-	      *out = ( (uint16_t) v << 8) | (uint16_t) y;
-	    }
+	      y = (FIX(0.299) * r + FIX(0.587) * g + FIX(0.114) * b + ONE_HALF) >> SCALEBITS;
+	      if (!(col & 0x0001)) {
+	        /* even pixel, do u */
+	        u = (- FIX(0.16874) * r - FIX(0.33126) * g + FIX(0.5) * b + CBCR_OFFSET + ONE_HALF-1) >> SCALEBITS;
+	        *out = ( (uint16_t) u << 8) | (uint16_t) y;
+	      } else {
+	        /* odd pixel, do v */
+	        v = (FIX(0.5) * r - FIX(0.41869) * g - FIX(0.08131) * b + CBCR_OFFSET + ONE_HALF-1) >> SCALEBITS;
+	        *out = ( (uint16_t) v << 8) | (uint16_t) y;
+	      }
 #endif
-	    //printf("r %02x g %02x b %02x y %02x u %02x v %02x\n",r,g,b,y,u,v);
+	      //printf("r %02x g %02x b %02x y %02x u %02x v %02x\n",r,g,b,y,u,v);
+	    }
 	  }
-	}
-
-	xine_profiler_stop_count (this->prof_rgb2yuv);
-      }
-
-      img->PTS = buf->PTS;
-      img->SCR = buf->SCR;
-      if(ret) {
-	printf("w32codec: Error decompressing frame, err=%ld\n", (long)ret); 
-	img->bad_frame = 1;
-      } else {
-	img->bad_frame = 0;
+  
+	  xine_profiler_stop_count (this->prof_rgb2yuv);
+        }
       }
       
-      if (img->copy) {
+      img->pts = buf->pts;
+      if (ret || this->skipframes) {
+        if (!this->skipframes)
+	  printf("w32codec: Error decompressing frame, err=%ld\n", (long)ret); 
+	img->bad_frame = 1;
+#ifdef LOG
+	printf ("w32codec: BAD FRAME, duration is %lld\n", img->duration);
+#endif
+      } else {
+	img->bad_frame = 0;
+#ifdef LOG
+	printf ("w32codec: GOOD FRAME, duration is %lld\n\n", img->duration);
+#endif
+      }
+      
+      if (img->copy && !this->skipframes) {
 	int height = abs(this->o_bih.biHeight);
 	int stride = this->o_bih.biWidth;
 	uint8_t* src[3];
@@ -729,7 +801,14 @@ static void w32v_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 	}
       }
 
-      img->draw(img);
+      this->skipframes = img->draw(img);
+
+#ifdef LOG
+      printf ("w32codec: skipframes is %d\n", this->skipframes);
+#endif
+
+      if (this->skipframes < 0)
+        this->skipframes = 0;
       img->free(img);
 
       this->size = 0;
@@ -742,16 +821,57 @@ static void w32v_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 static void w32v_flush (video_decoder_t *this_gen) {
 }
 
+static void w32v_reset (video_decoder_t *this_gen) {
+
+  w32v_decoder_t *this = (w32v_decoder_t *) this_gen;
+  
+  /* FIXME: need to improve this function. currently it
+     doesn't avoid artifacts when seeking. */
+  
+  pthread_mutex_lock(&win32_codec_mutex);
+  if ( !this->ds_driver ) {
+    if( this->hic )
+    {
+      if (!this->ex_functions) 
+        ICDecompressBegin(this->hic, &this->bih, &this->o_bih);
+      else
+        ICDecompressBeginEx(this->hic, &this->bih, &this->o_bih);
+    }
+  } else {
+  }
+  this->size = 0;
+  pthread_mutex_unlock(&win32_codec_mutex);
+}
+
+
 static void w32v_close (video_decoder_t *this_gen) {
 
   w32v_decoder_t *this = (w32v_decoder_t *) this_gen;
 
-  if ( !this->ds_driver ) {
-  } else {
-    if( this->ds_dec )
-      DS_VideoDecoder_Destroy(this->ds_dec);
+  pthread_mutex_lock(&win32_codec_mutex);
+#ifdef SYNC_SHUTDOWN
+  if( !w32a_instance || (w32a_instance && !w32a_instance->decoder_ok) )
+#endif
+  {
+    if ( !this->ds_driver ) {
+      if( this->hic ) {
+        ICDecompressEnd(this->hic);
+        ICClose(this->hic);
+      }
+    } else {
+      if( this->ds_dec )
+        DS_VideoDecoder_Destroy(this->ds_dec);
+    }
+    w32v_instance = NULL;
+    Restore_LDT_Keeper( this->ldt_fs );
   }
-  
+  pthread_mutex_unlock(&win32_codec_mutex);
+
+#ifdef SYNC_SHUTDOWN
+  if( w32a_instance && !w32a_instance->decoder_ok )
+    w32a_close ((audio_decoder_t *)w32a_instance);
+#endif
+
   if ( this->img_buffer ) {
     free (this->img_buffer);
     this->img_buffer = NULL;
@@ -761,16 +881,22 @@ static void w32v_close (video_decoder_t *this_gen) {
     free (this->buf);
     this->buf = NULL;
   }
-  
-  this->video_out->close(this->video_out);
-  
-  Restore_LDT_Keeper( this->ldt_fs );
+
+  if( this->decoder_ok )
+  {  
+    this->decoder_ok = 0;
+    this->video_out->close(this->video_out);
+  }
 }
 
 static char *w32v_get_id(void) {
   return "vfw (win32) video decoder";
 }
 
+static void w32v_dispose (video_decoder_t *this_gen) {
+  free (this_gen);
+}
+
 /*
  * audio stuff
  */
@@ -844,21 +970,20 @@ static void w32a_init (audio_decoder_t *this_gen, ao_instance_t *audio_out) {
   this->outbuf = NULL;
 }
 
-static int w32a_init_audio (w32a_decoder_t *this,
-			    WAVEFORMATEX *in_fmt_,
-			    int buf_type) {
+static int w32a_init_audio (w32a_decoder_t *this, buf_element_t *buf ) {
 
   HRESULT ret;
   void *ret2;
   static WAVEFORMATEX wf;     
   static WAVEFORMATEX *in_fmt;
-  unsigned long in_size=in_fmt_->nBlockAlign;
+  unsigned long in_size;
   unsigned long out_size;
   audio_buffer_t *audio_buffer;
+  int audio_buffer_mem_size;
 
-  in_fmt = (WAVEFORMATEX *) malloc (64);
-
-  memcpy (in_fmt, in_fmt_, sizeof (WAVEFORMATEX) + in_fmt_->cbSize);
+  in_fmt = (WAVEFORMATEX *) malloc (buf->size);
+  memcpy (in_fmt, buf->content, buf->size);
+  in_size=in_fmt->nBlockAlign;
 
   this->srcstream = 0;
   this->num_channels  = in_fmt->nChannels;
@@ -874,9 +999,13 @@ static int w32a_init_audio (w32a_decoder_t *this,
     printf("w32codec: (ACM_Decoder) Cannot open audio output device\n");
     return 0;
   }
+  
   audio_buffer = this->audio_out->get_buffer (this->audio_out);
+  audio_buffer_mem_size = audio_buffer->mem_size;
+  audio_buffer->num_frames = 0;
+  audio_buffer->vpts       = 0;
+  this->audio_out->put_buffer (this->audio_out, audio_buffer);
 
-  this->ldt_fs = Setup_LDT_Keeper();
   
   wf.nChannels       = in_fmt->nChannels;
   wf.nSamplesPerSec  = in_fmt->nSamplesPerSec;
@@ -886,44 +1015,44 @@ static int w32a_init_audio (w32a_decoder_t *this,
   wf.wBitsPerSample  = 16;
   wf.cbSize          = 0;
   
-  pthread_mutex_lock(&win32_codec_name_mutex);
-  win32_codec_name = get_auds_codec_name (this, buf_type);
+  pthread_mutex_lock(&win32_codec_mutex);
+  this->ldt_fs = Setup_LDT_Keeper();
+  win32_codec_name = get_auds_codec_name (this, buf->type);
   
   if( !this->ds_driver ) {
     ret=acmStreamOpen(&this->srcstream,(HACMDRIVER)NULL,
                       in_fmt,
                       &wf,
                       NULL,0,0,0);
-    pthread_mutex_unlock(&win32_codec_name_mutex);
-
     if(ret){
       if(ret==ACMERR_NOTPOSSIBLE)
         printf("w32codec: (ACM_Decoder) Unappropriate audio format\n");
       else
         printf("w32codec: (ACM_Decoder) acmStreamOpen error %d", (int) ret);
       this->srcstream = 0;
+      pthread_mutex_unlock(&win32_codec_mutex);
       return 0;
     }
 
     acmStreamSize(this->srcstream, in_size, &out_size, ACM_STREAMSIZEF_SOURCE);
     out_size*=2;
-    if(out_size < audio_buffer->mem_size) 
-      out_size=audio_buffer->mem_size;
+    if(out_size < audio_buffer_mem_size) 
+      out_size=audio_buffer_mem_size;
     printf("w32codec: Audio buffer min. size: %d\n",(int)out_size);
   
     acmStreamSize(this->srcstream, out_size, (LPDWORD) &this->rec_audio_src_size, 
       ACM_STREAMSIZEF_DESTINATION);
   } else {
     ret2 = this->ds_dec=DS_AudioDecoder_Open(win32_codec_name,this->guid, in_fmt);
-    pthread_mutex_unlock(&win32_codec_name_mutex);
     
     if( ret2 == NULL ) {
       printf("w32codec: Error initializing DirectShow Audio\n");
       this->srcstream = 0;
+      pthread_mutex_unlock(&win32_codec_mutex);
       return 0;
     }
     
-    out_size = audio_buffer->mem_size;  
+    out_size = audio_buffer_mem_size;  
     printf("w32codec: output buffer size: %d\n",(int)out_size);
     this->rec_audio_src_size=DS_AudioDecoder_GetSrcSize(this->ds_dec,out_size);
     
@@ -932,6 +1061,7 @@ static int w32a_init_audio (w32a_decoder_t *this,
        impossible */
     this->rec_audio_src_size*=2; 
   }
+  pthread_mutex_unlock(&win32_codec_mutex);
   printf("w32codec: Recommended source buffer size: %d\n", this->rec_audio_src_size); 
 
   if( this->buf )
@@ -963,8 +1093,7 @@ static void w32a_decode_audio (w32a_decoder_t *this,
 			       unsigned char *data, 
 			       uint32_t size, 
 			       int frame_end, 
-			       uint32_t pts,
-			       uint32_t scr) {
+			       int64_t pts) {
 
   static ACMSTREAMHEADER ash;
   HRESULT hr;
@@ -975,10 +1104,9 @@ static void w32a_decode_audio (w32a_decoder_t *this,
      issues with some streams.
   */
   
-  /* buffer empty -> take pts/scr from package */
+  /* buffer empty -> take pts from package */
   if( !this->size ) {
     this->pts = pts;
-    this->scr = scr;
     /*
     printf("w32codec: resync pts (%d)\n",this->pts);
     */
@@ -992,7 +1120,6 @@ static void w32a_decode_audio (w32a_decoder_t *this,
   /* force resync every 4 seconds */
   if( this->sumpts >= 4 * 90000 && pts ) {
     this->pts = pts - this->size * this->sumpts / this->sumsize;
-    this->scr = scr - this->size * this->sumpts / this->sumsize; 
     /*
     printf("w32codec: estimated resync pts (%d)\n",this->pts);
     */
@@ -1023,18 +1150,19 @@ static void w32a_decode_audio (w32a_decoder_t *this,
     ash.pbDst=this->outbuf;
     ash.cbDstLength=this->outsize;
     
-    /*
+#ifdef LOG
     printf ("decoding %d of %d bytes (%02x %02x %02x %02x ... %02x %02x)\n", 
 	    this->rec_audio_src_size, this->size,
 	    this->buf[0], this->buf[1], this->buf[2], this->buf[3],
 	    this->buf[this->rec_audio_src_size-2], this->buf[this->rec_audio_src_size-1]); 
-    */
+#endif
     
+    pthread_mutex_lock(&win32_codec_mutex);
     if( !this->ds_driver ) {
       hr=acmStreamPrepareHeader(this->srcstream,&ash,0);
       if(hr){
         printf("w32codec: (ACM_Decoder) acmStreamPrepareHeader error %d\n",(int)hr);
-
+        pthread_mutex_unlock(&win32_codec_mutex);
         return;
       }
       
@@ -1046,18 +1174,20 @@ static void w32a_decode_audio (w32a_decoder_t *this,
        ash.cbSrcLengthUsed = size_read;
        ash.cbDstLengthUsed = size_written;
     }
+    pthread_mutex_unlock(&win32_codec_mutex);
    
     if(hr){
-      printf("w32codec: stream convert error %d, used %d bytes\n",(int)hr,(int)ash.cbSrcLengthUsed);
+      printf ("w32codec: stream convert error %d, used %d bytes\n",
+	      (int)hr,(int)ash.cbSrcLengthUsed);
       this->size-=ash.cbSrcLength;
     } else {
       int DstLengthUsed, bufsize;
       audio_buffer_t *audio_buffer;
       char *p;
-      /*
+#ifdef LOG
       printf ("acmStreamConvert worked, used %d bytes, generated %d bytes\n",
 	      ash.cbSrcLengthUsed, ash.cbDstLengthUsed);
-      */
+#endif
       DstLengthUsed = ash.cbDstLengthUsed;
       p = this->outbuf;
       
@@ -1076,11 +1206,10 @@ static void w32a_decode_audio (w32a_decoder_t *this,
         */
 	audio_buffer->num_frames = bufsize / (this->num_channels*2);
 	audio_buffer->vpts       = this->pts;
-	audio_buffer->scr        = this->scr;
 
 	this->audio_out->put_buffer (this->audio_out, audio_buffer);
         
-	this->pts = this->scr = 0;
+	this->pts = 0;
         DstLengthUsed -= bufsize;
 	p += bufsize;
       }
@@ -1092,12 +1221,14 @@ static void w32a_decode_audio (w32a_decoder_t *this,
       xine_fast_memcpy( this->buf, &this->buf [ash.cbSrcLengthUsed], this->size);
     }
 
+    pthread_mutex_lock(&win32_codec_mutex);
     if( !this->ds_driver ) {
       hr=acmStreamUnprepareHeader(this->srcstream,&ash,0);
       if(hr){
         printf("w32codec: (ACM_Decoder) acmStreamUnprepareHeader error %d\n",(int)hr);
       }
     }
+    pthread_mutex_unlock(&win32_codec_mutex);
   }
 }
 
@@ -1105,17 +1236,33 @@ static void w32a_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 
   w32a_decoder_t *this = (w32a_decoder_t *) this_gen;
 
-  if (buf->decoder_info[0] == 0) {
+  if (buf->decoder_flags & BUF_FLAG_PREVIEW)
+    return;
+  
+  if (buf->decoder_flags & BUF_FLAG_HEADER) {
     /* init package containing bih */
 
-    this->decoder_ok = w32a_init_audio (this, (WAVEFORMATEX *)buf->content, buf->type);
+#ifdef LOG
+    printf ("w32codec: got audio header\n");
+#endif
+
+    this->decoder_ok = w32a_init_audio (this, buf);
+
+#ifdef SYNC_SHUTDOWN
+    if( this->decoder_ok )
+      w32a_instance = this;
+#endif
   } else if (this->decoder_ok) {
+#ifdef LOG
+    printf ("w32codec: decoding %d data bytes...\n", buf->size);
+#endif
+
     if( (int)buf->size <= 0 )
       return;
 
     w32a_decode_audio (this, buf->content, buf->size,
-		       buf->decoder_info[0]==2, 
-		       buf->PTS, buf->SCR);
+		       buf->decoder_flags & BUF_FLAG_FRAME_END, 
+		       buf->pts);
   }
 }
 
@@ -1123,42 +1270,63 @@ static void w32a_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 static void w32a_close (audio_decoder_t *this_gen) {
 
   w32a_decoder_t *this = (w32a_decoder_t *) this_gen;
-
-  if( !this->ds_driver ) {
-    if( this->srcstream )
-      acmStreamClose(this->srcstream, 0);
-  } else {
-    if( this->ds_dec )
-      DS_AudioDecoder_Destroy(this->ds_dec);
-  }
-
-  Restore_LDT_Keeper(this->ldt_fs);
   
-  if (this->output_open) {
-    this->audio_out->close (this->audio_out);
-    this->output_open = 0;
+  pthread_mutex_lock(&win32_codec_mutex);
+#ifdef SYNC_SHUTDOWN
+  if( !w32v_instance || (w32v_instance && !w32v_instance->decoder_ok) )
+#endif
+  {
+    if( !this->ds_driver ) {
+      if( this->srcstream )
+        acmStreamClose(this->srcstream, 0);
+    } else {
+      if( this->ds_dec )
+        DS_AudioDecoder_Destroy(this->ds_dec);
+    }
+
+    w32a_instance = NULL;
+    Restore_LDT_Keeper(this->ldt_fs);
   }
-  
+  pthread_mutex_unlock(&win32_codec_mutex);
+
+#ifdef SYNC_SHUTDOWN
+  if( w32v_instance && !w32v_instance->decoder_ok )
+    w32v_close ((video_decoder_t *)w32v_instance);
+#endif
+    
   if( this->buf ) {
     free(this->buf);
     this->buf = NULL;
   }
+  
   if( this->outbuf ) {
     free(this->outbuf);
     this->outbuf = NULL;
   }
+  
+  this->decoder_ok = 0;
+  
+  if (this->output_open) {
+    this->audio_out->close (this->audio_out);
+    this->output_open = 0;
+  }
 }
 
 static char *w32a_get_id(void) {
   return "vfw (win32) audio decoder";
 }
 
+static void init_routine(void) {
+  pthread_mutex_init (&win32_codec_mutex, NULL);
+  w32v_init_rgb_ycc();
+}
+
 video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
 
   w32v_decoder_t *this ;
   config_values_t *cfg;
 
-  if (iface_version != 5) {
+  if (iface_version != 7) {
     printf( "w32codec: plugin doesn't support plugin API version %d.\n"
 	    "w32codec: this means there's a version mismatch between xine and this "
 	    "w32codec: decoder plugin.\nInstalling current input plugins should help.\n",
@@ -1178,23 +1346,33 @@ video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
   this->video_decoder.init                = w32v_init;
   this->video_decoder.decode_data         = w32v_decode_data;
   this->video_decoder.flush               = w32v_flush;
+  this->video_decoder.reset               = w32v_reset;
   this->video_decoder.close               = w32v_close;
   this->video_decoder.get_identifier      = w32v_get_id;
+  this->video_decoder.dispose             = w32v_dispose;
   this->video_decoder.priority            = 1;
 
-  pthread_mutex_init (&win32_codec_name_mutex, NULL);
+  pthread_once (&once_control, init_routine);
   
   this->prof_rgb2yuv = xine_profiler_allocate_slot ("w32codec rgb2yuv convert");
 
+#ifdef SYNC_SHUTDOWN
+  w32v_instance = NULL;
+#endif
+
   return (video_decoder_t *) this;
 }
 
+static void w32a_dispose (audio_decoder_t *this_gen) {
+  free (this_gen);
+}
+
 audio_decoder_t *init_audio_decoder_plugin (int iface_version, xine_t *xine) {
 
   w32a_decoder_t *this ;
   config_values_t *cfg;
   
-  if (iface_version != 5) {
+  if (iface_version != 7) {
     printf( "w32codec: plugin doesn't support plugin API version %d.\n"
 	    "w32codec: this means there's a version mismatch between xine and this "
 	    "w32codec: decoder plugin.\nInstalling current input plugins should help.\n",
@@ -1217,9 +1395,14 @@ audio_decoder_t *init_audio_decoder_plugin (int iface_version, xine_t *xine) {
   this->audio_decoder.reset               = w32a_reset;
   this->audio_decoder.close               = w32a_close;
   this->audio_decoder.get_identifier      = w32a_get_id;
+  this->audio_decoder.dispose             = w32a_dispose;
   this->audio_decoder.priority            = 1;
   
-  pthread_mutex_init (&win32_codec_name_mutex, NULL);
+  pthread_once (&once_control, init_routine);
+
+#ifdef SYNC_SHUTDOWN
+  w32a_instance = NULL;
+#endif
   
   return (audio_decoder_t *) this;
 }
