@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: yuv2rgb.c,v 1.27 2001/11/17 14:26:39 f1rmb Exp $
+ * $Id: yuv2rgb.c,v 1.29 2002/02/26 22:53:58 guenter Exp $
  */
 
 #include "config.h"
@@ -68,11 +68,11 @@ static void *my_malloc_aligned (size_t alignment, size_t size, void **chunk) {
 }
 
 
-int yuv2rgb_setup (yuv2rgb_t *this, 
-		   int source_width, int source_height,
-		   int y_stride, int uv_stride,
-		   int dest_width, int dest_height,
-		   int rgb_stride) {
+static int yuv2rgb_configure (yuv2rgb_t *this, 
+			      int source_width, int source_height,
+			      int y_stride, int uv_stride,
+			      int dest_width, int dest_height,
+			      int rgb_stride) {
   /*
   printf ("yuv2rgb setup (%d x %d => %d x %d)\n", source_width, source_height,
 	  dest_width, dest_height);
@@ -147,6 +147,7 @@ int yuv2rgb_setup (yuv2rgb_t *this,
 
 static void scale_line_gen (uint8_t *source, uint8_t *dest,
 			    int width, int step) {
+
   /*
    * scales a yuv source row to a dest row, with interpolation
    * (good quality, but slow)
@@ -161,22 +162,75 @@ static void scale_line_gen (uint8_t *source, uint8_t *dest,
   p2 = *source++;
   dx = 0;
 
-  while (width) {
+  /*
+   * the following code has been optimized by Scott Smith <ssmith@akamai.com>:
+   *
+   * ok now I have a meaningful optimization for yuv2rgb.c:scale_line_gen.
+   * it removes the loop from within the while() loop by separating it out
+   * into 3 cases: where you are enlarging the line (<32768), where you are
+   * between 50% and 100% of the original line (<=65536), and where you are
+   * shrinking it by a lot.  anyways, I went from 200 delivered / 100+
+   * skipped to 200 delivered / 80 skipped for the enlarging case.  I
+   * noticed when looking at the assembly that the compiler was able to
+   * unroll these while(width) loops, whereas before it was trying to
+   * unroll the while(dx>32768) loops.  so the compiler is better able to
+   * deal with this code.
+   */
 
-    *dest = (p1 * (32768 - dx) + p2 * dx) / 32768;
 
-    dx += step;
-    while (dx > 32768) {
-      dx -= 32768;
-      p1 = p2;
-      p2 = *source++;
+  if (step < 32768) {
+    while (width) {
+      *dest = p1 + (((p2-p1) * dx)>>15);
+
+      dx += step;
+      if (dx > 32768) {
+	dx -= 32768;
+	p1 = p2;
+	p2 = *source++;
+      }
+      
+      dest ++;
+      width --;
     }
+  } else if (step <= 65536) {
+    while (width) {
+      *dest = p1 + (((p2-p1) * dx)>>15);
 
-    dest ++;
-    width --;
+      dx += step;
+      if (dx > 65536) {
+	dx -= 65536;
+	p1 = *source++;
+	p2 = *source++;
+      } else {
+	dx -= 32768;
+	p1 = p2;
+	p2 = *source++;
+      }
+      
+      dest ++;
+      width --;
+    }
+  } else {
+    while (width) {
+      int offs;
+
+      *dest = p1 + (((p2-p1) * dx)>>15);
+
+      dx += step;
+      offs=((dx-1)>>15);
+      dx-=offs<<15;
+      source+=offs-2;
+      p1=*source++;
+      p2=*source++;
+      dest ++;
+      width --;
+    }
   }
-
   xine_profiler_stop_count(prof_scale_line);
+
+
+
+
 }
 
 /*
@@ -1265,15 +1319,15 @@ static void scale_line_4 (uint8_t *source, uint8_t *dest,
 
 #define DST1CMAP(i)							\
 	Y = py_1[2*i];                          			\
-	dst_1[2*i] = this->fast_rgb[r[Y] + g[Y] + b[Y]];		\
+	dst_1[2*i] = this->cmap[r[Y] + g[Y] + b[Y]];		\
 	Y = py_1[2*i+1];						\
-	dst_1[2*i+1] = this->fast_rgb[r[Y] + g[Y] + b[Y]];
+	dst_1[2*i+1] = this->cmap[r[Y] + g[Y] + b[Y]];
 
 #define DST2CMAP(i)							\
 	Y = py_2[2*i];							\
-	dst_2[2*i] = this->fast_rgb[r[Y] + g[Y] + b[Y]];		\
+	dst_2[2*i] = this->cmap[r[Y] + g[Y] + b[Y]];		\
 	Y = py_2[2*i+1];						\
-	dst_2[2*i+1] = this->fast_rgb[r[Y] + g[Y] + b[Y]];
+	dst_2[2*i+1] = this->cmap[r[Y] + g[Y] + b[Y]];
 
 static void yuv2rgb_c_32 (yuv2rgb_t *this, uint8_t * _dst,
 			  uint8_t * _py, uint8_t * _pu, uint8_t * _pv)
@@ -1298,7 +1352,7 @@ static void yuv2rgb_c_32 (yuv2rgb_t *this, uint8_t * _dst,
     dy = 0;
     dst_height = this->dest_height;
 
-    for (height = 0;; height++) {
+    for (height = 0;; ) {
       dst_1 = (uint32_t*)_dst;
       py_1  = this->y_buffer;
       pu    = this->u_buffer;
@@ -1339,22 +1393,25 @@ static void yuv2rgb_c_32 (yuv2rgb_t *this, uint8_t * _dst,
       if (dst_height <= 0)
 	break;
 
-      dy -= 32768;
-      _py += this->y_stride;
+      do {
+          dy -= 32768;
+          _py += this->y_stride;
 
-      scale_line (_py, this->y_buffer, 
-		  this->dest_width, this->step_dx);
+          scale_line (_py, this->y_buffer, 
+                      this->dest_width, this->step_dx);
 
-      if (height & 1) {
-	_pu += this->uv_stride;
-	_pv += this->uv_stride;
+          if (height & 1) {
+              _pu += this->uv_stride;
+              _pv += this->uv_stride;
 	  
-	scale_line (_pu, this->u_buffer,
-		    this->dest_width >> 1, this->step_dx);
-	scale_line (_pv, this->v_buffer,
-		    this->dest_width >> 1, this->step_dx);
+              scale_line (_pu, this->u_buffer,
+                          this->dest_width >> 1, this->step_dx);
+              scale_line (_pv, this->v_buffer,
+                          this->dest_width >> 1, this->step_dx);
 	  
-      }
+          }
+          height++;
+      } while( dy>=32768);
     }
   } else {
     height = this->source_height >> 1;
@@ -1426,7 +1483,7 @@ static void yuv2rgb_c_24_rgb (yuv2rgb_t *this, uint8_t * _dst,
     dy = 0;
     dst_height = this->dest_height;
 
-    for (height = 0;; height++) {
+    for (height = 0;; ) {
       dst_1 = _dst;
       py_1  = this->y_buffer;
       pu    = this->u_buffer;
@@ -1467,22 +1524,25 @@ static void yuv2rgb_c_24_rgb (yuv2rgb_t *this, uint8_t * _dst,
       if (dst_height <= 0)
 	break;
 
-      dy -= 32768;
-      _py += this->y_stride;
+      do {
+          dy -= 32768;
+          _py += this->y_stride;
 
-      scale_line (_py, this->y_buffer, 
-		  this->dest_width, this->step_dx);
+          scale_line (_py, this->y_buffer, 
+                      this->dest_width, this->step_dx);
 
-      if (height & 1) {
-	_pu += this->uv_stride;
-	_pv += this->uv_stride;
+          if (height & 1) {
+              _pu += this->uv_stride;
+              _pv += this->uv_stride;
 	  
-	scale_line (_pu, this->u_buffer,
-		    this->dest_width >> 1, this->step_dx);
-	scale_line (_pv, this->v_buffer,
-		    this->dest_width >> 1, this->step_dx);
+              scale_line (_pu, this->u_buffer,
+                          this->dest_width >> 1, this->step_dx);
+              scale_line (_pv, this->v_buffer,
+                          this->dest_width >> 1, this->step_dx);
 	  
-      }
+          }
+          height++;
+      } while (dy>=32768);
     }
   } else {
     height = this->source_height >> 1;
@@ -1554,7 +1614,7 @@ static void yuv2rgb_c_24_bgr (yuv2rgb_t *this, uint8_t * _dst,
     dy = 0;
     dst_height = this->dest_height;
 
-    for (height = 0;; height++) {
+    for (height = 0;; ) {
       dst_1 = _dst;
       py_1  = this->y_buffer;
       pu    = this->u_buffer;
@@ -1595,22 +1655,25 @@ static void yuv2rgb_c_24_bgr (yuv2rgb_t *this, uint8_t * _dst,
       if (dst_height <= 0)
 	break;
 
-      dy -= 32768;
-      _py += this->y_stride;
+      do {
+          dy -= 32768;
+          _py += this->y_stride;
 
-      scale_line (_py, this->y_buffer, 
-		  this->dest_width, this->step_dx);
+          scale_line (_py, this->y_buffer, 
+                      this->dest_width, this->step_dx);
 
-      if (height & 1) {
-	_pu += this->uv_stride;
-	_pv += this->uv_stride;
+          if (height & 1) {
+              _pu += this->uv_stride;
+              _pv += this->uv_stride;
 	  
-	scale_line (_pu, this->u_buffer,
-		    this->dest_width >> 1, this->step_dx);
-	scale_line (_pv, this->v_buffer,
-		    this->dest_width >> 1, this->step_dx);
+              scale_line (_pu, this->u_buffer,
+                          this->dest_width >> 1, this->step_dx);
+              scale_line (_pv, this->v_buffer,
+                          this->dest_width >> 1, this->step_dx);
 	  
-      }
+          }
+          height++;
+      } while( dy>=32768 );
     }
 
   } else {
@@ -1682,7 +1745,7 @@ static void yuv2rgb_c_16 (yuv2rgb_t *this, uint8_t * _dst,
     dy = 0;
     dst_height = this->dest_height;
 
-    for (height = 0;; height++) {
+    for (height = 0;; ) {
       dst_1 = (uint16_t*)_dst;
       py_1  = this->y_buffer;
       pu    = this->u_buffer;
@@ -1723,22 +1786,25 @@ static void yuv2rgb_c_16 (yuv2rgb_t *this, uint8_t * _dst,
       if (dst_height <= 0)
 	break;
 
-      dy -= 32768;
-      _py += this->y_stride;
+      do {
+          dy -= 32768;
+          _py += this->y_stride;
 
-      scale_line (_py, this->y_buffer, 
-		  this->dest_width, this->step_dx);
+          scale_line (_py, this->y_buffer, 
+                      this->dest_width, this->step_dx);
 
-      if (height & 1) {
-	_pu += this->uv_stride;
-	_pv += this->uv_stride;
+          if (height & 1) {
+              _pu += this->uv_stride;
+              _pv += this->uv_stride;
 	  
-	scale_line (_pu, this->u_buffer,
-		    this->dest_width >> 1, this->step_dx);
-	scale_line (_pv, this->v_buffer,
-		    this->dest_width >> 1, this->step_dx);
+              scale_line (_pu, this->u_buffer,
+                          this->dest_width >> 1, this->step_dx);
+              scale_line (_pv, this->v_buffer,
+                          this->dest_width >> 1, this->step_dx);
 	  
-      }
+          }
+          height++;
+      } while( dy>=32768);
     }
   } else {
     height = this->source_height >> 1;
@@ -1809,7 +1875,7 @@ static void yuv2rgb_c_8 (yuv2rgb_t *this, uint8_t * _dst,
     dy = 0;
     dst_height = this->dest_height;
 
-    for (height = 0;; height++) {
+    for (height = 0;; ) {
       dst_1 = (uint8_t*)_dst;
       py_1  = this->y_buffer;
       pu    = this->u_buffer;
@@ -1850,22 +1916,25 @@ static void yuv2rgb_c_8 (yuv2rgb_t *this, uint8_t * _dst,
       if (dst_height <= 0)
 	break;
 
-      dy -= 32768;
-      _py += this->y_stride;
+      do {
+          dy -= 32768;
+          _py += this->y_stride;
 
-      scale_line (_py, this->y_buffer, 
-		  this->dest_width, this->step_dx);
+          scale_line (_py, this->y_buffer, 
+                      this->dest_width, this->step_dx);
 
-      if (height & 1) {
-	_pu += this->uv_stride;
-	_pv += this->uv_stride;
+          if (height & 1) {
+              _pu += this->uv_stride;
+              _pv += this->uv_stride;
 	  
-	scale_line (_pu, this->u_buffer,
-		    this->dest_width >> 1, this->step_dx);
-	scale_line (_pv, this->v_buffer,
-		    this->dest_width >> 1, this->step_dx);
+              scale_line (_pu, this->u_buffer,
+                          this->dest_width >> 1, this->step_dx);
+              scale_line (_pv, this->v_buffer,
+                          this->dest_width >> 1, this->step_dx);
 	  
-      }
+          }
+          height++;
+      } while( dy>=32768 );
     }
   } else {
     height = this->source_height >> 1;
@@ -1942,8 +2011,11 @@ static void yuv2rgb_c_gray (yuv2rgb_t *this, uint8_t * _dst,
       if (dst_height <= 0)
 	break;
 
-      dy -= 32768;
-      _py += this->y_stride;
+      _py += this->y_stride*(dy>>15);
+      dy &= 32767;
+      /* dy -= 32768; 
+	 _py += this->y_stride;
+      */
     }
   } else {
     for (height = this->source_height; --height >= 0; ) {
@@ -1978,7 +2050,7 @@ static void yuv2rgb_c_palette (yuv2rgb_t *this, uint8_t * _dst,
     dy = 0;
     dst_height = this->dest_height;
 
-    for (height = 0;; height++) {
+    for (height = 0;; ) {
       dst_1 = _dst;
       py_1  = this->y_buffer;
       pu    = this->u_buffer;
@@ -2019,22 +2091,25 @@ static void yuv2rgb_c_palette (yuv2rgb_t *this, uint8_t * _dst,
       if (dst_height <= 0)
 	break;
 
-      dy -= 32768;
-      _py += this->y_stride;
+      do {
+          dy -= 32768;
+          _py += this->y_stride;
 
-      scale_line (_py, this->y_buffer,
-		  this->dest_width, this->step_dx);
+          scale_line (_py, this->y_buffer,
+                      this->dest_width, this->step_dx);
 
-      if (height & 1) {
-	_pu += this->uv_stride;
-	_pv += this->uv_stride;
+          if (height & 1) {
+              _pu += this->uv_stride;
+              _pv += this->uv_stride;
 	  
-	scale_line (_pu, this->u_buffer,
-		    this->dest_width >> 1, this->step_dx);
-	scale_line (_pv, this->v_buffer,
-		    this->dest_width >> 1, this->step_dx);
+              scale_line (_pu, this->u_buffer,
+                          this->dest_width >> 1, this->step_dx);
+              scale_line (_pv, this->v_buffer,
+                          this->dest_width >> 1, this->step_dx);
 	  
-      }
+          }
+          height++;
+      } while( dy>=32768 );
     }
   } else {
     height = this->source_height >> 1;
@@ -2088,7 +2163,7 @@ static int div_round (int dividend, int divisor)
     return -((-dividend + (divisor>>1)) / divisor);
 }
 
-static void yuv2rgb_setup_tables (yuv2rgb_t *this, int mode, int swapped) 
+static void yuv2rgb_setup_tables (yuv2rgb_factory_t *this, int mode, int swapped) 
 {
   int i;
   uint8_t table_Y[1024];
@@ -2334,13 +2409,13 @@ static uint32_t yuv2rgb_single_pixel_palette (yuv2rgb_t *this, uint8_t y, uint8_
   g = (void *) (((uint8_t *)this->table_gU[u]) + this->table_gV[v]);
   b = this->table_bU[u];
 
-  return this->fast_rgb[r[y] + g[y] + b[y]];
+  return this->cmap[r[y] + g[y] + b[y]];
 }
 
 
-static void yuv2rgb_c_init (yuv2rgb_t *this, int mode, int swapped)
+static void yuv2rgb_c_init (yuv2rgb_factory_t *this)
 {
-  switch (mode) {
+  switch (this->mode) {
   case MODE_32_RGB:
   case MODE_32_BGR:
     this->yuv2rgb_fun = yuv2rgb_c_32;
@@ -2349,7 +2424,7 @@ static void yuv2rgb_c_init (yuv2rgb_t *this, int mode, int swapped)
   case MODE_24_RGB:
   case MODE_24_BGR:
     this->yuv2rgb_fun =
-	(mode==MODE_24_RGB && !swapped) || (mode==MODE_24_BGR && swapped)
+	(this->mode==MODE_24_RGB && !this->swapped) || (this->mode==MODE_24_BGR && this->swapped)
 	    ? yuv2rgb_c_24_rgb
 	    : yuv2rgb_c_24_bgr;
     break;
@@ -2375,15 +2450,15 @@ static void yuv2rgb_c_init (yuv2rgb_t *this, int mode, int swapped)
     break;
 
   default:
-    fprintf (stderr, "mode %d not supported by yuv2rgb\n", mode);
+    printf ("yuv2rgb: mode %d not supported by yuv2rgb\n", this->mode);
     exit (1);
   }
 
 }
 
-static void yuv2rgb_single_pixel_init (yuv2rgb_t *this, int mode, int swapped)
-{
-  switch (mode) {
+static void yuv2rgb_single_pixel_init (yuv2rgb_factory_t *this) {
+
+  switch (this->mode) {
   case MODE_32_RGB:
   case MODE_32_BGR:
     this->yuv2rgb_single_pixel_fun = yuv2rgb_single_pixel_32;
@@ -2392,7 +2467,7 @@ static void yuv2rgb_single_pixel_init (yuv2rgb_t *this, int mode, int swapped)
   case MODE_24_RGB:
   case MODE_24_BGR:
     this->yuv2rgb_single_pixel_fun =
-	(mode==MODE_24_RGB && !swapped) || (mode==MODE_24_BGR && swapped)
+	(this->mode==MODE_24_RGB && !this->swapped) || (this->mode==MODE_24_BGR && this->swapped)
 	    ? yuv2rgb_single_pixel_24_rgb
 	    : yuv2rgb_single_pixel_24_bgr;
     break;
@@ -2418,7 +2493,7 @@ static void yuv2rgb_single_pixel_init (yuv2rgb_t *this, int mode, int swapped)
     break;
 
   default:
-    fprintf (stderr, "mode %d not supported by yuv2rgb\n", mode);
+    printf ("yuv2rgb: mode %d not supported by yuv2rgb\n", this->mode);
     exit (1);
   }
 }
@@ -2491,8 +2566,12 @@ static void yuy22rgb_c_32 (yuv2rgb_t *this, uint8_t * _dst, uint8_t * _p)
     if (height <= 0)
       break;
 
-    dy -= 32768;
-    _p += this->y_stride*2;
+    _p += this->y_stride*2*(dy>>15);
+    dy &= 32767;
+    /*
+      dy -= 32768;
+      _p += this->y_stride*2;
+    */
     
     scale_line_4 (_p+1, this->u_buffer,
 		  this->dest_width >> 1, this->step_dx);
@@ -2565,8 +2644,12 @@ static void yuy22rgb_c_24_rgb (yuv2rgb_t *this, uint8_t * _dst, uint8_t * _p)
     if (height <= 0)
       break;
 
-    dy -= 32768;
-    _p += this->y_stride*2;
+    _p += this->y_stride*2*(dy>>15);
+    dy &= 32767;
+    /*
+      dy -= 32768;
+      _p += this->y_stride*2;
+    */
     
     scale_line_4 (_p+1, this->u_buffer,
 		  this->dest_width >> 1, this->step_dx);
@@ -2639,8 +2722,8 @@ static void yuy22rgb_c_24_bgr (yuv2rgb_t *this, uint8_t * _dst, uint8_t * _p)
     if (height <= 0)
       break;
 
-    dy -= 32768;
-    _p += this->y_stride*2;
+    _p += this->y_stride*2*(dy>>15);
+    dy &= 32767;
 
     scale_line_4 (_p+1, this->u_buffer,
 		  this->dest_width >> 1, this->step_dx);
@@ -2713,8 +2796,8 @@ static void yuy22rgb_c_16 (yuv2rgb_t *this, uint8_t * _dst, uint8_t * _p)
     if (height <= 0)
       break;
 
-    dy -= 32768;
-    _p += this->y_stride*2;
+    _p += this->y_stride*2*(dy>>15);
+    dy &= 32767;
     
     scale_line_4 (_p+1, this->u_buffer,
 		  this->dest_width >> 1, this->step_dx);
@@ -2787,8 +2870,8 @@ static void yuy22rgb_c_8 (yuv2rgb_t *this, uint8_t * _dst, uint8_t * _p)
     if (height <= 0)
       break;
 
-    dy -= 32768;
-    _p += this->y_stride*2;
+    _p += this->y_stride*2*(dy>>15);
+    dy &= 32767;
     
     scale_line_4 (_p+1, this->u_buffer,
 		  this->dest_width >> 1, this->step_dx);
@@ -2827,8 +2910,8 @@ static void yuy22rgb_c_gray (yuv2rgb_t *this, uint8_t * _dst, uint8_t * _p)
       if (height <= 0)
 	break;
 
-      dy -= 32768;
-      _p += this->y_stride*2;
+      _p += this->y_stride*2*(dy>>15);
+      dy &= 32767;
     }
   } else {
     for (height = this->source_height; --height >= 0; ) { 
@@ -2904,8 +2987,8 @@ static void yuy22rgb_c_palette (yuv2rgb_t *this, uint8_t * _dst, uint8_t * _p)
     if (height <= 0)
       break;
 
-    dy -= 32768;
-    _p += this->y_stride*2;
+    _p += this->y_stride*2*(dy>>15);
+    dy &= 32767;
 
     scale_line_4 (_p+1, this->u_buffer,
 		  this->dest_width >> 1, this->step_dx);
@@ -2916,9 +2999,9 @@ static void yuy22rgb_c_palette (yuv2rgb_t *this, uint8_t * _dst, uint8_t * _p)
   }
 }
 
-static void yuy22rgb_c_init (yuv2rgb_t *this, int mode, int swapped)
+static void yuy22rgb_c_init (yuv2rgb_factory_t *this)
 {
-  switch (mode) {
+  switch (this->mode) {
   case MODE_32_RGB:
   case MODE_32_BGR:
     this->yuy22rgb_fun = yuy22rgb_c_32;
@@ -2927,7 +3010,7 @@ static void yuy22rgb_c_init (yuv2rgb_t *this, int mode, int swapped)
   case MODE_24_RGB:
   case MODE_24_BGR:
     this->yuy22rgb_fun =
-	(mode==MODE_24_RGB && !swapped) || (mode==MODE_24_BGR && swapped)
+	(this->mode==MODE_24_RGB && !this->swapped) || (this->mode==MODE_24_BGR && this->swapped)
 	    ? yuy22rgb_c_24_rgb
 	    : yuy22rgb_c_24_bgr;
     break;
@@ -2952,70 +3035,39 @@ static void yuy22rgb_c_init (yuv2rgb_t *this, int mode, int swapped)
     break;
 
   default:
-    printf ("yuv2rgb: mode %d not supported for yuy2\n", mode);
+    printf ("yuv2rgb: mode %d not supported for yuy2\n", this->mode);
   }
 }
 
-yuv2rgb_t *yuv2rgb_init (int mode, int swapped, uint8_t *colormap) {
+yuv2rgb_t *yuv2rgb_create_converter (yuv2rgb_factory_t *factory) {
 
-#ifdef ARCH_X86
-  uint32_t mm = xine_mm_accel();
-#endif
   yuv2rgb_t *this = xine_xmalloc (sizeof (yuv2rgb_t));
-
-
-  this->matrix_coefficients = 6;
-  this->fast_rgb = colormap;
+  
+  this->cmap                     = factory->cmap;
 
   this->y_chunk = this->y_buffer = NULL;
   this->u_chunk = this->u_buffer = NULL;
   this->v_chunk = this->v_buffer = NULL;
 
-  yuv2rgb_setup_tables(this, mode, swapped);
+  this->table_rV                 = factory->table_rV;
+  this->table_gU                 = factory->table_gU;
+  this->table_gV                 = factory->table_gV;
+  this->table_bU                 = factory->table_bU;
 
-  /*
-   * auto-probe for the best yuv2rgb function
-   */
-
-  this->yuv2rgb_fun = NULL;
-#ifdef ARCH_X86
-  if ((this->yuv2rgb_fun == NULL) && (mm & MM_ACCEL_X86_MMXEXT)) {
-    yuv2rgb_init_mmxext (this, mode, swapped);
-    if (this->yuv2rgb_fun != NULL)
-      printf ("yuv2rgb: using MMXEXT for colorspace transform\n");
-  }
-  if ((this->yuv2rgb_fun == NULL) && (mm & MM_ACCEL_X86_MMX)) {
-    yuv2rgb_init_mmx (this, mode, swapped);
-    if (this->yuv2rgb_fun != NULL)
-      printf ("yuv2rgb: using MMX for colorspace transform\n");
-  }
-#endif
-#if HAVE_MLIB
-  if (this->yuv2rgb_fun == NULL) {
-    yuv2rgb_init_mlib (this, mode, swapped);
-    if (this->yuv2rgb_fun != NULL)
-      printf ("yuv2rgb: using medialib for colorspace transform\n");
-  }
-#endif
-  if (this->yuv2rgb_fun == NULL) {
-    printf ("yuv2rgb: no accelerated colorspace conversion found\n");
-    yuv2rgb_c_init (this, mode, swapped);
-  }
-
-  yuv2rgb_single_pixel_init (this, mode, swapped);
-
-  /*
-   * auto-probe for the best yuy22rgb function
-   */
-
-  /* FIXME: implement mmx/mlib functions */
-  yuy22rgb_c_init (this, mode, swapped);
+  this->yuv2rgb_fun              = factory->yuv2rgb_fun;
+  this->yuy22rgb_fun             = factory->yuy22rgb_fun;
+  this->yuv2rgb_single_pixel_fun = factory->yuv2rgb_single_pixel_fun;
 
+  this->configure                = yuv2rgb_configure;
   return this;
 }
 
-void yuv2rgb_set_gamma (yuv2rgb_t *this, int gamma)
-{
+/*
+ * factory functions 
+ */
+
+void yuv2rgb_set_gamma (yuv2rgb_factory_t *this, int gamma) {
+
   int i;
   
   for (i = 0; i < 256; i++) {
@@ -3029,7 +3081,82 @@ void yuv2rgb_set_gamma (yuv2rgb_t *this, int gamma)
   this->gamma = gamma;
 }
 
-int yuv2rgb_get_gamma (yuv2rgb_t *this)
-{
+int yuv2rgb_get_gamma (yuv2rgb_factory_t *this) {
+
   return this->gamma;
 }
+
+yuv2rgb_factory_t* yuv2rgb_factory_init (int mode, int swapped, 
+					 uint8_t *cmap) {
+
+  yuv2rgb_factory_t *this;
+
+#ifdef ARCH_X86
+  uint32_t mm = xine_mm_accel();
+#endif
+
+  this = malloc (sizeof (yuv2rgb_factory_t));
+
+  this->mode                = mode;
+  this->swapped             = swapped;
+  this->cmap                = cmap;
+  this->create_converter    = yuv2rgb_create_converter;
+  this->set_gamma           = yuv2rgb_set_gamma;
+  this->get_gamma           = yuv2rgb_get_gamma;
+  this->matrix_coefficients = 6;
+
+
+  yuv2rgb_setup_tables (this, mode, swapped);
+
+  /*
+   * auto-probe for the best yuv2rgb function
+   */
+
+  this->yuv2rgb_fun = NULL;
+#ifdef ARCH_X86
+  if ((this->yuv2rgb_fun == NULL) && (mm & MM_ACCEL_X86_MMXEXT)) {
+
+    yuv2rgb_init_mmxext (this);
+
+    if (this->yuv2rgb_fun != NULL)
+      printf ("yuv2rgb: using MMXEXT for colorspace transform\n");
+  }
+
+  if ((this->yuv2rgb_fun == NULL) && (mm & MM_ACCEL_X86_MMX)) {
+
+    yuv2rgb_init_mmx (this);
+
+    if (this->yuv2rgb_fun != NULL)
+      printf ("yuv2rgb: using MMX for colorspace transform\n");
+  }
+#endif
+#if HAVE_MLIB
+  if (this->yuv2rgb_fun == NULL) {
+
+    yuv2rgb_init_mlib (this);
+
+    if (this->yuv2rgb_fun != NULL)
+      printf ("yuv2rgb: using medialib for colorspace transform\n");
+  }
+#endif
+  if (this->yuv2rgb_fun == NULL) {
+    printf ("yuv2rgb: no accelerated colorspace conversion found\n");
+    yuv2rgb_c_init (this);
+  }
+
+  /*
+   * auto-probe for the best yuy22rgb function
+   */
+
+  /* FIXME: implement mmx/mlib functions */
+  yuy22rgb_c_init (this);
+
+  /*
+   * set up single pixel function
+   */
+
+  yuv2rgb_single_pixel_init (this);
+
+  return this;
+}
+
