@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000, 2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with self program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_out.c,v 1.39 2001/12/24 12:36:19 miguelfreitas Exp $
+ * $Id: audio_out.c,v 1.52 2002/04/14 00:24:45 guenter Exp $
  * 
  * 22-8-2001 James imported some useful AC3 sections from the previous alsa driver.
  *   (c) 2001 Andy Lo A Foe <andy@alsaplayer.org>
@@ -67,9 +67,8 @@
 #include "resample.h"
 #include "metronom.h"
 
-
 /*
-#define AUDIO_OUT_LOG
+#define LOG
 */
 
 #define NUM_AUDIO_BUFFERS       32
@@ -77,6 +76,32 @@
 
 #define ZERO_BUF_SIZE         5000
 
+/* By adding gap errors (difference between reported and expected
+ * sound card clock) into metronom's vpts_offset we can use its 
+ * smoothing algorithms to correct sound card clock drifts.
+ * obs: previously this error was added to xine scr.
+ *
+ * audio buf ---> metronom --> audio fifo --> (buf->vpts - hw_vpts)
+ *           (vpts_offset + error)                     gap
+ *                    <---------- control --------------|
+ *
+ * Unfortunately audio fifo adds a large delay to our closed loop.
+ *
+ * These are designed to avoid updating the metronom too fast.
+ * - it will only be updated 1 time per second (so it has a chance of
+ *   distributing the error for several frames).
+ * - it will only be updated 2 times for the whole audio fifo size
+ *   length (so the control will wait to see the feedback effect)
+ * - each update will be of gap/SYNC_GAP_RATE.
+ *
+ * Sound card clock correction can only provide smooth playback for
+ * errors < 1% nominal rate. For bigger errors (bad streams) audio
+ * buffers may be dropped or gaps filled with silence.
+ */
+#define SYNC_TIME_INVERVAL  (1 * 90000)
+#define SYNC_BUF_INTERVAL   NUM_AUDIO_BUFFERS / 2
+#define SYNC_GAP_RATE       4
+
 struct audio_fifo_s {
   audio_buffer_t    *first;
   audio_buffer_t    *last;
@@ -204,13 +229,14 @@ void write_pause_burst(ao_instance_t *this, uint32_t num_frames)
 }
 
 
-static void ao_fill_gap (ao_instance_t *this, uint32_t pts_len) {
+static void ao_fill_gap (ao_instance_t *this, int64_t pts_len) {
 
   int num_frames ;
 
   num_frames = pts_len * this->frames_per_kpts / 1024;
 
-  printf ("audio_out: inserting %d 0-frames to fill a gap of %d pts\n",num_frames, pts_len);
+  printf ("audio_out: inserting %d 0-frames to fill a gap of %lld pts\n",
+	  num_frames, pts_len);
 
   if ((this->mode == AO_CAP_MODE_A52) || (this->mode == AO_CAP_MODE_AC5)) {
     write_pause_burst(this,num_frames);
@@ -233,24 +259,27 @@ static void ao_fill_gap (ao_instance_t *this, uint32_t pts_len) {
 static void *ao_loop (void *this_gen) {
 
   ao_instance_t  *this = (ao_instance_t *) this_gen;
-  uint32_t        hw_vpts;
+  int64_t         hw_vpts;
   audio_buffer_t *buf;
-  int32_t         gap;
+  int64_t         gap;
   int             delay;
-  uint8_t        *data;
-  uint32_t        cur_time;
+  int64_t         cur_time;
   int             num_output_frames ;
   int             paused_wait;
+  int64_t         last_sync_time;
+  int             bufs_since_sync;
+
+  last_sync_time = bufs_since_sync = 0;
   
   while ((this->audio_loop_running) ||
 	 (!this->audio_loop_running && this->out_fifo->first)) {
 
-
-#ifdef AUDIO_OUT_LOG
-    printf ("audio_out: fifo_remove\n");
-#endif
-
     buf = fifo_remove (this->out_fifo);
+    bufs_since_sync++;
+
+#ifdef LOG
+    printf ("audio_out: got a buffer\n");
+#endif
 
     do {
       delay = this->driver->delay(this->driver);
@@ -263,9 +292,9 @@ static void *ao_loop (void *this_gen) {
       cur_time = this->metronom->get_current_time (this->metronom);
       hw_vpts = cur_time;
   
-#ifdef AUDIO_OUT_LOG
-      printf ("audio_out: current delay is %d, current time is %d\n",
-	    delay, cur_time);
+#ifdef LOG
+      printf ("audio_out: current delay is %d, current time is %lld\n",
+	      delay, cur_time);
 #endif
 
       /* External A52 decoder delay correction */
@@ -291,47 +320,55 @@ static void *ao_loop (void *this_gen) {
         xine_usec_sleep (50000);
     } while ( paused_wait );
 
-        
-   /* 
-      printf ("vpts : %d   buffer_vpts : %d  gap %d\n",
-      hw_vpts, buf->vpts, gap);
-    */
+#ifdef LOG
+    printf ("audio_out: hw_vpts : %lld   buffer_vpts : %lld   gap : %lld\n",
+	    hw_vpts, buf->vpts, gap);
+#endif
 
     /*
      * output audio data synced to master clock
      */
-  
-    if (gap < (-1 * this->gap_tolerance) || !buf->num_frames || 
+    
+    if (gap < (-1 * AO_MAX_GAP) || !buf->num_frames || 
         this->audio_paused ) {
 
       /* drop package */
 
-#ifdef AUDIO_OUT_LOG
-      printf ("audio_out: audio package (vpts = %d %d) dropped\n", 
+#ifdef LOG
+      printf ("audio_out: audio package (vpts = %lld, gap = %lld) dropped\n", 
 	      buf->vpts, gap);
 #endif
 
     } else {
-
-      if (gap>this->gap_tolerance) {
-    
-	if (gap>15000) 
-	  ao_fill_gap (this, gap);
-	else {
-	  printf ("audio_out: adjusting master clock %d -> %d\n",
-		  cur_time, cur_time + gap);
-	  this->metronom->adjust_clock (this->metronom, 
-					cur_time + gap);
-	}
       
-      } 
+      /* for small gaps ( tolerance < abs(gap) < AO_MAX_GAP ) 
+       * feedback them into metronom's vpts_offset. 
+       */
+      if ( abs(gap) < AO_MAX_GAP && abs(gap) > this->gap_tolerance &&
+           cur_time > (last_sync_time + SYNC_TIME_INVERVAL) && 
+           bufs_since_sync >= SYNC_BUF_INTERVAL ) {
+           
+        this->metronom->set_option(this->metronom, METRONOM_ADJ_VPTS_OFFSET,
+                                   -gap/SYNC_GAP_RATE );
+        last_sync_time = cur_time;
+        bufs_since_sync = 0;
+      }
+      
+      /* for big gaps output silence */
+      if ( gap > AO_MAX_GAP ) {
+        ao_fill_gap (this, gap/2);
+      }
   
       /*
        * resample and output audio data
        */
-      
+
       num_output_frames = (double) buf->num_frames * this->frame_rate_factor;
       
+#ifdef LOG
+      printf ("audio_out: outputting %d frames\n", num_output_frames);
+#endif
+      
       if ((!this->do_resample) 
 	  && (this->mode != AO_CAP_MODE_A52) 
 	  && (this->mode != AO_CAP_MODE_AC5)) {
@@ -390,6 +427,10 @@ static int ao_open(ao_instance_t *this,
  
   int output_sample_rate, err;
 
+  xine_log (this->xine, XINE_LOG_FORMAT,
+	    "audio_out: stream audio format is %d kHz sampling rate, %d bits. mode is %d.\n",
+	    rate, bits, mode);
+
   if ((output_sample_rate=this->driver->open(this->driver,bits,(this->force_rate ? this->force_rate : rate),mode)) == 0) {
     printf("audio_out: open failed!\n");
     return 0;
@@ -427,7 +468,7 @@ static int ao_open(ao_instance_t *this,
   this->frame_rate_factor = (double) this->output_frame_rate / (double) this->input_frame_rate; 
   this->audio_step        = (uint32_t) 90000 * (uint32_t) 32768 / this->input_frame_rate;
   this->frames_per_kpts   = this->output_frame_rate * 1024 / 90000;
-#ifdef AUDIO_OUT_LOG
+#ifdef LOG
   printf ("audio_out : audio_step %d pts per 32768 frames\n", this->audio_step);
 #endif
 
@@ -463,18 +504,27 @@ static audio_buffer_t *ao_get_buffer (ao_instance_t *this) {
 
 static void ao_put_buffer (ao_instance_t *this, audio_buffer_t *buf) {
 
+  int64_t pts;
+
   if (buf->num_frames == 0) {
     fifo_append (this->free_fifo, buf);
     return;
   }
 
-  buf->vpts = this->metronom->got_audio_samples (this->metronom, buf->vpts, 
-						 buf->num_frames, buf->scr);
+  pts = buf->vpts;
+
+  buf->vpts = this->metronom->got_audio_samples (this->metronom, pts, 
+						 buf->num_frames);
+
+#ifdef LOG
+  printf ("audio_out: got buffer, pts=%lld, vpts=%lld\n",
+	  pts, buf->vpts);
+#endif
 
   if ( buf->vpts<this->last_audio_vpts) {
 
     /* reject buffer */
-    printf ("audio_out: rejected buffer vpts=%d, last_audio_vpts=%d\n", 
+    printf ("audio_out: rejected buffer vpts=%lld, last_audio_vpts=%lld\n", 
 	    buf->vpts, this->last_audio_vpts);
 
     fifo_append (this->free_fifo, buf);
@@ -513,7 +563,41 @@ static void ao_close(ao_instance_t *this) {
 }
 
 static void ao_exit(ao_instance_t *this) {
+
+  audio_buffer_t *buf, *next;
+
   this->driver->exit(this->driver);
+
+  free (this->frame_buffer);
+  free (this->zero_space);
+
+  buf = this->free_fifo->first;
+
+  while (buf != NULL) {
+
+    next = buf->next;
+
+    free (buf->mem);
+    free (buf);
+
+    buf = next;
+  }
+
+  buf = this->out_fifo->first;
+
+  while (buf != NULL) {
+
+    next = buf->next;
+
+    free (buf->mem);
+    free (buf);
+
+    buf = next;
+  }
+
+  free (this->free_fifo);
+  free (this->out_fifo);
+  free (this);
 }
 
 static uint32_t ao_get_capabilities (ao_instance_t *this) {
@@ -532,17 +616,32 @@ static int ao_set_property (ao_instance_t *this, int property, int value) {
   return(this->driver->set_property(this->driver, property, value));
 }
 
-ao_instance_t *ao_new_instance (ao_driver_t *driver, metronom_t *metronom, 
-				config_values_t *config) {
+static int ao_control (ao_instance_t *this, int cmd, ...) {
 
-  ao_instance_t *this;
-  int            i;
-  static         char *resample_modes[] = {"auto", "off", "on", NULL};
+  va_list args;
+  void *arg;
+  int rval;
+
+  va_start(args, cmd);
+  arg = va_arg(args, void*);
+  rval = this->driver->control(this->driver, cmd, arg);
+  va_end(args);
+
+  return rval;
+}
+
+ao_instance_t *ao_new_instance (ao_driver_t *driver, xine_t *xine) {
+ 
+  config_values_t *config = xine->config;
+  ao_instance_t   *this;
+  int              i;
+  static     char *resample_modes[] = {"auto", "off", "on", NULL};
 
   this = xine_xmalloc (sizeof (ao_instance_t)) ;
 
   this->driver                = driver;
-  this->metronom              = metronom;
+  this->metronom              = xine->metronom;
+  this->xine                  = xine;
 
   this->open                  = ao_open;
   this->get_buffer            = ao_get_buffer;
@@ -552,6 +651,7 @@ ao_instance_t *ao_new_instance (ao_driver_t *driver, metronom_t *metronom,
   this->get_capabilities      = ao_get_capabilities;
   this->get_property          = ao_get_property;
   this->set_property          = ao_set_property;
+  this->control		      = ao_control;
   this->audio_loop_running    = 0;
   this->audio_paused          = 0;
   /* FIXME: is 4* good enough for all resample cases?? */
