@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: buffer_types.c,v 1.9 2002/01/15 17:30:51 miguelfreitas Exp $
+ * $Id: buffer_types.c,v 1.16 2002/04/20 20:20:44 guenter Exp $
  *
  *
  * contents:
@@ -78,7 +78,23 @@ static video_db_t video_db[] = {
     0
   },
   BUF_VIDEO_MPEG4,
-  "ISO-MPEG4 or OpenDivx format"
+  "ISO-MPEG4/OpenDivx format"
+},
+{
+  {
+    mmioFOURCC('X', 'V', 'I', 'D'),
+    0
+  },
+  BUF_VIDEO_XVID,
+  "XviD format"
+},
+{
+  {
+    mmioFOURCC('D', 'X', '5', '0'),
+    0
+  },
+  BUF_VIDEO_DIVX5,
+  "DivX 5 format"
 },
 {
   {
@@ -123,7 +139,6 @@ static video_db_t video_db[] = {
     mmioFOURCC('d', 'i', 'v', '5'),
     mmioFOURCC('D', 'I', 'V', '6'),
     mmioFOURCC('d', 'i', 'v', '6'),
-    mmioFOURCC('3', 'I', 'V', '1'),
     mmioFOURCC('A', 'P', '4', '1'),
     mmioFOURCC('M', 'P', 'G', '3'),
     0
@@ -131,6 +146,14 @@ static video_db_t video_db[] = {
   BUF_VIDEO_MSMPEG4_V3,
   "Microsoft MPEG-4 format v3"
 },
+{
+  {
+    mmioFOURCC('3', 'I', 'V', '1'),
+    0
+  },
+  BUF_VIDEO_3IVX,
+  "3ivx MPEG-4"
+},
 {
   {
     mmioFOURCC('d', 'm', 'b', '1'),
@@ -213,13 +236,6 @@ static video_db_t video_db[] = {
   BUF_VIDEO_RV10,
   ""
 },
-{
-  {
-    0
-  },
-  BUF_VIDEO_FILL,
-  ""
-},
 {
   {
     mmioFOURCC('r','a','w',' '),
@@ -308,6 +324,14 @@ static video_db_t video_db[] = {
   BUF_VIDEO_H263,
   "H263 format"
 },
+{
+  {
+    mmioFOURCC('c', 'y', 'u', 'v'),
+    0
+  },
+  BUF_VIDEO_CYUV,
+  "cyuv format"
+},
 { { 0 }, 0, "last entry" }
 };
 
@@ -377,8 +401,10 @@ static audio_db_t audio_db[] = {
   "MS GSM"
 },
 {
-  {
-    0
+  {                                  
+    /* these formattags are used by Vorbis ACM encoder and
+       supported by NanDub, a variant of VirtualDub. */
+    0x674f, 0x676f, 0x6750, 0x6770, 0x6751, 0x6771, 0
   },
   BUF_AUDIO_VORBIS,
   "OggVorbis Audio"
