@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: load_plugins.c,v 1.67 2002/01/08 16:41:39 cvogler Exp $
+ * $Id: load_plugins.c,v 1.75 2002/04/23 15:45:26 esnel Exp $
  *
  *
  * Load input/demux/audio_out/video_out/codec plugins
@@ -48,29 +48,9 @@
 #include "compat.h"
 
 /*
-#define LOAD_LOG
+#define LOG
 */
 
-#ifdef __GNUC__
-#define LOG_MSG_STDERR(xine, message, args...) {                     \
-    xine_log(xine, XINE_LOG_PLUGIN, message, ##args);                \
-    fprintf(stderr, message, ##args);                                \
-  }
-#define LOG_MSG(xine, message, args...) {                            \
-    xine_log(xine, XINE_LOG_PLUGIN, message, ##args);                \
-    printf(message, ##args);                                         \
-  }
-#else
-#define LOG_MSG_STDERR(xine, ...) {                                  \
-    xine_log(xine, XINE_LOG_PLUGIN, __VA_ARGS__);                    \
-    fprintf(stderr, __VA_ARGS__);                                    \
-  }
-#define LOG_MSG(xine, ...) {                                         \
-    xine_log(xine, XINE_LOG_PLUGIN, __VA_ARGS__);                    \
-    printf(__VA_ARGS__);                                             \
-  }
-#endif
-
 /* transition code; between xine 0.9.7 and 0.9.8, the dxr3enc driver
  * was integrated in the dxr3 driver and no longer exists as a seperate
  * plugin. upgraded installs may have an old dxr3enc driver left in the 
@@ -117,8 +97,8 @@ void load_demux_plugins (xine_t *this,
 			 config_values_t *config, int iface_version) {
   DIR *dir;
 
-  if(this == NULL || config == NULL) {
-    LOG_MSG(this, _("%s(%s@%d): parameter should be non null, exiting\n"),
+  if (this == NULL || config == NULL) {
+    printf (_("%s(%s@%d): parameter should be non null, exiting\n"),
 	    __FILE__, __XINE_FUNCTION__, __LINE__);
     exit(1);
   }
@@ -155,8 +135,9 @@ void load_demux_plugins (xine_t *this,
 	if(!(plugin = dlopen (str, RTLD_LAZY))) {
 	  char *dl_error_msg = dlerror();
 
-	  LOG_MSG(this, _("load_plugins: cannot open demux plugin %s:\n%s\n"),
-		  str, dl_error_msg);
+	  xine_log (this, XINE_LOG_PLUGIN,
+		    _("load_plugins: cannot open demux plugin %s:\n%s\n"),
+		    str, dl_error_msg);
 
 	} else {
 
@@ -169,15 +150,16 @@ void load_demux_plugins (xine_t *this,
 	    if (dxp) {
 	      this->demuxer_plugins[this->num_demuxer_plugins] = dxp; 
 	    
-	      LOG_MSG(this, _("load_plugins: demux plugin found : %s\n"), 
-		      this->demuxer_plugins[this->num_demuxer_plugins]->get_identifier());
+	      xine_log (this, XINE_LOG_PLUGIN,
+			_("load_plugins: demux plugin found : %s\n"), 
+			this->demuxer_plugins[this->num_demuxer_plugins]->get_identifier());
 
 	      this->num_demuxer_plugins++;
 	    }
 	  }
 	  
 	  if(this->num_demuxer_plugins > DEMUXER_PLUGIN_MAX) {
-	    LOG_MSG(this, _("load_plugins: too many demux plugins installed, exiting.\n"));
+	    printf ( _("load_plugins: too many demux plugins installed, exiting.\n"));
 	    exit(1);
 	  }
 	}
@@ -315,8 +297,9 @@ void load_input_plugins (xine_t *this,
 	if(!(plugin = dlopen (str, RTLD_LAZY))) {
 	  char *dl_error_msg = dlerror();
 
-	  LOG_MSG(this, _("load_plugins: cannot open input plugin %s:\n%s\n"), 
-		  str, dl_error_msg);
+	  xine_log (this, XINE_LOG_PLUGIN,
+		    _("load_plugins: cannot open input plugin %s:\n%s\n"), 
+		    str, dl_error_msg);
 	} else {
 	  void *(*initplug) (int, xine_t *);
 	  
@@ -327,19 +310,21 @@ void load_input_plugins (xine_t *this,
 	    if (ip) {
 	      this->input_plugins[this->num_input_plugins] = ip; 
 	    
-	      LOG_MSG(this, _("load_plugins: input plugin found : %s\n"), 
-		      this->input_plugins[this->num_input_plugins]->get_identifier(this->input_plugins[this->num_input_plugins]));
+	      xine_log (this, XINE_LOG_PLUGIN,
+			_("load_plugins: input plugin found : %s\n"), 
+			this->input_plugins[this->num_input_plugins]->get_identifier(this->input_plugins[this->num_input_plugins]));
 
 	      this->num_input_plugins++;
 	    }
 
 	  } else {
-	    LOG_MSG(this, _("load_plugins: %s is no valid input plugin (lacks init_input_plugin() function)\n"), str);
+	    xine_log (this, XINE_LOG_PLUGIN,
+		      _("load_plugins: %s is no valid input plugin (lacks init_input_plugin() function)\n"), str);
 	  }
 	  
 	  if(this->num_input_plugins > INPUT_PLUGIN_MAX) {
-	    LOG_MSG_STDERR(this, _("%s(%d): too many input plugins installed, "
-				   "exiting.\n"), __FILE__, __LINE__);
+	    printf (_("%s(%d): too many input plugins installed, "
+		      "exiting.\n"), __FILE__, __LINE__);
 	    exit(1);
 	  }
 	}
@@ -351,8 +336,8 @@ void load_input_plugins (xine_t *this,
   remove_segv_handler();
   
   if (this->num_input_plugins == 0) {
-    LOG_MSG_STDERR(this, _("load_plugins: no input plugins found in %s! - "
-			   "Did you install xine correctly??\n"), XINE_PLUGINDIR);
+    printf (_("load_plugins: no input plugins found in %s! - "
+	      "Did you install xine correctly??\n"), XINE_PLUGINDIR);
     exit (1);
   }
   
@@ -467,11 +452,13 @@ void load_decoder_plugins (xine_t *this,
   int  i;
   int spu_prio[DECODER_PLUGIN_MAX], video_prio[DECODER_PLUGIN_MAX],
    audio_prio[DECODER_PLUGIN_MAX];
+  int *spu_used[DECODER_PLUGIN_MAX], *video_used[DECODER_PLUGIN_MAX],
+   *audio_used[DECODER_PLUGIN_MAX];
    
 
   if(this == NULL || config == NULL) {
-    LOG_MSG(this, _("%s(%s@%d): parameter should be non null, exiting\n"),
-	    __FILE__, __XINE_FUNCTION__, __LINE__);
+    printf ( _("%s(%s@%d): parameter should be non null, exiting\n"),
+	     __FILE__, __XINE_FUNCTION__, __LINE__);
     exit(1);
   }
 
@@ -482,18 +469,21 @@ void load_decoder_plugins (xine_t *this,
   for (i=0; i<DECODER_PLUGIN_MAX; i++) {
     this->spu_decoder_plugins[i] = NULL;
     spu_prio[i] = 0;
+    spu_used[i] = NULL;
   }
 
   this->cur_video_decoder_plugin = NULL;
   for (i=0; i<DECODER_PLUGIN_MAX; i++) {
     this->video_decoder_plugins[i] = NULL;
     video_prio[i] = 0;
+    video_used[i] = NULL;
   }
 
   this->cur_audio_decoder_plugin = NULL;
   for (i=0; i<DECODER_PLUGIN_MAX; i++) {
     this->audio_decoder_plugins[i] = NULL;
     audio_prio[i] = 0;
+    audio_used[i] = NULL;
   }
 
   /*
@@ -531,8 +521,9 @@ void load_decoder_plugins (xine_t *this,
 	if(!(plugin = dlopen (str, RTLD_LAZY))) {
 	  char *dl_error_msg = dlerror();
 
-	  LOG_MSG(this, _("load_plugins: failed to load plugin %s:\n%s\n"),
-		  str, dl_error_msg);
+	  xine_log (this, XINE_LOG_PLUGIN,
+		    _("load_plugins: failed to load plugin %s:\n%s\n"),
+		    str, dl_error_msg);
 
 	} else {
 	  int plugin_prio;
@@ -548,19 +539,35 @@ void load_decoder_plugins (xine_t *this,
 	      spu_decoder_t *sdp;
 	      int            streamtype;
 	      
-	      sdp = (spu_decoder_t *) initplug(4, this);
+	      sdp = (spu_decoder_t *) initplug(6, this);
 	      if (sdp) {
+		int *used = (int *)xine_xmalloc (sizeof (int));
+
 		sdp->metronom = this->metronom;
 		
 		for (streamtype = 0; streamtype<DECODER_PLUGIN_MAX; streamtype++)
 		  if ((plugin_prio =
 		       decide_spu_insert(sdp, streamtype, spu_prio[streamtype]))) {
+
+		    if (spu_used[streamtype] && --(*spu_used[streamtype]) == 0) {
+		      this->spu_decoder_plugins[streamtype]->dispose (this->spu_decoder_plugins[streamtype]);
+		      free (spu_used[streamtype]);
+		    }
+
 		    this->spu_decoder_plugins[streamtype] = sdp;
 		    spu_prio[streamtype] = plugin_prio;
+		    spu_used[streamtype] = used;
+		    (*used)++;
 		  }
 		
-		LOG_MSG(this, _("spu decoder plugin found : %s\n"),
-			sdp->get_identifier());
+		xine_log (this, XINE_LOG_PLUGIN,
+			  _("spu decoder plugin found : %s\n"),
+			  sdp->get_identifier());
+
+		if (*used == 0) {
+		  sdp->dispose (sdp);
+		  free (used);
+		}
 	      }
 	    }
 	  }
@@ -578,17 +585,33 @@ void load_decoder_plugins (xine_t *this,
 	      
 	      vdp = (video_decoder_t *) initplug(iface_version, this);
 	      if (vdp) {
+		int *used = (int *)xine_xmalloc (sizeof (int));
+
 		vdp->metronom = this->metronom;
 		
 		for (streamtype = 0; streamtype<DECODER_PLUGIN_MAX; streamtype++)
 		  if ((plugin_prio =
 		       decide_video_insert(vdp, streamtype, video_prio[streamtype]))) {
+
+		    if (video_used[streamtype] && --(*video_used[streamtype]) == 0) {
+		      this->video_decoder_plugins[streamtype]->dispose (this->video_decoder_plugins[streamtype]);
+		      free (video_used[streamtype]);
+		    }
+
 		    this->video_decoder_plugins[streamtype] = vdp;
 		    video_prio[streamtype] = plugin_prio;
+		    video_used[streamtype] = used;
+		    (*used)++;
 		  }
 		
-		LOG_MSG(this, _("video decoder plugin found : %s\n"), 
-			vdp->get_identifier());
+		xine_log (this, XINE_LOG_PLUGIN,
+			  _("video decoder plugin found : %s\n"), 
+			  vdp->get_identifier());
+
+		if (*used == 0) {
+		  vdp->dispose (vdp);
+		  free (used);
+		}
 	      }
 	    }
 	    
@@ -603,16 +626,31 @@ void load_decoder_plugins (xine_t *this,
 	      
 	      adp = (audio_decoder_t *) initplug(iface_version, this);
 	      if (adp) {
+		int *used = (int *)xine_xmalloc (sizeof (int));
 		
 		for (streamtype = 0; streamtype<DECODER_PLUGIN_MAX; streamtype++)
 		  if ((plugin_prio =
 		       decide_audio_insert(adp, streamtype, audio_prio[streamtype]))) {
+
+		    if (audio_used[streamtype] && --(*audio_used[streamtype]) == 0) {
+		      this->audio_decoder_plugins[streamtype]->dispose (this->audio_decoder_plugins[streamtype]);
+		      free (audio_used[streamtype]);
+		    }
+
 		    this->audio_decoder_plugins[streamtype] = adp; 
 		    audio_prio[streamtype] = plugin_prio;
+		    audio_used[streamtype] = used;
+		    (*used)++;
 		  }
 		
-		LOG_MSG(this, _("audio decoder plugin found : %s\n"), 
-			adp->get_identifier());
+		xine_log (this, XINE_LOG_PLUGIN,
+			  _("audio decoder plugin found : %s\n"), 
+			  adp->get_identifier());
+
+		if (*used == 0) {
+		  adp->dispose (adp);
+		  free (used);
+		}
 	      }
 	    }
 	    
@@ -624,6 +662,17 @@ void load_decoder_plugins (xine_t *this,
   }
   remove_segv_handler();
 
+  for (i=0; i<DECODER_PLUGIN_MAX; i++) {
+    if (spu_used[i] && --(*spu_used[i]) == 0)
+      free (spu_used[i]);
+
+    if (video_used[i] && --(*video_used[i]) == 0)
+      free (video_used[i]);
+
+    if (audio_used[i] && --(*audio_used[i]) == 0)
+      free (audio_used[i]);
+  }
+
   this->cur_spu_decoder_plugin = NULL;
   this->cur_video_decoder_plugin = NULL;
   this->cur_audio_decoder_plugin = NULL;
@@ -662,15 +711,16 @@ char **xine_list_video_output_plugins (int visual_type) {
 	   && (dir_entry->d_name[nLen-2]=='s')
 	   && (dir_entry->d_name[nLen-1]=='o'))) {
 
-	/*printf ("load_plugins: found a video output plugin: %s\n",
-	  dir_entry->d_name); */
+#ifdef LOG
+	printf ("load_plugins: found a video output plugin: %s\n",
+		dir_entry->d_name); 
+#endif
 
 #if IGNORE_DXR3ENC
-	if (! strncasecmp(dir_entry->d_name, 
-		XINE_VIDEO_OUT_PLUGIN_PREFIXNAME "dxr3enc", 
-		XINE_VIDEO_OUT_PLUGIN_PREFIXNAME_LENGTH + 7))
-	{
-		printf("(ignoring obsolete dxr3enc driver)");
+	if (!strncasecmp(dir_entry->d_name, 
+			 XINE_VIDEO_OUT_PLUGIN_PREFIXNAME "dxr3enc", 
+			 XINE_VIDEO_OUT_PLUGIN_PREFIXNAME_LENGTH + 7)) {
+		printf ("load_plugins: (ignoring obsolete dxr3enc driver)");
 		continue;
 	}
 #endif
@@ -684,7 +734,7 @@ char **xine_list_video_output_plugins (int visual_type) {
 	 * and finally if all this went through get it's id
 	 */
 
-	if(!(plugin = dlopen (str, RTLD_LAZY))) {
+	if(!(plugin = dlopen (str, RTLD_LAZY | RTLD_GLOBAL))) {
 
 	  printf("load_plugins: cannot load plugin %s:\n%s\n",
 		 str, dlerror());
@@ -698,9 +748,11 @@ char **xine_list_video_output_plugins (int visual_type) {
 	    vo_info = getinfo();
 
 	    if ( (vo_info->visual_type == visual_type)
-		 && (vo_info->interface_version == VIDEO_OUT_IFACE_VERSION) ) {
-	      /* printf("video output plugin found : %s (%s)\n",
-		     vo_info->id, vo_info->description); */
+		 && (vo_info->interface_version == VIDEO_OUT_DRIVER_IFACE_VERSION) ) {
+#ifdef LOG
+	       printf("video output plugin found : %s (%s)\n",
+		      vo_info->id, vo_info->description); 
+#endif
 
 	      /* sort the list by vo_info->priority */
 
@@ -721,6 +773,16 @@ char **xine_list_video_output_plugins (int visual_type) {
 
 	      num_plugins++;
 	      plugin_ids[num_plugins] = NULL;
+	    } else {
+	      
+	      if(vo_info->interface_version != VIDEO_OUT_DRIVER_IFACE_VERSION)
+		printf ("load_plugins: %s has wrong interface version (%d)\n",
+			str, vo_info->interface_version);
+#ifdef LOG
+	      else if(vo_info->visual_type != visual_type)
+		printf ("load_plugins: %s has wrong visual type (%d)\n",
+			str, vo_info->visual_type);
+#endif
 	    }
 	  } else {
 
@@ -734,14 +796,13 @@ char **xine_list_video_output_plugins (int visual_type) {
     }
     closedir(dir);
   } else {
-    fprintf(stderr, "load_plugins: %s - cannot access plugin dir: %s",
-	    __XINE_FUNCTION__, strerror(errno));
+    printf ("load_plugins: %s - cannot access plugin dir '%s': %s\n",
+	    __XINE_FUNCTION__, XINE_PLUGINDIR, strerror(errno));
   }
   
   remove_segv_handler();
 
-  if( !num_plugins )
-  {
+  if (!num_plugins) {
     fprintf(stderr, "load_plugins: no video plugins found, make sure you have them "
             "installed at %s\n", XINE_PLUGINDIR );
   }
@@ -800,7 +861,7 @@ vo_driver_t *xine_load_video_output_plugin(config_values_t *config,
 
 	sprintf (str, "%s/%s", XINE_PLUGINDIR, dir_entry->d_name);
 		
-	if(!(plugin = dlopen (str, RTLD_LAZY))) {
+	if(!(plugin = dlopen (str, RTLD_LAZY | RTLD_GLOBAL))) {
 	  printf("load_plugins: video output plugin %s failed to link:\n%s\n",
 		 str, dlerror());
 	} else {
@@ -812,7 +873,7 @@ vo_driver_t *xine_load_video_output_plugin(config_values_t *config,
 
 	    if (!strcasecmp(id, vo_info->id) ) {
 
-	      if (vo_info->interface_version == VIDEO_OUT_IFACE_VERSION) {
+	      if (vo_info->interface_version == VIDEO_OUT_DRIVER_IFACE_VERSION) {
 
 		void *(*initplug) (config_values_t *, void *);
 	    
@@ -904,14 +965,14 @@ char **xine_list_audio_output_plugins(void) {
 	  ao_info_t* (*getinfo) ();
 	  ao_info_t   *ao_info;
 
-#ifdef LOAD_LOG
+#ifdef LOG
 	  printf ("load_plugins: plugin %s successfully loaded.\n", str);  
 #endif
 
 	  if ((getinfo = dlsym(plugin, "get_audio_out_plugin_info")) != NULL) {
 	    ao_info = getinfo();
 
-#ifdef LOAD_LOG
+#ifdef LOG
 	    printf("load_plugins: id=%s, priority=%d, interface_version=%d\n", 
 		   ao_info->id, ao_info->priority, ao_info->interface_version); 
 #endif
@@ -955,8 +1016,8 @@ char **xine_list_audio_output_plugins(void) {
     }
     closedir(dir);
   } else {
-    fprintf (stderr, "load_plugins: %s - cannot access plugin dir: %s",
-	     __XINE_FUNCTION__, strerror(errno));
+    printf ("load_plugins: %s - cannot access plugin dir: %s",
+	    __XINE_FUNCTION__, strerror(errno));
   }
   
   remove_segv_handler();
