@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_overlay.c,v 1.14 2002/01/15 20:22:44 jcdutton Exp $
+ * $Id: video_overlay.c,v 1.20 2002/04/23 13:14:34 esnel Exp $
  *
  */
 
@@ -101,11 +101,12 @@ static void remove_showing_handle( video_overlay_t *this, int32_t handle )
   pthread_mutex_unlock( &this->video_overlay_showing_mutex );
 }
 
-static void remove_events_handle( video_overlay_t *this, int32_t handle )
+static void remove_events_handle( video_overlay_t *this, int32_t handle, int lock )
 {
   uint32_t   last_event,this_event;
 
-  pthread_mutex_lock( &this->video_overlay_events_mutex );
+  if( lock )
+    pthread_mutex_lock( &this->video_overlay_events_mutex );
   
   this_event=0;
   do {
@@ -134,7 +135,8 @@ static void remove_events_handle( video_overlay_t *this, int32_t handle )
     }
   } while ( this_event );
  
-  pthread_mutex_unlock( &this->video_overlay_events_mutex );
+  if( lock )
+    pthread_mutex_unlock( &this->video_overlay_events_mutex );
 }
 
 
@@ -187,7 +189,7 @@ static void video_overlay_free_handle(video_overlay_instance_t *this_gen, int32_
   video_overlay_t *this = (video_overlay_t *) this_gen;
 
   remove_showing_handle(this,handle);
-  remove_events_handle(this,handle);
+  remove_events_handle(this,handle,1);
   internal_video_overlay_free_handle(this,handle);
 }
 
@@ -241,7 +243,8 @@ static void video_overlay_init (video_overlay_instance_t *this_gen) {
  *
  * note: on success event->object.overlay is "taken" (caller will not have access
  *       to overlay data including rle).
- * note2: handle will be freed on HIDE events
+ * note2: handle will not be freed on HIDE events
+ *        the handle is removed from the currently showing list.
  */
 static int32_t video_overlay_add_event(video_overlay_instance_t *this_gen,  void *event_gen ) {
   video_overlay_event_t *event = (video_overlay_event_t *) event_gen;
@@ -326,16 +329,19 @@ static void video_overlay_print_overlay( vo_overlay_t *ovl ) {
 /*
    process overlay events
    if vpts == 0 will process everything now (used in flush)
+   return true if something has been processed
 */
-static void video_overlay_event( video_overlay_t *this, int vpts ) {
+static int video_overlay_event( video_overlay_t *this, int64_t vpts ) {
   int32_t      handle;
   uint32_t     this_event;
+  int          processed = 0;
   
   pthread_mutex_lock (&this->video_overlay_events_mutex);
   
   this_event=this->video_overlay_events[0].next_event;
   while ( this_event && (vpts > this->video_overlay_events[this_event].event->vpts ||
           vpts == 0) ) {
+    processed++;
     handle=this->video_overlay_events[this_event].event->object.handle;
     switch( this->video_overlay_events[this_event].event->event_type ) {
       case EVENT_SHOW_SPU:
@@ -346,9 +352,15 @@ static void video_overlay_event( video_overlay_t *this, int vpts ) {
 #ifdef LOG_DEBUG
           video_overlay_print_overlay( this->video_overlay_events[this_event].event->object.overlay ) ;
 #endif
-          /* This should not happen, the calling routine should do the free */
-          /* FIXME: Need to add new event to free handle */
-/* internal_video_overlay_free_handle( this, handle ); */
+          /* this->video_overlay_objects[handle].overlay is about to be
+           * overwritten by this event data. make sure we free it if needed.
+           */
+          if( this->video_overlay_objects[handle].overlay ) {
+            if( this->video_overlay_objects[handle].overlay->rle )
+              free( this->video_overlay_objects[handle].overlay->rle );
+            free( this->video_overlay_objects[handle].overlay );
+            this->video_overlay_objects[handle].overlay = NULL; 
+          }
           
           this->video_overlay_objects[handle].handle = handle;
           if( this->video_overlay_objects[handle].overlay ) {
@@ -368,10 +380,16 @@ static void video_overlay_event( video_overlay_t *this, int vpts ) {
 #ifdef LOG_DEBUG
         printf ("video_overlay: FREE SPU NOW\n");
 #endif
-        free(this->video_overlay_events[this_event].event->object.overlay);
+        /* free any overlay associated with this event */
+        if (this->video_overlay_events[this_event].event->object.overlay != NULL) {
+          free(this->video_overlay_events[this_event].event->object.overlay);
           this->video_overlay_events[this_event].event->object.overlay = NULL; 
+        }
+        /* this avoid removing this_event from the queue
+         * (it will be removed at the end of this loop) */
+        this->video_overlay_events[this_event].event->object.handle = -1;
         remove_showing_handle(this,handle);
-        remove_events_handle(this,handle);
+        remove_events_handle(this,handle,0);
         internal_video_overlay_free_handle( this, handle );
         break;
 
@@ -382,120 +400,56 @@ static void video_overlay_event( video_overlay_t *this, int vpts ) {
 #ifdef LOG_DEBUG
         printf ("video_overlay: HIDE SPU NOW\n");
 #endif
-        free(this->video_overlay_events[this_event].event->object.overlay);
+        /* free any overlay associated with this event */
+        if (this->video_overlay_events[this_event].event->object.overlay != NULL) {
+          free(this->video_overlay_events[this_event].event->object.overlay);
           this->video_overlay_events[this_event].event->object.overlay = NULL; 
+        }
         remove_showing_handle( this, handle );
-        /* This should not happen, the calling routine should do the free */
-        /* FIXME: Need to add new event to free handle */
-        /* internal_video_overlay_free_handle( this, handle ); */
         break;
   
       case EVENT_HIDE_MENU:
 #ifdef LOG_DEBUG
         printf ("video_overlay: HIDE MENU NOW %d\n",handle);
 #endif
-        free(this->video_overlay_events[this_event].event->object.overlay);
+        if (this->video_overlay_events[this_event].event->object.overlay != NULL) {
+          free(this->video_overlay_events[this_event].event->object.overlay);
           this->video_overlay_events[this_event].event->object.overlay = NULL; 
+        }
         remove_showing_handle( this, handle );
-        /* This should not happen, the calling routine should do the free */
-        /* FIXME: Need to add new event to free handle */
-        /* internal_video_overlay_free_handle( this, handle ); */
         break;
   
-      case EVENT_MENU_SPU:
-        /* mixes palette and copy rle */
+      case EVENT_SHOW_MENU:
 #ifdef LOG_DEBUG
-        printf ("video_overlay:MENU SPU NOW\n");
+        printf ("video_overlay: SHOW MENU NOW\n");
 #endif
         if (this->video_overlay_events[this_event].event->object.overlay != NULL) {
-          vo_overlay_t *event_overlay = this->video_overlay_events[this_event].event->object.overlay;
-          vo_overlay_t *overlay;
-          int menu_changed = 0;
-          printf ("video_overlay:overlay present\n");
 #ifdef LOG_DEBUG
           video_overlay_print_overlay( this->video_overlay_events[this_event].event->object.overlay ) ;
 #endif
-
-          /* we need to allocate overlay on first EVENT_MENU_SPU */          
-          if( !this->video_overlay_objects[handle].overlay ) {
-            this->video_overlay_objects[handle].overlay
-               = xine_xmalloc( sizeof(vo_overlay_t) );
-            menu_changed = 1;
+          /* this->video_overlay_objects[handle].overlay is about to be
+           * overwritten by this event data. make sure we free it if needed.
+           */
+          if( this->video_overlay_objects[handle].overlay ) {
+            if( this->video_overlay_objects[handle].overlay->rle )
+              free( this->video_overlay_objects[handle].overlay->rle );
+            free( this->video_overlay_objects[handle].overlay );
+            this->video_overlay_objects[handle].overlay = NULL; 
           }
-          overlay = this->video_overlay_objects[handle].overlay;
           
           this->video_overlay_objects[handle].handle = handle;
-          
-          /* If rle is not empty, free it first */
-          if(overlay->rle) {
-            free (overlay->rle);
+          if( this->video_overlay_objects[handle].overlay ) {
+            fprintf(stderr,"video_overlay: error: object->overlay was not freed!\n");
           }
-#ifdef LOG_DEBUG
-	  printf("video_overlay.c:Menu SPU area is (%u,%u)-(%u,%u), display = 1\n",
-	       event_overlay->x, event_overlay->y,
-	       event_overlay->x + event_overlay->width,
-	       event_overlay->y + event_overlay->height);
-#endif
-          if( overlay->num_rle != event_overlay->num_rle ||
-              overlay->width != event_overlay->width || 
-              overlay->height != event_overlay->height ||
-              overlay->x != event_overlay->x ||
-              overlay->y != event_overlay->y ) {
-            menu_changed = 1;
-          }
-
-          overlay->rle = event_overlay->rle;
-          
-          overlay->data_size = event_overlay->data_size;
-          overlay->num_rle = event_overlay->num_rle;
-          overlay->x = event_overlay->x;
-          overlay->y = event_overlay->y;
-          overlay->width = event_overlay->width;
-          overlay->height = event_overlay->height;
-          
-          if( menu_changed ) {
-            if((event_overlay->color[0] +
-                event_overlay->color[1] +
-                event_overlay->color[2] +
-                event_overlay->color[3]) > 0 ) {
-              overlay->color[0] = event_overlay->color[0];
-              overlay->color[1] = event_overlay->color[1];
-              overlay->color[2] = event_overlay->color[2];
-              overlay->color[3] = event_overlay->color[3];
-              
-              overlay->rgb_clut = event_overlay->rgb_clut;
-            }
-            if((event_overlay->trans[0] +
-                event_overlay->trans[1] +
-                event_overlay->trans[2] +
-                event_overlay->trans[3]) > 0 ) {
-              overlay->trans[0] = event_overlay->trans[0];
-              overlay->trans[1] = event_overlay->trans[1];
-              overlay->trans[2] = event_overlay->trans[2];
-              overlay->trans[3] = event_overlay->trans[3];
-            }
-          }
-#ifdef LOG_DEBUG
-          video_overlay_print_overlay( this->video_overlay_events[this_event].event->object.overlay ) ;
-#endif
-                    
-          /* let EVENT_MENU_BUTTON set the correct clipping box */
-          if( menu_changed ) {
-            overlay->clip_top = overlay->height;
-            overlay->clip_bottom = 0;
-            overlay->clip_left = overlay->width;
-            overlay->clip_right = 0;
-          }
-          
-          add_showing_handle( this, handle );
-          
-          /* The null test was done at the start of this case statement */
-          free (this->video_overlay_events[this_event].event->object.overlay);
+          this->video_overlay_objects[handle].overlay = 
+             this->video_overlay_events[this_event].event->object.overlay;
+          this->video_overlay_objects[handle].pts = 
+             this->video_overlay_events[this_event].event->object.pts;
           this->video_overlay_events[this_event].event->object.overlay = NULL;
-        } else {
-          printf ("video_overlay:overlay present\n");
+        
+          add_showing_handle( this, handle );
         }
-        break;
+break;
   
       case EVENT_MENU_BUTTON:
         /* mixes palette and copy clip coords */
@@ -554,7 +508,7 @@ static void video_overlay_event( video_overlay_t *this, int vpts ) {
         }
         if ( (this->video_overlay_events[this_event].event->object.pts != 
                 this->video_overlay_objects[handle].pts) ) {
-          printf ("MENU BUTTON DROPPED menu pts=%u spu pts=%u\n",      
+          printf ("MENU BUTTON DROPPED menu pts=%lld spu pts=%lld\n",      
             this->video_overlay_events[this_event].event->object.pts,
             this->video_overlay_objects[handle].pts);
         }
@@ -584,12 +538,14 @@ static void video_overlay_event( video_overlay_t *this, int vpts ) {
   }
   
   pthread_mutex_unlock (&this->video_overlay_events_mutex);
+
+  return processed;
 }
   
 /* This is called from video_out.c 
  * must call output->overlay_blend for each active overlay.
  */
-static void video_overlay_multiple_overlay_blend(video_overlay_instance_t *this_gen, int vpts, 
+static void video_overlay_multiple_overlay_blend(video_overlay_instance_t *this_gen, int64_t vpts, 
                                          vo_driver_t *output, vo_frame_t *vo_img, int enabled) {
   video_overlay_t *this = (video_overlay_t *) this_gen;
   int i;
@@ -623,6 +579,44 @@ static void video_overlay_flush_events(video_overlay_instance_t *this_gen )
   video_overlay_event( this, 0 );
 }
 
+/* this is called from video_out.c on still frames to check 
+   if a redraw is needed.
+*/
+static int video_overlay_redraw_needed(video_overlay_instance_t *this_gen, int64_t vpts )
+{
+  video_overlay_t *this = (video_overlay_t *) this_gen;
+  
+  return video_overlay_event( this, vpts );
+}
+
+
+static void video_overlay_dispose(video_overlay_instance_t *this_gen) {
+
+  video_overlay_t *this = (video_overlay_t *) this_gen;
+  int i;
+
+  for (i=0; i < MAX_EVENTS; i++) {
+    if (this->video_overlay_events[i].event != NULL) {
+      if (this->video_overlay_events[i].event->object.overlay != NULL) {
+        if (this->video_overlay_events[i].event->object.overlay->rle)
+	  free (this->video_overlay_events[i].event->object.overlay->rle);
+        free (this->video_overlay_events[i].event->object.overlay);
+      }
+      free (this->video_overlay_events[i].event);
+    }
+  }
+
+  for (i=0; i < MAX_OBJECTS; i++) {
+    if (this->video_overlay_objects[i].overlay != NULL) {
+      if (this->video_overlay_objects[i].overlay->rle)
+	free (this->video_overlay_objects[i].overlay->rle);
+      free (this->video_overlay_objects[i].overlay);
+    }
+  }
+
+  free (this);
+}
+
 
 video_overlay_instance_t *video_overlay_new_instance () {
 
@@ -631,10 +625,12 @@ video_overlay_instance_t *video_overlay_new_instance () {
   this = (video_overlay_t *) xine_xmalloc (sizeof (video_overlay_t));
 
   this->video_overlay.init                = video_overlay_init;
+  this->video_overlay.dispose             = video_overlay_dispose;
   this->video_overlay.get_handle          = video_overlay_get_handle;
   this->video_overlay.free_handle         = video_overlay_free_handle;
   this->video_overlay.add_event           = video_overlay_add_event;
   this->video_overlay.flush_events        = video_overlay_flush_events;
+  this->video_overlay.redraw_needed       = video_overlay_redraw_needed;
   this->video_overlay.multiple_overlay_blend = video_overlay_multiple_overlay_blend;
 
   return (video_overlay_instance_t *) &this->video_overlay;
