@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine.c,v 1.98 2002/01/15 13:51:10 guenter Exp $
+ * $Id: xine.c,v 1.121 2002/04/28 17:39:36 guenter Exp $
  *
  * top-level xine functions
  *
@@ -56,26 +56,6 @@
 #include "xineutils.h"
 #include "compat.h"
 
-#ifdef __GNUC__
-#define LOG_MSG_STDERR(xine, message, args...) {                     \
-    xine_log(xine, XINE_LOG_MSG, message, ##args);                   \
-    fprintf(stderr, message, ##args);                                \
-  }
-#define LOG_MSG(xine, message, args...) {                            \
-    xine_log(xine, XINE_LOG_MSG, message, ##args);                   \
-    printf(message, ##args);                                         \
-  }
-#else
-#define LOG_MSG_STDERR(xine, ...) {                                  \
-    xine_log(xine, XINE_LOG_MSG, __VA_ARGS__);                       \
-    fprintf(stderr, __VA_ARGS__);                                    \
-  }
-#define LOG_MSG(xine, ...) {                                         \
-    xine_log(xine, XINE_LOG_MSG, __VA_ARGS__);                       \
-    printf(__VA_ARGS__);                                             \
-  }
-#endif
-
 void * xine_notify_stream_finished_thread (void * this_gen) {
   xine_t *this = this_gen;
   xine_event_t event;
@@ -86,6 +66,8 @@ void * xine_notify_stream_finished_thread (void * this_gen) {
 
   xine_send_event (this, &event);
 
+  pthread_detach( pthread_self() );
+
   return NULL;
 }
 
@@ -104,19 +86,46 @@ void xine_notify_stream_finished (xine_t *this) {
   */
   if ((err = pthread_create (&finished_thread,
 			     NULL, xine_notify_stream_finished_thread, this)) != 0) {
-    LOG_MSG_STDERR(this, _("xine_notify_stream_finished: can't create new thread (%s)\n"),
-		   strerror(err));
+    printf (_("xine_notify_stream_finished: can't create new thread (%s)\n"),
+	    strerror(err));
     exit (1);
   }
 }
 
+/* internal use only - called from demuxers on seek/stop
+ * warning: after clearing decoders fifos an absolute discontinuity
+ *          indication must be sent. relative discontinuities are likely
+ *          to cause "jumps" on metronom.
+ */
+void xine_flush_engine (xine_t *this) {
+
+  buf_element_t *buf;
+
+  this->video_fifo->clear(this->video_fifo);
+  if( this->audio_fifo )
+    this->audio_fifo->clear(this->audio_fifo);
+  
+  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
+  buf->type            = BUF_CONTROL_RESET_DECODER;
+  this->video_fifo->put (this->video_fifo, buf);
+
+  if(this->audio_fifo) {
+    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+    buf->type            = BUF_CONTROL_RESET_DECODER;
+    this->audio_fifo->put (this->audio_fifo, buf);
+  }
+  
+  this->metronom->adjust_clock(this->metronom,
+			       this->metronom->get_current_time(this->metronom) + 30 * 90000 );
+}
+
 static void xine_internal_osd (xine_t *this, char *str, 
 			       uint32_t start_time, uint32_t duration) {
 
   uint32_t seconds;
   char tstr[256];
 
-  if( this->osd_display ) {
+  if (this->osd_display) {
    
     this->osd_renderer->filled_rect (this->osd, 0, 0, 299, 99, 0);
     this->osd_renderer->render_text (this->osd, 0, 5, str, OSD_TEXT1);
@@ -142,34 +151,54 @@ static void update_osd_display(void *this_gen, cfg_entry_t *entry)
   this->osd_display = entry->num_value;
 }
 
+
+static void xine_set_speed_internal (xine_t *this, int speed) {
+
+  this->metronom->set_speed (this->metronom, speed);
+
+  /* see coment on audio_out loop about audio_paused */
+  if( this->audio_out ) {
+    this->audio_out->audio_paused = (speed != SPEED_NORMAL) + 
+                                    (speed == SPEED_PAUSE);
+
+    if (speed != SPEED_NORMAL && speed != SPEED_PAUSE)
+	this->audio_out->control(this->audio_out, AO_CTRL_FLUSH_BUFFERS);
+
+    this->audio_out->control(this->audio_out,
+			     speed == SPEED_PAUSE ? AO_CTRL_PLAY_PAUSE : AO_CTRL_PLAY_RESUME);
+  }
+
+  this->speed = speed;
+}
+
+
 void xine_stop_internal (xine_t *this) {
 
   pthread_mutex_lock (&this->xine_lock);
 
-  LOG_MSG(this, _("xine_stop\n"));
+  printf ("xine_stop\n");
 
   /* xine_internal_osd (this, "}", this->metronom->get_current_time (this->metronom), 30000); never works */
 
   if (this->status == XINE_STOP) {
-    LOG_MSG(this, _("xine_stop ignored\n"));
+    printf ("xine_stop ignored\n");
     pthread_mutex_unlock (&this->xine_lock);
     return;
   }
 
+  if (this->audio_out)
+    this->audio_out->control(this->audio_out, AO_CTRL_FLUSH_BUFFERS);
 
-  this->metronom->set_speed (this->metronom, SPEED_NORMAL);
-  this->speed      = SPEED_NORMAL;
-  this->video_out->video_paused = 0;
-  if( this->audio_out )
-    this->audio_out->audio_paused = 0;
+  xine_set_speed_internal(this, SPEED_NORMAL);
 
   this->status = XINE_STOP;
-  LOG_MSG(this, _("xine_stop: stopping demuxer\n"));
-
+    
+  printf ("xine_stop: stopping demuxer\n");
   if(this->cur_demuxer_plugin) {
     this->cur_demuxer_plugin->stop (this->cur_demuxer_plugin);
     this->cur_demuxer_plugin = NULL;
   }
+  printf ("xine_stop: stopped demuxer\n");
 
   if(this->cur_input_plugin) {
     this->cur_input_plugin->close(this->cur_input_plugin);
@@ -181,7 +210,7 @@ void xine_stop_internal (xine_t *this) {
      */
   }
 
-  LOG_MSG(this, _("xine_stop: done\n"));
+  printf ("xine_stop: done\n");
 
   pthread_mutex_unlock (&this->xine_lock);
 }
@@ -194,7 +223,7 @@ void xine_stop (xine_t *this) {
      am i abusing of xine architeture? :)
   */
   this->metronom->adjust_clock(this->metronom,
-    this->metronom->get_current_time(this->metronom) + 30 * 90000 );
+			       this->metronom->get_current_time(this->metronom) + 30 * 90000 );
 }
 
 
@@ -212,8 +241,8 @@ static int try_demux_with_stages(xine_t *this, const char *MRL,
   stages[2] = -1;
 
   if(stages[0] == -1) {
-    LOG_MSG_STDERR(this, _("%s(%d) wrong first stage = %d !!\n"), 
-		   __XINE_FUNCTION__, __LINE__, stage1);
+    printf (_("%s(%d) wrong first stage = %d !!\n"), 
+	    __XINE_FUNCTION__, __LINE__, stage1);
     return 0;
   }
 
@@ -269,22 +298,24 @@ static int find_demuxer(xine_t *this, const char *MRL) {
 }
 
 int xine_play (xine_t *this, char *mrl, 
-		int start_pos, int start_time) {
+	       int start_pos, int start_time) {
 
   double     share ;
   off_t      pos, len;
   int        i;
 
-  LOG_MSG(this, _("xine_play: xine open %s, start pos = %d, start time = %d (sec)\n"), 
+  printf ("xine_play: xine open %s, start pos = %d, start time = %d (sec)\n", 
 	  mrl, start_pos, start_time);
 
   pthread_mutex_lock (&this->xine_lock);
 
+  xine_set_speed_internal (this, SPEED_NORMAL);
+  
   /*
-   * stop engine?
+   * stop engine only for different mrl
    */
 
-  if (this->status == XINE_PLAY) {
+  if (this->status == XINE_PLAY && strcmp (mrl, this->cur_mrl) ) {
     
     if(this->cur_demuxer_plugin) {
       this->cur_demuxer_plugin->stop (this->cur_demuxer_plugin);
@@ -298,88 +329,104 @@ int xine_play (xine_t *this, char *mrl,
 	this->cur_input_plugin->stop(this->cur_input_plugin);
     }
 
-    /* this will make output threads discard about everything
-       (seeking should be faster!) */
-    this->metronom->adjust_clock(this->metronom,
-      this->metronom->get_current_time(this->metronom) + 30 * 90000 );
+    if (this->audio_out)
+      this->audio_out->control(this->audio_out, AO_CTRL_FLUSH_BUFFERS);
 
     this->status = XINE_STOP;
   }
 
-  /*
-   * find input plugin
-   */
+  if (this->status == XINE_STOP ) {
+    /*
+     * find input plugin
+     */
+    this->cur_input_plugin = NULL;
 
-  this->cur_input_plugin = NULL;
-
-  for (i = 0; i < this->num_input_plugins; i++) {
-    if (this->input_plugins[i]->open(this->input_plugins[i], mrl)) {
-      this->cur_input_plugin = this->input_plugins[i];
-      break;
+    for (i = 0; i < this->num_input_plugins; i++) {
+      if (this->input_plugins[i]->open(this->input_plugins[i], mrl)) {
+        this->cur_input_plugin = this->input_plugins[i];
+        break;
+      }
     }
-  }
 
-  if (!this->cur_input_plugin) {
-    LOG_MSG(this, _("xine: cannot find input plugin for this MRL\n"));
-    this->cur_demuxer_plugin = NULL;
-    this->err = XINE_ERROR_NO_INPUT_PLUGIN;
-    pthread_mutex_unlock (&this->xine_lock);
+    if (!this->cur_input_plugin) {
+      xine_log (this, XINE_LOG_FORMAT,
+	        _("xine: cannot find input plugin for this MRL\n"));
+      this->cur_demuxer_plugin = NULL;
+      this->err = XINE_ERROR_NO_INPUT_PLUGIN;
+      pthread_mutex_unlock (&this->xine_lock);
 
-    return 0;
-  }
+      return 0;
+    }
   
-  LOG_MSG(this, _("xine: using input plugin >%s< for this MRL (%s).\n"), 
-	  this->cur_input_plugin->get_identifier(this->cur_input_plugin), mrl);
+    printf ("xine: using input plugin >%s< for this MRL (%s).\n", 
+	    this->cur_input_plugin->get_identifier(this->cur_input_plugin), mrl);
 
-  /*
-   * find demuxer plugin
-   */
+    xine_log (this, XINE_LOG_FORMAT,
+	      _("using input plugin '%s' for MRL '%s'\n"),
+	      this->cur_input_plugin->get_identifier(this->cur_input_plugin), 
+	      mrl);
 
-  if (!find_demuxer(this, mrl)) {
-    LOG_MSG(this, _("xine: couldn't find demuxer for >%s<\n"), mrl);
-    this->err = XINE_ERROR_NO_DEMUXER_PLUGIN;
-    pthread_mutex_unlock (&this->xine_lock);
-    return 0;
+    /*
+     * find demuxer plugin
+     */
+
+    if (!find_demuxer(this, mrl)) {
+      xine_log (this, XINE_LOG_FORMAT, 
+	        _("xine: couldn't find demuxer for >%s<\n"), mrl);
+      this->cur_input_plugin->close(this->cur_input_plugin);
+      this->err = XINE_ERROR_NO_DEMUXER_PLUGIN;
+      pthread_mutex_unlock (&this->xine_lock);
+      return 0;
+    }
+
+    xine_log (this, XINE_LOG_FORMAT,
+	      _("system layer format '%s' detected.\n"),
+	      this->cur_demuxer_plugin->get_identifier());
   }
-
-  LOG_MSG(this, _("xine: using demuxer plugin >%s< for this MRL.\n"),
-	  this->cur_demuxer_plugin->get_identifier());
-  
+    
   /*
    * start demuxer
    */
 
   if (start_pos) {
+    /* FIXME: do we need to protect concurrent access to input plugin here? */
     len = this->cur_input_plugin->get_length (this->cur_input_plugin);
     share = (double) start_pos / 65535;
     pos = (off_t) (share * len) ;
   } else
     pos = 0;
-
-  this->cur_demuxer_plugin->start (this->cur_demuxer_plugin,
-				   this->video_fifo,
-				   this->audio_fifo, 
-				   pos, start_time);
+  
+  if( this->status == XINE_STOP )
+    this->cur_demuxer_plugin->start (this->cur_demuxer_plugin,
+				     this->video_fifo,
+				     this->audio_fifo, 
+				     pos, start_time);
+  else
+    this->cur_demuxer_plugin->seek (this->cur_demuxer_plugin,
+				     pos, start_time);
   
   if (this->cur_demuxer_plugin->get_status(this->cur_demuxer_plugin) != DEMUX_OK) {
-    LOG_MSG(this, _("xine_play: demuxer failed to start\n"));
     
-    this->cur_input_plugin->close(this->cur_input_plugin);
+    xine_log (this, XINE_LOG_MSG, 
+	      _("xine_play: demuxer failed to start\n"));
+    
+    this->err = XINE_ERROR_DEMUXER_FAILED;
+    
+    if( this->status == XINE_STOP )      
+      this->cur_input_plugin->close(this->cur_input_plugin);
+  
+    pthread_mutex_unlock (&this->xine_lock);
 
-    this->status = XINE_STOP;
+    return 0;
+    
   } else {
 
     this->status = XINE_PLAY;
     strncpy (this->cur_mrl, mrl, 1024);
     
-    this->metronom->set_speed (this->metronom, SPEED_NORMAL);
-    this->video_out->video_paused = 0;
-    if( this->audio_out )
-      this->audio_out->audio_paused = 0;
-    this->speed = SPEED_NORMAL;
-
     /* osd */
-    xine_internal_osd (this, ">", 0, 300000);
+    xine_usec_sleep(100000); /* FIXME: how do we assure an updated cur_input_time? */
+    xine_internal_osd (this, ">", this->metronom->get_current_time (this->metronom), 300000);
 
   }
 
@@ -409,22 +456,68 @@ int xine_eject (xine_t *this) {
 
 void xine_exit (xine_t *this) {
 
+  int i, j;
+
   xine_stop(this);
     
-  LOG_MSG(this, _("xine_exit: shutdown audio\n"));
+  printf ("xine_exit: shutdown audio\n");
 
   audio_decoder_shutdown (this);
 
-  LOG_MSG(this, _("xine_exit: shutdown video\n"));
+  printf ("xine_exit: shutdown video\n");
 
   video_decoder_shutdown (this);
 
+  this->osd_renderer->close( this->osd_renderer );
+  this->video_out->exit (this->video_out);
+  this->video_fifo->dispose (this->video_fifo);
+
   this->status = XINE_QUIT;
 
-  LOG_MSG(this, _("xine_exit: bye!\n"));
+  printf ("xine_exit: bye!\n");
+
+  for (i = 0; i < XINE_LOG_NUM; i++)
+    this->log_buffers[i]->dispose (this->log_buffers[i]);
+
+  this->metronom->exit (this->metronom);
+
+  for (i = 0; i < this->num_demuxer_plugins; i++)
+    this->demuxer_plugins[i]->close (this->demuxer_plugins[i]);
+
+  for (i = 0; i < this->num_input_plugins; i++)
+    this->input_plugins[i]->dispose (this->input_plugins[i]);
+
+  for (i = 0; i < DECODER_PLUGIN_MAX; i++) {
+    if (this->audio_decoder_plugins[i]) {
+      this->audio_decoder_plugins[i]->dispose (this->audio_decoder_plugins[i]);
+
+      for (j = i + 1; j < DECODER_PLUGIN_MAX; j++) {
+	if (this->audio_decoder_plugins[j] == this->audio_decoder_plugins[i])
+	  this->audio_decoder_plugins[j] = NULL;
+      }
+    }
+    if (this->video_decoder_plugins[i]) {
+      this->video_decoder_plugins[i]->dispose (this->video_decoder_plugins[i]);
+
+      for (j = i + 1; j < DECODER_PLUGIN_MAX; j++) {
+	if (this->video_decoder_plugins[j] == this->video_decoder_plugins[i])
+	  this->video_decoder_plugins[j] = NULL;
+      }
+    }
+    if (this->spu_decoder_plugins[i]) {
+      this->spu_decoder_plugins[i]->dispose (this->spu_decoder_plugins[i]);
+
+      for (j = i + 1; j < DECODER_PLUGIN_MAX; j++) {
+	if (this->spu_decoder_plugins[j] == this->spu_decoder_plugins[i])
+	  this->spu_decoder_plugins[j] = NULL;
+      }
+    }
+  }
 
   xine_profiler_print_results ();
 
+  free (this);
+
 }
 
 xine_t *xine_init (vo_driver_t *vo, 
@@ -446,7 +539,7 @@ xine_t *xine_init (vo_driver_t *vo,
   bindtextdomain("xine-lib", XINE_LOCALEDIR);
 #endif 
 
-  LOG_MSG(this, _("xine_init entered\n"));
+  printf ("xine: xine_init entered\n");
   
   this->err     = XINE_ERROR_NONE;
   this->config  = config;
@@ -454,9 +547,6 @@ xine_t *xine_init (vo_driver_t *vo,
   /* probe for optimized memcpy or config setting */
   xine_probe_fast_memcpy(config);
   
-  /* initialize aligned mem allocator */
-  xine_init_mem_aligned();
-
   /*
    * init locks
    */
@@ -494,6 +584,7 @@ xine_t *xine_init (vo_driver_t *vo,
   this->spu_channel_auto   = -1;
   this->spu_channel_user   = -1;
   this->cur_input_pos      = 0;
+  this->cur_input_length   = 0;
 
   /*
    * init and start decoder threads
@@ -504,7 +595,7 @@ xine_t *xine_init (vo_driver_t *vo,
   this->video_out = vo_new_instance (vo, this);
   video_decoder_init (this);
 
-  this->osd_renderer = osd_renderer_init( this->video_out->overlay_source, config );
+  this->osd_renderer = osd_renderer_init (this->video_out->get_overlay_instance (this->video_out), config );
   
   this->osd = this->osd_renderer->new_object (this->osd_renderer, 300, 100);
   this->osd_renderer->set_font (this->osd, "cetus", 24);
@@ -516,10 +607,29 @@ xine_t *xine_init (vo_driver_t *vo,
                                             update_osd_display, this );
   
   if(ao) 
-    this->audio_out = ao_new_instance (ao, this->metronom, config);
+    this->audio_out = ao_new_instance (ao, this);
 
   audio_decoder_init (this);
-  LOG_MSG(this, _("xine_init returning\n"));
+
+  /*
+   * start metronom clock (needed for osd)
+   */
+
+  this->metronom->start_clock (this->metronom, 0);
+
+  if (this->osd_display) {
+   
+    char tstr[30];
+
+    this->osd_renderer->filled_rect (this->osd, 0, 0, 299, 99, 0);
+    sprintf (tstr, "xine-lib v%01d.%01d.%01d", XINE_MAJOR, XINE_MINOR, XINE_SUB);
+
+    this->osd_renderer->render_text (this->osd, 5, 5, tstr, OSD_TEXT1);
+  
+    this->osd_renderer->show (this->osd, 0);
+    this->osd_renderer->hide (this->osd, 300000);
+  }
+
 
   return this;
 }
@@ -560,14 +670,14 @@ int xine_get_current_position (xine_t *this) {
   pthread_mutex_lock (&this->xine_lock);
 
   if (!this->cur_input_plugin) {
-    LOG_MSG(this, _("xine: xine_get_current_position: no input source\n"));
+    printf ("xine: xine_get_current_position: no input source\n");
     pthread_mutex_unlock (&this->xine_lock);
     return 0;
   }
   
   /* pos = this->mCurInput->seek (0, SEEK_CUR); */
-  len = this->cur_input_plugin->get_length (this->cur_input_plugin);
-
+  len = this->cur_input_length;
+  if (len == 0) len = this->cur_input_plugin->get_length (this->cur_input_plugin); 
   share = (double) this->cur_input_pos / (double) len * 65535;
 
   pthread_mutex_unlock (&this->xine_lock);
@@ -630,11 +740,11 @@ int xine_check_version(int major, int minor, int sub) {
  */
 
 void xine_set_av_offset (xine_t *this, int offset_pts) {
-  this->metronom->set_av_offset (this->metronom, offset_pts);
+  this->metronom->set_option (this->metronom, METRONOM_AV_OFFSET, offset_pts);
 }
 
 int xine_get_av_offset (xine_t *this) {
-  return this->metronom->get_av_offset (this->metronom);
+  return this->metronom->get_option (this->metronom, METRONOM_AV_OFFSET);
 }
 
 /*
@@ -643,8 +753,6 @@ int xine_get_av_offset (xine_t *this) {
 
 void xine_set_speed (xine_t *this, int speed) {
 
-  struct timespec tenth_sec;
-
   pthread_mutex_lock (&this->xine_lock);
 
   if (speed <= SPEED_PAUSE) 
@@ -676,22 +784,10 @@ void xine_set_speed (xine_t *this, int speed) {
   } 
 
   /* make sure osd can be displayed */
-  tenth_sec.tv_sec = 0;
-  tenth_sec.tv_nsec = 100000000;
+  xine_usec_sleep(100000);
 
-  nanosleep (&tenth_sec, NULL);
-
-  LOG_MSG(this, _("xine: set_speed %d\n"), speed);
-
-  this->metronom->set_speed (this->metronom, speed);
-
-  this->video_out->video_paused = (speed == SPEED_PAUSE);
-  /* see coment on audio_out loop about audio_paused */
-  if( this->audio_out )
-    this->audio_out->audio_paused = (speed != SPEED_NORMAL) + 
-                                    (speed == SPEED_PAUSE);
-
-  this->speed      = speed;
+  printf ("xine: set_speed %d\n", speed);
+  xine_set_speed_internal (this, speed);
 
   pthread_mutex_unlock (&this->xine_lock);
 }
@@ -818,12 +914,12 @@ void xine_get_audio_lang (xine_t *this, char *str) {
   }
 }
 
-
 int xine_is_stream_seekable (xine_t *this) {
 
-  if (this->cur_input_plugin) 
+  if (this->cur_input_plugin)
     return this->cur_input_plugin->get_capabilities (this->cur_input_plugin) & INPUT_CAP_SEEKABLE;
-  return 0;
+
+  return -1;
 }
 
 osd_renderer_t *xine_get_osd_renderer (xine_t *this) {
@@ -834,20 +930,16 @@ osd_renderer_t *xine_get_osd_renderer (xine_t *this) {
 /*
  * log functions
  */
-unsigned int xine_get_log_section_count(void) {
+int xine_get_log_section_count (xine_t *this) {
   return XINE_LOG_NUM;
 }
 
-const char **xine_get_log_names(void) {
-  static const char *log_sections[XINE_LOG_NUM + 1];
+char **xine_get_log_names (xine_t *this) {
+  static char *log_sections[XINE_LOG_NUM + 1];
 
-  log_sections[XINE_LOG_MSG]      = _("messages");  /* XINE_LOG_MSG      */
-  log_sections[XINE_LOG_INPUT]    = _("inputs");    /* XINE_LOG_INPUT    */
-  log_sections[XINE_LOG_DEMUX]    = _("demuxers");  /* XINE_LOG_DEMUX    */
-  log_sections[XINE_LOG_CODEC]    = _("codecs");    /* XINE_LOG_CODEC    */
-  log_sections[XINE_LOG_VIDEO]    = _("video");     /* XINE_LOG_VIDEO    */
-  log_sections[XINE_LOG_METRONOM] = _("metronom");  /* XINE_LOG_METRONOM */
-  log_sections[XINE_LOG_PLUGIN]   = _("plugin");    /* XINE_LOG_PLUGIN   */
+  log_sections[XINE_LOG_FORMAT]   = _("stream format");
+  log_sections[XINE_LOG_MSG]      = _("messages");  
+  log_sections[XINE_LOG_PLUGIN]   = _("plugin");
   log_sections[XINE_LOG_NUM]      = NULL;
   
   return log_sections;
@@ -859,7 +951,12 @@ void xine_log (xine_t *this, int buf, const char *format, ...) {
 
   va_start (argp, format);
 
-  this->log_buffers[buf]->printf (this->log_buffers[buf], format, argp);
+  this->log_buffers[buf]->scratch_printf (this->log_buffers[buf], format, argp);
+  va_end (argp);
+
+  va_start (argp, format);
+
+  vprintf (format, argp);
 
   va_end (argp);
 }
