@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_internal.h,v 1.69 2002/01/09 22:33:04 jcdutton Exp $
+ * $Id: xine_internal.h,v 1.79 2002/04/23 15:45:26 esnel Exp $
  *
  */
 
@@ -55,7 +55,7 @@ extern "C" {
 #define INPUT_PLUGIN_MAX       50
 #define DEMUXER_PLUGIN_MAX     50
 #define DECODER_PLUGIN_MAX     256
-#define DECODER_PLUGIN_IFACE_VERSION      5
+#define DECODER_PLUGIN_IFACE_VERSION      7
 #define AUDIO_OUT_PLUGIN_MAX   50
 #define VIDEO_OUT_PLUGIN_MAX   50
 #define XINE_MAX_EVENT_LISTENERS 50
@@ -80,12 +80,16 @@ struct video_decoder_s {
 
   void (*decode_data) (video_decoder_t *this, buf_element_t *buf);
 
+  void (*reset) (video_decoder_t *this);
+  
   void (*flush) (video_decoder_t *this);
 
   void (*close) (video_decoder_t *this);
 
   char* (*get_identifier) (void);
 
+  void (*dispose) (video_decoder_t *this);
+
   int priority;
 
   metronom_t *metronom;
@@ -118,6 +122,8 @@ struct audio_decoder_s {
 
   char* (*get_identifier) (void);
 
+  void (*dispose) (audio_decoder_t *this);
+
   int priority;
 
 };
@@ -138,14 +144,10 @@ struct audio_decoder_s {
 /*
  * log output
  */
-#define XINE_LOG_MSG       0
-#define XINE_LOG_INPUT     1
-#define XINE_LOG_DEMUX     2
-#define XINE_LOG_CODEC     3
-#define XINE_LOG_VIDEO     4
-#define XINE_LOG_METRONOM  5
-#define XINE_LOG_PLUGIN    6
-#define XINE_LOG_NUM       7 /* # of log buffers defined */
+#define XINE_LOG_FORMAT    0 /* stream format, decoders, video size... */
+#define XINE_LOG_MSG       1 /* warnings, errors, ... */
+#define XINE_LOG_PLUGIN    2
+#define XINE_LOG_NUM       3 /* # of log buffers defined */
 
 typedef void (*xine_event_listener_t) (void *user_data, xine_event_t *);
 
@@ -169,6 +171,7 @@ struct xine_s {
   int                        status;
   int                        speed;
   off_t                      cur_input_pos;
+  off_t                      cur_input_length;
   int                        cur_input_time;
   char                       cur_mrl[1024];
 
@@ -240,7 +243,7 @@ struct xine_s {
  * read config file and init a config object
  * (if it exists)
  */
-config_values_t *config_file_init (char *filename);
+config_values_t *xine_config_file_init (char *filename);
 
 /*
  * init xine - call once at startup
@@ -411,6 +414,7 @@ char **xine_get_autoplay_mrls (xine_t *this, char *plugin_id, int *num_mrls);
  */
 
 void xine_notify_stream_finished (xine_t *this);
+void xine_flush_engine (xine_t *this);
 
 /*
  * video decoder stuff
@@ -502,6 +506,7 @@ void load_decoder_plugins (xine_t *this,
 #define VISUAL_TYPE_AA    2
 #define VISUAL_TYPE_FB    3
 #define VISUAL_TYPE_GTK   4
+#define VISUAL_TYPE_DFB   5
 
 /*
  * list_video_output_plugins
@@ -596,11 +601,11 @@ osd_renderer_t *xine_get_osd_renderer (xine_t *this);
  * xine log functions 
  */
   
-const char **xine_get_log_names(void);
-
-void xine_log (xine_t *this, int buf, const char *format, ...);
-
+char **xine_get_log_names (xine_t *this);
+void   xine_log (xine_t *this, int buf, const char *format, ...);
 char **xine_get_log (xine_t *this, int buf);
+int    xine_get_log_section_count (xine_t *this);
+
 
 /*
  * xine error reporting
@@ -609,6 +614,7 @@ char **xine_get_log (xine_t *this, int buf);
 #define XINE_ERROR_NONE              0
 #define XINE_ERROR_NO_INPUT_PLUGIN   1
 #define XINE_ERROR_NO_DEMUXER_PLUGIN 2
+#define XINE_ERROR_DEMUXER_FAILED    3
 
 int xine_get_error (xine_t *this);
 
