@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xineutils.h,v 1.6 2002/01/02 18:16:08 jkeil Exp $
+ * $Id: xineutils.h,v 1.13 2002/04/27 14:35:39 richwareham Exp $
  *
  */
 #ifndef XINEUTILS_H
@@ -28,10 +28,32 @@ extern "C" {
 #endif
 
 #include <unistd.h>
+#include <stdlib.h>
+#include <string.h>
 #include <inttypes.h>
+#include <pthread.h>
 #include "attributes.h"
 #include "compat.h"
 
+  /*
+   * debugable mutexes
+   */
+
+  typedef struct {
+    pthread_mutex_t  mutex;
+    char             id[80];
+    char            *locked_by;
+  } xine_mutex_t;
+
+  int xine_mutex_init    (xine_mutex_t *mutex, const pthread_mutexattr_t *mutexattr,
+			  char *id);
+  
+  int xine_mutex_lock    (xine_mutex_t *mutex, char *who);
+  int xine_mutex_unlock  (xine_mutex_t *mutex, char *who);
+  int xine_mutex_destroy (xine_mutex_t *mutex);
+
+
+
 			/* CPU Acceleration */
 
 /*
@@ -543,13 +565,7 @@ void xine_probe_fast_memcpy(config_values_t *config);
 
 		      /* Debugging/Monitoring */
 
-#ifdef __GNUC__
-#define perr(FMT,ARGS...) {fprintf(stderr, FMT, ##ARGS);fflush(stderr);}
-#else /* C99 version: */
-#define perr(...)	  {fprintf(stderr, __VA_ARGS__);fflush(stderr);}
-#endif
-
-#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 95) || !defined(__GNUC__) 
+#if (__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 95) || !defined(__GNUC__)) && !defined(__cplusplus)
 #define xlerror(...) do {                                                     \
         printf("XINE lib %s:%d:(%s) ", __FILE__, __LINE__, __XINE_FUNCTION__);   \
         printf(__VA_ARGS__);                                                     \
@@ -596,18 +612,10 @@ void *xine_xmalloc(size_t size);
 
 /*
  * Same as above, but memory is aligned to 'alignement'.
+ * **base is used to return pointer to un-aligned memory, use
+ * this to free the mem chunk
  */
-void *xine_xmalloc_aligned(size_t alignment, size_t size);
-
-/*
- * Free memory allocated with the above function
- */
-void *xine_free_aligned(void *p);
-
-/* 
- * Initialize aligned memory allocator
- */
-void xine_init_mem_aligned(void);
+void *xine_xmalloc_aligned(size_t alignment, size_t size, void **base);
 
 /* 
  * Get user home directory.
@@ -699,6 +707,26 @@ static inline char *_x_strsep(char **stringp, const char *delim) {
 #endif
 
 
+#ifdef	HAVE_SETENV
+#define	xine_setenv	setenv
+#else
+static inline void _x_setenv(const char *name, const char *val, int _xx)
+{
+  int len  = strlen(name) + strlen(val) + 2;
+  char *env;
+  
+  env = (char*)malloc(len);
+
+  if (env != NULL) {
+    strcpy(env, name);
+    strcat(env, "=");
+    strcat(env, val);
+    putenv(env);
+  }
+}
+#define	xine_setenv	_x_setenv
+#endif
+
 #ifdef __cplusplus
 }
 #endif
